# Utilities for the Tdyn Interface
proc TIU_getbooknames { book } {
    set matlist ""
    foreach matname [.central.s info materials] {
	if { [string equal $book [.central.s info materials $matname BOOK]] } {
	    lappend matlist $matname
	}
    }
    return $matlist
}

proc TIU_getmaterialnames { book {field ""} {value ""} } {
    if { $field == "" } {
	return [TIU_getbooknames $book]
    }
    set matlist ""
    foreach matname [.central.s info materials] {
	if { [string equal $book [.central.s info materials $matname BOOK]] } {
	    if { [string equal $value [TUI_getmatfieldvalue $matname $field]] } {
		lappend matlist $matname
	    }
	}
    }
    return $matlist
}

proc TIU_getmaterialnamesbyfield { field values } {

    set matlist ""
    foreach matname [.central.s info materials] {
	set value [TUI_getmatfieldvalue $matname $field]
	if { [lsearch $values $value] != -1 } {
	    lappend matlist $matname
	}
    }
    return $matlist
}

proc TUI_getmatfieldvalue { matname fieldname } {
    set value ""
    set matinfo [.central.s info materials $matname]
 
    if { [regexp {#CB#\(} $fieldname] } {
	set idx [lsearch -exact $matinfo $fieldname]
	if { $idx == -1 } {
	    set idx [lsearch $matinfo $fieldname]
	}
    } else {
	set idx [lsearch -exact $matinfo $fieldname]
	if { $idx == -1 } {
	    set idx [lsearch -regexp $matinfo [eval set temp ^${fieldname}(\$|\#CB|\:)]]
	}
    }
    if { $idx != -1 } {
	set value [lindex $matinfo [expr $idx+1]]
    }
#    for { set i 0 } { $i < [lindex $aa 0] } { incr i } {
#        if { [lindex $aa [expr $i*2+1]] == $fieldname } {
#            set value [lindex $aa [ expr $i*2+2]]
#            break
#        }
#    }
    return $value
}

proc TUI_getfieldvalue { fieldname info } {
    set value ""
 
    if { [regexp {#CB#\(} $fieldname] } {
	set idx [lsearch -exact $info $fieldname]
	if { $idx == -1 } {
	    set idx [lsearch $info $fieldname]
	}
    } else {
	set idx [lsearch -exact $info $fieldname]
	if { $idx == -1 } {
	    set idx [lsearch -regexp $info [eval set temp ^${fieldname}(\$|\#CB|\:)]]
	}
    }
    if { $idx != -1 } {
	set value [lindex $info [expr $idx+1]]
    }

    return $value
}

proc TUI_getmatvalueslist { matname } {

    set vallist " "
    set aa [.central.s info materials $matname]
    for { set i 0 } { $i < [lindex $aa 0] } { incr i } {
	lappend vallist [lindex $aa [expr $i*2+2]]
    }
    return $vallist
}

proc TUI_setprbvalue { field value } {
    #.central.s process escape escape escape escape Data \
    #    ProblemData -SingleField- $field "\"$value\""
    #.central.s process escape escape escape escape
    GiD_AccessValue set gendata $field $value
}

proc TUI_setnewmatvalues { matname values } {

    GidUtils::DisableWarnLine
    #.central.s process escape escape escape escape Data Materials \
    #        NewMaterial $matname $matname Yes $values
    #.central.s process escape escape escape escape
    GiD_ModifyData materials $matname $values
    GidUtils::EnableWarnLine

    return 1
}

proc TUI_exectdyn { params } {
    
    global ProblemTypePriv
    global ProgramName

    if { $::tcl_platform(platform) == "windows" } {
	switch $ProgramName {
	    "Tdyn2D" {
		set exename "tdyn.exe"
	    }
	    "Tdyn3D" {
		set exename "tdyn.exe"
	    }
	}
    } else {
	switch $ProgramName {
	    "Tdyn2D" {
		set exename "tdyn"
	    }
	    "Tdyn3D" {
		set exename "tdyn"
	    }
	}
    }

    set retv ""
    catch {
	set ExecFile [file join $ProblemTypePriv(problemtypedir) "../scripts" $exename]
	eval exec [list $ExecFile] $params
    } retv

    if { $retv == "\n" || $retv == "Empty String" } {
	set retv ""
    }

    return $retv
}

proc TUI_exectdyntool { params {echo 0} } {
    
    global ProblemTypePriv
    global ProgramName

    if { $::tcl_platform(platform) == "windows" } {
	switch $ProgramName {
	    "Tdyn2D" {
		set exename "tdyn.exe"
	    }
	    "Tdyn3D" {
		set exename "tdyn.exe"
	    }
	}
    } else {
	switch $ProgramName {
	    "Tdyn2D" {
		set exename "tdyn"
	    }
	    "Tdyn3D" {
		set exename "tdyn"
	    }
	}
    }

    set retv ""
    catch {
	set ExecFile [file join $ProblemTypePriv(problemtypedir) "../scripts" $exename]
	set retv [eval exec [list $ExecFile] $params]
    } rete

    if { $retv == "\n" || $retv == "Empty String" } {
	set retv ""
    }

    if { $retv != "" } {
	return $retv
    } else {
	if { $echo } {
	    GiveError "[= {Error: Cannot use Tdyn tool (%s)} $params].\n$rete"
	}
	return ""
    }
}

proc TUI_getcbxsize { {minsize 7} vallist } {
    set size $minsize
    foreach item $vallist {
	if { [string length $item] > $size } {
	    set size [string length $item]
	}
    }
    return $size
}

proc TUI_deletespaces { str } {
    regsub -all { } $str {} str
    return $str
}

proc TUI_valuecheck { val type {ln 0} {tag ""}} {
    
    set result "ok"

    set val [string trim $val]

    switch $type {
	"integer" {
	    if { [string is integer -strict $val] != 1 } {
		set result "[= {Value is not an integer}]."
	    }
	}
	"float" {
	    regsub -all {,} $val {.} val
	    if { [string is double -strict $val] != 1 } {   
		set result "[= {Value is not a number}]."
	    }
	}
	"posfloat" {
	    # Positive Float
	    regsub -all {,} $val {.} val
	    if { [string is double -strict $val] != 1 } {   
		set result "[= {Value is not a number}]."
	    }
	    if { $val <= 0.0 } {
		set result "[= {Value must be positive}]."
	    } 
	}
	"posfloat0" {
	    # Non negative Float
	    regsub -all {,} $val {.} val
	    if { [string is double -strict $val] != 1 } {   
		set result "[= {Value is not a number}]."
	    }
	    if { $val < 0.0 } {
		set result "[= {Value must be non negative}]."
	    } 
	}
	"string" {
	    if { [string length $val] > $ln && $ln > 0 } {
		set result "[= {Value is longer then required number of characters of %s} $ln]."
	    } elseif { [regexp {^[A-Za-z0-9{ }]+$} "$val"] != 1 } {
		# [regexp {^[A-Za-z0-9_]+$} "$val"]
		set result "[= {String includes illegal characters or is empty}].\n[= {Allowed\
		            characters are A-z, a-z and 0-9}]."
	    }
	}
	"yesno" {
	    if { ![string is boolean -strict $val] } {
		set result "[= {Answer should be Yes/No}]."
	    }
	}
	"yn" {
	    if { ![string is boolean -strict $val] } {
		set result "[= {Answer should be Y/N}]."
	    }
	}
	"onoff" {
	    if { ![string is boolean -strict $val] } {
		set result "[= {Answer should be 0/1}]."
	    }
	}
	"func" { 
	    global ProblemTypePriv
	    if { $ProblemTypePriv(check_functions) } {
		set checkres false
		set params [list -no_timetable -tool check_function "$val;"]
		set checkres [TUI_exectdyn $params]
		if { $checkres != "" } {
		    regsub -all {\r} $checkres "" checkres
		    regsub -all {\n} $checkres "" checkres
		    regsub -all { }  $checkres "" checkres
		    set result [= {Value is not a number nor a correct function. ( %s )} $checkres]
		}
	    }
	}
	"unit" {
#            global ProblemTypePriv
#            if { $ProblemTypePriv(check_units) } {
#                if { [string length $tag] > 0 } {
#                    set checkres false
#                    set params [list -tool check_units "$val" "$tag"]
#                    set checkres [TUI_exectdyn $params]
#                    if { $checkres != "" } {
#                        regsub -all {\r} $checkres "" checkres
#                        regsub -all {\n} $checkres "" checkres
#                        set result "Incorrect units. ( $checkres )"
#                    }
#                }
#            }
	}
       "gobviscosity" {
	    if { ![regexp -lineanchor {^exp\((.*)\+(.*)/\(Tm-(.*)\)\)$} $val] } {
		set result "[= {Field must have a function of type: exp(a+b\(Tm-c))}]."
	    }
	}
       "list" {
	    set vallist [split $val {,}]
	    set len [llength $vallist]
	    if { [expr int($len/2)]!=[expr 0.5*$len] } {
		set result "[= {Field must be a list of pairs (x,y) like: 1.0,2.0,2.0,2.0,}]."
	    } else {
		foreach item $vallist {
		    set result [TUI_valuecheck $item float]
		    if { $result != "ok" } {
		        set result "[= {Field must be a list of float pairs (x,y) like: 1.0,2.0,2.0,2.0,}]."
		        break
		    }
		}
	    }
	}
    }

    return $result
}



#################################################################################
# VarList has to be a list of Variables defining QUESTIONS
# and their types in i.e. ProblemData.
# EXAMPLE of VarList definition:
#      variable Fluid_Solver [list \
#              [list VariableOne "VariableOne#CB#" string 0] \
#              [list VariableTwo "VariableTwo" float 1] \
#              [list VariableThree "VariableThree" float 1] \
#              [list VariableFour "VariableFour#CB#" string 0] \
#              [list VariableFive "VariableFive" float 1] \
#              ]
#################################################################################

proc TUI_dumpsinglefields { NamespaceName VarList DataType } {

    # The following condition is only a protection against errors.
    # Add another possible data type to expand this function to 
    # new types of data.
    switch $DataType {
	"ProblemData" {
	    foreach item $VarList {
		set itt $NamespaceName
		append itt "::" [lindex $item 0]
		# Esto se ha cambiado porque en [lindex $item 0] est  el nombre de la variable.
		# set savename [lindex $item 1]                
		set savename [lindex $item 0]
		set savetype [lindex $item 2]
		if { $savetype == "func" } {
		    GiD_AccessValue set gendata $savename [string trim [eval set nothing $$itt]]
		} else {
		    GiD_AccessValue set gendata $savename [DWSpace2Under \
		            [string trim [eval set nothing $$itt]]]
		}
		# set val [DWSpace2Under [string trim [eval set nothing $$itt]]]
		# .central.s process escape escape escape escape Data \
		#         $DataType -SingleField- "$savename" "\"$val\""
		# .central.s process escape escape escape escape
	    }  
	}
    }
    return 1
}

proc TUI_dumpsingleunits { NamespaceName VarList DataType } {

    # The following condition is only a protection against errors.
    # Add another possible data type to expand this function to 
    # new types of data.
    switch $DataType {
	"ProblemData" {
	    foreach item $VarList {
		set itt $NamespaceName
		append itt "::" [lindex $item 0]
		# Esto se ha cambiado porque en [lindex $item 0] est  el nombre de la variable.
		# set savename [lindex $item 1]        
		# set val [DWSpace2Under [string trim [eval set nothing $$itt]]]
		set savename [lindex $item 0]
		set val [TUI_formatunits Unicode2Nums [eval set nothing $$itt] 1]

		.central.s process escape escape escape escape Data \
		        $DataType -SingleField- "$savename" "\"$val\""
		.central.s process escape escape escape escape
	    }  
	}
    }
    return 1
}

#################################################################################
#
#   Read & Format Values - from GiD database to Tdyn GUI format
#
#################################################################################

proc TUI_readsinglefield { Type Field } {

    return [GiD_AccessValue get $Type $Field]

#    set pos [lsearch -regexp $VarList [eval set nothing ^${Field}(\$|\#CB|\:)]]
#    return [DWUnder2Space "[lindex $VarList [expr $pos + 1]]"]
#    return [lindex $VarList [expr $pos + 1]]
}

proc TUI_formatsinglefield { Value {Type "string"} } {

    if { $Type == "func" } {
	return $Value
    }
    return [DWUnder2Space "$Value"]
}

#################################################################################
#
#   Format Units - unicode character to numbers and the other way
#
#################################################################################

proc TUI_formatunits { way input {addbrack 0} } {

    set output $input

    switch $way {
	"Nums2Unicode" {
	    regsub -nocase -all "gt" $output "\u2070C" output
	    regsub -nocase -all "fh" $output "\u2070F" output
	    regsub -nocase -all "An" $output "\u00c5" output
	    regsub -nocase -all "mdeg" $output "\u0027" output
	    regsub -nocase -all "sdeg" $output "\u0022" output
	    regsub -nocase -all "deg" $output "\u2070" output
#            regsub -nocase -all "deg" $output "\u00b0" output
	    regsub -all "2" $output "\u00b2" output
	    regsub -all "3" $output "\u00b3" output
	    regsub -all "4" $output "\u2074" output
	    regsub -all "5" $output "\u2075" output
	    regsub -all {\[} $output "" output
	    regsub -all {\]} $output "" output
	}
	"Unicode2Nums" {
	    regsub -all "\u2070C" $output "Gt" output
	    regsub -all "\u2070F" $output "Fh" output
	    regsub -all "\u00c5" $output "An" output
	    regsub -all "\u0027" $output "mdeg" output
	    regsub -all "\u0022" $output "sdeg" output
	    regsub -all "\u2070" $output "deg" output
#            regsub -all "\u00b0" $output "deg" output
	    regsub -all "\u00b2" $output "2" output
	    regsub -all "\u00b3" $output "3" output
	    regsub -all "\u2074" $output "4" output
	    regsub -all "\u2075" $output "5" output

	    if { $addbrack == 1 } {
		if { [regexp {\[.*\]} $output] != 1 } {
		    set output "\[$output\]"
		}
	    }
	}
	"NumsToPlain" {
	    regsub -all {\[} $output "" output
	    regsub -all {\]} $output "" output
	}
    }

    return $output
}

#################################################################################
# The following function takes care of all possible Tdyn values to be
# synchronized with NUMBER OF SPECIES or URSolver VARIABLES defined.
# It goes through every Material and Boundary, ALL Solver Data and All Initial 
# Data and checks for proper lengths of these values. If existing values 
# do not refer to the correct number pf Species, values are or multiplicated
# or Truncated to respect the Species number.
#
# type should be or "advect" or "ursolver" or more later definitions
# depending on what type of values are to be synchronized. This will be 
# usefull for example if the user turns off the ADVECT MODULE. 
#################################################################################
proc TUI_synchrospecies { type rebuild currmat } {

    # Nothing to update
    if { $rebuild == "" } { return }

    # Update species values in other materials and general data info
    GidUtils::DisableWarnLine

    switch $type {
	"advect" {
	    # Read Species Names from Problem Data
	    set Names [TUI_formatlistvalues String2List "[lindex [.central.s info gendata] \
		    [expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]"]

	    set Lookfor "Species_Properties"
	    set ULookfor "Spc_Units"
	    set solverdata_Lookfor "Advect_Solverdata"
	    set initdata_Lookfor "Advect_Initdata"

	    set Fluid_Valnames $matfluid::Species_Valnames
	    set Fluid_UValnames $matfluid::Species_UValnames
	    set Solid_Valnames $matsolid::Species_Valnames
	    set Solid_UValnames $matsolid::Species_UValnames
	    set Bound_Valnames [list \
		    [list "Flux " Flux_Spc func 1 ""] \
		    [list "Reactive Flux " Reactive_Flux_Spc func 1 ""] \
		    ]
	    set Bound_UValnames \
		    [list \
		    [list "Flux Units " Flux_Units unit 1 concentration] \
		    [list "Reactive Flux Units " Reactive_Flux_Units unit 1 concentration] \
		    ]
	    set Contc_Valnames [list \
		    [list "Resistance " Resistance_Spc func 1 ""] \
		    [list "On/Off " OnOff_Spc onoff 1 ""] \
		    ]
	    set Contc_UValnames \
		    [list \
		    [list "Resistance Spc Units " Resistance_Spc_Units unit 1 concentration] \
		    ]
	}
	"ursolver" {
	    # Read URSolver Variables Names from Problem Data
	    set Names [TUI_formatlistvalues String2List "[lindex [.central.s info gendata] \
		    [expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]"]

	    set Lookfor "Variables_Properties"
	    set ULookfor "URV_Units"
	    set solverdata_Lookfor "URSolver_Solverdata"
	    set initdata_Lookfor "URSolver_Initdata:"

	    set Fluid_Valnames $matfluid::URVariables_Valnames
	    set Fluid_UValnames $matfluid::URVariables_UValnames
	    set Solid_Valnames $matsolid::URVariables_Valnames
	    set Solid_UValnames $matsolid::URVariables_UValnames
	    set Bound_Valnames [list \
		    [list "Flux " Flux_Phi func 1 ""] \
		    [list "Reactive Flux " Reactive_Flux_Phi func 1 ""] \
		    ]
	    set Bound_UValnames \
		    [list \
		    [list "Flux Units " Flux_Phi_Units unit 1 concentration] \
		    [list "Reactive Flux Units " Reactive_Flux_Phi_Units unit 1 concentration] \
		    ]
	    set Contc_Valnames [list \
		    [list "Resistance " Resistance_Phi func 1 ""] \
		    [list "On/Off " OnOff_Phi onoff 1 ""] \
		    ]
	    set Contc_UValnames \
		    [list \
		    [list "Resistance Phi Units " Resistance_Phi_Units unit 1 concentration] \
		    ]
	}
    }

    set Loop_Types [list "Fluid" "Solid" "BFluid" "BSolid" "Contact"]
#    set Loop_Types [list "Fluid" "Solid"]

    foreach Type $Loop_Types {

	if { $Type == "Fluid" } {
	    set Book "MATERIALS_(FLUID)"
	    set MType ""
	    set Valnames $Fluid_Valnames
	    set UValnames $Fluid_UValnames
	} elseif { $Type == "Solid" } {
	    set Book "MATERIALS_(SOLID)"
	    set MType ""
	    set Valnames $Solid_Valnames
	    set UValnames $Solid_UValnames
	} elseif { $Type == "BFluid" } {
	    set Book "BOUNDARIES_(FLUID)"
	    set MType "Fluid_Body"
	    set Valnames $Bound_Valnames
	    set UValnames $Bound_UValnames
	} elseif { $Type == "BSolid" } { 
	    set Book "BOUNDARIES_(SOLID)"
	    set MType "Solid_Body"
	    set Valnames $Bound_Valnames
	    set UValnames $Bound_UValnames
	} elseif { $Type == "Contact" } { 
	    set Book "CONTACTS"
	    set MType "Interf"
	    set Valnames $Contc_Valnames
	    set UValnames $Contc_UValnames
	}

	### Materials Data in Materials
	set Materials_List [TIU_getbooknames $Book]
	foreach matname $Materials_List {

	    set proplist [.central.s info materials $matname]
	    set exvalues [TUI_getmatvalueslist $matname]
	    set newvalues $exvalues
	    if {($MType=="" || $MType==[lindex $exvalues 0]) && $currmat!=$matname} {
		# Get existing values of the respective MATERIAL and QUESTION (values and units)
		set propvalpos [expr [lsearch -regexp $proplist $Lookfor] + 1]
		set exval [lindex $proplist $propvalpos]
		set propuvalpos [expr [lsearch -regexp $proplist $ULookfor] + 1]
		set exuval [lindex $proplist $propuvalpos]

		# Get index position of the requested values in the values 
		# list (for later replacement, values and units)
		set exvalpos [expr ($propvalpos+1)/2-1]
		set exuvalpos [expr ($propuvalpos+1)/2-1]

		# Trim the "#N# Num" from the begining of the value string (values and units)
		set exval [lrange $exval 2 end]
		set exuval [lrange $exuval 2 end]
		
		# Count appropriate length of values lists (totval for values, utotval for units)
		set totval [expr [llength $Valnames] * [llength $Names]]
		set utotval [expr [llength $UValnames] * [llength $Names]]
		set extotval [llength $exval]
		set exutotval [llength $exuval]
		
		# El formato con el que se guarda la informacin de las especies es 
		# una lista de la forma:
		# #N# 4 1.0 {0.0 + 1.0} 2.0 {1.0 + 1.0}
		# Values & Units rebuild
		if { $rebuild != "" } {
		    set rowlen [llength $Valnames] 
		    set urowlen [llength $UValnames]
		    set result [list #N# $totval]
		    set uresult [list #N# $utotval]
		    set valist $exval                 
		    set uvalist $exuval                  
		    for { set i 0 } { $i < [llength $rebuild] } { incr i 2 } {
		        set code [lindex $rebuild $i]
		        set index [lindex $rebuild [expr $i+1]]
		        if { $code=="r" } {
		            if { $propvalpos > 0 } { 
		                set valist [lreplace $valist [expr $index*$rowlen] [expr ($index+1)*$rowlen-1]] 
		            }
		            if { $propuvalpos > 0 } { 
		                set uvalist [lreplace $uvalist [expr $index*$urowlen] [expr ($index+1)*$urowlen-1]]
		            }
		        } elseif { $code=="a"} {
		            if { $propvalpos > 0 } {
		                eval lappend valist [lrange $valist [expr $index*$rowlen] [expr ($index+1)*$rowlen-1]]
		            }
		            if { $propuvalpos > 0 } { 
		                eval lappend uvalist [lrange $uvalist [expr $index*$urowlen] [expr ($index+1)*$urowlen-1]]
		            }
		        } else {
		            WarnWin [= {Error found synchronising species or variables}]
		        }
		    }
		    if { $propvalpos > 0 } {
		        eval lappend result $valist
		        set newvalues [lreplace $newvalues $exvalpos $exvalpos $result]
		    }
		    if { $propuvalpos > 0 } { 
		        eval lappend uresult $uvalist
		        set newvalues [lreplace $newvalues $exuvalpos $exuvalpos $uresult]
		    }
		}

		# If existing values != new values, new values are written
		if { $exvalues != $newvalues } {
		    TUI_setnewmatvalues $matname $newvalues
		}
	    }
	}
    }

    ### SolverData and InitialData in ProblemData guardada como lista de la forma:
    ### #N# 4 1.0 {0.0 + 1.0} 2.0 {1.0 + 1.0}

    if { $rebuild != "" } {
	set gendlist [.central.s info gendata]

	# Get existing values of the respective SOLVER AND INITIAL values lists
	set propsolpos [expr [lsearch -regexp "$gendlist\*" $solverdata_Lookfor] + 1]
	set sdw_numvlnms [expr [regexp -all "," [lindex $gendlist [expr $propsolpos - 1]]] + 1]
	set exsol [lindex $gendlist $propsolpos]
 
	set propinipos [expr [lsearch -regexp "$gendlist\*" $initdata_Lookfor] + 1]
	set idw_numvlnms [expr [regexp -all "," [lindex $gendlist [expr $propinipos - 1]]] + 1]
	set exini [lindex $gendlist $propinipos]

	# Trim the "#N# Num" from the begining of the value string (values and units)
	set exsol [lrange $exsol 2 end]
	set exini [lrange $exini 2 end]

	# Count appropriate length of values lists (totval for values, utotval for units)
	set totsol [expr [llength $Names] * $sdw_numvlnms]
	set totini [expr [llength $Names] * $idw_numvlnms]
	set extotsol [llength $exsol]
	set extotini [llength $exini]

	# Values rebuild
	if { $rebuild != "" } {
	    set solrowlen $sdw_numvlnms 
	    set inirowlen $idw_numvlnms
	    set sollist $exsol                 
	    set inilist $exini                  
	    for { set i 0 } { $i < [llength $rebuild] } { incr i 2 } {
		set code [lindex $rebuild $i]
		set index [lindex $rebuild [expr $i+1]]
		if { $code=="r" } {
		    if { $propsolpos > 0 } { 
		        set sollist [lreplace $sollist [expr $index*$solrowlen] [expr ($index+1)*$solrowlen-1]] 
		    }
		    if { $propinipos > 0 } { 
		        set inilist [lreplace $inilist [expr $index*$inirowlen] [expr ($index+1)*$inirowlen-1]]
		    }
		} elseif { $code=="a"} {
		    if { $propsolpos > 0 } {
		        eval lappend sollist [lrange $sollist [expr $index*$solrowlen] [expr ($index+1)*$solrowlen-1]]
		    }
		    if { $propinipos > 0 } { 
		        eval lappend inilist [lrange $inilist [expr $index*$inirowlen] [expr ($index+1)*$inirowlen-1]]
		    }
		} else {
		    WarnWin [= {Error found synchronising species or variables}]
		}
	    }
	}
	set sollist [eval list "#N#" [llength $sollist] $sollist]
	TUI_setprbvalue $solverdata_Lookfor $sollist
	set inilist [eval list "#N#" [llength $inilist] $inilist]
	TUI_setprbvalue $initdata_Lookfor $inilist
    }

    GidUtils::EnableWarnLine

    return 1

    ### SolverData in ProblemData (el formato que se usa para SolverData no es el de
    ### lista tcl, esto habra que cambiarlo, pero como no hay campos complejos, 
    ### de momento funciona as)
    set temp "#N#"
    set proplength [expr [llength $Names] * $sdw_numvlnms]
    lappend temp $proplength
    set exvals [join [lindex [.central.s info gendata] \
	    [expr [lsearch [.central.s info gendata] $solverdata_Lookfor] + 1]]]
    set actlength [lindex $exvals 1]
    if { $actlength > $proplength } {
	set ttex [lreplace $exvals 0 1]
	set firstcut [expr $proplength]
	lappend temp [lreplace $ttex $firstcut end]
    } elseif { $actlength < $proplength } {
	set ttex [lreplace $exvals 0 1]
	lappend temp $ttex
	set lastex [join [lrange $ttex [expr $actlength - $sdw_numvlnms] end]]
	for { set i 0 } { $i < [expr ($proplength - $actlength) / $sdw_numvlnms] } { incr i } {
	    lappend temp $lastex
	}
    }
    if { $actlength != $proplength } {
	TUI_setprbvalue $solverdata_Lookfor $temp
    }

    ### InitialData en ProblemData guardada como lista de la forma:
    ### #N# 4 1.0 {0.0 + 1.0} 2.0 {1.0 + 1.0}
    set proplength [expr [llength $Names] * $idw_numvlnms]
    set temp [list "#N#" $proplength]
    set exvals [join [lindex [.central.s info gendata] \
	    [expr [lsearch [.central.s info gendata] $initdata_Lookfor] + 1]]]
    set actlength [lindex $exvals 1]
    if { $actlength > $proplength } {
	set ttex [lreplace $exvals 0 1]
	set firstcut [expr $proplength]
	eval lappend temp [lreplace $ttex $firstcut end] 
    } elseif { $actlength < $proplength } {
	set ttex [lrange $exvals 2 end]
	eval lappend temp $ttex
	set lastex [join [lrange $ttex [expr $actlength - $idw_numvlnms] end]]
	for { set i 0 } { $i < [expr ($proplength - $actlength) / $idw_numvlnms] } { incr i } {
	    eval lappend temp $lastex
	}
    }
    if { $actlength != $proplength } {
	TUI_setprbvalue $initdata_Lookfor $temp
    }

    GidUtils::EnableWarnLine

    return 1
}

proc TUI_savespeciesnames { writeto {nameslist ""} } {

    set numspecies [llength $nameslist]

    if { $numspecies > 0 } {
	set temp "#N#"
	append temp " " $numspecies
	foreach val $nameslist {
	    append temp " " $val
	}
	.central.s process escape escape escape escape Data \
		ProblemData -SingleField- $writeto "\"$temp\""
	.central.s process escape escape escape escape
    }

    return $numspecies
}

#################################################################################
# The following function is purely custom and formats Species Type data saved 
# in list of lists to GiD Matrix Strings and the other way around.
# The function is used for units saved in GiD Matrix too.
# 
# ==> way == List2String: Returns GiD Matrix String made from data saved in 
#                         list of lists.
# ==? way == String2List: Returns list of lists made from GiD Matrix String
#
# ==> Values: GiD Matrix String or list of lists with Species data
#
# ==> Valnames: list defining the structure of the Species data. For example: 
#               set Valnames [list \
#                [list "F1 Spc" F1_Spc func 1 ""] \
#                [list "F2 Spc" F2_Spc func 1 ""] \
#                [list "F3 Spc" F3_Spc func 1 ""] \
#                ]
#               This could basically be substituted by a number of values for each 
#               Specie (GiD Matrix row), but since this interface uses Valnames
#               definitions widely, I just passed them in.
#################################################################################
proc TUI_formatspeciesvalues { way Valnames Values } {

    set result ""
    set vnl [llength $Valnames]

    # El formato con el que se guarda la informacin de las especies es 
    # una lista de la forma:
    # #N# 4 1.0 {0.0 + 1.0} 2.0 {1.0 + 1.0}
    switch $way {
	"List2String" {
	 # Tdyn format (list format)
	    set tn [expr $vnl * [llength $Values]]
	    set result [list "#N#" "$tn"]
	    foreach sv $Values {
		foreach svv $sv {
		    lappend result "$svv"
		}
	    }
	}
	"Field2String" {
	# GiD format (string instead of list)
	    set tn [expr $vnl * [llength $Values]]
	    set result "#N# $tn "
	    foreach sv $Values {
		foreach svv $sv {
		    append result "[DWSpace2Under [string trim $svv]] "
		}
	    }
	    set result [string trim $result]
	}
	"String2List" {
	# Tdyn format (list format)
	    set valstr [lreplace $Values 0 1]
	    for { set i 0 } { $i < [expr [llength $valstr] / $vnl] } { incr i } {
		set aa [lrange $valstr [expr $i * $vnl] \
		        [expr [expr ($i+1) * $vnl] -1]]
		lappend result $aa
	    }
	}
    }

    return $result
}

proc TUI_formatlistentries { way Values i } {

    set result ""
    switch $way {
	"String2Field" {
	   # GiD format (string instead of list)
	   set result [DWUnder2Space [lindex $Values $i]]
	}
    }
    return $result
}

proc TUI_formatlistvalues { way Values } {

    set result ""
    switch $way {
	"List2String" {
	    set result [list "#N#" "[llength $Values]"]
	    foreach sv $Values {
		lappend result "$sv"
	    }
	}
	"String2List" {
	    set result [lrange $Values 2 end]
	}
       "Field2String" {
	# GiD format (string instead of list)
	    set result "#N# [llength $Values] "
	    foreach sv $Values {
		    append result "[DWSpace2Under [string trim $sv]] "
	    }
	    set result [string trim $result]
	}
    }
    return $result
}

#
#   Limpia una cadena de "elementos errneos o peligrosos"
#
proc TUI_cleanvalue { val type } {   
    # Nota: \r retorno de carro, \n nueva lnea, \t tabulador
    set val [string trim $val]
    # \n not supported by GiD
    regsub -all {\n} $val {} val 
    switch $type {
	"integer" {
	    regsub -all { } $val {} val
	    regsub -all {\t} $val {} val
	}
	"float" {
	    regsub -all { } $val {} val
	    regsub -all {\t} $val {} val
	    regsub -all {,} $val {.} val
	}
	"string" {
	    set val [DWSpace2Under $val]
	}
	"yesno" {
	    regsub -all { } $val {} val
	    regsub -all {\t} $val {} val
	}
	"yn" {
	    regsub -all { } $val {} val
	    regsub -all {\t} $val {} val
	}
	"onoff" {
	    regsub -all { } $val {} val
	    regsub -all {\t} $val {} val
	}
	"func" {
	    # regsub -all {\t} $val {} val
	    # regsub -all {;}  $val {} val 
	    # regsub -all { }  $val {} val
	}
	"unit" {
	    regsub -all { } $val {} val
	    regsub -all {\t} $val {} val
	}
    }
    return $val
}

#
#   Compara dos listas elemento a elemento (pueden diferir en el nivel de {})
#
proc TUI_comparelists { list1 list2 } {
    set len [llength $list1]
    for { set i 0 } { $i < $len } { incr i } {
	if { [lindex $list1 $i] != [lindex $list2 $i] } { return 0 }
    }
    return 1
}

proc TUI_closeotherwindows { w {asksave 1} } {

    set owc 1

    # Fluid Material Window
    if { [winfo exists .gid.fluidmat] == 1 && $w != ".gid.fluidmat" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [matfluid::checkclose .gid.fluidmat 1 1 1]
	} else {
	    destroy .gid.fluidmat
	    set owc 1
	}
    }

    # Solid Material Window
    if { [winfo exists .gid.solidmat] == 1 && $w != ".gid.solidmat" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [matsolid::checkclose .gid.solidmat 1 1 1]
	} else {
	    destroy .gid.solidmat
	    set owc 1
	}
    }

    # Fluid Boundaries Window
    if { [winfo exists .gid.fluidbond] == 1 && $w != ".gid.fluidbond" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [bondfluid::checkclose .gid.fluidbond 1 1 1]
	} else {
	    destroy .gid.fluidbond
	    set owc 1
	}
    }

    # Solid Boundaries Window
    if { [winfo exists .gid.solidbond] == 1 && $w != ".gid.solidbond" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [bondsolid::checkclose .gid.solidbond 1 1 1]
	} else {
	    destroy .gid.solidbond
	    set owc 1
	}
    }

    # Fluid-Solid Contacts Window
    if { [winfo exists .gid.contacts] == 1 && $w != ".gid.contacts" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [bondsolid::checkclose .gid.contacts 1 1 1]
	} else {
	    destroy .gid.contacts
	    set owc 1
	}
    }

    # Solver Data Window`
    if { [winfo exists .gid.tdw] == 1 && $w != ".gid.tdw" && $owc == 1 } {
	if { $asksave == 1 } {
	    if { [winfo exists .gid.sdadvturb] == 1 } {
		set owc [solvdatwin::ransoladvturbulencecheckclose .gid.sdadvturb 1 1 1]
	    }
	    if { $owc != "dataerror" } {
		set owc [solvdatwin::checkclose .gid.tdw 1 1 1]
	    }
	} else {
	    destroy .gid.sdadvturb
	    destroy .gid.tdw
	    set owc 1
	}
    }

    # Initial Data Window
    if { [winfo exists .gid.tiw] == 1 && $w != ".gid.tiw" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [initdatwin::checkclose .gid.tiw 1 1 1]
	} else {
	    destroy .gid.tiw
	    set owc 1
	}
    }
    
    # Problem and Solver Data Window
    if { [winfo exists .gid.prb] == 1 && $w != ".gid.prb" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [prbdatwin::checkclose .gid.prb 1 1 1]
	} else {
	    destroy .gid.prb
	    set owc 1
	}
    }

    # Units Data Window
    if { [winfo exists .gid.unw] == 1 && $w != ".gid.unw" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [unitdatwin::checkclose .gid.unw 1 1 1]
	} else {
	    destroy .gid.unw
	    set owc 1
	}
    }

    # Process Data Window
    if { [winfo exists .gid.prc] == 1 && $w != ".gid.prc" && $owc == 1 } {
	if { $asksave == 1 } {
	    set owc [processdatwin::checkclose .gid.prc 1 1 1]
	} else {
	    destroy .gid.prc
	    set owc 1
	}
    }

    if { $owc != "dataerror" } {
	set owc 1
    }

    return $owc
}

proc TDYN_getcbxsize { intu vals {intm 16} } {
    set len1 [TUI_getcbxsize $intu $vals]
    if { $len1 < $intm && $intm > 0 } {
	return $intm 
    }
    return $len1
}

proc ListBoxWindow { wn title labeltext entries { command "" } } { 

    # Makes a List Box Window with entries
    # $parent -  Parent window path
    # $title -  ListBox Window Title
    # $labeltext - Label text inserted above of the values
    # $entries - Entries names
    # $command - executed command when finished

    # Defining the size of the window.
    # A minimum of 30 characters is set

    set lwidth 30
    foreach item $entries {
	if { [string length $item] > $lwidth } {
	    set lwidth [string length $item]
	}
    }

    set w $wn.lsbox
    InitWindow $w $title PreFEditorOperatorsWindowGeom ""

    # Label and listbox with entries
    set main [frame $w.main -bd 2 -relief groove]
    set label [label $main.label -justify left -text $labeltext -padx 3]
    set entriesframe [frame $main.entriesframe -bd .1c]
    set scrollbar [scrollbar $entriesframe.scrollbar -command "$entriesframe.listbox yview"]
    set listbox [listbox $entriesframe.listbox -yscroll "$scrollbar set" \
	    -selectmode single -height 12 -bg white -width $lwidth ]

    grid $main -row 0 -column 0 -sticky nsew
    grid $label -row 0 -column 0 -sticky nw
    grid $entriesframe -row 1 -column 0 -sticky nsew
    grid $listbox -row 1 -column 0 -sticky nwes
    grid $scrollbar -row 1 -column 1 -sticky nse
    grid columnconfigure $entriesframe 0 -weight 1
    grid rowconfigure $entriesframe 0 -weight 1
    grid columnconfigure $main 0 -weight 1
    grid rowconfigure $main 0 -weight 1

    proc MakeSelection { w listbox command } {

	set selent [$listbox curselection]
	  if { ![string length $selent] > 0 } {
	      set selent "-1"
	  }

	eval $command $selent
	destroy $w
    } 

    bind $listbox <Return> "MakeSelection $w $listbox \"$command\""
    bind $listbox <Double-1> "MakeSelection $w $listbox \"$command\""

    eval $listbox insert 0 $entries

    # Buttons
    set btnframe [frame $w.btnframe]
    set btn_ok [button $btnframe.btn_ok -text [= {OK}] -width 5 \
	    -command "MakeSelection $w $listbox \"$command\""]
    set btn_cancel [button $btnframe.btn_cancel -text [= {Cancel}] -width 5 \
	    -command "destroy $w"]

    grid $btnframe -row 1 -column 0 -sticky nsew -padx 3 -pady 1
    grid $btn_ok -row 0 -column 0 -sticky s -padx 3 -pady 1
    grid $btn_cancel -row 0 -column 1 -sticky s -padx 3 -pady 1  
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

    bind $w <Escape> "tkButtonInvoke $btn_cancel"

    update
    focus $w
}

# Images for menus of the Tdyn Interface
namespace eval buttons {
    global ProblemTypePriv

    # Function Editor button
    variable ibtn_func [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images nb_fx.gif]]

    # Initial Conditions button
    variable ibtn_initcond [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images initial.gif]]

    # Solver Data button
    variable ibtn_solvdat [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images solvers.gif]]

    # Advanced button
    variable image64_btn_advanced {
	R0lGODlhSwARAKIAAMTZ25S5vvT4+Nzo6nCiqNDg4k1NTf///yH5BAAAAAAA
	LAAAAABLABEAAAP/eLrc/jDKSesxxurtMF/YIHwkFY4cFgxlC6lsahCFazMY
	3WA8zuceUEbxG3p+viARKfzpQIJBoYAZhaZAg2gpMkSxXt63etFirWaqtyxV
	PxWCQmCOAdTngexquYrj6yoBAHkGdoV4AGqDdIV3jG9lRYRziTmENTk0RSqW
	lQaTggUAhpugnm91iW45AGgEhq2WqWAErK62BwICnXIEtTOtZaiffzM5NcKj
	PwGvgY/HS7+QlngEhE/Q0QG0l5E0A04rlrTZsgbIGWrbWAOGmNkHakXWhGEz
	BQKGvq3yTuUzjKr4KaJLjhh4A5itY/Zq0SiGNRL6IsBC4kJfEBUU2OdQSxSA
	dpSmjAIpaJAvZLkWVevjEA+LfPtG5MMzx9qiGAlZ0sQ3c6eulnNiwGkzpcCW
	LwOSbsmVFAXTolOULo1ytM1UoleNKnV6o+uNBAA7
    }
    variable ibtn_advanced [image create photo -data [set image64_btn_advanced]]

    # Plus sign white background
    variable image64_btn_plus {
	R0lGODlhEAAPAKIAAFqUmvT4+JS5vtDg4nCiqNzo6k1NTf///yH5BAAAAAAA
	LAAAAAAQAA8AAAM5eLrc/nAZE9ssoaopij4TMUBTaVLWaWJMIRDEBABw176x
	MdfeEhSDQUgQZD1CI80Alqy4bJqfcZEAADs=
    }
    variable ibtn_plus [image create photo -data [set image64_btn_plus]]

    # Minus sign white background
    variable image64_btn_minus {
	R0lGODlhEAAPAKIAAJS5vvT4+Nzo6tDg4nCiqE1NTf///wAAACH5BAAAAAAA
	LAAAAAAQAA8AAAMkaLrc/jDKSSssOGunexEBIwBEaZaAIJKnmTKBMMz0DFp4
	ri8JADs=
    }
    variable ibtn_minus [image create photo -data [set image64_btn_minus]]

    # Update Changes button - copy from GiD
    variable image64_btn_uchang {
	R0lGODlhGAAYAOMAAAICBObm5NTPwYuCYKlYV6annXJoSLqMYP//////////
	/////////////////////yH5BAEKAAgALAAAAAAYABgAAAR4EMlJq704680v
	AJ0HBAJImdknBCyKAKU4snT8CoP7wjTpByBYrjKjEXw4klB36xEKgcKghWNK
	ioGjgbZMkQiHI7dq87AIYiBZieH1litBtd07GFSFvNT6egP0emVmUgMFcnEf
	G249bB1YLXxmNZFeJZQaiSGamyERADs=
    }
    variable ibtn_uchang [image create photo -data [set image64_btn_uchang]]

    # New button - copy from GiD
    variable image64_btn_new {
	R0lGODlhGAAYAOMAAAICBK2tmo2NgOXlztXVvzY2MlxcWfLy2///////////
	/////////////////////yH5BAEKAAgALAAAAAAYABgAAAR4EMlJq704awu2
	BkDQeRVIDERBTibhnuPWuugwiLLw7vagfr5Xr3eIXQCHQ4E3SKKMpWZBN0wm
	oRSkQdi0WrGSVs3mTeI43G45ScACaCfy+tA+DtVlEOZ9Lw/AFAZyZQKAWU14
	fysISFZPi4wEbAaGGUh1kFmVmRoRADs=
    }
    variable ibtn_new [image create photo -data [set image64_btn_new]]

    # Erase button - copy from GiD
    variable image64_btn_erase {
	R0lGODlhGAAYAMIAAAAAAMTC6Ht44l1U2P///////////////yH+Dk1hZGUg
	d2l0aCBHSU1QACH5BAEKAAQALAAAAAAYABgAAANRSLrc/jDKSatlIGttQfhg
	AAgA5YXfSE4nqq5RG77wI4N0Bt2p4NcOnugHbAhzMVTvV7Idic3gZgqNDa7Y
	6ytqy2ZV3K5XWxSPdabp5sJuu98JADs=
    }
    variable ibtn_erase [image create photo -data [set image64_btn_erase]]
}

namespace eval TUI_images {
    global ProblemTypePriv

    # Fluid Advect equation image
    variable image64_fladveq {
	R0lGODlhKQEtAOcAAGxsbDMzM9jY2D09Pb29vT4+PgMDA1FRUeLi4kxMTMvL
	y2JiYqmpqbW1tSEhIY6Ojp6ensrKypmZmZubmzY2Nuzs7IWFhdra2tHR0b+/
	v+np6aioqIqKil1dXVlZWYyMjMbGxldXV8TExOjo6LS0tLa2tqGhoWRkZOfn
	50NDQ+vr61BQUHd3d8zMzC8vL7m5uaCgoCcnJ1hYWGtraxwcHD8/P9/f38nJ
	yRgYGJqamnBwcLGxsWNjYzExMdfX1wEBAePj46amppCQkAAAAB0dHWZmZnl5
	edvb2xMTE6qqqkhISNTU1A0NDdzc3GpqaiAgIKKioktLS1paWnFxcbq6ukBA
	QI+Pj5eXl9DQ0CYmJjo6Om5ubouLixsbG+Hh4ZSUlEJCQo2Njbu7u319fdnZ
	2bi4uC0tLeTk5CsrK5OTkywsLFRUVAUFBd7e3oCAgBoaGn9/f7y8vNXV1RIS
	EsfHx3h4eJGRkUVFRSIiIqurq3t7e0FBQZKSkurq6h4eHuXl5aWlpa2trcLC
	wikpKTc3N8jIyKSkpIeHh1VVVdbW1re3t7KysjU1NR8fH93d3URERHx8fCoq
	KomJic/Pz6enp6ysrBUVFXR0dMPDw15eXl9fXxcXF5+fn01NTX5+frOzs0ZG
	Rs3NzWVlZa+vr9PT07CwsObm5gkJCcHBwWFhYTs7O87OzsXFxYGBgTAwMFZW
	VmdnZ2BgYHZ2doiIiIKCgkdHRzw8PGhoaHV1dTk5OUlJSQICAkpKSoaGhjQ0
	NG1tbRkZGTg4OAwMDJWVlRYWFmlpaQQEBA4ODoODg66urp2dneDg4CQkJMDA
	wJiYmNLS0np6epaWlm9vb1xcXCMjI1NTU09PT76+vqOjo1tbW1JSUiUlJf//
	////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////
	/////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQB
	CgD/ACwAAAAAKQEtAAAI/gD/CRxIsKBBAAgTGlzIsKHDhxAjSjTIICHCEhMz
	atzIsaPHjyA1BhBAkmTIkyg9oigpgJOblDBjypxJ8+GAmjhzEiTwUqfPn0B9
	3nRIoICBAwIYIkgQNCjPplCjSt04lKGCBQwaLHDA8AGEqTmfgh1LFmzVhREI
	SpiwkEKFsjPFwp1Lt+ZZiBYMXjhQF6bcvoADd7y7EEMGDf82EETAoYOHD4JB
	/o1MufJCwgRBhBAxgkQJBgNNnECRQsUKFpY3Tk7NOjJmgS1cjBj4AoZABjH+
	XZDxbwaN1hNXxywENhHwoK//1YA80MYNgThy/NOx4x+PHscjCk8pggBYKF6y
	/gtl6OMHEIJBBAoZIjCAQCJFxD/cjnIMWQDyU6owcqTgaxNIFJSEQDIo8c8S
	IfwjAhNNeOTDKFKR8gJRPc00xgUZoWDBGtDAtIMENAFiDRxT4dJfRE5gSNBr
	JDxBEBQDRSHFP1NQ8U8VVnz0AFilBNIQfSF1opECAkmRVkoUzNSHQK9IJUcI
	Ki4ExBUYCARAlAIl9wQWAm1ApHpZ/KPFP1tw8ZETpkAF40BSqMAQkB9FI01E
	U1Y5kB52drTmQGugglKdBLHS0J4oUbFnKFcyNMEEFXRhJZb/JOcFCyR8ER5B
	V+gARhg+fBSKBx9NcAojM5Rq6gydHMIQKk4QNIon/m9WGJGopJ5aaqoE8UIC
	RIs2ShAfAtFq662qLsRqQSzEF1KvjgrESS2rtorSCDwAwUEd/2CUqEFi2Heg
	QDOQ4V9HTpQRUirSfYSDKgvN4OZCPKxS0BwMxUFiRuu2+65Au5zYULcCLTGQ
	IPvma5C7DMVb0DV+hATwtwNdAgK88qL0yh//mFJBBpVgog0gC5lxRkHhFoRG
	R9iFNIccIMHxg6AE+dBKQykT5AolC+Vx70QuwzyQzASFwt5DIhdEylsD9VwQ
	0AzVPBAZQ4z8UdEGLSKuQU6DBIWyAo1ACikefGJQGlUYJIV3BKnBUQQnTGRB
	BmgN/ZEpTKxQ0BRpLkQJ/qgFeSBKzrJGRLfdBOFNkB1yM0R2QZwcAMsJPmZc
	9915G7S3QSgMMbFBb0e0OEERSJKBInrzfRIzcB+E5Q5rsIFGGBAl55APbUzk
	BtoFeZK4QJ+8AuE/Z8ACUSwG+AuELAvNQkstV9hCB0EzJGkQnAsRbzzyBP3S
	7EKsuw77Q9YPdHzyyzf/PEFD2LHQ7Q91/3pBNpDPvPMgwbLNEJkkFdEbqduk
	ExxxWAgsGFGQCEyCAjn6Bx+E9BA5DOEWA8GF1AiSi1L8IwX/KIIuCMKCXSzE
	XhtxIAQFIsGCpEJ2/+BfREQYwQkOpIIXzOAG0XeJhQBwf/1zCAwxqEGC7AAa
	/kAMIhAD2JAHQEsicqBXRAhDhwc48YlQjKIUz1eQG5qNMH0wAMv+kQshBO0L
	C9EFEt5yBuwRBBEzYgAkemMJgsBhCEgjCAg3IkYympFNskviROoIvDsKBI3/
	UCMbCzKEGdiQiA7RI0QAKcgZtLE5CoikJCOJsYa8YhYTqYPp/FcQNwzhk6AM
	pShHWTaB2OEOqMRDAFB5hz1JgTATcIVA6LCL2g0kFrs4T0EIMIS8lJAgN9jF
	JP7xCoHxYoYC8eS+BiIXTkzjmdB8ZuR20st//JIgr3SIJoNTzWsKJJjDLOY/
	jklI/JiSlapkJaEIss2HgJOYxkSmR4xxtYiAIQ8S/kHhSfSAkSsaJApr/Ecv
	ZLkYMTCEAl04gh//4YRm+eIfWUxXMncnEOodNKEL/Uc2G3LPjCBUoQZpqEAe
	GtFyLoSfEOnoQ0T6D5IaQKL/OAIrZkrTmSKgIYIww0RQYAAU5PN/iMweMAqS
	RYGRwAm/mIgEhpAFfw2EF7wJwsw4EAxkCeODO9PIUpu6ECdQoyE89alSmepU
	gUD1H1L9B1UNMgRYGcSKYO0pRM6a1rUSpBQsyKte82pQhjhhhBGBwh4mos+Q
	wJUgw9hdA4jxDxCcgQKs+NL+NnlLvrzCEXJgRFmLcQeseuQNlB2IESg6EMFu
	BLTVsyxmNVuQCgzhd1UM/upCTDs81Wa2rBzZRjMm0gFgSURtOWGfQaDmQkAY
	AxLxO0YYliQRZOivIBjIQjJcoYwtLMMgiCgG4DziXIYogrQC6e1GumuQ6E63
	utctyBmGIDCDCLch4oWIealr3ZA04Q0ZUQNzI9KwqfjhSAPxV3pRwgJZuJAg
	qFuIMgKHEhsM4VIG0W9MCnzggWAAvBGRsEQojJJoeCsidNBDRoggkT8AYpoo
	cYMRfpKABi1kBC5ScFZhQoTNFSTEMmlxQ7DRWY7geCI6RokHEPMQEKgPNSOO
	iCAGQYguoAHCIdEAU775hSpb2coh6UMNGqKMBBpEGTNOSTHcKhAj/wPJMNGy
	/kN+YaaMmBnNEVFzSKQxghvAtCGFYIEhKjmR/j7kC+fpwyEQARM77PYfDRiW
	qUJSBmc0pG0MWTBNJlENguR5zzJptENqUOGHXJrPEdF0SG6xg+rgxM8TWcBA
	EkE6kPhWKht4BpcZnBIwlOUP2chPTVAtkUW46Q+MIDJIKiGVVXBp1jWJwPfA
	YoXn6jomJIbICL7AgB0IIg4qIAEjGgEFRzyby2EuNKihcgTbfFsmvF7IDe6g
	okKs+B+PCOi5FUxrmJhbKhacd0wiARE82PgfkhAIG4apb4Mkod4FT/hJUHiI
	AhQEZ5V4pMIHMseJW3zhD1FCDQfiBUz8IxOauHhFdREu8pLH7iGbQDEn3oNP
	kVvU5DBf0UM6QeZPVGkJQ1ABFSf+8pjHHIUYAIUILpADQZTZEi2/eM99bnJ9
	jiAUctgvxERecaZb3SCFvfrItc71gfSgBWAHe9cFsoywtyAaJB+7yEWhg7a3
	Xe2VcLsOAJBvtcsnIAA7
    }
    variable i_fladveq [image create photo -data [set image64_fladveq]]

    # Fluid URSolver equation image
    variable image64_flurseq {
	R0lGODlhLgEoAOcAAM7OzqSkpNzc3Le3t29vbzExMSgoKBcXF5KSksjIyGNj
	Y7CwsLKysubm5urq6r6+visrK9XV1WxsbMPDw9HR0YCAgDIyMpubm1hYWL29
	vaioqK+vr7a2tqmpqZ2dnYKCgktLS4mJiU1NTdLS0nh4eIODg6enp1FRUZ6e
	nkNDQ2FhYVNTU5eXl0dHR9jY2GhoaE5OTrW1tXR0dBYWFh0dHbS0tMnJyd7e
	3gcHB1tbW83NzVBQUEREROLi4srKygAAAKWlpXJychQUFDMzMwkJCY6Ojl5e
	Xtra2ikpKd/f31VVVbGxsS4uLgoKClxcXN3d3Tc3N6urq4yMjIeHh2dnZ4qK
	iuXl5aOjo+np6cLCwiUlJb+/v2pqakVFRWtra9nZ2cvLy8/Pz9vb22lpaaqq
	quDg4B4eHjs7O8XFxby8vBAQEElJSZmZmXFxcWVlZXNzc6amprm5uYuLi2Zm
	ZtTU1Li4uNDQ0Dk5Oejo6KGhoa2trZ+fn0pKSlRUVK6urpWVlYaGhmBgYCMj
	I8fHx+fn50hISIGBgbu7u+vr66CgoBkZGeHh4YSEhMbGxqKiojo6OqysrIiI
	iC0tLRgYGFdXVzAwMHd3dycnJ2JiYpycnAsLC3Z2dlpaWpiYmEFBQWRkZJSU
	lMDAwHt7e8TExJOTk35+fhwcHF1dXXV1ddPT0ywsLAgICH9/f1lZWZCQkDQ0
	NA0NDRUVFQYGBrq6uiQkJFJSUszMzBISEjw8PB8fHw8PD8HBwXBwcEBAQI+P
	j9bW1uPj4zY2NtfX1+Tk5Ozs7Dg4OAUFBXp6ej09PX19fW5ubi8vL0JCQhMT
	EyoqKjU1NSEhIQwMDFZWVo2NjbOzs19fX///////////////////////////
	////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////
	/////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQB
	CgD/ACwAAAAALgEoAAAI/gD/CRxIsKDBggACCDg4cADDhxAjSpxIsaLFixgz
	atzIsaPHigQKGDiAgGECBR9TqlzJsqXLlzA/LmDQwMEDCBEOSpgQs6fPn0CD
	ClXpgCCFCgctDF3KtKnTpy8vGGSAAarVq1izMs2gYQOHfx0IevgAIoSIEVrT
	ql3LtiKJEgIbeDAx8ASKfyn+qVjRtq/fv1hZtCDoQoPAFzD+xZDxT8IMwJAL
	RrDlNNSiyE1p1CBo48a/ETh45tDxbwcPzJGHeYDqCzXMHgd9/CgIRGAQIQKH
	CCRS5KUL1ysJXCRW49fKL8lgpgnzFO1EB0aOGGSBhGCSBwJFKPm3hMQ/Fkxe
	/lrhg4ghhyRNwyTwaePDxT0LckRZaQR9y1k1jGBn2qhaRQnSFWRDE2INlANf
	TkTwBBQ+wMRJeQYtAMZTQOzSUw7OUdSAQF84sVIUlsAEwFdLERMKJxBFIcUU
	AgF4EBVVWPHAFVgMlIUWW1iwgASkfbiiQA8a1MMbHHVi5JFIdvJQBcwN5CFG
	SUZJECIHTKQiiwP90to/USa5ZJMCLSJISmjAkUwjA9HFEJMqlXmmQA9ggSJD
	XKDVhUBefMFQFmCAOZAYMozhZ0p1/nPnP0EWhMmEG3nyAxkPRCrpAGs89EhB
	eWxpkaOQShoppQRpMFhEhR4qEBwDcerpp5UydGlB/kIA4NECgPyTCQsCCaDC
	H66qRKutuI7yDyJzGhQCGQW5eNEKObF0bEGJDmQFDR3F8ENvBUWhB0MTfFIQ
	IWZgZC22BGlL0BwoQfQsQoQMNK5B5h7UrUEgIMWRAzswdIq83n6Eb0FTBNBJ
	MYMYVEa4yQZo0RkV6QCbRAcbFK1AoJzgUQFaGJTuQS+EYpABm12EscYFFcPY
	QxEX1EgEhDQo0MgFbWxQxwYZ0QpDC1CkDCMHLRIAxx5/tPNBSWxXkBg7gFAj
	QVOgcVEaUlRUQQYSIa10QaLQUdAOEhzkWUV7/FDbQHXkcdAoG0hCin0DKcEF
	RmGPLVDZBRlj70FWLy2Q/gtetNEGKQPFTRDdBqGtNtsCcREMQ8hIRAgkmmyB
	dQf7EWT42h09HjlD5ArEARdqrMHGUFNH9Hnoo0ekSSkFUcCJKQOd8nNEk0BB
	EBUHofIHGVz4AUFBL+CSUe23F4TFD0uYB7roExE/EO4G6c677wXJYAzjErXB
	TDMElCGR9L3/zpH23HtPURNiLFW6ROhLdDwkBaWCgioC4bHKZRGx8gNP/8SR
	iEGt8M4LwOCAH4RlIDJYRUb0xz//FUQAP6DaQ9o3EQYKxIEFCeA/BljAAwqk
	ArM5SOMkIoocTESDHDQgQQLBwha2ECIltEgMwiMRFxTghjjMoQ532IaBjGAJ
	/kDEQAmAuITfHGSGExFDBA3ShyAIxBWvKMgfPCEkWPDlH9AjyB5iIZAoNuAH
	avpgCAXigTKa0YxVvGIWB3KEJTIEiRTpgRUFskaBbLGL//hiGP8BQghxh4jO
	IGLyGPKM+UTkjv/wIhg7UkiLuIFIEoGDCCZJyUpa8pIiGEgc5MDJLsyBk3Jw
	2kEeOZEv/CANBmmCy6DgHYIwgoYFmYMsIjCLPXwMdzXooQZ+sJCB6M+PGgim
	MIV5EFnS0pYFMSUqGULKihizlrf8Ry7/scteCuSXAwGlHKChTYZYgQhFiYgB
	cKlLXnLkm+GkiBZESToJQmSdE2mjOwWigYz9ow60/nAZQdJJkAj8gAoyy9UP
	4vCPUzToBLUoCAg14k+AEs2NB4FnRRoa0H9AkKAG/QdCFTrGgowQIokw1UMu
	WtCDJpQgBEipSlX6kJBaJAJqaMr6HgJTihzPbARpBe5ssYVbWIQPTcASQWQj
	HeH5IBcKE8gxiLARoAq1ID+I2kFqehGnGoSo/zAqUgtiiWhgLyLS4FlEsKrV
	pP7DFmhNa1ofElaLfMCEBNHaT2bKkLdWRBeiKIgppjCKCLAABvqUSBqqdJBb
	LKAGovjCCSxUEC58FCODfYguknMQuz6NsAYxLGIVy1iCSKAXX4VINMzKEM0m
	drEeGa1FWlAHgjDgqTEB/oIdIsJa493AfAWpRR15gYnZdgATuJ2IYQ5yiE/0
	whdSMGIG+6WR4R6kEF07SG0x4tyCFPe4yT2IEYzAkE1EZBAMm8h1kavcjYD3
	IncoiBtkhZX0EgQNvwBGrQwihcT8hBC6eQgTGPASZdzsIO5dCX4h0oJpaIQE
	yKLIgFOCYIrg4R952BZBqmKRPfChDzFYyYMjXBCxMqQBigAKGfL6kMe85AHL
	KMiGJbySEUMEFqSNCDAE8iSKuLgjM/5HjSPihzZEgFcEucEcLPIHDAAiEILY
	QEp6/OOCTCDBDxlDFgayiWBY+cpW9m5HlCCMh5CBdTDBzUCYDGSWcPkhSYji
	/kVw8YU8zLYiZ+4Im93s4AAUjDZSUTA7fVAIgqRPI3iws0EEcIo0MOogPfDC
	QCJRiEY7utGR8IiSHvKCnqAiZP8I9J1bMmmG5EEOGKHGIb5WkU5zRNSk3ogV
	EHcRQwwkD5teCRg6+0ZaL6UO83SJANxwlUAAxykCgM0hypMIReCPKWlgdVAi
	IDmfyGFQS9HBFX69FCAwohEJcAQDEMEBKDwCEg+jtl8A95TUiRsokciiA6An
	CWSe2y8OgHZQ5P1ul4RhEhsayP+e8INw1/vfAEcNFShREMOEQCkBT7jC/VKJ
	pzbCM2sI0cInTnGsXKK60/5HE2Jd8Y57HCgywMRAJjIhkARoogwc/7jKV76S
	TQRgCyxo1z++wAOHsPzmOM+5znfukYAAADs=
    }
    variable i_flurseq [image create photo -data [set image64_flurseq]]

    # Solid Advect equation image
    variable image64_sladveq {
	R0lGODlh0gAnAOcAAN7e3oCAgEVFRZSUlHV1dePj41hYWGZmZnJyci4uLk5O
	TqioqISEhLa2tufn52FhYbS0tExMTH19fUBAQOrq6uLi4qampsHBwYKCgllZ
	WcTExOTk5H9/f6urq8XFxampqdTU1MzMzLCwsERERJ+fn729vaSkpNDQ0Coq
	KtLS0mpqajIyMmJiYm1tbbm5uTQ0NHt7e4yMjJeXl2hoaCIiIggICNPT07Oz
	s62trQAAAOHh4VdXV6ysrOXl5bW1tQkJCV9fXzExMdjY2AYGBru7u15eXtra
	2qKioisrK9nZ2UpKSlpaWouLi56enmVlZevr62xsbA4ODsvLy7GxsXZ2dj4+
	PkhISObm5qWlpcnJyUNDQ2tra8fHx5mZmb6+vujo6Dc3N46OjpycnE1NTUFB
	QWlpadfX15KSknNzc1NTU2NjY7i4uGRkZM3Nzdvb21BQUA0NDRwcHODg4NXV
	1RoaGoeHhzo6OhYWFuzs7J2dnZGRkUlJSVFRUbKysiYmJlJSUtbW1ikpKXl5
	ed/f3zU1Ndzc3Ly8vIODg1VVVR0dHcPDw8DAwKCgoN3d3RUVFenp6ZiYmEdH
	R3BwcBISEqenpycnJ6qqqgcHBzAwMJOTkz8/P8bGxqOjo4mJiV1dXb+/vxQU
	FHp6ej09PUJCQo+PjyEhIWdnZ2BgYIqKilRUVG5ubhMTE35+foGBgZWVlZub
	m5aWlktLS6+vry8vL8/Pz6GhocrKynR0dB4eHhsbG4WFhVtbW9HR0be3tzw8
	PAoKChgYGHx8fFxcXIaGhk9PTwQEBEZGRnd3d7q6usLCwq6urhkZGTMzMx8f
	H3h4eP//////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////
	/////////////////////yH+FUNyZWF0ZWQgd2l0aCBUaGUgR0lNUAAh+QQB
	CgD/ACwAAAAA0gAnAAAI/gD/CRxIsKBBggACCBhwcCCBhhAjSpxIsaLFixgz
	atw4sYCBAwgSKGi4gAHHkyhTqlzJcmUDBwMfQDgYAUDLmzhz6tyJUYJBBxN4
	EjzhRWhECBSMKlVKoYJACwUvYMigQanJpQ07Yd16cwOHDh4+gPgwMAQCESMu
	TCAhNE8brgYpeYBLl+OGESUGmjAh8ASKFA4i/FOxQuiBugU9IV5skUULggVc
	CHwB418MGf9m0OD5iQNjgSxsfh5tsEANGwRvCMSRQ8c/AQX+vdjBk0WI0V0w
	kF7Mo4dBHz8KLhAIJMi/CiP+CRlChCeojGbqhErS0owonUcIDELszGIRIwaP
	/iAhWIL6PyVL/jFp8s8JC55J/GQU0ePGqJs5cs6R0wtK3RTyRZTCEwJBAV5B
	AETh2z9STDEQFVX8Y8UVWKTHEykjUaQDIAXxcZJTBZWyCUsDEkTWQSC2VOI/
	j1gHURYD+KCFQFuYV9AULXDRRVEDfbFDDGCEIQZHY5BRxhZIJrkFHwQKZMop
	FKFiCAyYCfRFFwIVeaSSSDLZUCpmFBTJMCrBKONAaHDxZZgrmTnjP0Vdd5AZ
	Tgh0RoEHHvRFQ2hUydEWOcxVUCEqEKSEKhMJYsg/F6gp0Ew0BmoQoRDdV9AS
	QKREp50DXRDhQZaqtOk/d7rASw9gPHJQGicUZGBG/lUkdZIQNahh0CFzELQK
	KxJtkoFBqA1Eq60F4doQI60YdIAxKbF60DEHIcuSswMZMsUUgTRX0BooGMTG
	Wxe58YZKO8DhGkFlFJRDHRJNABVBrrzyASww/VPuuQOl21AsrbpaSkOIUsRt
	QZ8IYQYyB/G70sAHyTmQDTLEMYG2A3kgB0YaUHwSFzlUNlAYUhD0SA4NQGRJ
	ETm4cgVBIrS8xkAceywQyAZVgIorhCBwREEE5NfwRBBLrHEyyghRkM0464xS
	0BMfpEhBc+BARx2OkmbHHQQVSpAbOeTVkCySzCILRVdnbRAtVrRRyxlG/GD0
	QAH4bJDDEEU9ddUNoa02/ttuE2SGIYAHDngjDdlNdUUV1IDHbgJ9kEMeApFi
	S0FGdB0RC7dU5Djk/0heUA+4qBZBBU/kAAlBcTdZEN0QJb54RKCLTrrpBLnQ
	ye24315VQ65bpMceFC3Ax/DEF2/88R4KhIUTzDfPfB8G+THeYAYlkcOiEN2R
	hUXSC6Q1QaoQ8k8B9wmRg8Yc5KC6Keybskz7pkD0+0Thj1/++SjNb9Efd04E
	CA4ADKAAB0hAHBDEAQhMIAIPoocciMBzlLNcQ1KQi4s08IGTKwgcdBOGTPxD
	F1EoSOoGAoISgoAZJgQBRPg3kQ3+o4MfDGFBvkDDGtIwIiy0CBzcwDiCwEEU
	/ooxSCMkeBAO7AIjPwwiQXiRg1btwSkr0Arq5LY6iuxQIkx0IhSlOJBXpOKL
	YPziiQ5yxYoQIRA9JEjPenGQ0u2sIb54l0XWCKwcAGIDsfiHDIBXEAlQkSCs
	O8gZgWZHPOqRjycZpEVmIIg0DmQQZIBIDgLQkC/8IiOQbMgEWNEJJphgBgdp
	QTMaEkiDMJIim+zkJ1VySosQwkaOzJNBgBGMhvAAShmRJUHw0IRA4OACDVEB
	sw7yiYm8kiK89CUwVXLMirjBQicpxHbgooQtNIQFKbhJI5QQEWHgMiPPtMg2
	WRLOiQxiUYd420bkgAgaJOIwXJlBEQpClg1QASfD/mAPRIjhmYuc8x/ptEg+
	U/LPgE5kCioQwdNOwoAFpMAHY2jkVpqQR0BqAEs4yRBEUFDMiyBUoRfR6Ek+
	ulCKsOkkqhNI8v7xhEUYxQyOKAgFYFkdiRQjpRU5qUV0ehKe0oURTVID4YxC
	A1V9phCacKQjs7CGC1yhBE+YQwYcoQGj8kQFXmNMB3yi1N044AFYeMIjLHA6
	G4ygCP0SShasKZAMuPWtcI1rXHnASh52lTSRYARBJCGQSfBIKQYYCBYGS9jC
	Gtaw2UzJA+5KGkpUoiCW+IcHLrEVSrxMIG7IrGY3y9nNqgQCkGIsY/jA1oGo
	Bgrj2kot/zGBHLj2tbCNWS1sKYkS3Yj2M3HgnEDo+g9MeHArRigKLzhB3OIa
	97jG5QVKGrCB235GE7/9BwheNogcDKIQzs2udiGygAgkZROcEIgiqKrL7Zo3
	u0RYgCGySqrynve9OwkIADs=
    }
    variable i_sladveq [image create photo -data [set image64_sladveq]]

    # Solid URSolver equation image
    variable image64_slurseq $image64_sladveq
    variable i_slurseq [image create photo -data [set image64_slurseq]]
    
    # Fluid blue drop
    variable i_fluiddrop [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images drop.gif]]

    # Solid grey column
    variable i_solidcolumn [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images column.gif]]

    # Fluid boundaries icon
    variable i_fbond [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images fboundaries.gif]]

    # Solid boundaries icon
    variable i_sbond [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images sboundaries.gif]]

    # Contact icon
    variable i_contact [image create photo -file \
	    [file join $ProblemTypePriv(problemtypedir) images contacts.gif]]

    # Folder icon
    variable image64_folder {
	R0lGODlhEQANAPcAAAgICJubCNDQZ/rQm/r6m/r60Pr6+gAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	AAAAAAAAAAAAAAAAAP///yH5BAEAAP8ALAAAAAARAA0AQAhjAP8JDECQIACB
	CBMGKECgocMBBCAOEHDw30KHGB9SRAigo8ePHwcKGElSQMGCCC9mbChxo8UC
	EiPKHEDTpUUDBRgSCFAx4cCcQIPmJOBSZcaYRXU+nEm0otGYEic6LUm15MGA
	ADs=
    }
    variable i_folder [image create photo -data [set image64_folder]]

    # Point 16x16 icon
    variable image64_point16 {
	R0lGODlhEAAQAKEAAAAA/////39/f////yH5BAEKAAMALAAAAAAQABAAAAIb
	nI+py+0PYwQUghCquxmITVEf6I2OgErqyioFADs=
    }
    variable i_point16 [image create photo -data [set image64_point16]]
}