### Interfaz con la estructura XML de  informacin de Tdyn
### El formato es el siguiente:
### <Tdyn_XML_Database version="0.1">
###    <geometry><\geometry>
###    <mesh><\mesh>
###    <conditions><\conditions>
###    <materials><\materials>
###    <properties>
###       <tdyn><\tdyn>
###       <glass><\glass>
###    <\properties>
###    <postprocess><\postprocess>
###    <other><\other>
###    <\Tdyn_XML_Database>
### Los campos de texto que guarden variables pueden escribirse en formato 
### gid (QUESTION - VALUE), xml (<question> - <value>) o tcl (script ejecutable).
namespace eval processxmlfile {
    # XML doc info
    variable XmlFile ""
    variable XmlDoc ""
    variable XmlRoot ""
    
    variable rootname Tdyn_XML_database
    variable version 0.1

    variable XML_version {<?xml version="1.0"?>}
    variable DTD_format {\
	<!DOCTYPE Tdyn_XML_database [
	  <!ELEMENT Tdyn_XML_database (geometry,mesh,conditions,materials,properties,postprocess,other)>
	  <!ELEMENT geometry (#PCDATA)>
	  <!ELEMENT mesh (#PCDATA)>
	  <!ELEMENT conditions (#PCDATA)>
	  <!ELEMENT materials (#PCDATA)>
	  <!ELEMENT properties (#PCDATA)>
	  <!ELEMENT postprocess (#PCDATA)>
	  <!ELEMENT other (#PCDATA)>
	]>\
    }
}

proc processxmlfile::initRead { filename {new 0} } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    package require tdom

    if { $XmlFile != "" } {
	$XmlDoc delete 
    }

    if { [catch {
	    set fin [open $filename r]
	    set XmlDoc [dom parse -channel $fin]
	    set XmlRoot [$XmlDoc documentElement]
	    close $fin
	    set XmlFile $filename
	} errormsg ] } {
	if { $new == 1 } {
	    initCreate $filename
	} else {
	    WarnWin "[= {Cannot read Xml file %s due to} $filename]:\n$errormsg"
	    set XmlFile ""
	    set XmlDoc ""
	    set XmlRoot ""
	}
    }
}

proc processxmlfile::initCreate { filename } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    variable rootname
    variable version

    package require tdom

    if { $XmlFile != "" } {
	$XmlDoc delete 
    }
    if { [catch {
	    set XmlDoc [dom createDocument $rootname]
	    set XmlRoot [$XmlDoc documentElement]
	    set XmlFile $filename
	    $XmlRoot setAttribute version $version
	} errormsg ] } {
	WarnWin "[= {Cannot create Xml document %s due to} $filename]:\n$errormsg"
	set XmlFile ""
	set XmlDoc ""
	set XmlRoot ""
    }
}

# Esta funcin lee un texto entre los tags (node)
# node puede ser una lista de tags
proc processxmlfile::read { node } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    if { $XmlDoc == "" } {
	return ""
    }

    set ret ""
    foreach inode $node {
	foreach jnode [$XmlDoc getElementsByTagName $inode] {
	    append ret [$jnode text] 
	}
    }
    return $ret
}

# Escribe un listado de variables de un namespace en un formado dado
proc processxmlfile::putsvars { nodepath vars namesp { type xml } {version 1.0} } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    variable rootname

    if { $XmlDoc == "" } {
	WarnWin [= {XML file not initialised}]
	return
    }

    switch $type {
	tcl {
	    set text "\n"
	    foreach ivar $vars {
		set varname ${namesp}::${ivar}
		catch {
		    append text "set $varname [eval set temp $$varname]\n"
		}
	    }
	}
	gid {
	    set text "\n"
	    foreach ivar $vars {
		set varname ${namesp}::${ivar}
		catch {
		    set val [eval set temp $$varname]
		    append text "QUESTION: $ivar\nVALUE: $val\n"
		}
	    }
	}
	xml {
	    set prcnode [$XmlRoot selectNodes $nodepath]
	    if { $prcnode == "" } {
		set prcnode [createElement /$rootname/$nodepath]
	    }
	    foreach ivar $vars {
		set xpath "$nodepath/param\[@name='$ivar'\]"
		set paramNode [$XmlRoot selectNodes $xpath]
		if { $paramNode != "" } {
		    [$paramNode firstChild] nodeValue [set ${namesp}::${ivar}]
		} else {
		    set newnode [$XmlDoc createElement param]
		    $newnode setAttribute name $ivar
		    $prcnode appendChild $newnode
		    $newnode appendChild [$xmldoc createTextNode [set ${namesp}::${ivar}]]
		}
	    }
	    return
	}
    }
    # Look for the node and fill XML document
    set oldnode [$XmlRoot selectNodes /$rootname/$nodepath]
    if { $oldnode == "" } {
	set newnode [processxmlfile::createElement /$rootname/$nodepath]
	if { $newnode == "" } {
	    return
	}
    } else {
	set path [split $nodepath /]
	set newnode [$XmlDoc createElement [lindex $path end]]
	set parenode [$oldnode parentNode]
	$oldnode delete
	$parenode appendChild $newnode
    } 
    set textnode [$XmlDoc createTextNode $text]
    $newnode appendChild $textnode
}

# Comprueba si ha cambiado el valor del listado de variables
proc processxmlfile::checkvars { nodepath vars namesp { type xml } } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    variable rootname

    if { $XmlDoc == "" } {
	WarnWin [= {XML database not initialised}]
	return -1
    }

    set node [$XmlRoot selectNodes /$rootname/$nodepath]
    if { $node == "" } {
	WarnWin [= {XML database not correct}]
	return -1
    }

    switch $type {
	tcl {
	    set text [split [string trim [$node text]] "\n"]
	    foreach line $text {
		if { ![catch {
		        regexp "set\ *${namesp}::(.+)\ +(.+)" $line --> question value
		        set varname ${namesp}::${question}
		    }] } {
		    if { [eval set temp $$varname] != $value } {
		        return 1
		    }
		}
	    }   
	}
	gid {
	}
	xml {
	    foreach ivar $vars {
		set xpath "$nodepath/param\[@name='$ivar'\]"
		set paramNode [$XmlRoot selectNodes $xpath]
		if { $paramNode == "" } {
		    return 1
		}
		set value [set ${namesp}::${ivar}]
		if { [$paramNode text] != $value } {
		    return 1
		}
	    }
	}
    }
    return 0
}

# Crea un elemento a partir de un path 
proc processxmlfile::createElement { nodepath } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    set path [split $nodepath /]
    set node $XmlRoot
    if { [$node nodeName] != [lindex $path 1] } {
	WarnWin [= {XML nodepath not correct}]
	return
    }
    for { set i 2 } { $i < [llength $path] } { incr i } {
	set childnode [lindex [$node selectNodes [lindex $path $i]] 0]
	if { $childnode == "" } {
	    set childnode [$XmlDoc createElement [lindex $path $i]]
	    $node appendChild $childnode
	}
	set node $childnode
    }
    return $node
}

proc processxmlfile::getsvars { nodepath namesp { type xml } } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    variable rootname

    if { $XmlDoc == "" } {
	WarnWin [= {XML database not initialised}]
	return
    }

    set node [$XmlRoot selectNodes /$rootname/$nodepath]
    if { $node == "" } {
	WarnWin [= {XML database not correct}]
	return
    }

    switch $type {
	tcl {
	    set text [string trim [$node text]]
	    catch {
		eval $text
	    }
	    return 
	    #set text lindex [lindex [lindex [$node asList] 2] 0] 1]
	    #set text [$node text]
	    #catch {
	    #    set text [split $text "\n"]
	    #    foreach line $text {
	    #        eval $line
	    #    }
	    #}
	}
	gid {
	    set text [$node asList]
	    set questions [regexp -inline -all -line {QUESTION: *(.*) *$} $text]
	    set values [regexp -inline -all -line {VALUE:\s+?(.*)$} $text]
	    set qlen [llength $questions]
	    for { set i 1 } { $i < $qlen } { incr i 2 } {
		set question [lindex $questions $i]
		set value [lindex $values $i]
		set varname ${namesp}::${question}
		catch {
		    set $varname $value
		}
	    }
	}
	xml {
	    foreach node [$XmlRoot selectNodes $nodepath] {
		foreach paramNode [$node selectNodes param] {
		    catch {
		        set name [$paramNode getAttribute name ""]            
		        set [set ${namesp}::${name}] [$paramNode text]
		    }  
		}
	    }
	}
    }
}

# Esta funcin escribe el archivo XML
proc processxmlfile::write { {filename ""} } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    variable version
    variable XML_version
    variable rmat

    if { $XmlDoc == "" } {
	WarnWin [= {Cannot write XML file}]
	return
    }

    $XmlRoot setAttribute version $version

    if { $filename == "" } {
	set fout [open $XmlFile w]
	puts $fout $XML_version
#        puts $fout $DTD_format
	puts $fout [$XmlRoot asXML]
	close $fout
    } else {
	set fout [open $filename w]
	puts $fout $XML_version
#        puts $fout $DTD_format
	puts $fout [$XmlRoot asXML]
	close $fout
    }
}

# Esta funcin lee cdigo tcl escrito entre los tags (node) y lo ejecuta
# node puede ser una lista de tags
proc processxmlfile::execTcl { node } {
    # XML doc info
    variable XmlFile
    variable XmlDoc
    variable XmlRoot

    if { $XmlDoc == "" } {
	return ""
    }

    set ret ""
    foreach inode $node {
	foreach jnode [$XmlDoc getElementsByTagName $inode] {
	    append ret [$jnode text] 
	}
    }
    eval $ret
}

# Write info in bas file
proc processxmlfile::writeXMLbas { type } {
    switch $type {
	pbsim {
	    return [processdatwin::writepbsim]
	}
    }
}