namespace eval matfluid {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_ransol [IsVarOnTdynDefaults show_ransol 1]
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    variable show_advect [IsVarOnTdynDefaults show_advect 1]
    variable show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    variable edit_units [IsVarOnTdynDefaults edit_units 0]

    # Selected material name
    variable Material_Name
    variable ValList

    # Top Frame Buttons (Modify, New and Erase material)
    variable button_modify 
    variable button_new
    variable button_erase

    # Advect Species variables
    variable Species_Rebuild

    # Advect Species variables
    variable Species_Names
    variable Species_Values
    variable SpeciesUpdateCmd ""
    variable Species_UValues
    variable Species_Valnames \
	    [list \
	    [list "[= {Advection}] f \u00b9 " F1_Spc func 1 ""] \
	    [list "[= {Diffusion (Fick's law)}] f \u00b2 " F2_Spc func 1 ""] \
	    [list "[= {Degradation}] f \u00b3 " F3_Spc func 1 ""] \
	    [list "[= {Source}] f \u2074 " F4_Spc func 1 ""] \
	    [list "[= {Decantation}] f \u2075 " F5_Spc func 1 ""] ]
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    variable Species_UValnames \
	    [list \
	    [list "F2 Units" F2_Units unit 1 concentration] \
	    [list "F3 Units" F3_Units unit 1 concentration] \
	    [list "F4 Units" F4_Units unit 1 concentration] \
	    [list "F5 Units" F5_Units unit 1 concentration] ]
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    # Advect Species support variables
    variable spcdelerrwarn 0
    variable spcdellasterrwarn 0
    variable spcnlastsel -1
    variable snlist
    variable sname

    # URSolver Species variables
    variable URVariables_Rebuild 
    # URSolver Species variables
    variable URVariables_Names
    variable URVariables_Values
    variable URVariablesUpdateCmd ""
    variable URVariables_UValues
    variable URVariables_Valnames \
	    [list \
	    [list "ft\u00b9 " F1t_Phi func 1] \
	    [list "fc\u00b9 " F1c_Phi func 1] \
	    [list "f \u00b2 " F2_Phi func 1] \
	    [list "f \u00b3 " F3_Phi func 1] \
	    [list "f \u2074 " F4_Phi func 1] \
	    ]
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    variable URVariables_UValnames \
	    [list \
	    [list "F1t Phi Units" F1t_Phi_Units unit 1 concentration] \
	    [list "F1c Phi Units" F1c_Phi_Units unit 1 concentration] \
	    [list "F2 Phi Units" F2_Phi_Units unit 1 concentration] \
	    [list "F3 Phi Units" F3_Phi_Units unit 1 concentration] \
	    [list "F4 Phi Units" F4_Phi_Units unit 1 concentration] \
	    ]
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    # URSolver Species support variables
    variable ursdelerrwarn 0
    variable ursdellasterrwarn 0
    variable ursnlastsel -1
    variable ursnlist
    variable ursname

    # List of available Fluid Material Questions in Mat File.
    # Variables names are local script variables and in some 
    # cases do not correspond the MAT file QUESTION name
    if { $ProgramName == "Tdyn2D" } {
	variable Fluid_Questions [list \
		[list Type string 0 ""]\
		[list Density func 1 ransol]\
		[list Density_Units unit 1 ransol density] \
		[list Viscosity func 1 ransol] \
		[list Viscosity_Units unit 1 ransol viscosity] \
		[list Darcys_Coefficient_D11 func 1 ransol ""] \
		[list Darcys_Coefficient_D12 func 1 ransol ""] \
		[list Darcys_Coefficient_D22 func 1 ransol ""] \
		[list Darcys_Coefficient_Units unit 1 permeability_inverse] \
		[list Compressibility func 1 ransol] \
		[list Compressibility_Units unit 1 ransol compressibility] \
		[list AccOX_Field func 1 ransol] \
		[list AccOY_Field func 1 ransol] \
		[list Acceleration_Units unit 1 ransol acceleration] \
		[list Specific_Heat_Cp func 1 heatrans] \
		[list Cp_Units unit 1 heatrans heat] \
		[list Thermal_Conductivity_k func 1 heatrans] \
		[list k_Units unit 1 heatrans conductivity] \
		[list Floatability func 1 heatrans] \
		[list Heat_Source_Field func 1 heatrans] \
		[list Heat_Source_Units unit 1 heatrans heat] \
		[list Heat_Reaction_Field func 1 heatrans] \
		[list Heat_Reaction_Units unit 1 heatrans heat] \
		[list Species_Properties string 0 advect] \
		[list Spc_Units string 0 advect] \
		[list Variables_Properties string 0 ursolver] \
		[list URV_Units string 0 ursolver] \
		]
    } elseif { $ProgramName == "Tdyn3D" } {
	variable Fluid_Questions [list \
		[list Type string 0 ""]\
		[list Density func 1 ransol]\
		[list Density_Units unit 1 ransol density] \
		[list Viscosity func 1 ransol] \
		[list Viscosity_Units unit 1 ransol viscosity] \
		[list Compressibility func 1 ransol] \
		[list Compressibility_Units unit 1 ransol compressibility] \
		[list Darcys_Coefficient_D11 func 1 ransol ""] \
		[list Darcys_Coefficient_D12 func 1 ransol ""] \
		[list Darcys_Coefficient_D13 func 1 ransol ""] \
		[list Darcys_Coefficient_D22 func 1 ransol ""] \
		[list Darcys_Coefficient_D23 func 1 ransol ""] \
		[list Darcys_Coefficient_D33 func 1 ransol ""] \
		[list Darcys_Coefficient_Units unit 1 permeability_inverse] \
		[list AccOX_Field func 1 ransol] \
		[list AccOY_Field func 1 ransol] \
		[list AccOZ_Field func 1 ransol] \
		[list Acceleration_Units unit 1 ransol acceleration] \
		[list Specific_Heat_Cp func 1 heatrans] \
		[list Cp_Units unit 1 heatrans heat] \
		[list Thermal_Conductivity_k func 1 heatrans] \
		[list k_Units unit 1 heatrans conductivity] \
		[list Floatability func 1 heatrans] \
		[list Heat_Source_Field func 1 heatrans] \
		[list Heat_Source_Units unit 1 heatrans heat] \
		[list Heat_Reaction_Field func 1 heatrans] \
		[list Heat_Reaction_Units unit 1 heatrans heat] \
		[list Species_Properties string 0 advect] \
		[list Spc_Units string 0 advect] \
		[list Variables_Properties string 0 ursolver] \
		[list URV_Units string 0 ursolver] \
		]
    }
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }

    # Function Editor variables lists
    switch $ProgramName {
	"Tdyn2D" {
	    variable ransol_ff \
		    [list \
		    {matfluid::Density} {Density} \
		    {matfluid::Viscosity} {Viscosity} \
		    {matfluid::Compressibility} {Compressibility} \
		    {matfluid::Darcys_Coefficient_D11} [= {Darcy's Coefficient D11}] \
		    {matfluid::Darcys_Coefficient_D12} [= {Darcy's Coefficient D12 & 21}] \
		    {matfluid::Darcys_Coefficient_D22} [= {Darcy's Coefficient D22}] \
		    {matfluid::AccOX_Field} {Acceleration Field OX} \
		    {matfluid::AccOY_Field} {Acceleration Field OY} \
		    ]
	}
	"Tdyn3D" {
	    variable ransol_ff \
		    [list \
		    {matfluid::Density} {Density} \
		    {matfluid::Viscosity} {Viscosity} \
		    {matfluid::Compressibility} {Compressibility} \
		    {matfluid::Darcys_Coefficient_D11} [= {Darcy's Coefficient D11}] \
		    {matfluid::Darcys_Coefficient_D12} [= {Darcy's Coefficient D12 & 21}] \
		    {matfluid::Darcys_Coefficient_D13} [= {Darcy's Coefficient D13 & 31}] \
		    {matfluid::Darcys_Coefficient_D22} [= {Darcy's Coefficient D22}] \
		    {matfluid::Darcys_Coefficient_D23} [= {Darcy's Coefficient D23 & 32}] \
		    {matfluid::Darcys_Coefficient_D33} [= {Darcy's Coefficient D33}] \
		    {matfluid::AccOX_Field} {Acceleration Field OX} \
		    {matfluid::AccOY_Field} {Acceleration Field OY} \
		    {matfluid::AccOZ_Field} {Acceleration Field OZ} \
		    ]
	}
    }
    variable heatrans_ff \
	    [ list \
	    {matfluid::Specific_Heat_C} {Specific Heat C} \
	    {matfluid::Thermal_Conductivity_k} {Thermal Conductivity k} \
	    {matfluid::Floatability} {Floatability} \
	    {matfluid::Heat_Source_Field} {Heat Source Field} \
	    {matfluid::Heat_Reaction_Field} {Heat Reaction Field} \
	    ]
    variable advect_ff \
	    [ list \
	    {matfluid::F1_Spc} {f1} \
	    {matfluid::F2_Spc} {f2} \
	    {matfluid::F3_Spc} {f3} \
	    {matfluid::F4_Spc} {f4} \
	    {matfluid::F5_Spc} {f5} \
	    ]
    variable ursolver_ff \
	    [ list \
	    {matfluid::F1t_Phi} {f1t} \
	    {matfluid::F1c_Phi} {f1c} \
	    {matfluid::F2_Phi} {f2} \
	    {matfluid::F3_Phi} {f3} \
	    {matfluid::F4_Phi} {f4} \
	    ]
    variable fluid_variables "x y z nx ny nz t it rand dt exp(1) pi vx vy vz pr tm\
	    vt v_ tau wc ds ke ep om kt pk dn bt sp# ph#"
    variable example_fluid_variables "x y z nx ny nz t it rand dt exp(1)\
	    pi vx vy vz pr tm vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable fluid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
    variable predefined_ff [list $fluid_variables $example_fluid_variables $fluid_operators]
}

################################################################################
#
#   Initialize Fluid Material Window
#
################################################################################

proc matfluid::init { BASE selmatname } {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable edit_units

    # Selected Material Name
    variable Material_Name $selmatname
    # Variables Lists
    variable Fluid_Questions
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }
    # Function Editor Variables
    variable ransol_ff
    variable heatrans_ff
    variable advect_ff
    variable ursolver_ff
    variable fluid_variables
    variable example_fluid_variables
    variable fluid_operators
    variable predefined_ff

    # Advect Species support variables
    variable Species_Rebuild 
    variable Species_Names
    variable Species_Values
    variable Species_Valnames
    variable snlist

    # URSolver support variables
    variable URVariables_Rebuild 
    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_Valnames
    variable ursnlist

    # Initialise support variables
    set Species_Rebuild ""
    set URVariables_Rebuild ""

    # Read values of selected material
    matfluid::getvalues

    #####  HELP Text variables #####
    set help_btn_initcond "[= {Define Initial Conditions}]."
    set help_btn_solvdat "[= {Define Solver Data}]."
    set help_btn_func "[= {Custom Function Editor}].\n[= {Edit a function and transfer it automatically to a field}].\n[= {You can also RIGHT-CLICK a field to open the Function Editor}]."
    # RANSOL Fields
    set help_density "[= {Density of the fluid}]."
    set help_viscosity "[= {Viscosity of the fluid}].\n[= {May be a constant or a funtion ( i.e. 0.001*Tm )}].\n[= {Function editor can be started by pressing 'fx' button}]."
    set help_compressibility "[= {Compressibility parameter defined by 1/c\u00b2,}]\n[= {being c the velocity of sound in the fluid}]."
    switch $ProgramName {
	"Tdyn2D" {
	    set ht_acc "[= {External acceleration (NOT GRAVITY) acting on fluid (OX OY OZ component)}].\n[= {The units are inserted in the Data>Problem_Data>UNITS menu}].\n[= {May be a constant or a function ( i.e. 1.0-x*tt )}]\n[= {Function editor can be started by pressing 'fx' button}]."
	    set labeltextacc [= {Acceleration Field (X Y)}]
	    set help_Darcys_Coefficient "[= {Position (x,y) of the symmetric Darcy's law (DP=D.Mu.V) resistance coeffient matrix (inverse of permeability)}]."
	}
	"Tdyn3D" {
	    set ht_acc "[= {External acceleration (NOT GRAVITY) acting on fluid (OX OY component)}].\n[= {The units are inserted in the Problem_Data menu}].\n[= {May be a constant or a function ( i.e. 1.0-x*tt )}]\n[= {Function editor can be started by pressing 'fx' button}]."
	    set labeltextacc [= {Acceleration Field (X Y Z)}]
	    set help_Darcys_Coefficient "[= {Position (x,y,z) of the symmetric Darcy's law (DP=D.Mu.V) resistance coeffient matrix (inverse of permeability)}]."
	}
    }
    # HEATRANS Fields
    set help_specificheatcp "[= {Specific Heat (Cp) of the Fluid}]."
    set help_thermalconductivity "[= {Thermal Conductivity (k) of the Fluid}]."
    set help_thermalexpansion "[= {Floatability of the Fluid}]\n[= {( to take into account a thermal flotability effect, i.e. 0.001*(Tm-25) )}]"
    set help_heatsource "[= {Heat source for the thermal analysis}].\n[= {May be a constant or a funtion ( i.e. 100*sin(tt) )}].\n[= {Function editor can be started by pressing 'fx' button}].\n[= {Units are defined in Problem Data>UNITS>General Units window}]."
    set help_heatreaction "[= {Heat reaction for the thermal analysis}].\n[= {May be a constant or a funtion ( i.e. 1.1e-5*tm^3 )}].\n[= {Function editor can be started by pressing 'fx' button}].\n[= {Units are defined in Problem Data>UNITS>General Units window}]."
    # ADVECT Fields
    set help_Advect "[= {ADVECT module data}].\n[= {Select species from the list and edit their properties}].\n[= {New species may be created by pressing '+' button}].\n[= {It is also possible to delete current species by pressing '-' button}].\n[= {Changing species names or their number will affect all existing fluid materials}]."
    set help_Advect_Pic "[= {ADVECT module data}].\n[= {Properties of the current species may be inserted below}].\n[= {Properties names correspond to those given in the equation}]."
    # URSOLVER Fields
    set help_URSolver "[= {URSOLVER module data}].\n[= {Select variables from the list and edit their properties}].\n[= {New variables may be created by pressing '+' button}].\n[= {It is also possible to delete current variables by pressing '-' button}].\n[= {Changing variable names or their number will affect all existing fluid materials}]."
    set help_URSolver_Pic "[= {URSOLVER module data}].\n[= {Properties of the current variable may be inserted below}].\n[= {Properties names correspond to those given in the equation}]."

    # Main Notebook pages GRID
    set pages [NoteBook $BASE.pages -arcradius 0]
    if { $show_ransol == 1 } {
	set page1 [$pages insert 0 page1 -text "RANSOL"]
	set fpage1 [$pages getframe page1]
    }
    if { $show_heatrans == 1 } {
	set page2 [$pages insert 1 page2 -text "HEATRANS"]
	set fpage2 [$pages getframe page2]
    }
    if { $show_advect == 1 } {
	set page3 [$pages insert 2 page3 -text "ADVECT"]
	set fpage3 [$pages getframe page3]
    }
    if { $show_ursolver == 1 } {
	set page4 [$pages insert 3 page4 -text "URSOLVER"]
	set fpage4 [$pages getframe page4]
    }

    if { $show_ransol == 1 } {
	$pages raise page1
    } elseif { $show_heatrans == 1 } {
	$pages raise page2
    } elseif { $show_advect == 1 } {
	$pages raise page3
    } elseif { $show_ursolver == 1 } {
	$pages raise page4
    }

    # GRIDS PAGES
    grid $pages -row 1 -column 0 -sticky nwes -pady 5
    if { $show_ransol == 1 } {
	grid columnconf $fpage1 10 -weight 1
	grid rowconf $fpage1 10 -weight 1
    }
    if { $show_heatrans == 1 } {
	grid columnconf $fpage2 10 -weight 1
	grid rowconf $fpage2 10 -weight 1
    }
    if { $show_advect == 1 } {
	grid columnconf $fpage3 1 -weight 1
	grid rowconf $fpage3 1 -weight 1
    }
    if { $show_ursolver == 1 } {
	grid columnconf $fpage4 1 -weight 1
	grid rowconf $fpage4 1 -weight 1
    }


    # Density Units is outside of the show_ransol condition because 
    # it is also used in HEATRANS if RANSOL is hidden. 
    lappend units_dens "Kg/m\u00b3" "Kg/l" "g/l" "t/m\u00b3" "lb/in\u00b3" "Kg/cc" "g/cc"

    if { $show_ransol == 1 } {

	#--------------------------- PAGE RANSOL ---------------------------#
	
	# Combo boxes Units definitions
	lappend units_visc "Pa.s" "Kg/m.s" "ton/m.s" "g/m.s" "lb/in.s"
	lappend units_cprs "s\u00b2/m\u00b2" "s\u00b2/cm\u00b2" "cs\u00b2/m\u00b2" "s\u00b2/ft\u00b2"
	lappend units_acc "m/s\u00b2" "G" "ft/s\u00b2"
	lappend units_DCC_Units "1/m\u00b2" "1/darcy"

	# Invisible size label
	set sizelabel [label $fpage1.l_sizelabel \
		-text "                                   " -bd 0 ]
	
	# Density Field
	set l_dens [Label $fpage1.l_dens -text "[= {Density}] " -helptext $help_density]
	set e_dens [entry $fpage1.e_dens -textvariable matfluid::Density]
	set cbx_dens [ComboBox $fpage1.lb_dens -values $units_dens \
		-editable $edit_units -width 11 -textvariable matfluid::Density_Units]
	
	# Viscosity Field
	set l_visc [Label $fpage1.l_visc -text "[= {Viscosity}] " -helptext $help_viscosity]
	set e_visc [entry $fpage1.e_visc -textvariable matfluid::Viscosity]
	set cbx_visc [ComboBox $fpage1.lb_visc -values $units_visc \
		-editable $edit_units -width 11 -textvariable matfluid::Viscosity_Units]
	
	# Compressibility Field
	set l_cprs [Label $fpage1.l_cprs -text "[= {Compressibility}] " -helptext $help_compressibility]
	set e_cprs [entry $fpage1.e_cprs -textvariable matfluid::Compressibility]
	set cbx_cprs [ComboBox $fpage1.lb_cprsc -values $units_cprs \
		-editable $edit_units -width 11 -textvariable matfluid::Compressibility_Units]

	# Darcy's law Resistance Coefficients Matrix
	set l_dcc [Label $fpage1.l_darcyfields -text \
		"[= {Darcy's law Resistance}]\n [= {Matrix}] " -justify right -helptext $help_Darcys_Coefficient]
	set f_darcyfields [frame $fpage1.f_darcyfields -relief flat]
	switch $ProgramName {
	    "Tdyn2D" {
		#  Darcy's law Resistance Coefficients Fields
		set e_dcc11 [entry $f_darcyfields.e_dcc11 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D11]
		set e_dcc12 [entry $f_darcyfields.e_dcc12 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D12]
		set e_dcc21 [entry $f_darcyfields.e_dcc21 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D12]
		set e_dcc22 [entry $f_darcyfields.e_dcc22 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D22]
	    }
	    "Tdyn3D" {
		# Darcy's law Resistance Coefficients Fields
		set e_dcc11 [entry $f_darcyfields.e_dcc11 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D11]
		set e_dcc12 [entry $f_darcyfields.e_dcc12 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D12]
		set e_dcc13 [entry $f_darcyfields.e_dcc13 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D13]
		set e_dcc21 [entry $f_darcyfields.e_dcc21 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D12]            
		set e_dcc22 [entry $f_darcyfields.e_dcc22 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D22]
		set e_dcc23 [entry $f_darcyfields.e_dcc23 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D23]
		set e_dcc31 [entry $f_darcyfields.e_dcc31 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D13]
		set e_dcc32 [entry $f_darcyfields.e_dcc32 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D23]
		set e_dcc33 [entry $f_darcyfields.e_dcc33 -width 6 \
		        -textvariable matfluid::Darcys_Coefficient_D33]
	    }
	}
	set cbx_DCC_Units [ComboBox $fpage1.cbx_dcc_units -values $units_DCC_Units \
		-editable $edit_units -width 11 -textvariable matfluid::Darcys_Coefficient_Units]

	# Acceleration Field
	set f_acc [frame $fpage1.f_acc ]
	set l_acc [Label $fpage1.l_acc -text $labeltextacc -helptext $ht_acc ]
	set e_accx [entry $f_acc.e_accx -textvariable matfluid::AccOX_Field -width 6]
	set e_accy [entry $f_acc.e_accy -textvariable matfluid::AccOY_Field -width 6]
	if { $ProgramName == "Tdyn3D" } {
	    set e_accz [entry $f_acc.e_accz -textvariable matfluid::AccOZ_Field -width 6]
	}
	set cbx_acc [ComboBox $fpage1.lb_acc -values $units_acc \
		-editable $edit_units -width 11 -textvariable matfluid::Acceleration_Units]

	# Buttons frame
	set btn_frame [frame $fpage1.btn_frame]

	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ransol_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matfluid::callinitdata $BASE \"\" ransol" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE \"\" ransol" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]
	
	# GRIDS
	# SizeLabel
	grid $sizelabel -row 0 -column 0 -sticky en
	# Density
	grid $l_dens -row 1 -column 0 -sticky en -pady 2
	grid $e_dens -row 1 -column 1 -sticky wn -pady 2
	grid $cbx_dens -row 1 -column 2 -sticky wn -pady 1 -padx 7
	# Viscosity
	grid $l_visc -row 2 -column 0 -sticky en -pady 2
	grid $e_visc -row 2 -column 1 -sticky wn -pady 2
	grid $cbx_visc -row 2 -column 2 -sticky wn -pady 1 -padx 7
	# Compressibility
	grid $l_cprs -row 3 -column 0 -sticky en -pady 2
	grid $e_cprs -row 3 -column 1 -sticky wn -pady 2
	grid $cbx_cprs -row 3 -column 2 -sticky wn -pady 1 -padx 7
	# Darcy's law Resistance Coefficients Matrix
	grid $l_dcc -row 4 -column 0 -sticky en -pady 4
	grid $f_darcyfields -row 4 -column 1 -sticky wn -pady 4
	switch $ProgramName {
	    "Tdyn2D" {
		#  Darcy's law Resistance Coefficients Matrix Fields
		grid $e_dcc11 -row 0 -column 0 -sticky en
		grid $e_dcc12 -row 0 -column 1 -sticky wn -padx 3
		grid $e_dcc21 -row 1 -column 0 -sticky en
		grid $e_dcc22 -row 1 -column 1 -sticky wn -padx 3
	    }
	    "Tdyn3D" {
		#  Darcy's law Resistance Coefficients Matrix Fields
		grid $e_dcc11 -row 0 -column 0 -sticky en
		grid $e_dcc12 -row 0 -column 1 -sticky wn -padx 3
		grid $e_dcc13 -row 0 -column 2 -sticky wn
		grid $e_dcc21 -row 1 -column 0 -sticky en
		grid $e_dcc22 -row 1 -column 1 -sticky wn -padx 3
		grid $e_dcc23 -row 1 -column 2 -sticky wn
		grid $e_dcc31 -row 2 -column 0 -sticky en
		grid $e_dcc32 -row 2 -column 1 -sticky wn -padx 3
		grid $e_dcc33 -row 2 -column 2 -sticky wn
	    }
	}
	grid $cbx_DCC_Units -row 4 -column 2 -sticky wn -padx 7 -pady 3
	# Acc Field
	grid $l_acc -row 5 -column 0 -sticky en -pady 2
	grid $f_acc -row 5 -column 1 -sticky wn -pady 2
	grid $e_accx -row 0 -column 0 -sticky wn -padx 1
	grid $e_accy -row 0 -column 1 -sticky wn -padx 1
	if { $ProgramName == "Tdyn3D" } {
	    grid $e_accz -row 0 -column 2 -sticky wn -padx 1
	}
	grid $cbx_acc -row 5 -column 2 -sticky wn -pady 1 -padx 7
	
	# Buttons frame
	grid $btn_frame -row 6 -column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -pady 2 -padx 1
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -pady 2 -padx 1
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Right Click menus
	bind $e_dens <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Density Density \"\" 1 %X %Y"
	bind $e_visc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Viscosity Viscosity \"\" 1 %X %Y"
	bind $e_cprs <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Compressibility Compressibility \"\" 1 %X %Y"
	bind $e_accx <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::AccOX_Field \"Acceleration (OX)\" \"\" 1 %X %Y"
	bind $e_accy <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::AccOY_Field \"Acceleration (OY)\" \"\" 1 %X %Y"
	if { $ProgramName == "Tdyn3D" } {
	    bind $e_accz <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::AccOZ_Field \"Acceleration (OZ)\" \"\" 1 %X %Y"
	}
	#  Darcy's law Resistance Coefficients Bindings
	if { $ProgramName == "Tdyn2D" } {
	    bind $e_dcc11 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D11 \"Darcy's Coefficient D11\" \"\" 1 %X %Y"
	    bind $e_dcc12 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D12 \"Darcy's Coefficient D12\" \"\" 1 %X %Y"
	    bind $e_dcc21 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D12 \"Darcy's Coefficient D21\" \"\" 1 %X %Y"
	    bind $e_dcc22 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D22 \"Darcy's Coefficient D22\" \"\" 1 %X %Y"
	} else {
	    bind $e_dcc11 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D11 \"Darcy's Coefficient D11\" \"\" 1 %X %Y"
	    bind $e_dcc12 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D12 \"Darcy's Coefficient D12\" \"\" 1 %X %Y"
	    bind $e_dcc13 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D13 \"Darcy's Coefficient D13\" \"\" 1 %X %Y"
	    bind $e_dcc21 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D12 \"Darcy's Coefficient D21\" \"\" 1 %X %Y"
	    bind $e_dcc22 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D22 \"Darcy's Coefficient D22\" \"\" 1 %X %Y"
	    bind $e_dcc23 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D23 \"Darcy's Coefficient D23\" \"\" 1 %X %Y"
	    bind $e_dcc31 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D13 \"Darcy's Coefficient D31\" \"\" 1 %X %Y"
	    bind $e_dcc32 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D23 \"Darcy's Coefficient D32\" \"\" 1 %X %Y"
	    bind $e_dcc33 <ButtonPress-3> \
		    "matfluid::popupmenu $BASE matfluid::Darcys_Coefficient_D33 \"Darcy's Coefficient D33\" \"\" 1 %X %Y"
	}
    }

    if { $show_heatrans == 1 } {

	#--------------------------- PAGE HEATRANS ---------------------------#
	
	# Combo boxes Units definitions
	lappend units_shc "J/Kg.\u2070C" "J/g.\u2070C" "btu/lb.\u2070F"
	lappend units_thc "W/m.\u2070C" "KW/m.\u2070C" "btu/h.in.\u2070F" "btu/h.ft.\u2070F"
	lappend units_heatsrc "Kcal/hour.m\u00b3" "KW/m\u00b3" "cal/hour.m\u00b3" \
	"W/m\u00b3" "cal/s.m\u00b3" "btu/h.in\u00b3" "btu/s.in\u00b3" "btu/h.ft\u00b3"
	lappend units_heatrct "Kcal/h.m\u00b3.\u2070C" "KW/m\u00b3.\u2070C" \
	"cal/h.m\u00b3.\u2070C" "W/m\u00b3.\u2070C" "cal/s.m\u00b3.\u2070C" \
    "btu/h.in\u00b3.\u2070F" "btu/s.in\u00b3.\u2070F" "btu/h.ft\u00b3.\u2070F"

	# Invisible label to keep size
	set sizelabel [label $fpage2.l_sizelabel \
		-text "                                   " -bd 0 ]

	# Density Field
	set l_dens [Label $fpage2.l_dens -text "[= {Density}] " -helptext $help_density]
	set e_dens [entry $fpage2.e_dens -textvariable matfluid::Density]
	set cbx_dens [ComboBox $fpage2.lb_dens -values $units_dens \
		-editable $edit_units -width 11 -textvariable matfluid::Density_Units]
	bind $e_dens <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Density \"Density\" \"\" 1 %X %Y"

	# Specific Heat Cp Field
	set l_shc [Label $fpage2.l_shc -text "[= {Specific Heat}] " -helptext $help_specificheatcp]
	set e_shc [entry $fpage2.e_shc -textvariable matfluid::Specific_Heat_Cp]
	set cbx_shc [ComboBox $fpage2.lb_shc -values  $units_shc \
		-editable $edit_units -width 11 -textvariable matfluid::Cp_Units]
	
	# Thermal Conductivity Field
	set l_thc [Label $fpage2.l_thc -text "[= {Thermal Conductivity}] " -helptext $help_thermalconductivity]
	set e_thc [entry $fpage2.e_thc -textvariable matfluid::Thermal_Conductivity_k]
	set cbx_thc [ComboBox $fpage2.lb_thc -values $units_thc \
		-editable $edit_units -width 11 -textvariable matfluid::k_Units]
	
	# Floatability Field
	if { $show_ransol == 1 } {
	    set l_tex [Label $fpage2.l_tex -text "[= {Floatability}] " -helptext $help_thermalexpansion]
	    set e_tex [entry $fpage2.e_tex -textvariable matfluid::Floatability]
	}
	# Heat Source Field
	set l_heatsrc [Label $fpage2.l_heatsrc -text "[= {Heat Source Field}] " -helptext $help_heatsource]
	set e_heatsrc [Entry $fpage2.e_heatsrc -textvariable matfluid::Heat_Source_Field]
	set cbx_heatsrc [ComboBox $fpage2.lb_heatsrc -values $units_heatsrc \
		-editable $edit_units -width 11 -textvariable matfluid::Heat_Source_Units]
	
	# Heat Reaction Field
	set l_heatrct [Label $fpage2.l_heatrct -text "[= {Heat Reaction Field}] " -helptext $help_heatreaction]
	set e_heatrct [Entry $fpage2.e_heatrct -textvariable matfluid::Heat_Reaction_Field]
	set cbx_heatrct [ComboBox $fpage2.lb_heatrct -values $units_heatrct \
		-editable $edit_units -width 11 -textvariable matfluid::Heat_Reaction_Units]

	# Buttons frame
	set btn_frame [frame $fpage2.btn_frame]
	
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -justify left -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$heatrans_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matfluid::callinitdata $BASE \"\" heatrans" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE \"\" heatrans" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# GRIDS
	# SizeLabel
	grid $sizelabel -row 0 -column 0 -sticky en
	# Density
	grid $l_dens -row 1 -column 0 -sticky en -pady 2
	grid $e_dens -row 1 -column 1 -sticky wn -pady 2
	grid $cbx_dens -row 1 -column 2 -sticky wn -pady 1 -padx 7
	# Specific Heat Cp
	grid $l_shc -row 2 -column 0 -sticky en -pady 2
	grid $e_shc -row 2 -column 1 -sticky wn -pady 2
	grid $cbx_shc -row 2 -column 2 -sticky wn -pady 1 -padx 7
	# Thermal Conductivity
	grid $l_thc -row 3 -column 0 -sticky en -pady 2
	grid $e_thc -row 3 -column 1 -sticky wn -pady 2
	grid $cbx_thc -row 3 -column 2 -sticky wn -pady 1 -padx 7
	# Floatability
	if { $show_ransol == 1 } {
	    grid $l_tex -row 4 -column 0 -sticky en -pady 2
	    grid $e_tex -row 4 -column 1 -sticky wn -pady 2
	}
	# Heat Source Field
	grid $l_heatsrc -row 5 -column 0 -sticky en -pady 2
	grid $e_heatsrc -row 5 -column 1 -sticky wn -pady 2
	grid $cbx_heatsrc -row 5 -column 2 -sticky wn -pady 1 -padx 7
	# Heat Reaction Field
	grid $l_heatrct -row 6 -column 0 -sticky en -pady 2
	grid $e_heatrct -row 6 -column 1 -sticky wn -pady 2
	grid $cbx_heatrct -row 6 -column 2 -sticky wn -pady 1 -padx 7
	# Buttons frame
	grid $btn_frame -row 7 -column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Right click menus
	bind $e_shc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Specific_Heat_Cp \"Specific Heat Cp\" \"\" 1 %X %Y"
	bind $e_thc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Thermal_Conductivity_k \"Thermal Conductivity k\" \"\" 1 %X %Y"
	if { $show_ransol == 1 } {
	    bind $e_tex <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Floatability \"Floatability\" \"\" 1 %X %Y"
	}
	bind $e_heatsrc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Heat_Source_Field \"Heat Source Field\" \"\" 1 %X %Y"
	bind $e_heatrct <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Heat_Reaction_Field \"Heat Reaction Field\" \"\" 1 %X %Y"
    }

    if { $show_advect == 1 } {

	#--------------------------- PAGE ADVECT ---------------------------#

	# Top Label Frame
	set topfr [frame $fpage3.topfr -relief flat]
	# Information label
	set infotext "[= {Changing Species names or their number will affect all existing fluid materials}]."
	set l_info [Label $topfr.l_info -text $infotext -justify left -wraplength 150]
	GidHelp $l_info $help_Advect

	# Equation label with image
	set l_ieqdf [Label $topfr.l_ieqdf -image $TUI_images::i_fladveq]
	GidHelp $l_ieqdf $help_Advect_Pic

	# Title Frame Species List
	set tfr_spclist [TitleFrame $fpage3.tfr_spclist -relief ridge -bd 3\
		-text [= {Defined Species}] -side left]
	set frame_spclist [$tfr_spclist getframe]

	# Paned window
	set pw [PanedWindow $frame_spclist.pw]
	set pane1 [$pw add -weight 1]
	set pane2 [$pw add -weight 5]

	# Species Names in scrolled window
	set spc_scrw [ScrolledWindow $pane1.spc_scrw -scrollbar both]
	set snlist [listbox $spc_scrw.snlist -selectbackground lightblue -selectforeground black \
		-selectborderwidth 1 -selectmode extended -height 5 -bg white -width 10]
	$spc_scrw setwidget $snlist

	# Selected Specie values frame
	set svlist [frame $pane2.svlist -relief flat]

	# Buttons frame
	set btn_frame [frame $svlist.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$advect_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matfluid::callinitdata $BASE $snlist advect" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE $snlist advect" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# Buttons PLUS MINUS - under the Species Names listbox
	set btnbox [ButtonBox $frame_spclist.btnbox -spacing 0 -padx 1 -pady 1 -homogeneous 0]
	$btnbox add -image $buttons::ibtn_plus -width 20 -height 20 \
		-highlightthickness 0 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Add New Specie}] \
		-command "matfluid::addspecie $snlist $svlist \"advect\""
	$btnbox add -image $buttons::ibtn_minus -width 20 -height 20 \
		-highlightthickness 1 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Delete Selected Specie}] \
		-command "matfluid::removespecie $snlist $svlist \"advect\""

	# GRIDS
	grid $topfr -row 0 -column 0 -columnspan 2 -sticky nw -pady 10
	grid $l_info -row 0 -column 0 -sticky nw -padx 5
	grid $l_ieqdf -row 0 -column 1 -sticky nw -padx 5
	# Title Frame
	grid $tfr_spclist -row 1 -column 0 -columnspan 2 -sticky wnes
	grid $pw -row 0 -column 0 -sticky wnes
	grid $spc_scrw -row 0 -column 0 -sticky wnes
	grid $btnbox -row 1 -column 0 -sticky wn
	# Buttons frame
	grid $btn_frame -row [expr [llength $Species_Valnames] + 1] \
		-column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Rows and columns configuration
	grid rowconf $pane1 0 -weight 1
	grid columnconf $pane1 0 -weight 1
	grid rowconf $pane2 1 -weight 1
	grid columnconf $pane2 0 -weight 1
	grid columnconf $frame_spclist 0 -weight 1
	grid rowconf $frame_spclist 0 -weight 1
	grid rowconf $svlist 10 -weight 1

	GidHelpRecursive $tfr_spclist $help_Advect

	# BINDS 
	bind $snlist <<ListboxSelect>> ";
	matfluid::valsavespecie $snlist \"last\" \"advect\"; 
	matfluid::displayspecievalues $snlist \"advect\";
	matfluid::displayspeciesnames $snlist \"advect\";
	"
	bind $snlist <ButtonPress-3> "+matfluid::popupmenu $snlist \"\" \"\" \"\" 2 %X %Y"

	# Insert Species Names to listbox
	matfluid::displayspeciesnames $snlist "advect"
	# Selecting first Specie
	matfluid::updatespcselection $snlist 0 "advect"
	# Create Species Values widgets
	matfluid::makespecie $snlist $svlist "advect"
	# Put specie values into entries
	matfluid::displayspecievalues $snlist "advect"
    }

    if { $show_ursolver == 1 } {

	#--------------------------- PAGE URSOLVER ---------------------------#

	# Top Label Frame
	set topfr [frame $fpage4.topfr -relief flat]

	# Information label
	set infotext "[= {Changing Variables names or their number will affect all existing fluid materials}]."
	set l_info [Label $topfr.l_info -text $infotext -justify left -wraplength 150]
	GidHelp $l_info $help_URSolver

	# Equation label with image
	set l_ieqdf [Label $topfr.l_ieqdf -image $TUI_images::i_flurseq]
	GidHelp $l_ieqdf $help_URSolver_Pic

	# Title Frame Species List
	set tfr_spclist [TitleFrame $fpage4.tfr_spclist -relief ridge -bd 3\
		-text [= {Defined Variables}] -side left ]
	set frame_spclist [$tfr_spclist getframe]

	# Paned window
	set upw [PanedWindow $frame_spclist.pw]
	set upane1 [$upw add -weight 1]
	set upane2 [$upw add -weight 5]

	# Variables Names in scrolled window
	set ursv_scrw [ScrolledWindow $upane1.ursv_scrw -scrollbar both]
	set ursnlist [listbox $ursv_scrw.ursnlist -selectbackground lightblue -selectforeground black \
		-selectborderwidth 1 -selectmode extended -height 5 -bg white -width 10]
	$ursv_scrw setwidget $ursnlist

	# Selected Specie values frame
	set ursvlist [frame $upane2.ursvlist -relief flat]

	# Buttons frame
	set btn_frame [frame $ursvlist.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ursolver_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions button
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matfluid::callinitdata $BASE $ursnlist ursolver" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE $ursnlist ursolver" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# Buttons PLUS MINUS - under the Species Names listbox
	set btnbox [ButtonBox $frame_spclist.btnbox -spacing 0 -padx 1 -pady 1 -homogeneous 0]
	$btnbox add -image $buttons::ibtn_plus -width 20 -height 20 \
		-highlightthickness 0 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Add New Variable}] \
		-command "matfluid::addspecie $ursnlist $ursvlist ursolver"
	$btnbox add -image $buttons::ibtn_minus -width 20 -height 20 \
		-highlightthickness 1 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Delete Selected Variable}] \
		-command "matfluid::removespecie $ursnlist $ursvlist ursolver"

	# GRIDS
	grid $topfr -row 0 -column 0 -columnspan 2 -sticky nw -pady 10
	grid $l_info -row 0 -column 0 -sticky nw -padx 5
	grid $l_ieqdf -row 0 -column 1 -sticky nw -padx 5
	# Title Frame
	grid $tfr_spclist -row 1 -column 0 -columnspan 2 -sticky wnes
	grid $upw -row 0 -column 0 -sticky wnes
	grid $ursv_scrw -row 0 -column 0 -sticky wnes
	grid $btnbox -row 1 -column 0 -sticky wn
	# Buttons frame
	grid $btn_frame -row [expr [llength $Species_Valnames] + 2] \
		-column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Rows and columns configuration
	grid rowconf $upane1 0 -weight 1
	grid columnconf $upane1 0 -weight 1
	grid rowconf $upane2 1 -weight 1
	grid columnconf $upane2 0 -weight 1
	grid columnconf $frame_spclist 0 -weight 1
	grid rowconf $frame_spclist 0 -weight 1
	grid rowconf $ursvlist 10 -weight 1

	# BINDS 
	bind $ursnlist <<ListboxSelect>> ";
	matfluid::valsavespecie $ursnlist \"last\" \"ursolver\"; 
	matfluid::displayspecievalues $ursnlist \"ursolver\";
	matfluid::displayspeciesnames $ursnlist \"ursolver\";
	"
	bind $ursnlist <ButtonPress-3> "+matfluid::popupmenu $ursnlist \"\" \"\" \"\" 2 %X %Y"

	GidHelpRecursive $tfr_spclist $help_URSolver

	# Insert Species Names to listbox
	matfluid::displayspeciesnames $ursnlist "ursolver"
	# Selecting first Specie
	matfluid::updatespcselection $ursnlist 0 "ursolver"
	# Create Species Values widgets
	matfluid::makespecie $ursnlist $ursvlist "ursolver"
	# Put specie values into entries
	matfluid::displayspecievalues $ursnlist "ursolver"
    }

    if { [info exists snlist] == 1 && [info exists ursnlist] == 1 } {
	$pages bindtabs <Button-1> "matfluid::selectpage $snlist $ursnlist"
    } elseif { [info exists snlist] == 1 && [info exists ursnlist] != 1 } {
	$pages bindtabs <Button-1> "matfluid::selectpage $snlist \"\""
    } elseif { [info exists snlist] != 1 && [info exists ursnlist] == 1 } {
	$pages bindtabs <Button-1> "matfluid::selectpage \"\" $ursnlist"
    }
}

proc matfluid::selectpage { snlist ursnlist {pg ""} } {
    
    variable spcnlastsel
    variable ursnlastsel

    set nlist ""

    if { $pg == "page3" } {
	set type "advect"
	set nlist $snlist
	set setsel $spcnlastsel
	set leavenlist $ursnlist
    } elseif { $pg == "page4" } {
	set type "ursolver"
	set nlist $ursnlist
	set setsel $ursnlastsel
	set leavenlist $snlist
    }

    if { [string length $nlist] > 0 && ( $type == "advect" || $type == "ursolver") } {
	matfluid::valsavespecie $leavenlist "" $type
	matfluid::updatespcselection $nlist $setsel $type
	matfluid::displayspecievalues $nlist $type
    }

    return 1
}


proc matfluid::callinitdata { parent nlist type } {

    variable spcnlastsel
    variable ursnlastsel

    if { [string length $nlist] > 0 && ($type == "advect" || $type == "ursolver") } {
	matfluid::valsavespecie $nlist "actual" $type
    }
    set datachange [matfluid::checkclose $parent 0 0 0]

    if { $datachange != 0 } {
	set answer ""
	set answer [tk_dialogRAM $parent [= {Save Data?}] \
		"[= {To open the Initial Data Window, edited values have to be saved}].\n[= {Do you want to save them?}]" \               
		question 0 Yes No]
    } else {
	set answer 0
    }

    if { $answer == 0 } {
	set saveres ""
	if { $datachange != 0 } {
	    # Save edited data
	    set saveres [matfluid::checkclose $parent 1 0 0]
	}

	if { $saveres != "dataerror" } {
	    switch $type {
		"ransol" {
		    initdatwin::init .gid $type ""
		}
		"heatrans" {
		    initdatwin::init .gid $type ""
		}
		"advect" {
		    initdatwin::init .gid $type $spcnlastsel
		}
		"ursolver" {
		    initdatwin::init .gid $type $ursnlastsel
		}
	    }
	}
    }

    return 1
}

proc matfluid::callsolverdata { parent nlist type } {

    variable spcnlastsel
    variable ursnlastsel

    if { [string length $nlist] > 0 && ($type == "advect" || $type == "ursolver") } {
	matfluid::valsavespecie $nlist "actual" $type
    }
    set datachange [matfluid::checkclose $parent 0 0 0]

    if { $datachange != 0 } {
	set answer ""
	set answer [tk_dialogRAM $parent [= {Save Data?}] \
		"[= {To open the Solver Data Window, edited values have to be saved}].\n[= {Do you want to save them?}]" \               
		question 0 Yes No]
    } else {
	set answer 0
    }

    if { $answer == 0 } {
	set saveres ""
	if { $datachange != 0 } {
	    # Save edited data
	    set saveres [matfluid::checkclose $parent 1 0 0]
	}

	if { $saveres != "dataerror" } {
	    switch $type {
		"ransol" {
		    solvdatwin::init .gid $type ""
		}
		"heatrans" {
		    solvdatwin::init .gid $type ""
		}
		"advect" {
		    solvdatwin::init .gid $type $spcnlastsel
		}
		"ursolver" {
		    solvdatwin::init .gid $type $ursnlastsel
		}
	    }
	}
    }

    return 1
}




################################################################################
#
#   Species Manipulation
#   The following functions are used to manipulate the ADVECT Species Data
#   as well as the URSolver Variables Data.
#
################################################################################

proc matfluid::makespecie { nlist vlist type } {

    # Tdyn Settings
    variable edit_units

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable SpeciesUpdateCmd
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariablesUpdateCmd
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    # Combo boxes Units definitions
    # ADVECT
    lappend units_F2_Units "m\u00b2/s" "m\u00b2/h" "ft\u00b2/h"
    lappend units_F3_Units "1/s" "1/ds" "1/h"
    lappend units_F4_Units "C/s" "ppmC/s" "ppbC/s"
    lappend units_F5_Units "C/s" "ppmC/s" "ppbC/s"
    # URSOLVER
    lappend units_F1t_Phi_Units "1/m\u00b3" "1/cm\u00b3" "1/ft\u00b3"
    lappend units_F1c_Phi_Units "1/m\u00b3" "1/cm\u00b3" "1/ft\u00b3"
    lappend units_F2_Phi_Units "1/m.s" "1/cm.s" "1/in.s" "1/ft.s"
    lappend units_F3_Phi_Units "1/m\u00b3.s" "1/cm\u00b3.s" "1/in\u00b3.s"
    lappend units_F4_Phi_Units "U/m\u00b3.s" "U/cm\u00b3.s" "U/in\u00b3.s"

    # Grid Values Frame
    grid $vlist -row 0 -column 0 -sticky wnes -padx 10

    switch $type {
	"advect" {
	    # Specie Name
	    set l_sname [Label $vlist.l_sname -text "  [= {Species Name}]:"]
	    set e_sname [entry $vlist.e_sname -width 21 -textvariable matfluid::sname]
	}
	"ursolver" {
	    # Variable Name
	    set l_sname [Label $vlist.l_vname -text "[= {Variable Name}]:"]
	    set e_sname [entry $vlist.e_vname -width 21 -textvariable matfluid::ursname]
	}
    }

    # GRIDS
    # Label
    grid $l_sname -row 0 -column 0 -sticky en -pady 5
    # Entry
    grid $e_sname -row 0 -column 1 -columnspan 2 -sticky wn -pady 5

    # BINDS
    bind $e_sname <Return> ";
    matfluid::valsavespecie $nlist \"actual\" \"$type\";
    matfluid::displayspeciesnames $nlist \"$type\";
    "
    bind $e_sname <ButtonPress-3> "+matfluid::popupmenu $vlist \"\" \"\" \"\" 2 %X %Y"

    switch $type {
	"advect" {
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set cbmove 2
	    set SpeciesUpdateCmd ";
		matfluid::valsavespecie $nlist \"actual\" \"$type\";
		matfluid::displayspeciesnames $nlist \"$type\";
		"
	}
	"ursolver" {
	    set type_Valnames $URVariables_Valnames
	    set type_UValnames $URVariables_UValnames
	    set cbmove 1
	    set URVariablesUpdateCmd ";
		matfluid::valsavespecie $nlist \"actual\" \"$type\";
		matfluid::displayspeciesnames $nlist \"$type\";
		"
	}
    }

    # Creating widgets
    for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	set tvar [lindex [lindex $type_Valnames $i] 1]
	set tname [lindex [lindex $type_Valnames $i] 0]
	# Defining a Label
	set lvar l_$tvar
	set $lvar [Label $vlist.$lvar -text $tname]
	# Defining an Entry
	set evar e_$tvar
	set $evar [entry $vlist.$evar -width 11 -textvariable matfluid::$tvar]

	if { "F5_Spc" != $tvar } {
	    # GRIDS
	    # Label
	    grid [eval set temp $$lvar] -row [expr $i+1] -column 0 -sticky en -pady 1
	    # Entry
	    grid [eval set temp $$evar] -row [expr $i+1] -column 1 -sticky wn -pady 1
	    
	    # BINDS
	    bind [eval set temp $$evar] <Return> ";
		matfluid::valsavespecie $nlist \"actual\" \"$type\";
		matfluid::displayspeciesnames $nlist \"$type\";
		"
	    bind [eval set temp $$evar] <ButtonPress-3> "+matfluid::popupmenu $vlist matfluid::$tvar \"$tname\" \"\" 1 %X %Y"
	}
    }
    for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	set tuvar [lindex [lindex $type_UValnames $i] 1]
	set cuvar "units_"
	append cuvar [lindex [lindex $type_UValnames $i] 1]
	# Defining Units Combobox
	set cvar c_$tuvar
	set $cvar [ComboBox $vlist.$cvar -values [eval set temp $$cuvar] \
		-editable $edit_units -width 7 -textvariable matfluid::$tuvar -takefocus 1]

	if { "F5_Units" != $tuvar } {
	    # GRIDS
	    # Units Combo
	    grid [eval set temp $$cvar] -row [expr $i+$cbmove] -column 2 -sticky wn -padx 2
	}
    }

    grid columnconf $vlist 3 -weight 1

    return 1
}

proc matfluid::displayspecievalues { nlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    if { [lindex [$nlist curselection] 0] > -1 } {
	set sel [lindex [$nlist curselection] 0]
    } else {
	matfluid::updatespcselection $nlist 0 $type
	set sel [lindex [$nlist curselection] 0]
    }

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set type_Values $Species_Values
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set type_UValues $Species_UValues
	    set vartype_name "sname"
	    set vartype_lastsel "spcnlastsel"
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set type_Values $URVariables_Values
	    set type_Valnames $URVariables_Valnames
	    set type_UValues $URVariables_UValues
	    set type_UValnames $URVariables_UValnames
	    set vartype_name "ursname"
	    set vartype_lastsel "ursnlastsel"
	}
    }

    # Display Specie / Variable name and set last selection
    set $vartype_name [lindex $type_Names $sel]
    set $vartype_lastsel $sel

    set currvals [lindex $type_Values $sel]
    for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	# Entries Values
	set tvar [lindex [lindex $type_Valnames $i] 1]
	#        set $tvar [lindex $currvals $i]
	set $tvar [TUI_formatlistentries String2Field $currvals $i]
    }
    set currvals [lindex $type_UValues $sel]
    for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	# Combo boxes Units values
	set tvar [lindex [lindex $type_UValnames $i] 1]
	set $tvar [lindex $currvals $i]
    }

    return 1
}

proc matfluid::displayspeciesnames { nlist type } {

    variable Species_Names
    variable URVariables_Names

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	}
    }

    if { [lindex [$nlist curselection] 0] > -1 } {
	set savesel [lindex [$nlist curselection] 0]
    } else {
	set savesel ""
    }
    set p 0
    $nlist delete 0 end
    foreach n $type_Names {
	$nlist insert $p $n
	incr p
    }

    if { $savesel > -1 } {
	$nlist selection clear 0 end
	$nlist selection set $savesel $savesel
    }

    return 1
}

proc matfluid::updatespcselection { nlist setsel type } {

    variable Species_Names
    variable URVariables_Names
    variable spcnlastsel
    variable ursnlastsel
    variable sname
    variable ursname

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	}
    }

    if { $setsel < 0 || $setsel > [expr [llength $type_Names] -1] } {
	set setsel "end"
    }

    $nlist selection clear 0 end
    $nlist selection set $setsel $setsel

    switch $type {
	"advect" {
	    set spcnlastsel $setsel
	    set sname [lindex $type_Names $setsel]
	}
	"ursolver" {
	    set ursnlastsel $setsel
	    set ursname [lindex $type_Names $setsel]
	}
    }

    return 1
}

proc matfluid::valsavespecie { nlist sel type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    if { $sel == "actual" && [string length [$nlist curselection]] > 0 } {
	set sel [lindex [$nlist curselection] 0]
    } else {
	switch $type {
	    "advect" {
		set sel $spcnlastsel
	    }
	    "ursolver" {
		set sel $ursnlastsel
	    }
	}
    }

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set vartype_Names "Species_Names"
	    set type_Values $Species_Values
	    set vartype_Values "Species_Values"
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set type_UValues $Species_UValues
	    set vartype_UValues "Species_UValues"
	    set type_name $sname
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set vartype_Names "URVariables_Names"
	    set type_Values $URVariables_Values
	    set vartype_Values "URVariables_Values"
	    set type_Valnames $URVariables_Valnames
	    set type_UValnames $URVariables_UValnames
	    set type_UValues $URVariables_UValues
	    set vartype_UValues "URVariables_UValues"
	    set type_name $ursname
	}
    }

    if { $sel > -1 && !($sel > [expr [llength $type_Values] -1]) } {

	# Saving Specie / UR Variable Name
	set $vartype_Names [lreplace $type_Names $sel $sel [string trim [DWSpace2Under $type_name]]]

	# Saving Specie / UR Variable values
	set newvals ""
	for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	    set tvar [lindex [lindex $type_Valnames $i] 1]
	    set type [lindex [lindex $type_Valnames $i] 2]
	    lappend newvals [TUI_cleanvalue [eval set temp $$tvar] $type]
	    #if { [lindex [lindex $type_Valnames $i] 2] == "func" } {
	    #    lappend newvals [TdynCleanString [eval set temp $$tvar]]
	    #} else {
	    #    lappend newvals [DWSpace2Under [eval set temp $$tvar]]
	    #}
	}
	set $vartype_Values [lreplace $type_Values $sel $sel $newvals]
	# Saving Specie / UR Variable Units values
	set newvals ""
	for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	    set tvar [lindex [lindex $type_UValnames $i] 1]
	    lappend newvals [TUI_cleanvalue [eval set temp $$tvar] units]
	    #regsub -all { } [eval set temp $$tvar] {} ttv
	    #lappend newvals $ttv
	}
	set $vartype_UValues [lreplace $type_UValues $sel $sel $newvals]
    }

    return 1
}

proc matfluid::addspecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable spcnlastsel
    variable sname
    variable Species_Rebuild
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable ursnlastsel
    variable ursname
    variable URVariables_Rebuild

    matfluid::valsavespecie $nlist "last" $type

    set sr [$nlist curselection]

    switch $type {
	"advect" {
	    # Check if any species is selected
	    if { $sr == "" } {
		set sr $spcnlastsel
	    }
	    # Update Species Values
	    lappend Species_Values [lindex $Species_Values $sr]
	    # Update Species Units Values
	    lappend Species_UValues [lindex $Species_UValues $sr]
	    
	    # Update Species Names
	    set nn "Species"
	    set ns [expr $sr + 1]
	    lappend Species_Rebuild "a" $sr
	    set nt $nn$ns
	    while { [lsearch $Species_Names $nt]!=-1 } {
		set ns [expr $ns + 1]
		set nt $nn$ns
	    }
	    set nt $nn$ns
	    lappend Species_Names $nt
	    $nlist insert end $nt
	}
	"ursolver" {
	    # Check if any variable is selected
	    if { $sr == "" } {
		set sr $ursnlastsel
	    }
	    # Update URSolver Variables Values
	    lappend URVariables_Values [lindex $URVariables_Values $sr]
	    # Update URSolver Units Values
	    lappend URVariables_UValues [lindex $URVariables_UValues $sr]
	    
	    # Update URSolver Variables Names
	    set nn "Variable"
	    set ns [expr $sr + 1]
	    lappend URVariables_Rebuild "a" $sr
	    set nt $nn$ns
	    while { [lsearch $URVariables_Names $nt]!=-1 } {
		set ns [expr $ns + 1]
		set nt $nn$ns
	    }
	    set nt $nn$ns
	    lappend URVariables_Names $nt
	    $nlist insert end $nt
	}        
    }
    matfluid::updatespcselection $nlist "end" $type
    matfluid::displayspecievalues $nlist $type
}

proc matfluid::removespecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable spcnlastsel
    variable sname
    variable spcdelerrwarn
    variable spcdellasterrwarn
    variable Species_Rebuild
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable ursnlastsel
    variable ursname
    variable ursdelerrwarn
    variable ursdellasterrwarn
    variable URVariables_Rebuild

    set sr [$nlist curselection]

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set vartype_Names Species_Names
	    set type_Values $Species_Values
	    set vartype_Values Species_Values
	    set type_UValues $Species_UValues
	    set vartype_UValues Species_UValues
	    set displaykind "Specie"
	    set lastcheckvar $spcdellasterrwarn
	    set var_lastcheckvar spcdellasterrwarn
	    set noselvar $spcdelerrwarn
	    set var_noselvar spcdelerrwarn
	    set type_Rebuild Species_Rebuild
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set vartype_Names URVariables_Names
	    set type_Values $URVariables_Values
	    set vartype_Values URVariables_Values
	    set type_UValues $URVariables_UValues
	    set vartype_UValues URVariables_UValues
	    set displaykind "Variable"
	    set lastcheckvar $ursdellasterrwarn
	    set var_lastcheckvar ursdellasterrwarn
	    set noselvar $ursdelerrwarn
	    set var_noselvar ursdelerrwarn
	    set type_Rebuild URVariables_Rebuild
	}
    }

    if { $sr > -1 && $sr < [llength $type_Names] } {
	# Check if the deleted Specie/Variable is not the last one
	if { [llength $type_Names] < 2 } {
	    if { $lastcheckvar == 1 } {
		return
	    } else {
		WarnWin "[= {There must be at least one %s defined} $displaykind]."
		set $var_lastcheckvar 1
	    }
	} else {
	    lappend $type_Rebuild "r" $sr
	    # Updating Species/URVariables Names
	    set $vartype_Names [lreplace $type_Names [lindex $sr 0] [lindex $sr end]]
	    # Updating Species/URVariables Values
	    set $vartype_Values [lreplace $type_Values [lindex $sr 0] [lindex $sr end]]
	    # Updating Species/URVariables Units Values
	    set $vartype_UValues [lreplace $type_UValues [lindex $sr 0] [lindex $sr end]]

	    matfluid::displayspeciesnames $nlist $type
	    matfluid::updatespcselection $nlist "end" $type
	}
    } else {
	if { $noselvar == 1 } {
	    return
	} else {
	    set temp "[= {Please select a}] "
	    append temp $displaykind "[= {(s) you wish to remove}]."
	    WarnWin $temp
	    set $noselvar 1
	}
    }

    matfluid::displayspecievalues $nlist $type
}




################################################################################
#
#   PopUp Menus
#
################################################################################

proc matfluid::popupmenu { parent {field ""} {fieldname ""} tags content x y } {

    variable predefined_ff

    catch { 
	destroy $parent.menu
    }

    set menu [menu $parent.menu -type normal]

    switch $content {
	1 {
	    # $menu add command -label [= {Help}] -command matfluid::definehelptagshere-stillmissing
	    if { [string length $field] > 0 } {
		eval set fval $$field
	    }
	    $menu add command -label [= {Define Function}] \
		    -command "FEditor::init $parent \"$predefined_ff\" $field [list $fieldname] [list $fval] \"\" \"\""
	}
    }
    
    tk_popup $menu $x $y

    return 1
}



################################################################################
#
#   GiD Values Manipulation - GET Values, TSAVE and DUMP Values
#   - GETVALUES reads GiD values and puts them to the variables
#   - CHECKCLOSE goes through all variables and checks for changes. If changes 
#     are found, asks the user SAVE YES or NO. If YES, runs DUMP procedure.
#     This procedure also checks the temporal space of Initial Conditions.
#   - TICSAVE saves values from Initial Conditions entries to temporal variables
#     from where in case of accept saving it saves the values tom GiD memory.
#   - DUMP saves values of entries to GiD memory and in case of initial condition
#
################################################################################

proc matfluid::getvalues { } {

    global ProgramName

    variable show_advect
    variable show_ursolver

    variable Fluid_Questions
    variable Material_Name
    variable ValList

    variable Species_Names
    variable Species_Values
    variable Species_UValues
    variable Species_Valnames
    variable Species_UValnames

    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Valnames
    variable URVariables_UValnames

    # Read all materials values
    set ValList [.central.s info materials $Material_Name]

    # Index of values started from 1 and not from 0 because the 
    # first values is number of questions.
    set i 0
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	variable $itt
	set pos [lsearch -regexp $ValList [eval set nothing ^${itt}(\$|\#CB|\:)]]
	if { $pos != -1 } {
	    #set tv [join [lindex $ValList [expr $pos + 1]]]
	    set tv "[lindex $ValList [expr $pos + 1]]"
	    if { [lindex $item 1] == "unit" } {
		set tv [TUI_formatunits Nums2Unicode $tv 0]
	    }
	    set $itt $tv
	}
	incr i
    }

    if { $show_advect == 1 } {
	# Read Species Names from Problem Data
	set temp "[lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]"
	set Species_Names [TUI_formatlistvalues String2List $temp]
	set Species_Values [TUI_formatspeciesvalues String2List $Species_Valnames $Species_Properties]
	set Species_UValues [TUI_formatspeciesvalues String2List $Species_UValnames $Spc_Units]

	# Format Species Units Lists for Mat File Format
	set ltemp ""
	foreach litem $Species_UValues {
	    set temp ""
	    foreach item $litem {
		lappend temp [TUI_formatunits Nums2Unicode $item 1]
	    }
	    lappend ltemp [string trim $temp]
	}
	set Species_UValues $ltemp
    }

    if { $show_ursolver == 1 } {
	# Read URSolver Variables Names from Problem Data
	set temp "[lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]"
	set URVariables_Names [TUI_formatlistvalues String2List $temp]
	set URVariables_Values [TUI_formatspeciesvalues String2List $URVariables_Valnames $Variables_Properties]
	set URVariables_UValues [TUI_formatspeciesvalues String2List $URVariables_UValnames $URV_Units]

	# Format URVariables Units Lists for Mat File Format
	set ltemp ""
	foreach litem $URVariables_UValues {
	    set temp ""
	    foreach item $litem {
		lappend temp [TUI_formatunits Nums2Unicode $item 1]
	    }
	    lappend ltemp [string trim $temp]
	}
	set URVariables_UValues $ltemp
    }

    return 1
}

proc matfluid::checkclose { win {dumpit 1} {askdump 1} {close 1} } {

    global tkPriv tcl_platform

    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver

    variable Material_Name
    variable ValList

    variable Fluid_Questions

    variable Species_Names
    variable Species_Values
    variable Species_UValues
    variable Species_Valnames
    variable Species_UValnames
    variable snlist

    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Valnames
    variable URVariables_UValnames
    variable ursnlist

    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	variable $itt
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Unicode2Nums [eval set nothing $$itt] 1]
	}
	#if { [lindex $item 1] == "func" } {
	#    regsub -all { } [eval set nothing $$itt] {} $itt
	#}
    }

    if { $show_advect == 1 } {
	
	set Species_UValues_Temp $Species_UValues

	matfluid::valsavespecie $snlist "actual" "advect"

	set ltemp ""
	foreach litem $Species_UValues {
	    set temp ""
	    foreach item $litem {
		lappend temp [TUI_formatunits Unicode2Nums $item 1]
	    }
	    lappend ltemp [string trim $temp]
	}
	set Species_UValues $ltemp

	set Species_Properties [TUI_formatspeciesvalues List2String $Species_Valnames $Species_Values]
	set Spc_Units [TUI_formatspeciesvalues List2String $Species_UValnames $Species_UValues]
    }

    if { $show_ursolver == 1 } {
	
	set URVariables_UValues_Temp $URVariables_UValues

	matfluid::valsavespecie $ursnlist "actual" "ursolver"

	set ltemp ""
	foreach litem $URVariables_UValues {
	    set temp ""
	    foreach item $litem {
		lappend temp [TUI_formatunits Unicode2Nums $item 1]
	    }
	    lappend ltemp [string trim $temp]
	}
	set URVariables_UValues $ltemp

	set Variables_Properties [TUI_formatspeciesvalues List2String $URVariables_Valnames $URVariables_Values]
	set URV_Units [TUI_formatspeciesvalues List2String $URVariables_UValnames $URVariables_UValues]
    }

    set valueschange 0
    set wtsave 0
    set vi 1
    set usr_err ""
    set num_usr_err 0
    set uerrpg_ran 0
    set uerrpg_htr 0
    set uerrpg_adv 0
    set uerrpg_urs 0

    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	set $itt [string trim [eval set nothing $$itt]]
	eval set cv $$itt
	set pos [lsearch -regexp $ValList [eval set nothing ^${itt}(\$|\#CB|\:)]]
	if { $pos != -1 } {
	    #set tv [join [lindex $ValList [expr $pos + 1]]]
	    set tv [lindex $ValList [expr $pos + 1]]
	    if { ![TUI_comparelists $cv $tv] } {
		set valueschange 1
		if { $itt == "Species_Properties" } {
		    set spc_pos 0
		    foreach item $Species_Values {
		        set tspn [lindex $Species_Names $spc_pos]
		        incr spc_pos
		        set valnamepos 0
		        foreach itt $item {
		            set valname [lindex $Species_Valnames $valnamepos]
		            incr valnamepos
		            if { [lindex $valname 3] == 1 } {
		                set errcheck [TUI_valuecheck $itt [lindex $valname 2] 0 [lindex $valname 4]]
		            } else {
		                set errcheck "ok"
		            }
		            if { $errcheck != "ok"} {
		                incr num_usr_err
		                if { $uerrpg_adv != 1 } {
		                    append usr_err "\n[= {ADVECT PAGE}]\n"
		                    set uerrpg_adv 1
		                }
		                append usr_err "$num_usr_err. " "$tspn; " [lindex $valname 0] ": $errcheck\n"
		            }
		        }
		    }
		} elseif { $itt == "Variables_Properties" } {
		    set spc_pos 0
		    foreach item $URVariables_Values {
		        set tspn [lindex $URVariables_Names $spc_pos]
		        incr spc_pos
		        set valnamepos 0
		        foreach itt $item {
		            set valname [lindex $URVariables_Valnames $valnamepos]
		            incr valnamepos
		            if { [lindex $valname 3] == 1 } {
		                set errcheck [TUI_valuecheck $itt [lindex $valname 2] 0  [lindex $valname 4]]
		            } else {
		                set errcheck "ok"
		            }
		            if { $errcheck != "ok"} {
		                incr num_usr_err
		                if { $uerrpg_urs != 1 } {
		                    append usr_err "\n[= {URSOLVER PAGE}]\n"
		                    set uerrpg_urs 1
		                }
		                append usr_err "$num_usr_err. " "$tspn; " [lindex $valname 0] ": $errcheck\n"
		            }
		        }
		    }
		} else {
		    if { [lindex $item 2] == 1 } {
		        set errcheck [TUI_valuecheck [eval set nothing $$itt] [lindex $item 1] 0 [lindex $item 4]]
		    } else {
		        set errcheck "ok"
		    }
		    if { $errcheck != "ok" && [lindex $item 2] == 1} {
		        incr num_usr_err
		        switch "[lindex $item 3]" {
		            "ransol" {
		                if { $uerrpg_ran != 1 } {
		                    append usr_err "\n[= {RANSOL PAGE}]\n"
		                    set uerrpg_ran 1
		                }
		            }
		            "heatrans" {
		                if { $uerrpg_htr != 1 } {
		                    append usr_err "\n[= {HEATRANS PAGE}]\n"
		                    set uerrpg_htr 1
		                }
		            }
		        }
		        append usr_err "$num_usr_err. " [DWUnder2Space $itt] ": $errcheck\n"
		    }
		}
	    }
	}
	incr vi
    }

    if { $show_advect == 1 } {
	# Check also Species Names
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]]
	set temp [lreplace $temp 0 1]
	if { $temp != $Species_Names } {
	    set valueschange 1
	    set warnedspcnames ""
	    foreach item $Species_Names {
		if { [llength [lsearch -all $Species_Names $item]] > 1 && [lsearch -exact $warnedspcnames $item] == -1} {
		    lappend warnedspcnames $item
		    incr num_usr_err
		    if { $uerrpg_adv != 1 } {
		        append usr_err "\n[= {ADVECT PAGE}]\n"
		        set uerrpg_adv 1
		    }
		    append usr_err "[= {%s. Specie Name: '%s' is included more than once in the Species List} $num_usr_err $item].\n"
		    append usr_err "[= {Species have to have unique names}].\n"
		}
		set errcheck [TUI_valuecheck $item string 0]
		if { $errcheck != "ok"} {
		    incr num_usr_err
		    if { $uerrpg_adv != 1 } {
		        append usr_err "\n[= {ADVECT PAGE}]\n"
		        set uerrpg_adv 1
		    }
		    append usr_err "[= {%s. Specie Name: '%s' is not correct. } $num_usr_err $item]\n"
		    append usr_err "[= {Species names can only include alphabetical characters, numbers and underscore}].\n"
		}
	    }
	}
    }

    if { $show_ursolver == 1 } {
	# Check also URSolver Variables Names
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]]
	set temp [lreplace $temp 0 1]
	if { $temp != $URVariables_Names } {
	    set valueschange 1
	    set warnedurvnames ""
	    foreach item $URVariables_Names {
		if { [llength [lsearch -all $URVariables_Names $item]] > 1 && [lsearch -exact $warnedurvnames $item] == -1} {
		    lappend warnedurvnames $item
		    incr num_usr_err
		    if { $uerrpg_ran != 1 } {
		        append usr_err "\n[= {URSOLVER PAGE}]\n"
		        set uerrpg_urs 1
		    }
		    append usr_err "[= {%s. URSolver Variable Name: '%s' is included more than once in the Variable List} $num_usr_err $item].\n"
		    append usr_err "[= {Variables have to have unique names}].\n"
		}
		set errcheck [TUI_valuecheck $item string 0]
		if { $errcheck != "ok"} {
		    incr num_usr_err
		    if { $uerrpg_urs != 1 } {
		        append usr_err "\n[= {URSOLVER PAGE}]\n"
		        set uerrpg_urs 1
		    }
		    append usr_err "[= {%s. URSolver Variable Name: '%s' is not correct. } $num_usr_err $item]\n"
		    append usr_err "[= {URSolver Variables names can only include alphabetical characters, numbers and underscore}]\n"
		}
	    }
	}
    }

    # Units are formated back to unicode so that they are not seen 
    # in MAT File format in fields while asking if changes should be saved. 
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Nums2Unicode [eval set nothing $$itt] 1]
	}
    }
    if { $show_advect == 1 } {  
	set Species_UValues $Species_UValues_Temp
    }
    if { $show_ursolver == 1 } { 
	set URVariables_UValues $URVariables_UValues_Temp
    }

    # Ahora se continua con el resto de operaciones
    if { $valueschange == 1 } {
	if { $askdump == 0 } {
	    set answer 0
	} else {
	    set answer [tk_dialogRAM $win [= {Material: '%s'} $Material_Name] \
		    "[= {Material data has changed}].\n[= {Do you want to save changes?}]" question 0 Yes No]
	}

	if { $answer == 0 && $dumpit == 1} {
	    set wtsave 1
	    if { $num_usr_err > 0 } {
		if { $num_usr_err > 1 } {
		    set err_display [= {The following %s error} $num_usr_err]
		    append err_display [= {(s) were found in the data input:\n}]
		} else {
		    set err_display [= {The following error was found in the data input:\n}]
		}
		append err_display $usr_err
		WarnWin $err_display
		set close 0
	    } else {
		matfluid::dump
	    }
	}
    }

    if { $close == 1 } {
	destroy $win
	return $valueschange
    } else {
	if { $num_usr_err == 0 } {
	    return $valueschange
	} else {
	    if { $wtsave == 1 } {
		return "dataerror"
	    } else {
		return "nosave.dataerror"
	    }
	}
    }

}

# Updates material data to file and ValList
proc matfluid::dump { } {

    variable show_advect
    variable show_ursolver

    variable Material_Name
    variable ValList

    variable Species_Names
    variable URVariables_Names

    variable Species_Rebuild
    variable URVariables_Rebuild

    variable Fluid_Questions
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	variable $itt
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Unicode2Nums [eval set nothing $$itt] 1]
	}
    }

    # Replacing values in ValList
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	set pos [lsearch -regexp $ValList [eval set nothing ^${itt}(\$|\#CB|\:)]]
	if { $pos != -1 } {
	    set pos [expr $pos + 1]
	    set ValList [lreplace $ValList $pos $pos [eval set nothing $$itt]]
	}
    }

    # Saving values in GID data structures
    # TUI_setnewmatvalues $Material_Name [string trim [matfluid::getcurrentvalset]]
    matfluid::setnewmatvalues $Material_Name

    if { $show_advect == 1 } {
	TUI_savespeciesnames "Species_Names:(SpecieName)" $Species_Names
	TUI_synchrospecies "advect" $Species_Rebuild $Material_Name
    }
    if { $show_ursolver == 1 } {
	TUI_savespeciesnames "URVariables_Names:(VariableName)" $URVariables_Names
	TUI_synchrospecies "ursolver" $URVariables_Rebuild $Material_Name
    }

    # Units are formated back to unicode.
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Nums2Unicode [eval set nothing $$itt] 1]
	}
    }
    # Actualizamos la variable de control de cambios en GiD
    GIDModifiedFileFlag Set
    return 1
}

proc matfluid::setnewmatvalues { material } {
    variable ValList
    set vals [lrange $ValList 1 end]
    set justvals ""
    foreach {q v} $vals {
	lappend justvals $v
    }
    GiD_ModifyData materials $material $justvals
}

proc matfluid::getcurrentvalset { } {
    
    variable ValList

    set vals [lrange $ValList 1 end]
    set JVL ""
    foreach {q v} $vals {
	append JVL $v " "
    }

    return $JVL
}