namespace eval matsolid {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_ransol [IsVarOnTdynDefaults show_ransol 1]
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    variable show_advect [IsVarOnTdynDefaults show_advect 1]
    variable show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    variable edit_units [IsVarOnTdynDefaults edit_units 0]
    if { $ProgramName == "Tdyn2D" } {
	variable show_ramsolid 0
    } elseif { $ProgramName == "Tdyn3D" } {
	variable show_ramsolid [IsVarOnTdynDefaults show_ramsolid 1]
    }

    # Top Frame Buttons (Modify, New and Erase material)
    variable button_modify 
    variable button_new
    variable button_erase

    # Selected material name
    variable Material_Name
    variable ValList

    # Advect Species variables
    variable Species_Rebuild 

    # List of available Solid Material Questions in Mat File.
    # Variables names are local script variables and in some 
    # cases do not correspond the MAT file QUESTION name
    if { $ProgramName == "Tdyn2D" } {
	  variable Solid_Questions [list \
		[list Fluid_Density func 1 ransol ""] \
		[list Fluid_Density_Units unit 1 ransol density] \
		[list Fluid_Viscosity func 1 ransol ""] \
		[list Fluid_Viscosity_Units unit 1 ransol viscosity] \
		[list Darcys_Coefficient_D11 func 1 ransol ""] \
		[list Darcys_Coefficient_D12 func 1 ransol ""] \
		[list Darcys_Coefficient_D22 func 1 ransol ""] \
		[list Darcys_Coefficient_Units unit 1 permeability_inverse] \
		[list AccOX_Field func 1 ransol ""] \
		[list AccOY_Field func 1 ransol ""] \
		[list Acceleration_Units unit 1 ransol acceleration] \
		[list Density func 1 heatrans ""] \
		[list Density_Units unit 1 heatrans density] \
		[list Specific_Heat_C func 1 heatrans ""] \
		[list C_Units unit 1 heatrans heat] \
		[list Thermal_Conductivity_k11 func 1 heatrans ""] \
		[list Thermal_Conductivity_k12 func 1 heatrans ""] \
		[list Thermal_Conductivity_k22 func 1 heatrans ""] \
		[list k_Units unit 1 heatrans conductivity] \
		[list Floatability func 1 heatrans ""] \
		[list Heat_Source_Field func 1 heatrans ""] \
		[list Heat_Source_Units unit 1 heatrans heat] \
		[list Heat_Reaction_Field func 1 heatrans ""] \
		[list Heat_Reaction_Units unit 1 heatrans heat] \
		[list Species_Properties nocheck 0 advect ""] \
		[list Spc_Units nocheck 0 advect ""] \
		[list Variables_Properties nocheck 0 ursolver ""] \
		[list URV_Units nocheck 0 ursolver ""] \
		]
    } elseif { $ProgramName == "Tdyn3D" } {
	  variable Solid_Questions [list \
		[list Fluid_Density func 1 ransol ""] \
		[list Fluid_Density_Units unit 1 ransol density] \
		[list Fluid_Viscosity func 1 ransol ""] \
		[list Fluid_Viscosity_Units unit 1 ransol viscosity] \
		[list Darcys_Coefficient_D11 func 1 ransol ""] \
		[list Darcys_Coefficient_D12 func 1 ransol ""] \
		[list Darcys_Coefficient_D13 func 1 ransol ""] \
		[list Darcys_Coefficient_D22 func 1 ransol ""] \
		[list Darcys_Coefficient_D23 func 1 ransol ""] \
		[list Darcys_Coefficient_D33 func 1 ransol ""] \
		[list Darcys_Coefficient_Units unit 1 permeability_inverse] \
		[list AccOX_Field func 1 ransol] \
		[list AccOY_Field func 1 ransol] \
		[list AccOZ_Field func 1 ransol] \
		[list Acceleration_Units unit 1 ransol acceleration] \
		[list Density func 1 heatrans ""] \
		[list Density_Units unit 1 heatrans density] \
		[list Specific_Heat_C func 1 heatrans ""] \
		[list C_Units unit 1 heatrans heat] \
		[list Thermal_Conductivity_k11 func 1 heatrans ""] \
		[list Thermal_Conductivity_k12 func 1 heatrans ""] \
		[list Thermal_Conductivity_k13 func 1 heatrans ""] \
		[list Thermal_Conductivity_k22 func 1 heatrans ""] \
		[list Thermal_Conductivity_k23 func 1 heatrans ""] \
		[list Thermal_Conductivity_k33 func 1 heatrans ""] \
		[list k_Units unit 1 heatrans conductivity] \
		[list Floatability func 1 heatrans ""] \
		[list Heat_Source_Field func 1 heatrans ""] \
		[list Heat_Source_Units unit 1 heatrans heat] \
		[list Heat_Reaction_Field func 1 heatrans ""] \
		[list Heat_Reaction_Units unit 1 heatrans heat] \
		[list Species_Properties nocheck 0 advect ""] \
		[list Spc_Units nocheck 0 advect ""] \
		[list Variables_Properties nocheck 0 ursolver ""] \
		[list URV_Units nocheck 0 ursolver ""] \
		[list E float 0 ramsolid ""] \
		[list nu float 0 ramsolid ""] \
		[list Specific_weight float 0 ramsolid ""] \
		[list Mechanical_strain func 1 ramsolid ""] \
		[list Ram_Units nocheck 0 ramsolid ""] \
		]
    }
    foreach item $Solid_Questions {
	variable [lindex $item 0]
    }

    # Advect Species variables
    # Species_Properties:{F1_Spc,F2xx_Spc,F2xy_Spc,F2yy_Spc,F3_Spc,F4_Spc}
    # Species_Properties:(F1_Spc,F2xx_Spc,F2xy_Spc,F2xz_Spc,F2yy_Spc,F2yz_Spc,F2zz_Spc,F3_Spc,F4_Spc)
    # Spc_Units:(F2_Spc_Units,F3_Spc_Units,F4_Spc_Units)
    variable Species_Names
    variable Species_Values
    variable SpeciesUpdateCmd ""
    variable Species_UValues
    switch $ProgramName {
	"Tdyn2D" {
	    variable Species_Valnames \
		    [list \
		    [list "f1 " F1_Spc func 1 ""] \
		    [list "f2xx " F2xx_Spc func 1 ""] \
		    [list "f2xy " F2xy_Spc func 1 ""] \
		    [list "f2yy " F2yy_Spc func 1 ""] \
		    [list "f3 " F3_Spc func 1 ""] \
		    [list "f4 " F4_Spc func 1 ""] \
		    ]
	}
	"Tdyn3D" {
	    variable Species_Valnames \
		    [list \
		    [list "f1 " F1_Spc func 1 ""] \
		    [list "f2xx " F2xx_Spc func 1 ""] \
		    [list "f2xy " F2xy_Spc func 1 ""] \
		    [list "f2xz " F2xz_Spc func 1 ""] \
		    [list "f2yy " F2yy_Spc func 1 ""] \
		    [list "f2yz " F2yz_Spc func 1 ""] \
		    [list "f2zz " F2zz_Spc func 1 ""] \
		    [list "f3 " F3_Spc func 1 ""] \
		    [list "f4 " F4_Spc func 1 ""] \
		    ]
	}
    }
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    variable Species_UValnames \
	    [list \
	    [list "F2 Units" F2_Units unit 1 concentration] \
	    [list "F3 Units" F3_Units unit 1 concentration] \
	    [list "F4 Units" F4_Units unit 1 concentration] \
	    ]
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    # Advect Species support variables
    variable spcdelerrwarn 0
    variable spcdellasterrwarn 0
    variable spcnlastsel -1
    variable snlist
    variable sname

    # URSolver Species variables
    variable URVariables_Rebuild

    # URSolver Species variables
    # UR_Variables:(F1_Phi,F2xx_Phi,F2xy_Phi,F2yy_Phi,F3_Phi,F4_Phi)
    # UR_Variables:(F1_Phi,F2xx_Phi,F2xy_Phi,F2xz_Phi,F2yy_Phi,F2yz_Phi,F2zz_Phi,F3_Phi,F4_Phi)
    # URV_Units:(F1_Phi_Units,F2_Phi_Units,F3_Phi_Units,F4_Phi_Units)
    variable URVariables_Names
    variable URVariables_Values
    variable URVariablesUpdateCmd ""
    variable URVariables_UValues
    switch $ProgramName {
	"Tdyn2D" {
	    variable URVariables_Valnames \
		    [list \
		    [list "f1 " F1_Phi func 1] \
		    [list "f2xx " F2xx_Phi func 1] \
		    [list "f2xy " F2xy_Phi func 1] \
		    [list "f2yy " F2yy_Phi func 1] \
		    [list "f3 " F3_Phi func 1] \
		    [list "f4 " F4_Phi func 1] \
		    ]
	}
	"Tdyn3D" {
	    variable URVariables_Valnames \
		    [list \
		    [list "f1 " F1_Phi func 1] \
		    [list "f2xx " F2xx_Phi func 1] \
		    [list "f2xy " F2xy_Phi func 1] \
		    [list "f2xz " F2xz_Phi func 1] \
		    [list "f2yy " F2yy_Phi func 1] \
		    [list "f2yz " F2yz_Phi func 1] \
		    [list "f2zz " F2zz_Phi func 1] \
		    [list "f3 " F3_Phi func 1] \
		    [list "f4 " F4_Phi func 1] \
		    ]
	}
    }
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    variable URVariables_UValnames \
	    [list \
	    [list "F1 Phi Units" F1_Phi_Units unit 1 concentration] \
	    [list "F2 Phi Units" F2_Phi_Units unit 1 concentration] \
	    [list "F3 Phi Units" F3_Phi_Units unit 1 concentration] \
	    [list "F4 Phi Units" F4_Phi_Units unit 1 concentration] \
	    ]
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    # URSolver Species support variables
    variable ursdelerrwarn 0
    variable ursdellasterrwarn 0
    variable ursnlastsel -1
    variable ursnlist
    variable ursname


    # Function Editor variables lists
    switch $ProgramName {
	"Tdyn2D" {
	    variable ransol_ff \
		    [ list \
		    {matsolid::Fluid_Density} [= {Fluid Density}] \
		    {matsolid::Fluid_Viscosity} [= {Fluid Viscosity}] \
		    {matsolid::Darcys_Coefficient_D11} [= {Darcy's Coefficient D11}] \
		    {matsolid::Darcys_Coefficient_D12} [= {Darcy's Coefficient D12 & 21}] \
		    {matsolid::Darcys_Coefficient_D22} [= {Darcy's Coefficient D22}] \
		    {matsolid::AccOX_Field} [= {Acceleration Field OX}] \
		    {matsolid::AccOY_Field} [= {Acceleration Field OY}] \
		    ]
	    variable heatrans_ff \
		    [ list \
		    {matsolid::Density} [= {Density}] \
		    {matsolid::Specific_Heat_C} [= {Specific Heat C}] \
		    {matsolid::Thermal_Conductivity_k11} [= {Thermal Conductivity k11}] \
		    {matsolid::Thermal_Conductivity_k12} [= {Thermal Conductivity k12 & 21}] \
		    {matsolid::Thermal_Conductivity_k22} [= {Thermal Conductivity k22}] \
		    {matsolid::Floatability} [= {Floatability}] \
		    {matsolid::Heat_Source_Field} [= {Heat Source Field}] \
		    {matsolid::Heat_Reaction_Field} [= {Heat Reaction Field}] \
		    ]
	    variable advect_ff \
		    [ list \
		    {matsolid::F1_Spc} {f1} \
		    {matsolid::F2xx_Spc} {f2xx} \
		    {matsolid::F2xy_Spc} {f2xy} \
		    {matsolid::F2yy_Spc} {f2yy} \
		    {matsolid::F3_Spc} {f3} \
		    {matsolid::F4_Spc} {f4} \
		    ]
	    variable ursolver_ff \
		    [ list \
		    {matsolid::F1_Phi} {f1t} \
		    {matsolid::F2xx_Phi} {f2xx} \
		    {matsolid::F2xy_Phi} {f2xy} \
		    {matsolid::F2yy_Phi} {f2yy} \
		    {matsolid::F3_Phi} {f3} \
		    {matsolid::F4_Phi} {f4} \
		    ]
	}
	"Tdyn3D" {
	    variable ransol_ff \
		    [ list \
		    {matsolid::Fluid_Density} [= {Fluid Density}] \
		    {matsolid::Fluid_Viscosity} [= {Fluid Viscosity}] \
		    {matsolid::Darcys_Coefficient_D11} [= {Darcy's Coefficient D11}] \
		    {matsolid::Darcys_Coefficient_D12} [= {Darcy's Coefficient D12 & 21}] \
		    {matsolid::Darcys_Coefficient_D13} [= {Darcy's Coefficient D13 & 31}] \
		    {matsolid::Darcys_Coefficient_D22} [= {Darcy's Coefficient D22}] \
		    {matsolid::Darcys_Coefficient_D23} [= {Darcy's Coefficient D23 & 32}] \
		    {matsolid::Darcys_Coefficient_D33} [= {Darcy's Coefficient D33}] \
		    {matsolid::AccOX_Field} [= {Acceleration Field OX}] \
		    {matsolid::AccOY_Field} [= {Acceleration Field OY}] \
		    {matsolid::AccOZ_Field} [= {Acceleration Field OZ}] \
		    ]
	    variable heatrans_ff \
		    [ list \
		    {matsolid::Density} [= {Density}] \
		    {matsolid::Specific_Heat_C} [= {Specific Heat C}] \
		    {matsolid::Thermal_Conductivity_k11} [= {Thermal Conductivity k11}] \
		    {matsolid::Thermal_Conductivity_k12} [= {Thermal Conductivity k12 & k21}] \
		    {matsolid::Thermal_Conductivity_k13} [= {Thermal Conductivity k13 & k31}] \
		    {matsolid::Thermal_Conductivity_k22} [= {Thermal Conductivity k22}] \
		    {matsolid::Thermal_Conductivity_k23} [= {Thermal Conductivity k23 & k32}] \
		    {matsolid::Thermal_Conductivity_k33} [= {Thermal Conductivity k33}] \
		    {matsolid::Floatability} [= {Floatability}] \
		    {matsolid::Heat_Source_Field} [= {Heat Source Field}] \
		    {matsolid::Heat_Reaction_Field} [= {Heat Reaction Field}] \
		    ]
	    variable advect_ff \
		    [ list \
		    {matsolid::F1_Spc} {f1} \
		    {matsolid::F2xx_Spc} {f2xx} \
		    {matsolid::F2xy_Spc} {f2xy} \
		    {matsolid::F2xz_Spc} {f2yz} \
		    {matsolid::F2yy_Spc} {f2yy} \
		    {matsolid::F2yz_Spc} {f2yz} \
		    {matsolid::F2zz_Spc} {f2zz} \
		    {matsolid::F3_Spc} {f3} \
		    {matsolid::F4_Spc} {f4} \
		    ]
	    variable ursolver_ff \
		    [ list \
		    {matsolid::F1_Phi} {f1t} \
		    {matsolid::F2xx_Phi} {f2xx} \
		    {matsolid::F2xy_Phi} {f2xy} \
		    {matsolid::F2yz_Phi} {f2yz} \
		    {matsolid::F2yy_Phi} {f2zz} \
		    {matsolid::F2yz_Phi} {f2yz} \
		    {matsolid::F2zz_Phi} {f2zz} \
		    {matsolid::F3_Phi} {f3} \
		    {matsolid::F4_Phi} {f4} \
		    ]
	}
    }
    variable ramsolid_ff \
	    [ list \
	    {matsolid::Mechanical_strain} {Mechanical strain} \
	    ]
    variable solid_variables "x y z t it rand dt exp(1) pi vx vy vz pr tm dn sp# ph#"
    variable example_solid_variables "x y z t it tm rand dt exp(1) pi"
    variable solid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
    variable predefined_ff [list $solid_variables $example_solid_variables $solid_operators]
}



################################################################################
#
#   Initialize Solid Material Window
#
################################################################################

proc matsolid::init { BASE selmatname } {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable edit_units
    variable show_ramsolid

    # Selected Material Name
    variable Material_Name $selmatname

    # Variables Lists
    variable Solid_Questions
    foreach item $Solid_Questions {
	variable [lindex $item 0]
    }

    # Function Editor Variables
    variable ransol_ff
    variable heatrans_ff
    variable advect_ff
    variable ursolver_ff
    variable ramsolid_ff
    variable fluid_variables
    variable example_fluid_variables
    variable fluid_operators
    variable predefined_ff

    # Advect Species support variables
    variable Species_Rebuild 
    variable Species_Names
    variable Species_Values
    variable Species_Valnames
    variable snlist

    # URSolver support variables
    variable URVariables_Rebuild 
    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_Valnames
    variable ursnlist

    # Initialise support variables
    set Species_Rebuild ""
    set URVariables_Rebuild ""

    # Read values of selected material
    matsolid::getvalues

    #####  HELP Text variables #####
    set help_btn_initcond "[= {Define Initial Conditions}]."
    set help_btn_solvdat "[= {Define Solver Data}]."
    set help_btn_func "[= {Custom Function Editor}].\n\
	    [= {Edit a function and transfer it automatically to a field}].\n\
	    [= {You can also RIGHT-CLICK a field to open the Function Editor}]."
    # RANSOL Fields
    set help_Fluid_Density "[= {Density of the fluid}].\n\
	    [= {May be a function if compressibility effects are considered}]."
    set help_Fluid_Viscosity "[= {Viscosity of the fluid}].\n\
	    [= {May be a constant or a funtion ( i.e. 0.001*Tm )}].\n\
	    [= {Function editor can be started by pressing 'fx' button}]."
    switch $ProgramName {
	"Tdyn2D" {
	    set ht_acc "[= {External acceleration (NOT GRAVITY) acting on fluid (OX OY component)}].\n[= {The units are inserted in the Data>Problem_Data>UNITS menu}].\n[= {May be a constant or a function ( i.e. 1.0-x*tt )}]\n[= {Function editor can be started by pressing 'fx' button}]."
	    set labeltextacc [= {Acceleration Field (X Y)}]
	    set help_Darcys_Coefficient "[= {Position (x,y) of the symmetric Darcy's law resistance coeffient matrix (inverse of permeability)}]."
	}
	"Tdyn3D" {
	    set ht_acc "[= {External acceleration (NOT GRAVITY) acting on fluid (OX OY OZ component)}].\n[= {The units are inserted in the Problem_Data menu}].\n[= {May be a constant or a function ( i.e. 1.0-x*tt )}].\n[= {Function editor can be started by pressing 'fx' button}]."
	    set labeltextacc [= {Acceleration Field (X Y Z)}]
	    set help_Darcys_Coefficient "[= {Position (x,y,z) of the symmetric Darcy's law resistance coeffient matrix (inverse of permeability)}]."
	}
    }
    # HEATRANS Fields
    set help_Density ""
    set help_Specific_Heat_C ""
    switch $ProgramName {
	"Tdyn2D" {
	    set help_Thermal_Conductivity [= {Position (x,y) of the symmetric thermal_conductivity tensor (matrix)}]
	}
	"Tdyn3D" {
	    set help_Thermal_Conductivity [= {Position (x,y,z) of the symmetric thermal_conductivity tensor (matrix)}]
	}
    }
    set help_Floatability "[= {Floatability of the Fluid}]\n[= {( Boussinesq type, i.e. 0.001*(Tm-25) )}]"
    set help_Heat_Source "[= {Heat source for the thermal analysis}].\n[= {May be a constant or a funtion ( i.e. 100*sin(tt) )}].\n[= {Function editor can be started by pressing 'fx' button}].\n[= {Units are defined in Problem Data>UNITS>General Units window}]."
    set help_Heat_Reaction "[= {Heat reaction for the thermal analysis}].\n[= {May be a constant or a funtion ( i.e. 1.1e-5*tm^3 )}].\n[= {Function editor can be started by pressing 'fx' button}].\n[= {Units are defined in Problem Data>UNITS>General Units window}]."
    # ADVECT Fields
    set help_Advect "[= {ADVECT module data}].\n[= {Select species from the list and edit their properties}].\n[= {New species may be created by pressing '+' button}].\n[= {It is also possible to delete current species by pressing '-' button}].\n[= {Changing species names or their number will affect all existing solid materials}]."
    set help_Advect_Pic "[= {ADVECT module data}].\n[= {Properties of the current species may be inserted below}].\n[= {Properties names correspond to those given in the equation}]."
    # URSOLVER Fields
    set help_URSolver "[= {URSOLVER module data}].\n[= {Select variables from the list and edit their properties}].\n[= {New variables may be created by pressing '+' button}].\n[= {It is also possible to delete current variables by pressing '-' button}].\n[= {Changing variable names or their number will affect all existing solid materials}]."
    set help_URSolver_Pic "[= {URSOLVER module data}].\n[= {Properties of the current variable may be inserted below}].\n[= {Properties names correspond to those given in the equation}]."

    # RAMSOLID Fields
    set help_E [= {Young Modulus. Units in International System are N/m2}]
    set help_nu [= {Poisson coefficient. nu is adimensional}]
    set help_Specific_weight "[= {This is the specific weight of the solid}].\n[= {Units in International System are N/m3}]"
    set help_eo "[= {This (initial) strain is passed to the structural analysis}]\n[= {module to calculate the corresponding stresses.\nThis option}]\n[= {can be used to define a thermal expansion effect given by \u03b1*(Tm-To),}]\n[= {being To the (initial) reference temperature}]."

    # Main Notebook pages GRID
    set pages [NoteBook $BASE.pages -arcradius 0]
    if { $show_ransol == 1 } {
	set page1 [$pages insert 0 page1 -text "RANSOL"]
	set fpage1 [$pages getframe page1]
    }
    if { $show_heatrans == 1 } {
	set page2 [$pages insert 1 page2 -text "HEATRANS"]
	set fpage2 [$pages getframe page2]
    }
    if { $show_advect == 1 } {
	set page3 [$pages insert 2 page3 -text "ADVECT"]
	set fpage3 [$pages getframe page3]
    }
    if { $show_ursolver == 1 } {
	set page4 [$pages insert 3 page4 -text "URSOLVER"]
	set fpage4 [$pages getframe page4]
    }
    if { $show_ramsolid == 1 } {
	set page5 [$pages insert 4 page5 -text "RAMSOLID"]
	set fpage5 [$pages getframe page5]
    }

    if { $show_ransol == 1 } {
	$pages raise page1
    } elseif { $show_heatrans == 1 } {
	$pages raise page2
    } elseif { $show_advect == 1 } {
	$pages raise page3
    } elseif { $show_ursolver == 1 } {
	$pages raise page4
    } elseif { $show_ramsolid == 1 } {
	$pages raise page5
    }

    # GRIDS PAGES
    grid $pages -row 1 -column 0 -sticky nwes -pady 5
    if { $show_ransol == 1 } {
	grid columnconf $fpage1 10 -weight 1
	grid rowconf $fpage1 10 -weight 1
    }
    if { $show_heatrans == 1 } {
	grid columnconf $fpage2 10 -weight 1
	grid rowconf $fpage2 10 -weight 1
    }
    if { $show_advect == 1 } {
	grid columnconf $fpage3 1 -weight 1
	grid rowconf $fpage3 1 -weight 1
    }
    if { $show_ursolver == 1 } {
	grid columnconf $fpage4 1 -weight 1
	grid rowconf $fpage4 1 -weight 1
    }
    if { $show_ramsolid == 1 } {
	grid columnconf $fpage5 10 -weight 1
	grid rowconf $fpage5 10 -weight 1
    }

    if { $show_ransol == 1 } {

	#--------------------------- PAGE RANSOL ---------------------------#
	
	# Combo boxes Units definitions
	lappend units_Fluid_Density "Kg/m\u00b3" "Kg/l" "g/l" "t/m\u00b3" "lb/in\u00b3" "Kg/cc" "g/cc"        
	lappend units_Fluid_Viscosity_Units "Pa.s" "Kg/m.s" "ton/m.s" "g/m.s" "lb/in.s"
	lappend units_Acceleration "m/s\u00b2" "G" "ft/s\u00b2"
	lappend units_DCC_Units "1/m\u00b2" "1/darcy"

	# Invisible size label
	set sizelabel [label $fpage1.l_sizelabel \
		-text "                                                       " -bd 0 ]
	
	# Density Field
	set l_Fluid_Density [Label $fpage1.l_fluiddensity -text "[= {Fluid Density}] " -helptext $help_Fluid_Density]
	set e_Fluid_Density [entry $fpage1.e_fluiddensity -textvariable matsolid::Fluid_Density]
	set cbx_Fluid_Density [ComboBox $fpage1.lb_fluiddensity -values $units_Fluid_Density \
		-editable $edit_units -width 11 -textvariable matsolid::Fluid_Density_Units]
	
	# Viscosity Field
	set l_Fluid_Viscosity [Label $fpage1.l_fluidviscosity -text "[= {Fluid Viscosity}] " -helptext $help_Fluid_Viscosity]
	set e_Fluid_Viscosity [entry $fpage1.e_fluidviscosity -textvariable matsolid::Fluid_Viscosity]
	set cbx_Fluid_Viscosity [ComboBox $fpage1.lb_fluidviscosity -values $units_Fluid_Viscosity_Units \
		-editable $edit_units -width 11 -textvariable matsolid::Fluid_Viscosity_Units]

	# Darcy's law Resistance Coefficients Matrix
	set l_dcc [Label $fpage1.l_darcyfields -text \
		"[= {Darcy's law Resistance}] \n[= {Matrix}] " -justify right -helptext $help_Darcys_Coefficient]
	set f_darcyfields [frame $fpage1.f_darcyfields -relief flat]
	switch $ProgramName {
	    "Tdyn2D" {
		#  Darcy's law Resistance Coefficients Fields
		set e_dcc11 [entry $f_darcyfields.e_dcc11 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D11]
		set e_dcc12 [entry $f_darcyfields.e_dcc12 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D12]
		set e_dcc21 [entry $f_darcyfields.e_dcc21 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D12]
		set e_dcc22 [entry $f_darcyfields.e_dcc22 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D22]
	    }
	    "Tdyn3D" {
		# Darcy's law Resistance Coefficients Fields
		set e_dcc11 [entry $f_darcyfields.e_dcc11 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D11]
		set e_dcc12 [entry $f_darcyfields.e_dcc12 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D12]
		set e_dcc13 [entry $f_darcyfields.e_dcc13 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D13]
		set e_dcc21 [entry $f_darcyfields.e_dcc21 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D12]            
		set e_dcc22 [entry $f_darcyfields.e_dcc22 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D22]
		set e_dcc23 [entry $f_darcyfields.e_dcc23 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D23]
		set e_dcc31 [entry $f_darcyfields.e_dcc31 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D13]
		set e_dcc32 [entry $f_darcyfields.e_dcc32 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D23]
		set e_dcc33 [entry $f_darcyfields.e_dcc33 -width 6 \
		        -textvariable matsolid::Darcys_Coefficient_D33]
	    }
	}
	set cbx_DCC_Units [ComboBox $fpage1.cbx_dcc_units -values $units_DCC_Units \
		-editable $edit_units -width 11 -textvariable matsolid::Darcys_Coefficient_Units]

	# Acceleration Field
	set f_Acceleration [frame $fpage1.f_acc ]
	set l_Acceleration [Label $fpage1.l_acc -text $labeltextacc -helptext $ht_acc ]
	set e_AccelerationX [entry $f_Acceleration.e_accx -textvariable matsolid::AccOX_Field -width 6]
	set e_AccelerationY [entry $f_Acceleration.e_accy -textvariable matsolid::AccOY_Field -width 6]
	if { $ProgramName == "Tdyn3D" } {
	    set e_AccelerationZ [entry $f_Acceleration.e_accz -textvariable matsolid::AccOZ_Field -width 6]
	}
	set cbx_Acceleration_Units [ComboBox $fpage1.lb_acc -values $units_Acceleration \
		-editable $edit_units -width 11 -textvariable matsolid::Acceleration_Units]

	# Buttons frame
	set btn_frame [frame $fpage1.btn_frame]

	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ransol_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matsolid::callinitdata $BASE \"\" ransol" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matsolid::callsolverdata $BASE \"\" ransol" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]
	
	# GRIDS
	# SizeLabel
	grid $sizelabel -row 0 -column 0 -sticky en
	# Density
	grid $l_Fluid_Density -row 1 -column 0 -sticky en -pady 2
	grid $e_Fluid_Density -row 1 -column 1 -sticky wn -pady 2
	grid $cbx_Fluid_Density -row 1 -column 2 -sticky wn -pady 1 -padx 7
	# Viscosity
	grid $l_Fluid_Viscosity -row 2 -column 0 -sticky en -pady 2
	grid $e_Fluid_Viscosity -row 2 -column 1 -sticky wn -pady 2
	grid $cbx_Fluid_Viscosity -row 2 -column 2 -sticky wn -pady 1 -padx 7
	# Darcy's law Resistance Coefficients Matrix
	grid $l_dcc -row 3 -column 0 -sticky en -pady 4
	grid $f_darcyfields -row 3 -column 1 -sticky wn -pady 4
	switch $ProgramName {
	    "Tdyn2D" {
		#  Darcy's law Resistance Coefficients Matrix Fields
		grid $e_dcc11 -row 0 -column 0 -sticky en
		grid $e_dcc12 -row 0 -column 1 -sticky wn -padx 3
		grid $e_dcc21 -row 1 -column 0 -sticky en
		grid $e_dcc22 -row 1 -column 1 -sticky wn -padx 3
	    }
	    "Tdyn3D" {
		#  Darcy's law Resistance Coefficients Matrix Fields
		grid $e_dcc11 -row 0 -column 0 -sticky en
		grid $e_dcc12 -row 0 -column 1 -sticky wn -padx 3
		grid $e_dcc13 -row 0 -column 2 -sticky wn
		grid $e_dcc21 -row 1 -column 0 -sticky en
		grid $e_dcc22 -row 1 -column 1 -sticky wn -padx 3
		grid $e_dcc23 -row 1 -column 2 -sticky wn
		grid $e_dcc31 -row 2 -column 0 -sticky en
		grid $e_dcc32 -row 2 -column 1 -sticky wn -padx 3
		grid $e_dcc33 -row 2 -column 2 -sticky wn
	    }
	}
	grid $cbx_DCC_Units -row 3 -column 2 -sticky wn -padx 7 -pady 3
	# Acc Field
	grid $l_Acceleration -row 4 -column 0 -sticky en -pady 2
	grid $f_Acceleration -row 4 -column 1 -sticky wn -pady 2
	grid $e_AccelerationX -row 0 -column 0 -sticky wn -padx 1
	grid $e_AccelerationY -row 0 -column 1 -sticky wn -padx 1
	if { $ProgramName == "Tdyn3D" } {
	    grid $e_AccelerationZ -row 0 -column 2 -sticky wn -padx 1
	}
	grid $cbx_Acceleration_Units -row 4 -column 2 -sticky wn -pady 1 -padx 7
	
	# Buttons frame
	grid $btn_frame -row 5 -column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -pady 2 -padx 1
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -pady 2 -padx 1
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Right Click menus
	bind $e_Fluid_Density <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Fluid_Density \"Fluid Density\" \"\" 1 %X %Y"
	bind $e_Fluid_Viscosity <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Fluid_Viscosity \"Fluid Viscosity\" \"\" 1 %X %Y"
	bind $e_AccelerationX <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::AccOX_Field \"Acceleration X\" \"\" 1 %X %Y"
	bind $e_AccelerationY <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::AccOY_Field \"Acceleration Y\" \"\" 1 %X %Y"
	if { $ProgramName == "Tdyn3D" } {
	    bind $e_AccelerationZ <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::AccOZ_Field \"Acceleration Z\" \"\" 1 %X %Y"
	}
	#  Darcy's law Resistance Coefficients Bindings
	if { $ProgramName == "Tdyn2D" } {
	    bind $e_dcc11 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D11 \"Darcy's Coefficient D11\" \"\" 1 %X %Y"
	    bind $e_dcc12 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D12 \"Darcy's Coefficient D12\" \"\" 1 %X %Y"
	    bind $e_dcc21 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D12 \"Darcy's Coefficient D21\" \"\" 1 %X %Y"
	    bind $e_dcc22 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D22 \"Darcy's Coefficient D22\" \"\" 1 %X %Y"
	} else {
	    bind $e_dcc11 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D11 \"Darcy's Coefficient D11\" \"\" 1 %X %Y"
	    bind $e_dcc12 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D12 \"Darcy's Coefficient D12\" \"\" 1 %X %Y"
	    bind $e_dcc13 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D13 \"Darcy's Coefficient D13\" \"\" 1 %X %Y"
	    bind $e_dcc21 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D12 \"Darcy's Coefficient D21\" \"\" 1 %X %Y"
	    bind $e_dcc22 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D22 \"Darcy's Coefficient D22\" \"\" 1 %X %Y"
	    bind $e_dcc23 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D23 \"Darcy's Coefficient D23\" \"\" 1 %X %Y"
	    bind $e_dcc31 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D13 \"Darcy's Coefficient D31\" \"\" 1 %X %Y"
	    bind $e_dcc32 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D23 \"Darcy's Coefficient D32\" \"\" 1 %X %Y"
	    bind $e_dcc33 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Darcys_Coefficient_D33 \"Darcy's Coefficient D33\" \"\" 1 %X %Y"
	}
    }

    if { $show_heatrans == 1 } {

	#--------------------------- PAGE HEATRANS ---------------------------#

	# Combo boxes Units definitions
	lappend units_Density_Units "Kg/m\u00b3" "Kg/l" "g/l" "t/m\u00b3" "lb/in\u00b3" "Kg/cc" "g/cc"        
	lappend units_C_Units "J/Kg.\u2070C" "J/g.\u2070C" "btu/lb.\u2070F"
	lappend units_k_Units "W/m.\u2070C" "KW/m.\u2070C" "btu/h.in.\u2070F" "btu/h.ft.\u2070F" 
	lappend units_Heat_Source_Units "Kcal/hour.m\u00b3" "KW/m\u00b3" "cal/hour.m\u00b3" \
	    "W/m\u00b3" "cal/s.m\u00b3" "btu/h.in\u00b3" "btu/s.in\u00b3" "btu/h.ft\u00b3"
	lappend units_Heat_Reaction_Units "Kcal/h.m\u00b3.\u2070C" "KW/m\u00b3.\u2070C" \
	    "cal/h.m\u00b3.\u2070C" "W/m\u00b3.\u2070C" "cal/s.m\u00b3.\u2070C" \
	    "btu/h.in\u00b3.\u2070F" "btu/s.in\u00b3.\u2070F" "btu/h.ft\u00b3.\u2070F"

	# Invisible size label
	set sizelabel [ label $fpage2.l_sizelabel \
		-text "                                                       " -bd 0 ]
	
	# Density Field
	set l_Density [Label $fpage2.l_density -text \
		"[= {Density}] " -helptext $help_Density]
	set e_Density [entry $fpage2.e_density -textvariable matsolid::Density]
	set cbx_Density_Units [ComboBox $fpage2.lb_densityunits -values $units_Density_Units \
		-editable $edit_units -width 11 -textvariable matsolid::Density_Units]
	
	# Specific Heat C
	set l_Specific_Heat_C [Label $fpage2.l_specificheatc -text \
		"[= {Specific Heat}] " -helptext $help_Specific_Heat_C]
	set e_Specific_Heat_C [entry $fpage2.e_specificheatc -textvariable matsolid::Specific_Heat_C]
	set cbx_C_Units [ComboBox $fpage2.lb_specificheatunits -values $units_C_Units \
		-editable $edit_units -width 11 -textvariable matsolid::C_Units]
	
	# Thermal Conductivity Matrix
	set l_thmc [Label $fpage2.l_thmc -text \
		"[= {Thermal Conductivity}] \n[= {Matrix}] " -justify right -helptext $help_Thermal_Conductivity]
	set f_thcmtxflds [frame $fpage2.f_thcmtxflds -relief flat]
	switch $ProgramName {
	    "Tdyn2D" {
		# Thermal Conductivity Fields
		set e_thmc11 [entry $f_thcmtxflds.e_thmc11 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k11]
		set e_thmc12 [entry $f_thcmtxflds.e_thmc12 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k12]
		set e_thmc21 [entry $f_thcmtxflds.e_thmc21 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k12]
		set e_thmc22 [entry $f_thcmtxflds.e_thmc22 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k22]
	    }
	    "Tdyn3D" {
		# Thermal Conductivity Fields
		set e_thmc11 [entry $f_thcmtxflds.e_thmc11 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k11]
		set e_thmc12 [entry $f_thcmtxflds.e_thmc12 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k12]
		set e_thmc13 [entry $f_thcmtxflds.e_thmc13 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k13]
		set e_thmc21 [entry $f_thcmtxflds.e_thmc21 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k12]            
		set e_thmc22 [entry $f_thcmtxflds.e_thmc22 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k22]
		set e_thmc23 [entry $f_thcmtxflds.e_thmc23 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k23]
		set e_thmc31 [entry $f_thcmtxflds.e_thmc31 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k13]
		set e_thmc32 [entry $f_thcmtxflds.e_thmc32 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k23]
		set e_thmc33 [entry $f_thcmtxflds.e_thmc33 -width 6 \
		        -textvariable matsolid::Thermal_Conductivity_k33]
		
	    }
	}
	# K Units
	set cbx_k_Units [ComboBox $fpage2.cbx_k_units -values $units_k_Units \
		-editable $edit_units -width 11 -textvariable matsolid::k_Units]
	
	# Floatability
	if { $show_ransol == 1 } {
	    set l_Floatability [Label $fpage2.l_flotability -text \
		    "[= {Floatability}] " -helptext $help_Floatability]
	    set e_Floatability [Entry $fpage2.e_flotability -textvariable matsolid::Floatability]
	}

	# Heat Source 
	set l_Heat_Source [Label $fpage2.l_heatsrc -text \
		"[= {Heat Source Field}] " -helptext $help_Heat_Source]
	set e_Heat_Source [Entry $fpage2.e_heatsrc -textvariable matsolid::Heat_Source_Field]
	set cbx_Heat_Source_Units [ComboBox $fpage2.lb_heatsrc -values $units_Heat_Source_Units \
		-editable $edit_units -width 11 -textvariable matsolid::Heat_Source_Units]
	
	# Heat Reaction
	set l_Heat_Reaction [Label $fpage2.l_heatrct -text \
		"[= {Heat Reaction Field}] " -helptext $help_Heat_Reaction]
	set e_Heat_Reaction [Entry $fpage2.e_heatrct -textvariable matsolid::Heat_Reaction_Field]
	set cbx_Heat_Reaction_Units [ComboBox $fpage2.lb_heatrct -values $units_Heat_Reaction_Units \
		-editable $edit_units -width 11 -textvariable matsolid::Heat_Reaction_Units]

	# Buttons frame
	set btn_frame [frame $fpage2.btn_frame]
	
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -justify left -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$heatrans_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func]
	# Initial Conditions
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matsolid::callinitdata $BASE \"\" ransol" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matsolid::callsolverdata $BASE \"\" ransol" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]
	
	# GRIDS
	# SizeLabel
	grid $sizelabel -row 0 -column 0 -sticky en
	# Density
	grid $l_Density -row 1 -column 0 -sticky en -pady 2
	grid $e_Density -row 1 -column 1 -sticky wn -pady 2
	grid $cbx_Density_Units -row 1 -column 2 -sticky wn -pady 2 -padx 7
	# Specific Heat C
	grid $l_Specific_Heat_C -row 2 -column 0 -sticky en -pady 2
	grid $e_Specific_Heat_C -row 2 -column 1 -sticky wn -pady 2
	grid $cbx_C_Units -row 2 -column 2 -sticky ewn -pady 2 -padx 7
	# Thermal Conductivity Matrix
	grid $l_thmc -row 3 -column 0 -sticky en -pady 4
	grid $f_thcmtxflds -row 3 -column 1 -sticky wn -pady 4
	switch $ProgramName {
	    "Tdyn2D" {
		# Thermal Conductivity Fields
		grid $e_thmc11 -row 0 -column 0 -sticky en
		grid $e_thmc12 -row 0 -column 1 -sticky wn -padx 3
		grid $e_thmc21 -row 1 -column 0 -sticky en
		grid $e_thmc22 -row 1 -column 1 -sticky wn -padx 3
	    }
	    "Tdyn3D" {
		# Thermal Conductivity Fields
		grid $e_thmc11 -row 0 -column 0 -sticky en
		grid $e_thmc12 -row 0 -column 1 -sticky wn -padx 3
		grid $e_thmc13 -row 0 -column 2 -sticky wn
		grid $e_thmc21 -row 1 -column 0 -sticky en
		grid $e_thmc22 -row 1 -column 1 -sticky wn -padx 3
		grid $e_thmc23 -row 1 -column 2 -sticky wn
		grid $e_thmc31 -row 2 -column 0 -sticky en
		grid $e_thmc32 -row 2 -column 1 -sticky wn -padx 3
		grid $e_thmc33 -row 2 -column 2 -sticky wn
	    }
	}
	grid $cbx_k_Units -row 3 -column 2 -sticky wn -padx 7 -pady 3
	# Floatability
	if { $show_ransol == 1 } {
	    grid $l_Floatability -row 4 -column 0 -sticky en -pady 2
	    grid $e_Floatability -row 4 -column 1 -sticky wn -pady 2
	}
	# Heat Source
	grid $l_Heat_Source -row 5 -column 0 -sticky en -pady 2
	grid $e_Heat_Source -row 5 -column 1 -sticky wn -pady 2
	grid $cbx_Heat_Source_Units -row 5 -column 2 -sticky wn -pady 1 -padx 7
	# Heat Reaction
	grid $l_Heat_Reaction -row 6 -column 0 -sticky en -pady 2
	grid $e_Heat_Reaction -row 6 -column 1 -sticky wn -pady 2
	grid $cbx_Heat_Reaction_Units -row 6 -column 2 -sticky wn -pady 1 -padx 7

	# Buttons frame
	grid $btn_frame -row 7 -column 1 -columnspan 2 -sticky wn -pady 5
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -pady 2 -padx 1
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -pady 2 -padx 1
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Right click menus
	bind $e_Density <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Density Density \"\" 1 %X %Y"
	bind $e_Specific_Heat_C <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Specific_Heat_C \"Specific Heat C\" \"\" 1 %X %Y"
	if { $show_ransol == 1 } {
	    bind $e_Floatability <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Floatability \"Floatability\" \"\" 1 %X %Y"
	}
	bind $e_Heat_Source <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Heat_Source_Field \"Heat Source\" \"\" 1 %X %Y"
	bind $e_Heat_Reaction <ButtonPress-3> "matsolid::popupmenu $BASE matsolid::Heat_Reaction_Field \"Heat Reaction\" \"\" 1 %X %Y"
	# Thermal Conductivity Matrix Fields Bindings
	if { $ProgramName == "Tdyn2D" } {
	    bind $e_thmc11 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k11 \"Thermal Conductivity k11\" \"\" 1 %X %Y"
	    bind $e_thmc12 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k12 \"Thermal Conductivity k12\" \"\" 1 %X %Y"
	    bind $e_thmc21 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k12 \"Thermal Conductivity k21\" \"\" 1 %X %Y"
	    bind $e_thmc22 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k22 \"Thermal Conductivity k22\" \"\" 1 %X %Y"
	} else {
	    bind $e_thmc11 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k11 \"Thermal Conductivity k11\" \"\" 1 %X %Y"
	    bind $e_thmc12 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k12 \"Thermal Conductivity k12\" \"\" 1 %X %Y"
	    bind $e_thmc13 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k13 \"Thermal Conductivity k13\" \"\" 1 %X %Y"
	    bind $e_thmc21 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k12 \"Thermal Conductivity k21\" \"\" 1 %X %Y"
	    bind $e_thmc22 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k22 \"Thermal Conductivity k22\" \"\" 1 %X %Y"
	    bind $e_thmc23 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k23 \"Thermal Conductivity k23\" \"\" 1 %X %Y"
	    bind $e_thmc31 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k13 \"Thermal Conductivity k31\" \"\" 1 %X %Y"
	    bind $e_thmc32 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k23 \"Thermal Conductivity k32\" \"\" 1 %X %Y"
	    bind $e_thmc33 <ButtonPress-3> \
		    "matsolid::popupmenu $BASE matsolid::Thermal_Conductivity_k33 \"Thermal Conductivity k33\" \"\" 1 %X %Y"
	}
    }

    if { $show_advect == 1 } {

	#--------------------------- PAGE ADVECT ---------------------------#

	# Top Label Frame
	set topfr [frame $fpage3.topfr -relief flat]

	# Information label
	set infotext "[= {Changing Species names or their number will affect all existing solid materials}]."
	set l_info [Label $topfr.l_info -text $infotext -justify left -wraplength 150]
	GidHelp $l_info $help_Advect

	# Equation label with image
	set l_ieqdf [Label $topfr.l_ieqdf -image $TUI_images::i_sladveq]
	GidHelp $l_ieqdf $help_Advect_Pic

	# Title Frame Species List
	set tfr_spclist [TitleFrame $fpage3.tfr_spclist -relief ridge -bd 3\
		-text [= {Defined Species}] -side left ]
	set frame_spclist [$tfr_spclist getframe]

	# Paned window
	set pw [PanedWindow $frame_spclist.pw]
	set pane1 [$pw add -weight 1]
	set pane2 [$pw add -weight 5]

	# Species Names in scrolled window
	set spc_scrw [ScrolledWindow $pane1.spc_scrw -scrollbar both]
	set snlist [listbox $spc_scrw.snlist -selectbackground lightblue -selectforeground black \
		-selectborderwidth 1 -selectmode extended -height 5 -bg white -width 10]
	$spc_scrw setwidget $snlist

	# Selected Specie values frame
	set svlist [frame $pane2.svlist -relief flat]

	# Buttons frame
	set btn_frame [frame $svlist.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$advect_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matsolid::callinitdata $BASE $snlist advect" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matsolid::callsolverdata $BASE $snlist advect" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# Buttons PLUS MINUS - under the Species Names listbox
	set btnbox [ButtonBox $frame_spclist.btnbox -spacing 0 -padx 1 -pady 1 -homogeneous 0]
	$btnbox add -image $buttons::ibtn_plus -width 20 -height 20 \
		-highlightthickness 0 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Add New Specie}] \
		-command "matsolid::addspecie $snlist $svlist \"advect\""
	$btnbox add -image $buttons::ibtn_minus -width 20 -height 20 \
		-highlightthickness 1 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Delete Selected Specie}] \
		-command "matsolid::removespecie $snlist $svlist \"advect\""

	# GRIDS
	grid $topfr -row 0 -column 0 -columnspan 2 -sticky nw -pady 10
	grid $l_info -row 0 -column 0 -sticky nw -padx 5
	grid $l_ieqdf -row 0 -column 1 -sticky nw -padx 5
	# Title Frame
	grid $tfr_spclist -row 1 -column 0 -columnspan 5 -sticky wnes
	grid $pw -row 0 -column 0 -sticky wnes
	grid $spc_scrw -row 0 -column 0 -sticky wnes
	grid $btnbox -row 1 -column 0 -sticky wn
	# Buttons frame
	grid $btn_frame -row [expr [llength $Species_Valnames] + 1] \
		-column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Rows and columns configuration
	  grid rowconf $pane1 0 -weight 1
	  grid columnconf $pane1 0 -weight 1
	  grid rowconf $pane2 1 -weight 1
	  grid columnconf $pane2 0 -weight 1
	grid columnconf $frame_spclist 0 -weight 1
	  grid rowconf $frame_spclist 0 -weight 1
	    grid rowconf $svlist 10 -weight 1

	# BINDS 
	bind $snlist <<ListboxSelect>> ";
	matsolid::valsavespecie $snlist \"last\" \"advect\"; 
	matsolid::displayspecievalues $snlist \"advect\";
	matsolid::displayspeciesnames $snlist \"advect\";
	"
	bind $snlist <ButtonPress-3> "+matsolid::popupmenu $snlist \"\" \"\" \"\" 2 %X %Y"

	GidHelpRecursive $tfr_spclist $help_Advect

	# Insert Species Names to listbox
	matsolid::displayspeciesnames $snlist "advect"
	# Selecting first Specie
	matsolid::updatespcselection $snlist 0 "advect"
	# Create Species Values widgets
	matsolid::makespecie $snlist $svlist "advect"
	# Put specie values into entries
	matsolid::displayspecievalues $snlist "advect"

    }

    if { $show_ursolver == 1 } {

	#--------------------------- PAGE URSOLVER ---------------------------#

	# Top Label Frame
	set topfr [frame $fpage4.topfr -relief flat]

	# Information label
	set infotext "[= {Changing Variables names or their number will affect all existing solid materials}]."
	set l_info [Label $topfr.l_info -text $infotext -justify left -wraplength 150]
	GidHelp $l_info $help_URSolver

	# Equation label with image
	set l_ieqdf [Label $topfr.l_ieqdf -image $TUI_images::i_slurseq]
	GidHelp $l_ieqdf $help_URSolver_Pic

	# Title Frame Species List
	set tfr_spclist [TitleFrame $fpage4.tfr_spclist -relief ridge -bd 3\
		-text [= {Defined Variables}] -side left ]
	set frame_spclist [$tfr_spclist getframe]

	# Paned window
	set upw [PanedWindow $frame_spclist.pw]
	set upane1 [$upw add -weight 1]
	set upane2 [$upw add -weight 5]

	# Variables Names in scrolled window
	set ursv_scrw [ScrolledWindow $upane1.ursv_scrw -scrollbar both]
	set ursnlist [listbox $ursv_scrw.ursnlist -selectbackground lightblue -selectforeground black \
		-selectborderwidth 1 -selectmode extended -height 5 -bg white -width 10]
	$ursv_scrw setwidget $ursnlist
	GidHelp $ursv_scrw $help_URSolver

	# Selected Specie values frame
	set ursvlist [frame $upane2.ursvlist -relief flat]

	# Buttons frame
	set btn_frame [frame $ursvlist.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ursolver_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions button
	set btn_initcond [Button $btn_frame.btn_initcond -underline 0 \
		-command "matsolid::callinitdata $BASE $ursnlist ursolver" \
		-image $buttons::ibtn_initcond -helptext $help_btn_initcond -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matsolid::callsolverdata $BASE $ursnlist ursolver" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# Buttons PLUS MINUS - under the Species Names listbox
	set btnbox [ButtonBox $frame_spclist.btnbox -spacing 0 -padx 1 -pady 1 -homogeneous 0]
	$btnbox add -image $buttons::ibtn_plus -width 20 -height 20 \
		-highlightthickness 0 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Add New Variable}] \
		-command "matsolid::addspecie $ursnlist $ursvlist ursolver"
	$btnbox add -image $buttons::ibtn_minus -width 20 -height 20 \
		-highlightthickness 1 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext [= {Delete Selected Variable}] \
		-command "matsolid::removespecie $ursnlist $ursvlist ursolver"

	# GRIDS
	grid $topfr -row 0 -column 0 -columnspan 2 -sticky nw -pady 10
	grid $l_info -row 0 -column 0 -sticky nw -padx 5
	grid $l_ieqdf -row 0 -column 1 -sticky nw -padx 5
	# Title Frame
	grid $tfr_spclist -row 1 -column 0 -columnspan 2 -sticky wnes
	grid $upw -row 0 -column 0 -sticky wnes
	grid $ursv_scrw -row 0 -column 0 -sticky wnes
	grid $btnbox -row 1 -column 0 -sticky wn
	# Buttons frame
	grid $btn_frame -row [expr [llength $Species_Valnames] + 1] \
		-column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_initcond -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Rows and columns configuration
	  grid rowconf $upane1 0 -weight 1
	  grid columnconf $upane1 0 -weight 1
	  grid rowconf $upane2 1 -weight 1
	  grid columnconf $upane2 0 -weight 1
	grid columnconf $frame_spclist 0 -weight 1
	  grid rowconf $frame_spclist 0 -weight 1
	    grid rowconf $ursvlist 10 -weight 1

	# BINDS 
	bind $ursnlist <<ListboxSelect>> ";
	matsolid::valsavespecie $ursnlist \"last\" \"ursolver\"; 
	matsolid::displayspecievalues $ursnlist \"ursolver\";
	matsolid::displayspeciesnames $ursnlist \"ursolver\";
	"
	bind $ursnlist <ButtonPress-3> "+matsolid::popupmenu $ursnlist \"\" \"\" \"\" 2 %X %Y"

	GidHelpRecursive $tfr_spclist $help_URSolver

	# Insert Species Names to listbox
	matsolid::displayspeciesnames $ursnlist "ursolver"
	# Selecting first Specie
	matsolid::updatespcselection $ursnlist 0 "ursolver"
	# Create Species Values widgets
	matsolid::makespecie $ursnlist $ursvlist "ursolver"
	# Put specie values into entries
	matsolid::displayspecievalues $ursnlist "ursolver"
    }

    if { [info exists snlist] == 1 && [info exists ursnlist] == 1 } {
	$pages bindtabs <Button-1> "matsolid::selectpage $snlist $ursnlist"
    } elseif { [info exists snlist] == 1 && [info exists ursnlist] != 1 } {
	$pages bindtabs <Button-1> "matsolid::selectpage $snlist \"\""
    } elseif { [info exists snlist] != 1 && [info exists ursnlist] == 1 } {
	$pages bindtabs <Button-1> "matsolid::selectpage \"\" $ursnlist"
    }

    if { $show_ramsolid == 1 && $ProgramName == "Tdyn3D" } {

	#--------------------------- PAGE RAMSOLID ---------------------------#
	
	variable UnitsList [list N-m N-cm N-mm Kp-cm]
	variable EunitsList [list "N/m\u00b2" "N/cm\u00b2" "N/mm\u00b2" "Kp/cm\u00b2"]
	variable Specific_weight_unitsList [list "N/m\u00b3" "N/cm\u00b3" "N/mm\u00b3" "Kp/cm\u00b3"]
	set ipos [lsearch $UnitsList $matsolid::Ram_Units]
	if { $ipos == -1 } { error [= {Unknown value in material field 'Units'}] }
	variable Eunits [lindex $EunitsList $ipos]
	variable Specific_weight_units [lindex $Specific_weight_unitsList $ipos]

	proc UpdateRamSolidUnits {} {
	    foreach i [list Eunits Ram_Units Specific_weight_units UnitsList \
		    EunitsList Specific_weight_unitsList] { variable $i }

	    set ipos [lsearch $EunitsList $Eunits]
	    set Ram_Units [lindex $UnitsList $ipos]
	    set Specific_weight_units [lindex $Specific_weight_unitsList $ipos]
	}

	# Invisible size label
	set sizelabel [label $fpage5.l_sizelabel \
		-text "                                                       " -bd 0 ]

	# E Field
	set l_E [Label $fpage5.l_E -text "[= {E (Young modulus)}] " -helptext $help_E]
	set e_E [Entry $fpage5.e_E -textvariable matsolid::E]
	set cb_E [ComboBox $fpage5.cb_E -textvariable matsolid::Eunits -editable 0 -width 6 \
		-values $EunitsList -modifycmd [namespace code UpdateRamSolidUnits]] 
	# nu Field
	set l_nu [Label $fpage5.l_nu -text "\u03bd [= {(Poisson coefficient)}] " -helptext $help_nu]
	set e_nu [Entry $fpage5.e_nu -textvariable matsolid::nu]
	# Specific weight Field
	set l_SpecWeight [Label $fpage5.l_SpecWeight -text "[= {Specific weight}] " \
		-helptext $help_Specific_weight]
	set e_SpecWeight [Entry $fpage5.e_SpecWeight -textvariable matsolid::Specific_weight]
	set lu_SpecWeight [label $fpage5.lu_SpecWeight -textvariable matsolid::Specific_weight_units]
	# Mechanical strain
	set l_Mechanical_strain [Label $fpage5.l_Mechanical_strain \
		-text "\u03b5\u2080 [= {(Mechanical strain)}] " -helptext $help_eo]
	set e_Mechanical_strain [Entry $fpage5.e_Mechanical_strain -textvariable matsolid::Mechanical_strain]

	# Buttons frame
	set btn_frame [frame $fpage5.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -justify left -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ramsolid_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matsolid::callsolverdata $BASE \"\" ramsolid" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# GRIDS
	# SizeLabel
	set row 0
	grid $sizelabel -row $row -column 0 -sticky en
	incr row
	# Data
	grid $l_E -row $row -column 0 -sticky en -pady 2
	grid $e_E -row $row -column 1 -sticky wn -pady 2
	grid $cb_E -row $row -column 2 -sticky wn -padx 3 -pady 2
	incr row
	grid $l_nu -row $row -column 0 -sticky en -pady 2
	grid $e_nu -row $row -column 1 -sticky wn -pady 2
	incr row
	grid $l_SpecWeight -row $row -column 0 -sticky en -pady 2
	grid $e_SpecWeight -row $row -column 1 -sticky wn -pady 2
	grid $lu_SpecWeight -row $row -column 2 -sticky wn -padx 3 -pady 2
	incr row
	grid $l_Mechanical_strain -row $row -column 0 -sticky en -pady 2
	grid $e_Mechanical_strain -row $row -column 1 -sticky wn -pady 2
	bind $e_Mechanical_strain <ButtonPress-3> \
		"matsolid::popupmenu $BASE matsolid::Mechanical_strain \"Mechanical strain\" \"\" 1 %X %Y"
	incr row

	# Buttons frame
	grid $btn_frame -row $row -column 1 -columnspan 2 -sticky wn -pady 5
	incr row
	set column 0
	if { $ProgramName == "Tdyn3D" } {
	    
	    grid $btn_functions -row 0 -column $column -sticky en -pady 2 -padx 1
	    incr column
	}
	# Solver Data Button
	grid $btn_solvdat -row 0 -column $column -sticky en -padx 1 -pady 2
    }
}

proc matsolid::initgob { BASE selmatname } {

    global ProgramName
    global tkPriv tcl_platform
    
    # Tdyn settings
    variable edit_units

    # Selected Material Name
    variable Material_Name $selmatname
    
    # Variables Lists
    variable Solid_Questions
    foreach item $Solid_Questions {
	variable [lindex $item 0]
    }
    if { 0 } {
	for { set i 0 } { $i<[llength $Solid_Questions] } { incr i } {
	    if { [lindex [lindex $Solid_Questions $i] 0]=="Fluid_Viscosity" } {
		set Solid_Questions [lreplace $Solid_Questions $i $i \
		        [list Fluid_Viscosity gobviscosity 1 ransol ""]]
	    } elseif { [lindex [lindex $Solid_Questions $i] 0]=="Density" } {
		set Solid_Questions [lreplace $Solid_Questions $i $i \
		        [list Density list 1 heatrans ""]]
	    } elseif { [lindex [lindex $Solid_Questions $i] 0]=="Specific_Heat_C" } {
		set Solid_Questions [lreplace $Solid_Questions $i $i \
		        [list Specific_Heat_C list 1 heatrans ""]]
	    } elseif { [lindex [lindex $Solid_Questions $i] 0]=="Thermal_Conductivity_k11" } {
		set Solid_Questions [lreplace $Solid_Questions $i $i \
		        [list Thermal_Conductivity_k11 list 1 heatrans ""]]
	    }
	}
    }
    # Initialise support variables
    set Species_Rebuild ""
    set URVariables_Rebuild ""
    
    # Read values of selected material
    matsolid::getvalues
    
    #####  HELP Text variables #####
    # RANSOL Fields
    set help_Fluid_Viscosity "[= {Viscosity of the material exp(av+bv/(Tm-cv))}]."
    # HEATRANS Fields
    set help_Density "[= {Density of the material (Tm1, D1, Tm2, D2, ...)}]."
    set help_Specific_Heat_C "[= {Specific heat of the material  (Tm1, Cp1, Tm2, Cp2, ...)}]."
    set help_Thermal_Conductivity "[= {Thermal conductivity of the material (Tm1, K1, Tm2, K2, ...)}]."
    
    # All info is written in the same frame
    set pages [NoteBook $BASE.pages -arcradius 0]
    set fpage [$pages insert 1 page1 -text "HEATRANS"]
    grid $pages -row 1 -column 0 -sticky nwes -pady 5
    $pages raise page1
    grid columnconf $fpage 10 -weight 1
    grid rowconf $fpage 10 -weight 1
    set irow 0 
    
    #--------------------------- PAGE RANSOL ---------------------------#
    
    # Combo boxes Units definitions    
    lappend units_Fluid_Viscosity_Units "Pa.s" "Kg/m.s" "ton/m.s" "g/m.s" "lb/in.s"
    
    # Invisible size label
    set sizelabel [label $fpage.l_sizelabel \
	    -text "                                                       " -bd 0 ]
    
    # Viscosity Field
    set l_Fluid_Viscosity [Label $fpage.l_fluidviscosity -text [= {Viscosity }] -helptext $help_Fluid_Viscosity]
    set e_Fluid_Viscosity [entry $fpage.e_fluidviscosity -textvariable matsolid::Fluid_Viscosity]
    set cbx_Fluid_Viscosity [ComboBox $fpage.lb_fluidviscosity -values $units_Fluid_Viscosity_Units \
	    -editable $edit_units -width 11 -textvariable matsolid::Fluid_Viscosity_Units]
    
    # GRIDS
    # SizeLabel
    grid $sizelabel -row $irow -column 0 -sticky en
    incr irow
    # Viscosity
    grid $l_Fluid_Viscosity -row $irow -column 0 -sticky en -pady 2
    grid $e_Fluid_Viscosity -row $irow -column 1 -sticky wn -pady 2
    grid $cbx_Fluid_Viscosity -row $irow -column 2 -sticky wn -pady 1 -padx 7
    incr irow
    
    #--------------------------- PAGE HEATRANS ---------------------------#
    
    # Combo boxes Units definitions
    lappend units_Density_Units "Kg/m\u00b3" "Kg/l" "g/l" "t/m\u00b3" "lb/in\u00b3" "Kg/cc" "g/cc"        
    lappend units_C_Units "J/Kg.\u2070C" "J/g.\u2070C" "btu/lb.\u2070F"
    lappend units_k_Units "W/m.\u2070C" "KW/m.\u2070C" "btu/h.in.\u2070F" "btu/h.ft.\u2070F" 
    
    # Density Field
    set l_Density [Label $fpage.l_density -text \
	    "[= {Density}] " -helptext $help_Density]
    set e_Density [entry $fpage.e_density -textvariable matsolid::Density]
    set cbx_Density_Units [ComboBox $fpage.lb_densityunits -values $units_Density_Units \
	    -editable $edit_units -width 11 -textvariable matsolid::Density_Units]
    
    # Specific Heat C
    set l_Specific_Heat_C [Label $fpage.l_specificheatc -text \
	    "[= {Specific Heat}] " -helptext $help_Specific_Heat_C]
    set e_Specific_Heat_C [entry $fpage.e_specificheatc -textvariable matsolid::Specific_Heat_C]
    set cbx_C_Units [ComboBox $fpage.lb_specificheatunits -values $units_C_Units \
	    -editable $edit_units -width 11 -textvariable matsolid::C_Units]
    
    # Thermal Conductivity K
    set l_Thermal_Conductivity [Label $fpage.l_thmc -text \
	    "[= {Thermal Conductivity}] " -helptext $help_Thermal_Conductivity]
    set e_Thermal_Conductivity [entry $fpage.thconductivity -textvariable matsolid::Thermal_Conductivity_k11]
    set cbx_k_Units [ComboBox $fpage.cbx_k_units -values $units_k_Units \
	    -editable $edit_units -width 11 -textvariable matsolid::k_Units]
    
    # GRIDS
    # Density
    grid $l_Density -row $irow -column 0 -sticky en -pady 2
    grid $e_Density -row $irow -column 1 -sticky wn -pady 2
    grid $cbx_Density_Units -row $irow -column 2 -sticky wn -pady 2 -padx 7
    incr irow
    # Specific Heat C
    grid $l_Specific_Heat_C -row $irow -column 0 -sticky en -pady 2
    grid $e_Specific_Heat_C -row $irow -column 1 -sticky wn -pady 2
    grid $cbx_C_Units -row $irow -column 2 -sticky wn -pady 2 -padx 7
    incr irow
    # Thermal Conductivity
    grid $l_Thermal_Conductivity -row $irow -column 0 -sticky en -pady 2
    grid $e_Thermal_Conductivity -row $irow -column 1 -sticky en -pady 2        
    grid $cbx_k_Units -row $irow -column 2 -sticky wn -pady 2 -padx 7

}

proc matsolid::selectpage { snlist ursnlist {pg ""} } {
    
    variable spcnlastsel
    variable ursnlastsel

    set nlist ""

    if { $pg == "page3" } {
	set type "advect"
	set nlist $snlist
	set setsel $spcnlastsel
	set leavenlist $ursnlist
    } elseif { $pg == "page4" } {
	set type "ursolver"
	set nlist $ursnlist
	set setsel $ursnlastsel
	set leavenlist $snlist
    }

    if { [string length $nlist] > 0 && ( $type == "advect" || $type == "ursolver") } {
	matsolid::valsavespecie $leavenlist "" $type
	matsolid::updatespcselection $nlist $setsel $type
	matsolid::displayspecievalues $nlist $type
    }

    return 1
}

proc matsolid::callinitdata { parent nlist type } {

    variable spcnlastsel
    variable ursnlastsel

    if { [string length $nlist] > 0 && ($type == "advect" || $type == "ursolver") } {
	matsolid::valsavespecie $nlist "actual" $type
    }
    set datachange [matsolid::checkclose $parent 0 0 0]

    if { $datachange != 0 } {
	set answer ""
	set answer [tk_dialogRAM $parent [= {Save Data?}] \
		"[= {To open the Initial Data Window, edited values have to be saved}].\n[= {Do you want to save them?}]"\
		question 0 Yes No]
    } else {
	set answer 0
    }

    if { $answer == 0 } {
	set saveres ""
	if { $datachange != 0 } {
	    # Save edited data
	    set saveres [matsolid::checkclose $parent 1 0 0]
	}

	if { $saveres != "dataerror" } {
	    switch $type {
		"ransol" {
		    initdatwin::init .gid $type ""
		}
		"heatrans" {
		    initdatwin::init .gid $type ""
		}
		"advect" {
		    initdatwin::init .gid $type $spcnlastsel
		}
		"ursolver" {
		    initdatwin::init .gid $type $ursnlastsel
		}
	    }
	}
    }

    return 1
}

proc matsolid::callsolverdata { parent nlist type } {

    variable spcnlastsel
    variable ursnlastsel

    if { [string length $nlist] > 0 && ($type == "advect" || $type == "ursolver") } {
	matsolid::valsavespecie $nlist "actual" $type
    }
    set datachange [matsolid::checkclose $parent 0 0 0]

    if { $datachange != 0 } {
	set answer ""
	set answer [tk_dialogRAM $parent [= {Save Data?}] \
		"[= {To open the Solver Data Window, edited values have to be saved}].\n[= {Do you want to save them?}]" \               
	    question 0 Yes No]
    } else {
	set answer 0
    }

    if { $answer == 0 } {
	set saveres ""
	if { $datachange != 0 } {
	    # Save edited data
	    set saveres [matsolid::checkclose $parent 1 0 0]
	}

	if { $saveres != "dataerror" } {
	    switch $type {
		"ransol" {
		    solvdatwin::init .gid $type ""
		}
		"heatrans" {
		    solvdatwin::init .gid $type ""
		}
		"advect" {
		    solvdatwin::init .gid $type $spcnlastsel
		}
		"ursolver" {
		    solvdatwin::init .gid $type $ursnlastsel
		}
		"ramsolid" {
		    solvdatwin::init .gid $type ""
		}
	    }
	}
    }

    return 1
}



################################################################################
#
#   Species Manipulation
#   The following functions are used to manipulate the ADVECT Species Data
#   as well as the URSolver Variables Data.
#
################################################################################

proc matsolid::makespecie { nlist vlist type } {

    global ProgramName

    # Tdyn Settings
    variable edit_units

    # Top Frame Buttons (Modify, New and Erase material)
    variable button_modify 
    variable button_new
    variable button_erase

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable SpeciesUpdateCmd
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariablesUpdateCmd
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    # Combo boxes Units definitions
    # ADVECT
    lappend units_F2_Units "m\u00b2/s" "m\u00b2/h" "ft\u00b2/h"
    lappend units_F3_Units "1/s" "1/ds" "1/h"
    lappend units_F4_Units "C/s" "ppmC/s" "ppbC/s"
    # URSOLVER
    lappend units_F1_Phi_Units "1/m\u00b3" "1/cm\u00b3" "1/ft\u00b3"
    lappend units_F2_Phi_Units "1/m.s" "1/cm.s" "1/in.s" "1/ft.s"
    lappend units_F3_Phi_Units "1/m\u00b3.s" "1/cm\u00b3.s" "1/in\u00b3.s"
    lappend units_F4_Phi_Units "U/m\u00b3.s" "U/cm\u00b3.s" "U/in\u00b3.s"

    # Grid Values Frame
    grid $vlist -row 0 -column 0 -sticky wnes -padx 10

    switch $type {
	"advect" {
	    # Specie Name
	    set l_sname [Label $vlist.l_sname -text [= {  Species Name:}]]
	    set e_sname [entry $vlist.e_sname -width 21 -textvariable matsolid::sname]
	    set type_Valnames $Species_Valnames
	}
	"ursolver" {
	    # Variable Name
	    set l_sname [Label $vlist.l_vname -text [= {Variable Name:}]]
	    set e_sname [entry $vlist.e_vname -width 21 -textvariable matsolid::ursname]
	    set type_Valnames $URVariables_Valnames
	}
    }

    # GRIDS
    # Label
    grid $l_sname -row 0 -column 0 -sticky en -pady 5
    # Entry
    grid $e_sname -row 0 -column 1 -columnspan 2 -sticky wn -pady 5

    # BINDS
    bind $e_sname <Return> ";
    matsolid::valsavespecie $nlist \"actual\" \"$type\";
    matsolid::displayspeciesnames $nlist \"$type\";
    "
    bind $button_modify <ButtonPress-1> ";
    matsolid::valsavespecie $nlist \"actual\" \"$type\";
    matsolid::displayspeciesnames $nlist \"$type\";
    "
    switch $type {
	  "advect" {
	    set l_F1_Spc [Label $vlist.l_f1spc -text "[= {Temporal Factor}] f \u00b9 "]
	    set l_F2_Spc [Label $vlist.l_f2spc -text "[= {Diffusion Matrix}] f \u00b2 \n[= {(Fick's law)}] "]
	    set l_F3_Spc [Label $vlist.l_f3spc -text "[= {Degradation}] f \u00b3 "]
	    set l_F4_Spc [Label $vlist.l_f4spc -text "[= {Source}] f \u2074 "]

	    set mxfr [frame $vlist.mxfr -relief flat]
	    switch $ProgramName {
		"Tdyn2D" {
		    set e_F1_Spc [entry $vlist.e_f1spc -width 21 -textvariable matsolid::F1_Spc]
		    set e_F2xx_Spc [entry $mxfr.e_f2xxspc -width 6 -textvariable matsolid::F2xx_Spc]
		    set e_F2xy_Spc [entry $mxfr.e_f2xyspc -width 6 -textvariable matsolid::F2xy_Spc]
		    set e_F2yx_Spc [entry $mxfr.e_f2yxspc -width 6 -textvariable matsolid::F2xy_Spc]
		    set e_F2yy_Spc [entry $mxfr.e_f2yyspc -width 6 -textvariable matsolid::F2yy_Spc]
		    set e_F3_Spc [entry $vlist.e_f3spc -width 21 -textvariable matsolid::F3_Spc]
		    set e_F4_Spc [entry $vlist.e_f4spc -width 21 -textvariable matsolid::F4_Spc]
		}
		"Tdyn3D" {
		    set e_F1_Spc [entry $vlist.e_f1spc -width 21 -textvariable matsolid::F1_Spc]
		    set e_F2xx_Spc [entry $mxfr.e_f2xxspc -width 6 -textvariable matsolid::F2xx_Spc]
		    set e_F2xy_Spc [entry $mxfr.e_f2xyspc -width 6 -textvariable matsolid::F2xy_Spc]
		    set e_F2xz_Spc [entry $mxfr.e_f2xzspc -width 6 -textvariable matsolid::F2xz_Spc]
		    set e_F2yx_Spc [entry $mxfr.e_f2yxspc -width 6 -textvariable matsolid::F2xy_Spc]
		    set e_F2yy_Spc [entry $mxfr.e_f2yyspc -width 6 -textvariable matsolid::F2yy_Spc]
		    set e_F2yz_Spc [entry $mxfr.e_f2yzspc -width 6 -textvariable matsolid::F2yz_Spc]
		    set e_F2zx_Spc [entry $mxfr.e_f2zxspc -width 6 -textvariable matsolid::F2xz_Spc]
		    set e_F2zy_Spc [entry $mxfr.e_f2zyspc -width 6 -textvariable matsolid::F2yz_Spc]
		    set e_F2zz_Spc [entry $mxfr.e_f2zzspc -width 6 -textvariable matsolid::F2zz_Spc]
		    set e_F3_Spc [entry $vlist.e_f3spc -width 21 -textvariable matsolid::F3_Spc]
		    set e_F4_Spc [entry $vlist.e_f4spc -width 21 -textvariable matsolid::F4_Spc]
		}
	    }
	    set c_F2_Units [ComboBox $vlist.c_f2units -values $units_F2_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F2_Units -takefocus 1]
	    set c_F3_Units [ComboBox $vlist.c_f3units -values $units_F3_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F3_Units -takefocus 1]
	    set c_F4_Units [ComboBox $vlist.c_f4units -values $units_F4_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F4_Units -takefocus 1]

	    # GRIDS
	    grid $l_F1_Spc -row 1 -column 0 -sticky en -pady 1
	    grid $l_F2_Spc -row 2 -column 0 -sticky en -pady 1
	    grid $l_F3_Spc -row 3 -column 0 -sticky en -pady 1
	    grid $l_F4_Spc -row 4 -column 0 -sticky en -pady 1
	    grid $e_F1_Spc -row 1 -column 1 -sticky wn -pady 1
	    grid $mxfr -row 2 -column 1 -columnspan 2 -sticky wn -pady 1
	    switch $ProgramName {
		"Tdyn2D" {
		    grid $e_F2xx_Spc -row 0 -column 0 -sticky wn -pady 1
		    grid $e_F2xy_Spc -row 0 -column 1 -sticky wn -pady 1
		    grid $e_F2yx_Spc -row 1 -column 0 -sticky wn -pady 1
		    grid $e_F2yy_Spc -row 1 -column 1 -sticky wn -pady 1
		    grid $e_F3_Spc -row 3 -column 1 -sticky wn -pady 1
		    grid $e_F4_Spc -row 4 -column 1 -sticky wn -pady 1
		}
		"Tdyn3D" {
		    grid $e_F2xx_Spc -row 0 -column 0 -sticky wn -pady 1
		    grid $e_F2xy_Spc -row 0 -column 1 -sticky wn -pady 1
		    grid $e_F2xz_Spc -row 0 -column 2 -sticky wn -pady 1
		    grid $e_F2yx_Spc -row 1 -column 0 -sticky wn -pady 1
		    grid $e_F2yy_Spc -row 1 -column 1 -sticky wn -pady 1
		    grid $e_F2yz_Spc -row 1 -column 2 -sticky wn -pady 1
		    grid $e_F2zx_Spc -row 2 -column 0 -sticky wn -pady 1
		    grid $e_F2zy_Spc -row 2 -column 1 -sticky wn -pady 1
		    grid $e_F2zz_Spc -row 2 -column 2 -sticky wn -pady 1
		    grid $e_F3_Spc -row 3 -column 1 -sticky wn -pady 1
		    grid $e_F4_Spc -row 4 -column 1 -sticky wn -pady 1
		}
	    }

	    grid $c_F2_Units -row 2 -column 2 -sticky wn -padx 2
	    grid $c_F3_Units -row 3 -column 2 -sticky wn -padx 2
	    grid $c_F4_Units -row 4 -column 2 -sticky wn -padx 2

	    set SpeciesUpdateCmd ";
		matsolid::valsavespecie $nlist \"actual\" \"$type\";
		matsolid::displayspeciesnames $nlist \"$type\";
		"
	}
	  "ursolver" {
	    set l_F1_Phi [Label $vlist.l_f1phi -text "f \u00b9 "]
	    set l_F2_Phi [Label $vlist.l_f2phi -text "f \u00b2 "]
	    set l_F3_Phi [Label $vlist.l_f3phi -text "f \u00b3 "]
	    set l_F4_Phi [Label $vlist.l_f4phi -text "f \u2074 "]

	    set mxfr [frame $vlist.mxfr -relief flat]
	    switch $ProgramName {
		"Tdyn2D" {
		    set e_F1_Phi [entry $vlist.e_f1spc -width 21 -textvariable matsolid::F1_Phi]
		    set e_F2xx_Phi [entry $mxfr.e_f2xxspc -width 6 -textvariable matsolid::F2xx_Phi]
		    set e_F2xy_Phi [entry $mxfr.e_f2xyspc -width 6 -textvariable matsolid::F2xy_Phi]
		    set e_F2yx_Phi [entry $mxfr.e_f2yxspc -width 6 -textvariable matsolid::F2xy_Phi]
		    set e_F2yy_Phi [entry $mxfr.e_f2yyspc -width 6 -textvariable matsolid::F2yy_Phi]
		    set e_F3_Phi [entry $vlist.e_f3spc -width 21 -textvariable matsolid::F3_Phi]
		    set e_F4_Phi [entry $vlist.e_f4spc -width 21 -textvariable matsolid::F4_Phi]
		}
		"Tdyn3D" {
		    set e_F1_Phi [entry $vlist.e_f1spc -width 21 -textvariable matsolid::F1_Phi]
		    set e_F2xx_Phi [entry $mxfr.e_f2xxspc -width 6 -textvariable matsolid::F2xx_Phi]
		    set e_F2xy_Phi [entry $mxfr.e_f2xyspc -width 6 -textvariable matsolid::F2xy_Phi]
		    set e_F2xz_Phi [entry $mxfr.e_f2xzspc -width 6 -textvariable matsolid::F2xz_Phi]
		    set e_F2yx_Phi [entry $mxfr.e_f2yxspc -width 6 -textvariable matsolid::F2xy_Phi]
		    set e_F2yy_Phi [entry $mxfr.e_f2yyspc -width 6 -textvariable matsolid::F2yy_Phi]
		    set e_F2yz_Phi [entry $mxfr.e_f2yzspc -width 6 -textvariable matsolid::F2yz_Phi]
		    set e_F2zx_Phi [entry $mxfr.e_f2zxspc -width 6 -textvariable matsolid::F2xz_Phi]
		    set e_F2zy_Phi [entry $mxfr.e_f2zyspc -width 6 -textvariable matsolid::F2yz_Phi]
		    set e_F2zz_Phi [entry $mxfr.e_f2zzspc -width 6 -textvariable matsolid::F2zz_Phi]
		    set e_F3_Phi [entry $vlist.e_f3spc -width 21 -textvariable matsolid::F3_Phi]
		    set e_F4_Phi [entry $vlist.e_f4spc -width 21 -textvariable matsolid::F4_Phi]
		}
	    }
	    set c_F1_Phi_Units [ComboBox $vlist.c_f1phiunits -values $units_F1_Phi_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F1_Phi_Units -takefocus 1]
	    set c_F2_Phi_Units [ComboBox $vlist.c_f2phiunits -values $units_F2_Phi_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F2_Phi_Units -takefocus 1]
	    set c_F3_Phi_Units [ComboBox $vlist.c_f3phiunits -values $units_F3_Phi_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F3_Phi_Units -takefocus 1]
	    set c_F4_Phi_Units [ComboBox $vlist.c_f4phiunits -values $units_F4_Phi_Units -editable $edit_units \
		    -width 7 -textvariable matsolid::F4_Phi_Units -takefocus 1]

	    # GRIDS
	    grid $l_F1_Phi -row 1 -column 0 -sticky en -pady 1
	    grid $l_F2_Phi -row 2 -column 0 -sticky en -pady 1
	    grid $l_F3_Phi -row 3 -column 0 -sticky en -pady 1
	    grid $l_F4_Phi -row 4 -column 0 -sticky en -pady 1
	    grid $e_F1_Phi -row 1 -column 1 -sticky wn -pady 1
	    grid $mxfr -row 2 -column 1 -columnspan 2 -sticky wn -pady 1
	    switch $ProgramName {
		"Tdyn2D" {
		    grid $e_F2xx_Phi -row 0 -column 0 -sticky wn -pady 1
		    grid $e_F2xy_Phi -row 0 -column 1 -sticky wn -pady 1
		    grid $e_F2yx_Phi -row 1 -column 0 -sticky wn -pady 1
		    grid $e_F2yy_Phi -row 1 -column 1 -sticky wn -pady 1
		    grid $e_F3_Phi -row 3 -column 1 -sticky wn -pady 1
		    grid $e_F4_Phi -row 4 -column 1 -sticky wn -pady 1
		}
		"Tdyn3D" {
		    grid $e_F2xx_Phi -row 0 -column 0 -sticky wn -pady 1
		    grid $e_F2xy_Phi -row 0 -column 1 -sticky wn -pady 1
		    grid $e_F2xz_Phi -row 0 -column 2 -sticky wn -pady 1
		    grid $e_F2yx_Phi -row 1 -column 0 -sticky wn -pady 1
		    grid $e_F2yy_Phi -row 1 -column 1 -sticky wn -pady 1
		    grid $e_F2yz_Phi -row 1 -column 2 -sticky wn -pady 1
		    grid $e_F2zx_Phi -row 2 -column 0 -sticky wn -pady 1
		    grid $e_F2zy_Phi -row 2 -column 1 -sticky wn -pady 1
		    grid $e_F2zz_Phi -row 2 -column 2 -sticky wn -pady 1
		    grid $e_F3_Phi -row 3 -column 1 -sticky wn -pady 1
		    grid $e_F4_Phi -row 4 -column 1 -sticky wn -pady 1
		}
	    }

	    grid $c_F1_Phi_Units -row 1 -column 2 -sticky wn -padx 2
	    grid $c_F2_Phi_Units -row 2 -column 2 -sticky wn -padx 2
	    grid $c_F3_Phi_Units -row 3 -column 2 -sticky wn -padx 2
	    grid $c_F4_Phi_Units -row 4 -column 2 -sticky wn -padx 2

	    set URVariablesUpdateCmd ";
		matsolid::valsavespecie $nlist \"actual\" \"$type\";
		matsolid::displayspeciesnames $nlist \"$type\";
		"
	 }
    }

    # BINDS
    for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	set tvar [lindex [lindex $type_Valnames $i] 1]
	set tname [lindex [lindex $type_Valnames $i] 0]
	set evar e_$tvar

	# BINDS
	bind [eval set temp $$evar] <Return> ";
	matsolid::valsavespecie $nlist \"actual\" \"$type\";
	matsolid::displayspeciesnames $nlist \"$type\";
	"
	bind [eval set temp $$evar] <ButtonPress-3> "+matsolid::popupmenu $vlist matsolid::$tvar \"$tname\" \"\" 1 %X %Y"
    }

    grid columnconf $vlist 3 -weight 1

    return 1
}

proc matsolid::displayspecievalues { nlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname

    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    if { [lindex [$nlist curselection] 0] > -1 } {
	set sel [lindex [$nlist curselection] 0]
    } else {
	matsolid::updatespcselection $nlist 0 $type
	set sel [lindex [$nlist curselection] 0]
    }

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set type_Values $Species_Values
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set type_UValues $Species_UValues
	    set vartype_name "sname"
	    set vartype_lastsel "spcnlastsel"
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set type_Values $URVariables_Values
	    set type_Valnames $URVariables_Valnames
	    set type_UValues $URVariables_UValues
	    set type_UValnames $URVariables_UValnames
	    set vartype_name "ursname"
	    set vartype_lastsel "ursnlastsel"
	}
    }

    # Display Specie / Variable name and set last selection
    set $vartype_name [lindex $type_Names $sel]
    set $vartype_lastsel $sel

    set currvals [lindex $type_Values $sel]
    for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	# Entries Values
	set tvar [lindex [lindex $type_Valnames $i] 1]
	set $tvar [lindex $currvals $i]
    }
    set currvals [lindex $type_UValues $sel]
    for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	# Combo boxes Units values
	set tvar [lindex [lindex $type_UValnames $i] 1]
	set $tvar [lindex $currvals $i]
    }

    return 1
}

proc matsolid::displayspeciesnames { nlist type } {

    variable Species_Names
    variable URVariables_Names

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	}
    }

    if { [lindex [$nlist curselection] 0] > -1 } {
	set savesel [lindex [$nlist curselection] 0]
    } else {
	set savesel ""
    }
    set p 0
    $nlist delete 0 end
    foreach n $type_Names {
	$nlist insert $p $n
	incr p
    }

    if { $savesel > -1 } {
	$nlist selection clear 0 end
	$nlist selection set $savesel $savesel
    }

    return 1
}

proc matsolid::updatespcselection { nlist setsel type } {

    variable Species_Names
    variable URVariables_Names
    variable spcnlastsel
    variable ursnlastsel
    variable sname
    variable ursname

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	}
    }

    if { $setsel < 0 || $setsel > [expr [llength $type_Names] -1] } {
	set setsel "end"
    }

    $nlist selection clear 0 end
    $nlist selection set $setsel $setsel

    switch $type {
	"advect" {
	    set spcnlastsel $setsel
	    set sname [lindex $type_Names $setsel]
	}
	"ursolver" {
	    set ursnlastsel $setsel
	    set ursname [lindex $type_Names $setsel]
	}
    }

    return 1
}

proc matsolid::valsavespecie { nlist sel type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    if { $sel == "actual" && [string length [$nlist curselection]] > 0 } {
	set sel [lindex [$nlist curselection] 0]
    } else {
	switch $type {
	    "advect" {
		set sel $spcnlastsel
	    }
	    "ursolver" {
		set sel $ursnlastsel
	    }
	}
    }

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set vartype_Names "Species_Names"
	    set type_Values $Species_Values
	    set vartype_Values "Species_Values"
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set type_UValues $Species_UValues
	    set vartype_UValues "Species_UValues"
	    set type_name $sname
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set vartype_Names "URVariables_Names"
	    set type_Values $URVariables_Values
	    set vartype_Values "URVariables_Values"
	    set type_Valnames $URVariables_Valnames
	    set type_UValnames $URVariables_UValnames
	    set type_UValues $URVariables_UValues
	    set vartype_UValues "URVariables_UValues"
	    set type_name $ursname
	}
    }

    if { $sel > -1 && !($sel > [expr [llength $type_Values] -1]) } {

	# Saving Specie / UR Variable Name
	set $vartype_Names [lreplace $type_Names $sel $sel [string trim [DWSpace2Under $type_name]]]

	# Saving Specie / UR Variable values
	set newvals ""
	for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	    set tvar [lindex [lindex $type_Valnames $i] 1]
	    set type [lindex [lindex $type_Valnames $i] 2]
	    lappend newvals [TUI_cleanvalue [eval set temp $$tvar] $type]
	    #if { [lindex [lindex $type_Valnames $i] 2] == "func" } {
	    #    lappend newvals [TdynCleanString [eval set temp $$tvar]]
	    #} else {
	    #    lappend newvals [DWSpace2Under [eval set temp $$tvar]]
	    #}
	}
	set $vartype_Values [lreplace $type_Values $sel $sel $newvals]
	# Saving Specie / UR Variable Units values
	set newvals ""
	for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	    set tvar [lindex [lindex $type_UValnames $i] 1]
	    lappend newvals [TUI_cleanvalue [eval set temp $$tvar] units]
	    #regsub -all { } [eval set temp $$tvar] {} ttv
	    #lappend newvals $ttv
	}
	set $vartype_UValues [lreplace $type_UValues $sel $sel $newvals]
    }

    return 1
}

proc matsolid::addspecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable spcnlastsel
    variable sname
    variable Species_Rebuild
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable ursnlastsel
    variable ursname
    variable URVariables_Rebuild

    matsolid::valsavespecie $nlist "last" $type

    set sr [$nlist curselection]

    switch $type {
	"advect" {
	    # Check if any species is selected
	    if { $sr == "" } {
		set sr $spcnlastsel
	    }
	    # Update Species Values
	    lappend Species_Values [lindex $Species_Values $sr]
	    # Update Species Units Values
	    lappend Species_UValues [lindex $Species_UValues $sr]
	    
	    # Update Species Names
	    set nn "Species"
	    set ns [expr $sr + 1]
	    lappend Species_Rebuild "a" $sr
	    set nt $nn$ns
	    while { [lsearch $Species_Names $nt]!=-1 } {
		set ns [expr $ns + 1]
		set nt $nn$ns
	    }
	    set nt $nn$ns
	    lappend Species_Names $nt
	    $nlist insert end $nt
	}
	"ursolver" {
	    # Check if any variable is selected
	    if { $sr == "" } {
		set sr $ursnlastsel
	    }
	    # Update URSolver Variables Values
	    lappend URVariables_Values [lindex $URVariables_Values $sr]
	    # Update URSolver Units Values
	    lappend URVariables_UValues [lindex $URVariables_UValues $sr]
	    
	    # Update URSolver Variables Names
	    set nn "Variable"
	    set ns [expr $sr + 1]
	    lappend URVariables_Rebuild "a" $sr
	    set nt $nn$ns
	    while { [lsearch $URVariables_Names $nt]!=-1 } {
		set ns [expr $ns + 1]
		set nt $nn$ns
	    }
	    set nt $nn$ns
	    lappend URVariables_Names $nt
	    $nlist insert end $nt
	}        
    }
    matsolid::updatespcselection $nlist "end" $type
    matsolid::displayspecievalues $nlist $type
}

proc matsolid::removespecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable spcnlastsel
    variable sname
    variable spcdelerrwarn
    variable spcdellasterrwarn
    variable Species_Rebuild
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable ursnlastsel
    variable ursname
    variable ursdelerrwarn
    variable ursdellasterrwarn
    variable URVariables_Rebuild

    set sr [$nlist curselection]

	switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set vartype_Names Species_Names
	    set type_Values $Species_Values
	    set vartype_Values Species_Values
	    set type_UValues $Species_UValues
	    set vartype_UValues Species_UValues
	    set displaykind "Specie"
	    set lastcheckvar $spcdellasterrwarn
	    set var_lastcheckvar spcdellasterrwarn
	    set noselvar $spcdelerrwarn
	    set var_noselvar spcdelerrwarn
	    set type_Rebuild Species_Rebuild
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set vartype_Names URVariables_Names
	    set type_Values $URVariables_Values
	    set vartype_Values URVariables_Values
	    set type_UValues $URVariables_UValues
	    set vartype_UValues URVariables_UValues
	    set displaykind "Variable"
	    set lastcheckvar $ursdellasterrwarn
	    set var_lastcheckvar ursdellasterrwarn
	    set noselvar $ursdelerrwarn
	    set var_noselvar ursdelerrwarn
	    set type_Rebuild URVariables_Rebuild
	}
    }

    if { $sr > -1 && $sr < [llength $type_Names] } {
	# Check if the deleted Specie/Variable is not the last one
	if { [llength $type_Names] < 2 } {
	    if { $lastcheckvar == 1 } {
		return
	    } else {
		WarnWin "[= {There must be at least one %s defined} $displaykind]."
		set $var_lastcheckvar 1
	    }
	} else {
	    lappend $type_Rebuild "r" $sr
	    # Updating Species/URVariables Names
	    set $vartype_Names [lreplace $type_Names [lindex $sr 0] [lindex $sr end]]
	    # Updating Species/URVariables Values
	    set $vartype_Values [lreplace $type_Values [lindex $sr 0] [lindex $sr end]]
	    # Updating Species/URVariables Units Values
	    set $vartype_UValues [lreplace $type_UValues [lindex $sr 0] [lindex $sr end]]

	    matsolid::displayspeciesnames $nlist $type
	    matsolid::updatespcselection $nlist "end" $type
	}
    } else {
	if { $noselvar == 1 } {
	    return
	} else {
	    set temp [= {Please select a }]
	    append temp $displaykind "[= {(s) you wish to remove}]."
	    WarnWin $temp
	    set $noselvar 1
	}
    }

    matsolid::displayspecievalues $nlist $type
}

################################################################################
#
#   PopUp Menus
#
################################################################################

proc matsolid::popupmenu { parent {field ""} {fieldname ""} tags content x y } {

    variable predefined_ff

    catch { 
	destroy $parent.menu
    }

    switch $content {
	1 {
	    set menu [menu $parent.menu -type normal]
	      if { [string length $field] > 0 } {
		  set fval [set $field]
	      }
	    $menu add command -label [= {Define Function}] \
		    -command "FEditor::init $parent \"$predefined_ff\" $field [list $fieldname] [list $fval] \"\" \"\""
	    tk_popup $menu $x $y
	}
    }
}



################################################################################
#
#   GiD Values Manipulation - GET Values, TSAVE and DUMP Values
#   - GETVALUES reads GiD values and puts them to the variables
#   - CHECKCLOSE goes through all variables and checks for changes. If changes 
#     are found, asks the user SAVE YES or NO. If YES, runs DUMP procedure.
#     This procedure also checks the temporal space of Initial Conditions.
#   - TICSAVE saves values from Initial Conditions entries to temporal variables
#     from where in case of accept saving it saves the values tom GiD memory.
#   - DUMP saves values of entries to GiD memory and in case of initial condition
#
################################################################################

proc matsolid::getvalues { } {

    global ProgramName

    variable show_advect
    variable show_ursolver

    variable Solid_Questions
    variable Material_Name
    variable ValList

    variable Species_Names
    variable Species_Values
    variable Species_UValues
    variable Species_Valnames
    variable Species_UValnames

    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Valnames
    variable URVariables_UValnames

    # Read all materials values
    set ValList [.central.s info materials $Material_Name]

    # Index of values started from 1 and not from 0 because the 
    # first values is number of questions. 
    set i 0
    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	variable $itt
	set pos [lsearch -regexp $ValList [eval set nothing ^${itt}(\$|\#CB|\:)]]
	if { $pos != -1 } {
	    #set tv [join [lindex $ValList [expr $pos + 1]]]
	    set tv "[lindex $ValList [expr $pos + 1]]"
	    if { [lindex $item 1] == "unit" } {
		set tv [TUI_formatunits Nums2Unicode $tv 0]
	    }
	    set $itt $tv
	}
	incr i
    }

    if { $show_advect == 1 } {
	# Read Species Names from Problem Data
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]]
	set Species_Names [lreplace $temp 0 1]
	set Species_Values [TUI_formatspeciesvalues String2List $Species_Valnames $Species_Properties]
	set Species_UValues [TUI_formatspeciesvalues String2List $Species_UValnames $Spc_Units]

	# Format Species Units Lists for Mat File Format
	set ltemp ""
	  foreach litem $Species_UValues {
	      set temp ""
	      foreach item $litem {
		lappend temp [TUI_formatunits Nums2Unicode $item 1]
	      }
	      lappend ltemp [string trim $temp]
	  }
	  set Species_UValues $ltemp
    }

    if { $show_ursolver == 1 } {
	# Read URSolver Variables Names from Problem Data
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]]
	set URVariables_Names [lreplace $temp 0 1]
	set URVariables_Values [TUI_formatspeciesvalues String2List $URVariables_Valnames $Variables_Properties]
	set URVariables_UValues [TUI_formatspeciesvalues String2List $URVariables_UValnames $URV_Units]

	# Format URVariables Units Lists for Mat File Format
	set ltemp ""
	  foreach litem $URVariables_UValues {
	      set temp ""
	      foreach item $litem {
		lappend temp [TUI_formatunits Nums2Unicode $item 1]
	      }
	      lappend ltemp [string trim $temp]
	  }
	  set URVariables_UValues $ltemp
    }

    return 1
}

proc matsolid::checkclose { win {dumpit 1} {askdump 1} {close 1} } {

    global tkPriv tcl_platform

    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver

    variable Material_Name
    variable ValList

    variable Solid_Questions

    variable Species_Names
    variable Species_Values
    variable Species_UValues
    variable Species_Valnames
    variable Species_UValnames
    variable snlist

    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Valnames
    variable URVariables_UValnames
    variable ursnlist

    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	variable $itt
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Unicode2Nums [eval set nothing $$itt] 1]
	}
	#if { [lindex $item 1] == "func" } {
	#    regsub -all { } [eval set nothing $$itt] {} $itt
	#}
    }

    if { $show_advect == 1 } {
	set Species_UValues_Temp $Species_UValues
	matsolid::valsavespecie $snlist "actual" "advect"
	# Format Species Units Lists for Mat File Format
	set ltemp ""
	  foreach litem $Species_UValues {
	      set temp ""
	      foreach item $litem {
		lappend temp [TUI_formatunits Unicode2Nums $item 1]
	      }
	      lappend ltemp [string trim $temp]
	  }
	  set Species_UValues $ltemp

	set Species_Properties [TUI_formatspeciesvalues List2String $Species_Valnames $Species_Values]
	set Spc_Units [TUI_formatspeciesvalues List2String $Species_UValnames $Species_UValues]
    }

    if { $show_ursolver == 1 } {
	set URVariables_UValues_Temp $URVariables_UValues
	matsolid::valsavespecie $ursnlist "actual" "ursolver"
	# Format URVariables Units Lists for Mat File Format
	set ltemp ""
	  foreach litem $URVariables_UValues {
	      set temp ""
	      foreach item $litem {
		lappend temp [TUI_formatunits Unicode2Nums $item 1]
	      }
	      lappend ltemp [string trim $temp]
	  }
	  set URVariables_UValues $ltemp

	set Variables_Properties [TUI_formatspeciesvalues List2String $URVariables_Valnames $URVariables_Values]
	set URV_Units [TUI_formatspeciesvalues List2String $URVariables_UValnames $URVariables_UValues]
    }

    set valueschange 0
    set wtsave 0
    set vi 1
    set usr_err ""
    set num_usr_err 0
    set uerrpg_ran 0
    set uerrpg_htr 0
    set uerrpg_adv 0
    set uerrpg_urs 0

    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	set $itt [string trim [eval set nothing $$itt]]
	eval set cv $$itt
	set pos [lsearch -regexp $ValList [eval set nothing ^${itt}(\$|\#CB|\:)]]
	if { $pos != -1 } {
	    #set tv [join [lindex $ValList [expr $pos + 1]]]
	    set tv [lindex $ValList [expr $pos + 1]]
	    if { ![TUI_comparelists $cv $tv] } {
		set valueschange 1
		if { $itt == "Species_Properties" } {
		    set spc_pos 0
		    foreach item $Species_Values {
		        set tspn [lindex $Species_Names $spc_pos]
		        incr spc_pos
		        set valnamepos 0
		        foreach itt $item {
		            set valname [lindex $Species_Valnames $valnamepos]
		            incr valnamepos
		            if { [lindex $valname 3] == 1 } {
		                set errcheck [TUI_valuecheck $itt [lindex $valname 2] 0 [lindex $valname 4]]
		            } else {
		                set errcheck "ok"
		            }
		            if { $errcheck != "ok"} {
		                incr num_usr_err
		                if { $uerrpg_adv != 1 } {
		                    append usr_err "\n[= {ADVECT PAGE}]\n"
		                    set uerrpg_adv 1
		                }
		                append usr_err "$num_usr_err. " "$tspn; " [lindex $valname 0] ": $errcheck\n"
		            }
		        }
		    }
		} elseif { $itt == "Variables_Properties" } {
		    set spc_pos 0
		    foreach item $URVariables_Values {
		        set tspn [lindex $URVariables_Names $spc_pos]
		        incr spc_pos
		        set valnamepos 0
		        foreach itt $item {
		            set valname [lindex $URVariables_Valnames $valnamepos]
		            incr valnamepos
		            if { [lindex $valname 3] == 1 } {
		                set errcheck [TUI_valuecheck $itt [lindex $valname 2] 0  [lindex $valname 4]]
		            } else {
		                set errcheck "ok"
		            }
		            if { $errcheck != "ok"} {
		                incr num_usr_err
		                if { $uerrpg_urs != 1 } {
		                    append usr_err "\n[= {URSOLVER PAGE}]\n"
		                    set uerrpg_urs 1
		                }
		                append usr_err "$num_usr_err. " "$tspn; " [lindex $valname 0] ": $errcheck\n"
		            }
		        }
		    }
		} else {
		    if { [lindex $item 2] == 1 } {
		        set errcheck [TUI_valuecheck [eval set nothing $$itt] [lindex $item 1] 0 [lindex $item 4]]
		    } else {
		        set errcheck "ok"
		    }
		    if { $errcheck != "ok" && [lindex $item 2] == 1} {
		        incr num_usr_err
		        switch "[lindex $item 3]" {
		            "ransol" {
		                if { $uerrpg_ran != 1 } {
		                    append usr_err "\n[= {RANSOL PAGE}]\n"
		                    set uerrpg_ran 1
		                }
		            }
		            "heatrans" {
		                if { $uerrpg_htr != 1 } {
		                    append usr_err "\n[= {HEATRANS PAGE}]\n"
		                    set uerrpg_htr 1
		                }
		            }
		        }
		        append usr_err "$num_usr_err. " [DWUnder2Space $itt] ": $errcheck\n"
		    }
		}
	    }
	}
	incr vi
    }

    if { $show_advect == 1 } {
	# Check also Species Names
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]]
	set temp [lreplace $temp 0 1]
	if { $temp != $Species_Names } {
	    set valueschange 1
	    set warnedspcnames ""
	    foreach item $Species_Names {
		if { [llength [lsearch -all $Species_Names $item]] > 1 && [lsearch -exact $warnedspcnames $item] == -1} {
		    lappend warnedspcnames $item
		    incr num_usr_err
		    if { $uerrpg_adv != 1 } {
		        append usr_err "\n[= {ADVECT PAGE}]\n"
		        set uerrpg_adv 1
		    }
		    append usr_err "[= {%s. Specie Name: '%s' is included more than once in the Species List} $num_usr_err $item].\n"
		    append usr_err "[= {Species have to have unique names}].\n"
		}
		set errcheck [TUI_valuecheck $item string 0]
		if { $errcheck != "ok"} {
		    incr num_usr_err
		    if { $uerrpg_adv != 1 } {
		        append usr_err "\n[= {ADVECT PAGE}]\n"
		        set uerrpg_adv 1
		    }
		    append usr_err "[= {%s. Specie Name: '%s' is not correct. } $num_usr_err $item].\n"
		    append usr_err "[= {Species names can only include alphabetical characters, numbers and underscore}].\n"
		}
	    }
	}
    }

    if { $show_ursolver == 1 } {
	# Check also URSolver Variables Names
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]]
	set temp [lreplace $temp 0 1]
	if { $temp != $URVariables_Names } {
	    set valueschange 1
	    set warnedurvnames ""
	    foreach item $URVariables_Names {
		if { [llength [lsearch -all $URVariables_Names $item]] > 1 && [lsearch -exact $warnedurvnames $item] == -1} {
		    lappend warnedurvnames $item
		    incr num_usr_err
		    if { $uerrpg_ran != 1 } {
		        append usr_err "\n[= {URSOLVER PAGE}]\n"
		        set uerrpg_urs 1
		    }
		    append usr_err "[= {%s. URSolver Variable Name: '%s' is included more than once in the Variable List} $num_usr_err $item].\n"
		    append usr_err "[= {Variables have to have unique names}].\n"
		}
		set errcheck [TUI_valuecheck $item string 0]
		if { $errcheck != "ok"} {
		    incr num_usr_err
		    if { $uerrpg_urs != 1 } {
		        append usr_err "\n[= {URSOLVER PAGE}]\n"
		        set uerrpg_urs 1
		    }
		    append usr_err "[= {%s. URSolver Variable Name: '%s' is not correct. } $num_usr_err $item].\n"
		    append usr_err "[= {URSolver Variables names can only include alphabetical characters, numbers and underscore}]\n"
		}
	    }
	}
    }

    # Units are formated back to unicode so that they are not seen 
    # in MAT File format in fields while asking if changes shouldbe saved. 
    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Nums2Unicode [eval set nothing $$itt] 1]
	}
    }
    if { $show_advect == 1 } {  
	set Species_UValues $Species_UValues_Temp
    }
    if { $show_ursolver == 1 } { 
	set URVariables_UValues $URVariables_UValues_Temp
    }

    if { $valueschange == 1 } {
	if { $askdump == 0 } {
	    set answer 0
	} else {
	    set answer [tk_dialogRAM $win [= {Material: '%s'} $Material_Name] \
		    "[= {Material data has changed}].\n[= {Do you want to save changes?}]" question 0 Yes No]
	}

	if { $answer == 0 && $dumpit == 1} {
	    set wtsave 1
	    if { $num_usr_err > 0 } {
		if { $num_usr_err > 1 } {
		    set err_display [= {The following %s error} $num_usr_err]
		    append err_display "[= {(s) were found in the data input:}]\n"
		} else {
		    set err_display "[= {The following error was found in the data input:}]\n"
		}
		append err_display $usr_err
		WarnWin $err_display
		set close 0
	    } else {
		matsolid::dump
	    }
	}
    }

    if { $close == 1 } {
	destroy $win
	return $valueschange
    } else {
	if { $num_usr_err == 0 } {
	    return $valueschange
	} else {
	    if { $wtsave == 1 } {
		return "dataerror"
	    } else {
		return "nosave.dataerror"
	    }
	}
    }

}

proc matsolid::dump { } {

    variable show_advect
    variable show_ursolver

    variable Material_Name
    variable ValList

    variable Species_Names
    variable URVariables_Names

    variable Species_Rebuild
    variable URVariables_Rebuild

    variable Solid_Questions
    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	variable $itt
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Unicode2Nums [eval set nothing $$itt] 1]
	}
    }

    # Replacing values in ValList
    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	variable $itt
	set pos [lsearch -regexp $ValList [eval set nothing ^${itt}(\$|\#CB|\:)]]
	if { $pos != -1 } {
	    set pos [expr $pos + 1]
	    set ValList [lreplace $ValList $pos $pos [eval set nothing $$itt]]
	}
    }
    # Saving values in GID Data
    #TUI_setnewmatvalues $Material_Name [string trim [matsolid::getcurrentvalset]]
    matsolid::setnewmatvalues $Material_Name

    if { $show_advect == 1 } {
	TUI_savespeciesnames "Species_Names:(SpecieName)" $Species_Names
	TUI_synchrospecies "advect" $Species_Rebuild $Material_Name
    }
    if { $show_ursolver == 1 } {
	TUI_savespeciesnames "URVariables_Names:(VariableName)" $URVariables_Names
	TUI_synchrospecies "ursolver" $URVariables_Rebuild $Material_Name
    }

    # Units are formated back to unicode.
    foreach item $Solid_Questions {
	set itt [lindex $item 0]
	if { [lindex $item 1] == "unit" } {
	    set $itt [TUI_formatunits Nums2Unicode [eval set nothing $$itt] 1]
	}
    }
    # Actualizamos la variable de control de cambios en GiD
    GIDModifiedFileFlag Set
    return 1
}

proc matsolid::setnewmatvalues { material } {
    variable ValList

    set vals [lrange $ValList 1 end]
    set justvals ""
    foreach {q v} $vals {
	lappend justvals $v
    }
    GiD_ModifyData materials $material $justvals
}

proc matsolid::getcurrentvalset { } {
    
    variable ValList

    set vals [lrange $ValList 1 end]
    set JVL ""
    foreach {q v} $vals {
	append JVL $v " "
    }

    return $JVL
}
