namespace eval materials {

    global ProgramName
}

namespace eval fluidmaterials {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name
}

namespace eval solidmaterials {
    
    variable Materials_List
    variable Material_Name
    variable Last_Material_Name
}

namespace eval fluidboundaries {
    
    variable Materials_List
    variable Materials_Type
    variable Material_Name
    variable Last_Material_Name
    # Tdyn settings
    variable show_naval [IsVarOnTdynDefaults show_naval 1]
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
}

namespace eval solidboundaries {
    
    variable Materials_List
    variable Materials_Type
    variable Material_Name
    variable Last_Material_Name
    # Tdyn settings
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
}

proc materials::init { book {title ""} {field ""} {value ""} } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set w ".gid.fluidmat"
	    if { $title == "" } {
		set wtitle [= {Fluid Material Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynFluidMatWindowGeom
	    set fluidmaterials::Materials_List [TIU_getmaterialnames $book $field $value]
	    set fluidmaterials::Material_Name [lindex $fluidmaterials::Materials_List 0]
	    set fluidmaterials::Last_Material_Name $fluidmaterials::Material_Name
	}
	"MATERIALS_(SOLID)" {
	    set w ".gid.solidmat"
	    if { $title == "" } {
		set wtitle [= {Solid Material Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynSolidMatWindowGeom
	    set solidmaterials::Materials_List [TIU_getmaterialnames $book $field $value]
	    set solidmaterials::Material_Name [lindex $solidmaterials::Materials_List 0]
	    set solidmaterials::Last_Material_Name $solidmaterials::Material_Name
	}
	"BOUNDARIES_(FLUID)" {
	    set w ".gid.fluidbond"
	    if { $title == "" } {
		set wtitle [= {Fluid Boundaries Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynFluidBondWindowGeom
	    set fluidboundaries::Materials_List [TIU_getmaterialnames $book $field $value]
	    set fluidboundaries::Material_Name [lindex $fluidboundaries::Materials_List 0]
	    set fluidboundaries::Last_Material_Name $fluidboundaries::Material_Name
	    set fluidboundaries::Materials_Type ""
	    foreach imat $fluidboundaries::Materials_List {
		set matinfo [.central.s info materials $imat]
		lappend fluidboundaries::Materials_Type [lindex $matinfo 2]
	    }    
	}
	"BOUNDARIES_(SOLID)" {
	    set w ".gid.solidbond"
	    if { $title == "" } {
		set wtitle [= {Solid Boundaries Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynSolidBondWindowGeom
	    set solidboundaries::Materials_List [TIU_getmaterialnames $book $field $value]
	    set solidboundaries::Material_Name [lindex $solidboundaries::Materials_List 0]
	    set solidboundaries::Last_Material_Name $solidboundaries::Material_Name
	    set solidboundaries::Materials_Type ""
	    foreach imat $solidboundaries::Materials_List {
		set matinfo [.central.s info materials $imat]
		lappend solidboundaries::Materials_Type [lindex $matinfo 2]
	    }    
	}
	"CONTACTS" {
	    set w ".gid.contacts"
	    if { $title == "" } {
		set wtitle [= {Fluid-Solid Contact Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynContactWindowGeom
	    set solidboundaries::Materials_List [TIU_getmaterialnames $book $field $value]
	    set solidboundaries::Material_Name [lindex $solidboundaries::Materials_List 0]
	    set solidboundaries::Last_Material_Name $solidboundaries::Material_Name
	    set solidboundaries::Materials_Type ""
	    foreach imat $solidboundaries::Materials_List {
		set matinfo [.central.s info materials $imat]
		lappend solidboundaries::Materials_Type [lindex $matinfo 2]
	    }    
	}
	default {
	    return
	}
    }

    # Closing other windows opened
    set owc [TUI_closeotherwindows $w]

    if { $owc == 1 } {

	# GiD Initialize Window
	InitWindow $w $wtitle $geomname ""

	grid columnconf $w 0 -weight 1
	grid rowconf $w 1 -weight 1

	# Window Title and Close Window Command

	switch $book {
	    "MATERIALS_(FLUID)" {
		wm protocol $w WM_DELETE_WINDOW "matfluid::checkclose $w 1 1 1"
		fluidmaterials::init $w $book
	    }
	    "MATERIALS_(SOLID)" {
		wm protocol $w WM_DELETE_WINDOW "matsolid::checkclose $w 1 1 1"
		solidmaterials::init $w $book
	    }
	    "BOUNDARIES_(FLUID)" {
		wm protocol $w WM_DELETE_WINDOW "bondfluid::checkclose $w 1 1 1"
		fluidboundaries::init $w $book
	    }
	    "BOUNDARIES_(SOLID)" {
		wm protocol $w WM_DELETE_WINDOW "bondsolid::checkclose $w 1 1 1"
		solidboundaries::init $w $book
	    }
	    "CONTACTS" {
		wm protocol $w WM_DELETE_WINDOW "bondsolid::checkclose $w 1 1 1"
		solidboundaries::init $w $book
	    }
	}

	# BINDS
	bind $w <Escape> "tkButtonInvoke $w.bf.btn_close"
	bind $w <Alt-KeyPress-c> "tkButtonInvoke $w.bf.btn_close"

	focus $w
    }

    return 1
}

proc materials::initone { book {title ""} {field ""} {value ""} } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set w ".gid.fluidmat"
	    if { $title == "" } {
		set wtitle [= {Fluid Material Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynFluidMatWindowGeom
	    set fluidmaterials::Materials_List [lindex [TIU_getmaterialnames $book $field $value] 0]
	    set fluidmaterials::Material_Name [lindex $fluidmaterials::Materials_List 0]
	    set fluidmaterials::Last_Material_Name $fluidmaterials::Material_Name
	}
	"MATERIALS_(SOLID)" {
	    set w ".gid.solidmat"
	    if { $title == "" } {
		set wtitle [= {Solid Material Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynSolidMatWindowGeom
	    set solidmaterials::Materials_List [lindex [TIU_getmaterialnames $book $field $value] 0]
	    set solidmaterials::Material_Name [lindex $solidmaterials::Materials_List 0]
	    set solidmaterials::Last_Material_Name $solidmaterials::Material_Name
	}
	"BOUNDARIES_(FLUID)" {
	    set w ".gid.fluidbond"
	    if { $title == "" } {
		set wtitle [= {Fluid Boundaries Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynFluidBondWindowGeom
	    set fluidboundaries::Materials_List [lindex [TIU_getmaterialnames $book $field $value] 0]
	    set fluidboundaries::Material_Name [lindex $fluidboundaries::Materials_List 0]
	    set fluidboundaries::Last_Material_Name $fluidboundaries::Material_Name
	    set fluidboundaries::Materials_Type ""
	    foreach imat $fluidboundaries::Materials_List {
		set matinfo [.central.s info materials $imat]
		lappend fluidboundaries::Materials_Type [lindex $matinfo 2]
	    }    
	}
	"BOUNDARIES_(SOLID)" {
	    set w ".gid.solidbond"
	    if { $title == "" } {
		set wtitle [= {Solid Boundaries Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynSolidBondWindowGeom
	    set solidboundaries::Materials_List [lindex [TIU_getmaterialnames $book $field $value] 0]
	    set solidboundaries::Material_Name [lindex $solidboundaries::Materials_List 0]
	    set solidboundaries::Last_Material_Name $solidboundaries::Material_Name
	    set solidboundaries::Materials_Type ""
	    foreach imat $solidboundaries::Materials_List {
		set matinfo [.central.s info materials $imat]
		lappend solidboundaries::Materials_Type [lindex $matinfo 2]
	    }    
	}
	"MATERIALS_(GOB)" {
	   set w ".gid.solidmat"
	    if { $title == "" } {
		set wtitle [= {Solid Material Window}]
	    } else {
		set wtitle [= {%s Window} $title]
	    }
	    set geomname PreTdynSolidMatWindowGeom
	    set solidmaterials::Materials_List [lindex [TIU_getmaterialnames $book $field $value] 0]
	    set solidmaterials::Material_Name [lindex $solidmaterials::Materials_List 0]
	    set solidmaterials::Last_Material_Name $solidmaterials::Material_Name
	}
	default {
	    return
	}
    }

    # Closing other windows opened
    set owc [TUI_closeotherwindows $w]

    if { $owc == 1 } {

	# GiD Initialize Window
	InitWindow $w $wtitle $geomname ""

	grid columnconf $w 0 -weight 1
	grid rowconf $w 1 -weight 1

	# Window Title and Close Window Command
	set initcmd [list materials::initone $book $title $field $value]
	switch $book {
	    "MATERIALS_(FLUID)" {
		wm protocol $w WM_DELETE_WINDOW "matfluid::checkclose $w 1 1 1"
		fluidmaterials::init $w $book $initcmd 1
	    }
	    "MATERIALS_(SOLID)" {
		wm protocol $w WM_DELETE_WINDOW "matsolid::checkclose $w 1 1 1"
		solidmaterials::init $w $book $initcmd 1
	    }
	    "BOUNDARIES_(FLUID)" {
		wm protocol $w WM_DELETE_WINDOW "bondfluid::checkclose $w 1 1 1"
		fluidboundaries::init $w $book $initcmd 1
	    }
	    "BOUNDARIES_(SOLID)" {
		wm protocol $w WM_DELETE_WINDOW "bondsolid::checkclose $w 1 1 1"
		solidboundaries::init $w $book $initcmd 1
	    }
	    "MATERIALS_(GOB)" {
		wm protocol $w WM_DELETE_WINDOW "matsolid::checkclose $w 1 1 1"
		solidmaterials::initgob $w $initcmd
	    }
	}

	# BINDS
	bind $w <Escape> "tkButtonInvoke $w.bf.btn_close"
	bind $w <Alt-KeyPress-c> "tkButtonInvoke $w.bf.btn_close"

	focus $w
    }

    return 1
}

proc fluidmaterials::init { w book {initcmd ""} {one 0} } {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name

    if { $one } {
	materials::maketopframeone $w $book
	materials::makelowerbuttonsone $w $book $Material_Name $initcmd
    } else {
	materials::maketopframe $w $book
	materials::makelowerbuttons $w $book $Material_Name
    }
    matfluid::init $w $Material_Name

    return 1
}

proc solidmaterials::init { w book {initcmd ""} {one 0} } {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name

    if { $one } {
	materials::maketopframeone $w $book
	materials::makelowerbuttonsone $w $book $Material_Name $initcmd
    } else {
	materials::maketopframe $w $book
	materials::makelowerbuttons $w $book $Material_Name
    }
    matsolid::init $w $Material_Name

    return 1
}

proc solidmaterials::initgob { w {initcmd ""} } {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name

    materials::maketopframeone $w MATERIALS_(SOLID)
    materials::makelowerbuttonsone $w MATERIALS_(SOLID) $Material_Name $initcmd

    matsolid::initgob $w $Material_Name

    return 1
}

proc fluidboundaries::init { w book {initcmd ""} {one 0} } {

    variable Materials_List
    variable Materials_Type
    variable Material_Name
    variable Last_Material_Name

    variable show_naval 
    variable show_heatrans

    # Checking for Free Surface - exclude if Naval Module is OFF
    if { $show_naval != 1 } {
	while { [lsearch -exact $Materials_Type "FreeS"] > -1 } {
	    set pos [lsearch -exact $Materials_Type "FreeS"]
	    set Materials_List [lreplace $Materials_List $pos $pos]
	    set Materials_Type [lreplace $Materials_Type $pos $pos]        
	}
    }

    # Checking for Contact - exclude if Heatrans Module is OFF
    if { $show_heatrans != 1 } {
	while { [lsearch -exact $Materials_Type "Interf"] > -1 } {
	    set pos [lsearch -exact $Materials_Type "Interf"]
	    set Materials_List [lreplace $Materials_List $pos $pos]
	    set Materials_Type [lreplace $Materials_Type $pos $pos]        
	}
    }
    if { $one } {
	materials::maketopframeone $w $book
	materials::makelowerbuttonsone $w $book $Material_Name $initcmd
    } else {
	materials::maketopframe $w $book
	materials::makelowerbuttons $w $book $Material_Name
    }
    bondfluid::init $w $Material_Name

    return 1
}

proc fluidboundaries::reinit { w book } {

    variable Materials_List
    variable Materials_Type
    variable Material_Name
    variable Last_Material_Name

    variable show_naval 
    variable show_heatrans

    # Update Materials Type list
    set fluidboundaries::Materials_Type ""
    foreach imat $fluidboundaries::Materials_List {
	set matinfo [.central.s info materials $imat]
	lappend fluidboundaries::Materials_Type [lindex $matinfo 2]
    }
    
    # Checking for Free Surface - exclude if Naval Module is OFF
    if { $show_naval != 1 } {
	while { [lsearch -exact $Materials_Type "FreeS"] > -1 } {
	    set pos [lsearch -exact $Materials_Type "FreeS"]
	    set Materials_List [lreplace $Materials_List $pos $pos]
	    set Materials_Type [lreplace $Materials_Type $pos $pos]        
	}
    }

    # Checking for Contact - exclude if Heatrans Module is OFF
    if { $show_heatrans != 1 } {
	while { [lsearch -exact $Materials_Type "Interf"] > -1 } {
	    set pos [lsearch -exact $Materials_Type "Interf"]
	    set Materials_List [lreplace $Materials_List $pos $pos]
	    set Materials_Type [lreplace $Materials_Type $pos $pos]        
	}
    }

    return 1
}

proc solidboundaries::init { w book {initcmd ""} {one 0} } {

    variable Materials_List
    variable Materials_Type
    variable Material_Name
    variable Last_Material_Name

    variable show_heatrans

    # Checking for Contact - exclude if Heatrans Module is OFF
    if { $show_heatrans != 1 } {
	while { [lsearch -exact $Materials_Type "Interf"] > -1 } {
	    set pos [lsearch -exact $Materials_Type "Interf"]
	    set Materials_List [lreplace $Materials_List $pos $pos]
	    set Materials_Type [lreplace $Materials_Type $pos $pos]        
	}
    }
    if { $one } {
	materials::maketopframeone $w $book
	materials::makelowerbuttonsone $w $book $Material_Name $initcmd
    } else {
	materials::maketopframe $w $book
	materials::makelowerbuttons $w $book $Material_Name
    }
    bondsolid::init $w $Material_Name

    return 1
}

proc solidboundaries::reinit { w book } {

    variable Materials_List
    variable Materials_Type
    variable Material_Name
    variable Last_Material_Name

    variable show_heatrans

    # Update Materials Type list
    set solidboundaries::Materials_Type ""
    foreach imat $solidboundaries::Materials_List {
	set matinfo [.central.s info materials $imat]
	lappend solidboundaries::Materials_Type [lindex $matinfo 2]
    }

    # Checking for Contact - exclude if Heatrans Module is OFF
    if { $show_heatrans != 1 } {
	while { [lsearch -exact $Materials_Type "Interf"] > -1 } {
	    set pos [lsearch -exact $Materials_Type "Interf"]
	    set Materials_List [lreplace $Materials_List $pos $pos]
	    set Materials_Type [lreplace $Materials_Type $pos $pos]        
	}
    }

    return 1
}

proc materials::maketopframe { w book } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set matvar fluidmaterials::Material_Name
	    set icon $TUI_images::i_fluiddrop
	    set sel_label "[= {Select Material}]:"
	    set matlist $fluidmaterials::Materials_List
	    set changecmd "materials::changematerial $w $book -1 1"
	    set modcmd "materials::modifymaterial $w $book \$fluidmaterials::Material_Name"
	    set newcmd "materials::newmaterial $w $book"
	    set erasecmd "materials::deletematerial $w $book \$fluidmaterials::Material_Name"
	}
	"MATERIALS_(SOLID)" {
	    set matvar solidmaterials::Material_Name
	    set icon $TUI_images::i_solidcolumn
	    set sel_label "[= {Select Material}]:"
	    set matlist $solidmaterials::Materials_List
	    set changecmd "materials::changematerial $w $book -1 1"
	    set modcmd "materials::modifymaterial $w $book \$solidmaterials::Material_Name"
	    set newcmd "materials::newmaterial $w $book"
	    set erasecmd "materials::deletematerial $w $book \$solidmaterials::Material_Name"
	}
	"BOUNDARIES_(FLUID)" {
	    set matvar fluidboundaries::Material_Name
	    set icon $TUI_images::i_fbond
	    set sel_label "[= {Select Boundary}]:"
	    set matlist $fluidboundaries::Materials_List
	    set changecmd "materials::changematerial $w $book -1 1"
	    set modcmd "materials::modifymaterial $w $book \$fluidboundaries::Material_Name"
	    set newcmd "materials::newmaterial $w $book"
	    set erasecmd "materials::deletematerial $w $book \$fluidboundaries::Material_Name"
	}
	"BOUNDARIES_(SOLID)" {
	    set matvar solidboundaries::Material_Name
	    set icon $TUI_images::i_sbond
	    set sel_label "[= {Select Boundary}]:"
	    set matlist $solidboundaries::Materials_List
	    set changecmd "materials::changematerial $w $book -1 1"
	    set modcmd "materials::modifymaterial $w $book \$solidboundaries::Material_Name"
	    set newcmd "materials::newmaterial $w $book"
	    set erasecmd "materials::deletematerial $w $book \$solidboundaries::Material_Name"
	}
	"CONTACTS" {
	    set matvar solidboundaries::Material_Name
	    set icon $TUI_images::i_contact
	    set sel_label "[= {Select Contact}]:"
	    set matlist $solidboundaries::Materials_List
	    set changecmd "materials::changematerial $w $book -1 1"
	    set modcmd "materials::modifymaterial $w $book \$solidboundaries::Material_Name"
	    set newcmd "materials::newmaterial $w $book"
	    set erasecmd "materials::deletematerial $w $book \$solidboundaries::Material_Name"
	}
    }

    # Top navigation frame
    set f [frame $w.f -bg lightblue]

    # Materials Combo Box
    set l_icon [label $f.l_icon -image $icon -bg lightblue]
    set l_selmat [label $f.l_selmat -text $sel_label -bg lightblue]
    set cbx_matlist [ComboBox $w.f.matlist -values $matlist -editable false \
	    -width [TUI_getcbxsize 15 $matlist] -takefocus 1 -textvariable $matvar \
	    -modifycmd $changecmd]
    $cbx_matlist setvalue first

    # Update, New, Erase buttons
    set btn_mod [button $w.f.mod -image $buttons::ibtn_uchang -command $modcmd -und 0 -bg lightblue]
    set btn_new [button $w.f.new -image $buttons::ibtn_new -command $newcmd -und 0 -bg lightblue]
    set btn_erase [button $w.f.erase -image $buttons::ibtn_erase -command $erasecmd -und 0 -bg lightblue]
    switch $book {
	"MATERIALS_(FLUID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	    GidHelp $w.f.new [= {Create New Material}]
	    GidHelp $w.f.erase [= {Delete Material}]
	    set matfluid::button_modify $w.f.mod
	    set matfluid::button_new $w.f.new 
	    set matfluid::button_erase $w.f.erase
	}
	"MATERIALS_(SOLID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	    GidHelp $w.f.new [= {Create New Material}]
	    GidHelp $w.f.erase [= {Delete Material}]
	    set matsolid::button_modify $w.f.mod
	    set matsolid::button_new $w.f.new 
	    set matsolid::button_erase $w.f.erase
	}
	"BOUNDARIES_(FLUID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	    GidHelp $w.f.new [= {Create New Boundary}]
	    GidHelp $w.f.erase [= {Delete Boundary}]
	}
	"BOUNDARIES_(SOLID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	    GidHelp $w.f.new [= {Create New Boundary}]
	    GidHelp $w.f.erase [= {Delete Boundary}]
	}
	"CONTACTS" {
	    GidHelp $w.f.mod [= {Update changes}]
	    GidHelp $w.f.new [= {Create New Contact}]
	    GidHelp $w.f.erase [= {Delete Contact}]
	}
    }

    # GRIDS
    grid $f -row 0 -column 0 -sticky wnes
    grid $l_icon -row 0 -column 0 -sticky w -padx 5
    grid $l_selmat -row 0 -column 1 -sticky w
    grid $cbx_matlist -row 0 -column 2 -sticky w -padx 5
    grid $btn_mod -row 0 -column 4 -sticky w -pady 1
    grid $btn_new -row 0 -column 5 -sticky w -pady 1
    grid $btn_erase -row 0 -column 6 -sticky w -pady 1

    grid columnconf $f 3 -weight 1

    return 1
}

proc materials::maketopframeone { w book } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set matvar fluidmaterials::Material_Name
	    set icon $TUI_images::i_fluiddrop
	    set sel_label "[= {Material}]: "
	    set matlist $fluidmaterials::Materials_List
	    set modcmd "materials::modifymaterial $w $book \$fluidmaterials::Material_Name"
	}
	"MATERIALS_(SOLID)" {
	    set matvar solidmaterials::Material_Name
	    set icon $TUI_images::i_solidcolumn
	    set sel_label "[= {Material}]: "
	    set matlist $solidmaterials::Materials_List
	    set modcmd "materials::modifymaterial $w $book \$solidmaterials::Material_Name"
	}
	"BOUNDARIES_(FLUID)" {
	    set matvar fluidboundaries::Material_Name
	    set icon $TUI_images::i_fbond
	    set sel_label "[= {Boundary}]: "
	    set matlist $fluidboundaries::Materials_List
	    set modcmd "materials::modifymaterial $w $book \$fluidboundaries::Material_Name"
	}
	"BOUNDARIES_(SOLID)" {
	    set matvar solidboundaries::Material_Name
	    set icon $TUI_images::i_sbond
	    set sel_label "[= {Boundary}]: "
	    set matlist $solidboundaries::Materials_List
	    set modcmd "materials::modifymaterial $w $book \$solidboundaries::Material_Name"
	}
    }

    # Top navigation frame
    set f [frame $w.f -bg lightblue]

    # Materials Combo Box
    set l_icon [label $f.l_icon -image $icon -bg lightblue]
    set l_selmat [label $f.l_selmat -text $sel_label -bg lightblue]
    set l_matnam [label $f.l_matnam -text [eval set temp $$matvar] \
	    -bg lightblue -relief ridge]

    # Update button
    set btn_mod [button $w.f.mod -image $buttons::ibtn_uchang -command $modcmd -und 0 -bg lightblue]
    switch $book {
	"MATERIALS_(FLUID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	    set matfluid::button_modify $w.f.mod
	}
	"MATERIALS_(SOLID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	    set matsolid::button_modify $w.f.mod
	}
	"BOUNDARIES_(FLUID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	}
	"BOUNDARIES_(SOLID)" {
	    GidHelp $w.f.mod [= {Update changes}]
	}
    }

    # GRIDS
    grid $f -row 0 -column 0 -sticky wnes
    grid $l_icon -row 0 -column 0 -sticky w -padx 5
    grid $l_selmat -row 0 -column 1 -sticky w
    grid $l_matnam -row 0 -column 2 -sticky w -padx 5
    grid $btn_mod -row 0 -column 4 -sticky e -padx 5 -pady 1

    grid columnconf $f 3 -weight 1

    return 1
}

proc materials::changematerial { w book mindex {checkch 1} } {

    set temp 1

    if { $mindex < 0 } {
	set mindex [$w.f.matlist getvalue]
    }

    switch $book {
	"MATERIALS_(FLUID)" {
	    set matlistvar fluidmaterials::Materials_List
	    set Material_Name [lindex [eval set nothing $$matlistvar] $mindex]
	    if { $checkch == 1 } {
		set temp [matfluid::checkclose $w 1 1 0]
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		matfluid::init $w $Material_Name
	    } else {
		$w.f.matlist setvalue @[lsearch $fluidmaterials::Materials_List $fluidmaterials::Last_Material_Name]
	    }
	    set fluidmaterials::Last_Material_Name $Material_Name
	}
	"MATERIALS_(SOLID)" {
	    set matlistvar solidmaterials::Materials_List
	    set Material_Name [lindex [eval set nothing $$matlistvar] $mindex]
	    if { $checkch == 1 } {
		set temp [matsolid::checkclose $w 1 1 0]
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		matsolid::init $w $Material_Name
	    } else {
		$w.f.matlist setvalue @[lsearch $solidmaterials::Materials_List $solidmaterials::Last_Material_Name]
	    }
	    set solidmaterials::Last_Material_Name $Material_Name
	}
	"BOUNDARIES_(FLUID)" {
	    set matlistvar fluidboundaries::Materials_List
	    set Material_Name [lindex [eval set nothing $$matlistvar] $mindex]
	    if { $checkch == 1 } {
		set temp [bondfluid::checkclose $w 1 1 0]
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		fluidboundaries::reinit $w $Material_Name
		bondfluid::init $w $Material_Name
	    } else {
		$w.f.matlist setvalue @[lsearch $fluidboundaries::Materials_List $fluidboundaries::Last_Material_Name]
	    }
	    set fluidboundaries::Last_Material_Name $Material_Name
	}
	"BOUNDARIES_(SOLID)" {
	    set matlistvar solidboundaries::Materials_List
	    set Material_Name [lindex [eval set nothing $$matlistvar] $mindex]
	    if { $checkch == 1 } {
		set temp [bondsolid::checkclose $w 1 1 0]
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		solidboundaries::reinit $w $Material_Name
		bondsolid::init $w $Material_Name
	    } else {
		$w.f.matlist setvalue @[lsearch $solidboundaries::Materials_List $solidboundaries::Last_Material_Name]
	    }
	    set solidboundaries::Last_Material_Name $Material_Name
	}
	"CONTACTS" {
	    set matlistvar solidboundaries::Materials_List
	    set Material_Name [lindex [eval set nothing $$matlistvar] $mindex]
	    if { $checkch == 1 } {
		set temp [bondsolid::checkclose $w 1 1 0]
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		solidboundaries::reinit $w $Material_Name
		bondsolid::init $w $Material_Name
	    } else {
		$w.f.matlist setvalue @[lsearch $solidboundaries::Materials_List $solidboundaries::Last_Material_Name]
	    }
	    set solidboundaries::Last_Material_Name $Material_Name
	}
    }

    destroy $w.bf
    materials::makelowerbuttons $w $book $Material_Name

    return 1
}

proc materials::modifymaterial { w book Material_Name } {

    set temp ""

    switch $book {
	"MATERIALS_(FLUID)" {
	    set temp [matfluid::checkclose $w 1 0 0]
	    if { $temp != "dataerror" } {
		if { [info exists matfluid::SpeciesUpdateCmd] } {
		    eval $matfluid::SpeciesUpdateCmd
		}
		if { [info exists matfluid::URVariablesUpdateCmd] } {
		    eval $matfluid::URVariablesUpdateCmd
		}
	    }
	    set type Material
	}
	"MATERIALS_(SOLID)" {
	    set temp [matsolid::checkclose $w 1 0 0]
	    if { $temp != "dataerror" } {
		if { [info exists matsolid::SpeciesUpdateCmd] } {
		    eval $matsolid::SpeciesUpdateCmd
		}
		if { [info exists matsolid::URVariablesUpdateCmd] } {
		    eval $matsolid::URVariablesUpdateCmd
		}
	    }
	    set type Material
	}
	"BOUNDARIES_(FLUID)" {
	    set temp [bondfluid::checkclose $w 1 0 0]
	    set type Boundary
	}
	"BOUNDARIES_(SOLID)" {
	    set temp [bondsolid::checkclose $w 1 0 0]
	    set type Boundary
	}
	"CONTACTS" {
	    set temp [bondsolid::checkclose $w 1 0 0]
	    set type Boundary
	}
    }

    if { $temp == 1 } {
	FloatMessage $w.f.flm [= {%s '%s' was modified} $type $Material_Name]
    } elseif { $temp != "dataerror" } {
	FloatMessage $w.f.flm "[= {No changes need to be saved}]."
    }

    return 1
}

proc materials::newmaterial { w book } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set savechanges [matfluid::checkclose $w 1 1 0]
	    set type Material
	    set matvar fluidmaterials::Material_Name
	    set matlistvar fluidmaterials::Materials_List
	}
	"MATERIALS_(SOLID)" {
	    set savechanges [matsolid::checkclose $w 1 1 0]
	    set type Material
	    set matvar solidmaterials::Material_Name
	    set matlistvar solidmaterials::Materials_List
	}
	"BOUNDARIES_(FLUID)" {
	    set savechanges [bondfluid::checkclose $w 1 1 0]
	    set type Boundary
	    set matvar fluidboundaries::Material_Name
	    set matlistvar fluidboundaries::Materials_List
	}
	"BOUNDARIES_(SOLID)" {
	    set savechanges [bondsolid::checkclose $w 1 1 0]
	    set type Boundary
	    set matvar solidboundaries::Material_Name
	    set matlistvar solidboundaries::Materials_List
	}
	"CONTACTS" {
	    set savechanges [bondsolid::checkclose $w 1 1 0]
	    set type Boundary
	    set matvar solidboundaries::Material_Name
	    set matlistvar solidboundaries::Materials_List
	}
    }

    # Existing Material Name
    set Material_Name [lindex [eval set nothing $$matlistvar] [$w.f.matlist getvalue]]

    if { $savechanges != "dataerror" } {
	set aa [tk_dialogEntryRAM $w [= {Creating New Material}] "[= {Enter new %s name} $type]." "" any ""]

	if { $aa != "--CANCEL--" && $aa != "" } {

	    # Formating new name
	    regsub -all "\"" $aa "" aa
	    set aa [DWSpace2Under [string trim $aa]]

	    # Exists the new name?
	    set matindex -1
	    set matlist [.central.s info materials]
	    set index -1
	    foreach mat $matlist {
		incr index
		if { [string match -nocase $mat $aa] } {
		    set matindex $index
		    break
		}
	    }
	    if { $matindex == -1 } { 

		foreach val [TUI_getmatvalueslist $Material_Name] {
		    append values $val " "
		}

		GidUtils::DisableWarnLine
		.central.s process escape escape escape escape Data Materials \
		        NewMaterial $Material_Name $aa [string trim $values]
		.central.s process escape escape escape escape
		GidUtils::EnableWarnLine

		set $matvar $aa
		set $matlistvar [TIU_getbooknames $book]

		materials::changematerial $w $book [expr [llength [TIU_getbooknames $book]] -1] 0

		$w.f.matlist configure -values [eval set nothing $$matlistvar]

	    } else {
		WarnWin "[= {A %s with this name already exists} $type].\n[= {Please select another name}]."
	    }
	}
    }

    return 1
}

proc materials::deletematerial { w book Material_Name } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set type Material
	    set types Materials
	    set matvar fluidmaterials::Material_Name
	    set matlistvar fluidmaterials::Materials_List
	    set matfields [.central.s info materials $Material_Name]
	    set submatlistvar [TIU_getmaterialnames $book \
		    [lindex $matfields 1] [lindex $matfields 2]]
	}
	"MATERIALS_(SOLID)" {
	    set type Material
	    set types Materials
	    set matvar solidmaterials::Material_Name
	    set matlistvar solidmaterials::Materials_List
	    set matfields [.central.s info materials $Material_Name]
	    set submatlistvar [TIU_getmaterialnames $book \
		    [lindex $matfields 1] [lindex $matfields 2]]
	}
	"BOUNDARIES_(FLUID)" {
	    set type Boundary
	    set types Boundaries
	    set matvar fluidboundaries::Material_Name
	    set matlistvar fluidboundaries::Materials_List
	    set matfields [.central.s info materials $Material_Name]
	    set submatlistvar [TIU_getmaterialnames $book \
		    [lindex $matfields 1] [lindex $matfields 2]]
	}
	"BOUNDARIES_(SOLID)" {
	    set type Boundary
	    set types Boundaries
	    set matvar solidboundaries::Material_Name
	    set matlistvar solidboundaries::Materials_List
	    set matfields [.central.s info materials $Material_Name]
	    set submatlistvar [TIU_getmaterialnames $book \
		    [lindex $matfields 1] [lindex $matfields 2]]
	}
	"CONTACTS" {
	    set type Boundary
	    set types Boundaries
	    set matvar solidboundaries::Material_Name
	    set matlistvar solidboundaries::Materials_List
	    set matfields [.central.s info materials $Material_Name]
	    set submatlistvar [TIU_getmaterialnames $book \
		    [lindex $matfields 1] [lindex $matfields 2]]
	}
    }

    set aa [tk_dialogRAM $w [= {Deleting %s} $Material_Name] \
	    [= {Sure you want to delete this %s and all its data?} $type] question 1 Yes No]

    if { $aa == 0 } {
	if { [llength $submatlistvar] < 2 } {
	    WarnWin "[= {You cannot delete all this type of %s} $types].\n[= {Create another %s you wish to use and then delete the last one} $type]."
	} else {
	    GidUtils::DisableWarnLine
	    .central.s process escape escape escape escape Data Materials DeleteMaterial $Material_Name
	    .central.s process escape escape escape escape
	    GidUtils::EnableWarnLine

	    set $matlistvar [TIU_getbooknames $book]
	    set $matvar [lindex [eval set nothing $$matlistvar] 0]
	    materials::changematerial $w $book 0 0

	    $w.f.matlist configure -values [eval set nothing $$matlistvar]

	}
    }

    return 1
}

proc materials::makelowerbuttons { w book Material_Name } {

    global ProgramName

    switch $book {
	"MATERIALS_(FLUID)" {
	    set closecmd "matfluid::checkclose $w 1 1 1"
	    set 2delements Surfaces
	    set 3delements Volumes
#            set type [= {Fluid Material}]
#            set types [= {Fluid Materials}]
	    set type [= {Material}]
	    set types [= {Materials}]
	    set assigncmd "materials::assignmaterial $w $Material_Name"
	    set drawcmd "materials::drawmaterial"
	    set unasscmd "materials::unassignmaterial"
	}
	"MATERIALS_(SOLID)" {
	    set closecmd "matsolid::checkclose $w 1 1 1"
	    set 2delements Surfaces
	    set 3delements Volumes
#            set type [= {Solid Material}]
#            set types [= {Solid Materials}]
	    set type [= {Material}]
	    set types [= {Materials}]
	    set assigncmd "materials::assignmaterial $w $Material_Name"
	    set drawcmd "materials::drawmaterial"
	    set unasscmd "materials::unassignmaterial"
	}
	"BOUNDARIES_(FLUID)" {
	    set closecmd "bondfluid::checkclose $w 1 1 1"
	    set 2delements Lines
	    set 3delements Surfaces
#            set type [= {Fluid Boundary}]
#            set types [= {Fluid Boundaries}]
	    set type [= {Boundary}]
	    set types [= {Boundaries}]
	    set assigncmd "materials::assignmaterial $w $Material_Name"
	    set drawcmd "materials::drawmaterial"
	    set unasscmd "materials::unassignmaterial"
	}
	"BOUNDARIES_(SOLID)" {
	    set closecmd "bondsolid::checkclose $w 1 1 1"
	    set 2delements Lines
	    set 3delements Surfaces
#            set type [= {Solid Boundary}]
#            set types [= {Solid Boundaries}]
	    set type [= {Boundary}]
	    set types [= {Boundaries}]
	    set assigncmd "materials::assignmaterial $w $Material_Name"
	    set drawcmd "materials::drawmaterial"
	    set unasscmd "materials::unassignmaterial"
	}
	"CONTACTS" {
	    set closecmd "bondsolid::checkclose $w 1 1 1"
	    set 2delements Lines
	    set 3delements Surfaces
	    set type [= {Fluid-Solid Contact}]
	    set types [= {Fluid-Solid Contacts}]
	    set assigncmd "materials::assigncondmaterial $w $Material_Name"
	    set drawcmd "materials::drawcondmaterial"
	    set unasscmd "materials::unassigncondmaterial"
	}
    }

    # Buttons frame
    set bf [frame $w.bf -relief raised -takefocus 1]

    # Assign button
    switch $ProgramName {
	"Tdyn2D" { append assigncmd " $2delements" }
	"Tdyn3D" { append assigncmd " $3delements" }
    }
    set btn_assign [button $bf.btn_assign -text [= {Assign}] -width 10 -relief raised -bd 1 \
	    -command $assigncmd -bd 1 -highlightthickness 1 -takefocus 1]

    # Draw button
    set MenuDraw $bf.btn_draw.menu
    set btn_draw [menubutton $bf.btn_draw -text [= {Draw}] -width 10 -relief raised -bd 1 \
	    -menu $MenuDraw -bd 1.08 -highlightthickness 1 -takefocus 1]
    set menu_draw [menu $MenuDraw -bd 1 -activeborderwidth 1]
    if { $book != "CONTACTS" } {
	$MenuDraw add command -label "[= {This %s %s} $type [DWUnder2Space $Material_Name]]" \
	    -command "$drawcmd $w thismaterial $Material_Name"
    }
    switch $ProgramName {
	"Tdyn2D" {
	    $MenuDraw add command -label "[= {All %s} $types]" \
		-command "$drawcmd $w allmaterials $Material_Name $2delements"
	}
	"Tdyn3D" {
	    $MenuDraw add command -label "[= {All %s} $types]" \
		-command "$drawcmd $w allmaterials $Material_Name $3delements"
	}
    }

    # Unassign button
    set MenuUnassign $bf.btn_unassign.menu
    set btn_unassign [menubutton $bf.btn_unassign -text [= {Unassign}] -bd 1 -relief raised\
	-width 10 -menu $MenuUnassign -bd 1 -highlightthickness 1 -takefocus 1]
    set menu_unassign [menu $MenuUnassign -bd 1 -activeborderwidth 1]
    set trelements ""
    switch $ProgramName {
	"Tdyn2D" { 
	    switch $2delements {
		"Lines" { set trelements [= {Lines}] }
		"Surfaces" { set trelements [= {Surfaces}] }
	    }
	}
	"Tdyn3D" { 
	    switch $3delements {
		"Surfaces" { set trelements [= {Surfaces}] }
		"Volumes" { set trelements [= {Volumes}] }
	    }
	}
    }
    switch $ProgramName {
	"Tdyn2D" {
	    $MenuUnassign add command -label $trelements \
		-command "$unasscmd $w $book $Material_Name $2delements $2delements"
	    $MenuUnassign add command -label "[= {All entities of %s %s} $type [DWUnder2Space $Material_Name]]" \
		-command "$unasscmd $w $book $Material_Name allthismat $2delements"
	    if { $book != "Default" } {
		$MenuUnassign add command -label "[= {All %s} [DWUnder2Space $book]]" \
		    -command "$unasscmd $w $book $Material_Name allthisbook $2delements"
	    }
	}
	"Tdyn3D" {
	    $MenuUnassign add command -label $trelements \
		-command "$unasscmd $w $book $Material_Name $3delements $3delements"
	    $MenuUnassign add command -label "[= {All entities of %s %s} $type [DWUnder2Space $Material_Name]]" \
		-command "$unasscmd $w $book $Material_Name allthismat $3delements"
	    if { $book != "Default" } {
		$MenuUnassign add command -label "[= {All %s} [DWUnder2Space $book]]" \
		    -command "$unasscmd $w $book $Material_Name allthisbook $3delements"
	    }
	}
    }
#       $MenuUnassign add command -label "[= {All %s} $types]" \
#                  -command "materials::unassignmaterial $w $book $Material_Name allmaterials"

    # Import/Export button
    set btn_impexp [button $bf.btn_impexp -text "[= {Database}]..."  -width 10 \
	    -relief raised -bd 1 -highlightthickness 1 -takefocus 1 \
	    -command [list DWMatImportExport $w $book]]
    # Close button
    set btn_close [button $bf.btn_close -text [= {Close}] -width 10 -relief raised \
	    -bd 1 -takefocus 1 -command "$closecmd"]    
    
    # GRIDS
    grid $bf -row 2 -column 0 -sticky wnes
    grid columnconf $bf 4 -weight 1
    grid $btn_assign -row 0 -column 0 -sticky w -padx 5 -pady 5 
    grid $btn_draw -row 0 -column 1 -sticky w -padx 5 -pady 5
    grid $btn_unassign -row 0 -column 2 -sticky w -padx 5 -pady 5 
    grid $btn_impexp -row 0 -column 3 -sticky w -padx 5 -pady 5 
    grid $btn_close -row 0 -column 4 -sticky e -padx 5 -pady 5 

    return 1
}

proc materials::makelowerbuttonsone { w book Material_Name {initcmd ""} } {

    global ProgramName

    switch $book {
	"MATERIALS_(FLUID)" {
	    set closecmd "matfluid::checkclose $w 1 1 1"
	    set type Material
	    set types Materials
	}
	"MATERIALS_(SOLID)" {
	    set closecmd "matsolid::checkclose $w 1 1 1"
	    set type Material
	    set types Materials
	}
	"BOUNDARIES_(FLUID)" {
	    set closecmd "bondfluid::checkclose $w 1 1 1"
	    set type Boundary
	    set types Boundaries
	}
	"BOUNDARIES_(SOLID)" {
	    set closecmd "bondsolid::checkclose $w 1 1 1"
	    set type Boundary
	    set types Boundaries
	}
    }

    # Buttons frame
    set bf [frame $w.bf -relief raised -takefocus 1]

    # Draw button
    set MenuDraw $bf.btn_draw.menu
    set btn_draw [menubutton $bf.btn_draw -text [= {Draw}] -relief raised \
	    -menu $MenuDraw -bd 1 -width 11 -highlightthickness 1 -takefocus 1]
    set menu_draw [menu $MenuDraw -bd 1 -activeborderwidth 1]
    $MenuDraw add command -label [= {This %s %s} $type [DWUnder2Space $Material_Name]] \
	    -command "materials::drawmaterial $w thismaterial $Material_Name"
    $MenuDraw add command -label [= {All Materials and Boundaries}] \
	    -command "materials::drawmaterial $w allmaterials $Material_Name"

    # Import/Export button
    set btn_impexp [button $bf.btn_impexp -text "[= {Database}]..." -relief raised \
	    -bd 1 -width 11 -highlightthickness 1 -takefocus 1 \
	    -command [list ::GMMaterialsDatabase::DWMatImportExport $w $book $initcmd]]
    # Close button
    set btn_close [button $bf.btn_close -text [= {Close}] -underline 0 -relief raised \
	    -bd 1 -width 11 -highlightthickness 1 -takefocus 1 \
	    -command "$closecmd"]
    
    # GRIDS
    grid $bf -row 2 -column 0 -sticky wnes
    grid columnconf $bf 4 -weight 1
    grid $btn_draw -row 0 -column 1 -sticky w -pady 1
    grid $btn_impexp -row 0 -column 3 -sticky w -pady 1
    grid $btn_close -row 0 -column 4 -sticky e -pady 1

    return 1
}

proc materials::assignmaterial { w Material_Name {type Elements} } {
 
    # Disable Matlist Combobox
    $w.f.matlist configure -state disabled

    set MeshViewInfo [lindex [.central.s info Project] 4]
    if { $MeshViewInfo == "GEOMETRYUSE" || $type == "Elements" } {
	.central.s process escape escape escape escape data mat assign \
		"[DWSpace2Under $Material_Name]" $type
    } else {
	.central.s process escape escape escape escape data mat assign \
		"[DWSpace2Under $Material_Name]"
    }

    set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
    FinishButton $w $w.bf.btn_assign [= "Press 'Finish' when selection\nis finished"] $endaction disableall

}

proc materials::assigncondmaterial { w Material_Name {type Elements} } {

    # Disable Matlist Combobox
    $w.f.matlist configure -state disabled

    # Asigna una condicin de tipo Line_Contact o Surface_Contact 
    if { $type == "Lines" } {   
	.central.s process escape escape escape escape data cond assign \
	    Line_Contact change $Material_Name
    } else {
	.central.s process escape escape escape escape data cond assign \
	    Surface_Contact change $Material_Name
    }

    set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
    FinishButton $w $w.bf.btn_assign [= "Press 'Finish' when selection\nis finished"] $endaction disableall
}

proc materials::unassignmaterial { w book Material_Name type { entity "Lines" } } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set tty Material
	}
	"MATERIALS_(SOLID)" {
	    set tty Material
	}
	"BOUNDARIES_(FLUID)" {
	    set tty Boundary
	}
	"BOUNDARIES_(SOLID)" {
	    set tty Boundary
	}
	"CONTACTS" {
	    return [materials::unassigncondmaterial $w $book $Material_Name $type]
	}
    }

    if { $type == "Surfaces" || $type == "Volumes" || $type == "Lines" } {
	.central.s process escape escape escape escape Data Materials UnAssign Entities $type
	
	# Disable Matlist Combobox
	$w.f.matlist configure -state disabled

	set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
	FinishButton $w $w.bf.btn_assign [= "Press 'Finish' when selection\nis finished"] $endaction disableall

    } elseif { $type == "allthismat" } {
	if { $Material_Name == ""} {
	    WarnWin "[= {Please select a %s first} $tty]." $w
	    return
	}
	set aa [tk_dialogRAMFull $w.tempwin [= {Unassign materials}] \
		"[= {Are you sure you want to unassign all %s %s?} $tty [DWUnder2Space $Material_Name]]" \
		"" "" gidquestionhead 0 OK Cancel]
	if { $aa == "0" } {
	    .central.s process escape escape escape escape Data Materials UnAssign SomeMat $Material_Name
	}
    } elseif { $type == "allthisbook" } {
	if { $book == ""} {
	    WarnWin "[= {Please select a Book first}]." $w
	    return
	}
	set aa [tk_dialogRAMFull $w.tempwin [= {Unassign materials}] \
		[= {Are you sure you want to unassign all Book %s?} [DWUnder2Space $book]] \
		"" "" gidquestionhead 0 OK Cancel]
	if { $aa == "0" } {
	    .central.s process escape escape escape escape Data Materials UnAssign SomeBook $book
	}
    } elseif { $type == "allmaterials" } {
	# Confirmation question is not here because GiD posts it itself.
	.central.s process escape escape escape escape Data Materials UnAssign AllMat
    }

    return 1
}

proc materials::unassigncondmaterial { w book Material_Name type { entity "Lines" } } {
   
 switch $book {
	"MATERIALS_(FLUID)" {
	    set tty Material
	}
	"MATERIALS_(SOLID)" {
	    set tty Material
	}
	"BOUNDARIES_(FLUID)" {
	    set tty Boundary
	}
	"BOUNDARIES_(SOLID)" {
	    set tty Boundary
	}
	"CONTACTS" {
	    set tty Contact
	}
    }

    if { $type == "Surfaces" || $type  == "Lines" } {
	if { $entity  == "Lines" } { 
	    set condname Line_Contact
	} else { 
	    set condname Surface_Contact 
	}
	.central.s process escape escape escape escape data cond assign $condname unassign

	# Disable Matlist Combobox
	$w.f.matlist configure -state disabled

	set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
	FinishButton $w $w.bf.btn_assign [= "Press 'Finish' when selection\nis finished"] $endaction disableall

    } elseif { $type == "allthismat" } {
	if { $Material_Name == ""} {
	    WarnWin "[= {Please select a %s first} $tty]." $w
	    return
	}
	set aa [tk_dialogRAMFull $w.tempwin [= {Unassign materials}] \
		"[= {Are you sure you want to unassign all %s %s?} $tty [DWUnder2Space $Material_Name]]" \
		"" "" gidquestionhead 0 OK Cancel]
	if { $aa == "0" } {
	    if { $entity  == "Lines" } { 
		set condname Line_Contact
	    } else { 
		set condname Surface_Contact 
	    }
	    set entities [.central.s info conditions $condname geometry]
	    set dellist ""
	    set entlist ""
	    set vallist ""
	    foreach item $entities {
		regexp {E\s([0-9]+)\s*-\s*(.*)} $item -> entity value
		if { $value != $Material_Name} { 
		    set inde [lsearch $entlist $entity]
		    set indv [lsearch $vallist $value]
		    if { $inde != $indv || $inde == -1 } {
		        set entlist [concat $entlist $entity]
		        set vallist [concat $vallist $value]
		    } 
		} else {
		set dellist [concat $dellist $entity] 
		}
	    }
	    if { $dellist != "" } {
		.central.s process escape escape escape escape data cond \
		    assign $condname unassign $dellist
		.central.s process escape escape escape
	    }
	    foreach ient $entlist ival $vallist {
		.central.s process escape escape escape escape data cond assign \
		    $condname change $ival $ient escape escape escape
	    }
	}
    } elseif { $type == "allthisbook" } {
	if { $book == ""} {
	    WarnWin "[= {Please select a Book first}]." $w
	    return
	}
	set aa [tk_dialogRAMFull $w.tempwin [= {Unassign materials}] \
		[= {Are you sure you want to unassign all %s?} [DWUnder2Space $book]] \
		"" "" gidquestionhead 0 OK Cancel]
	if { $aa == "0" } {
	    .central.s process escape escape escape escape data cond \
		assign Line_Contact unassign 1: escape 
	    .central.s process escape escape escape escape data cond \
		assign Surface_Contact unassign 1: escape 
	}
    }
    return 1
}

proc materials::drawmaterial { w type Material_Name { entity "Lines" } } {

    switch $type {
	thismaterial {
	    .central.s process escape escape escape escape data material draw $Material_Name
	}
	allmaterials {
	    .central.s process escape escape escape escape data material draw -drawall-
	}
    }

    # Disable Matlist combobox and show 'Finish' button
    if { [winfo exists $w.f.matlist] } {
	$w.f.matlist configure -state disabled
	set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
	FinishButton $w $w.bf.btn_assign [= "Press 'Finish' when finished\ndrawing"] $endaction disableall
    } else {
	set endaction ".central.s process escape escape escape;"
	FinishButton $w $w.bf.btn_draw [= "Press 'Finish' when finished\ndrawing"] $endaction disableall
    }
}

proc materials::drawcondmaterial { w type Material_Name { entity "Lines" } } {

    if { $entity == "Lines" } {
	.central.s process escape escape escape escape data cond draw \
	    -ByColor- Line_Contact -draw-
    } else {
	.central.s process escape escape escape escape data cond draw \
	    -ByColor- Surface_Contact -draw-
    } 
    # Disable Matlist combobox and show 'Finish' button
    if { [winfo exists $w.f.matlist] } {
	$w.f.matlist configure -state disabled
	set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
	FinishButton $w $w.bf.btn_assign [= "Press 'Finish' when finished\ndrawing"] $endaction disableall
    } else {
	set endaction ".central.s process escape escape escape;"
	FinishButton $w $w.bf.btn_draw [= "Press 'Finish' when finished\ndrawing"] $endaction disableall
    }
}
