#   NameSpace TdynOutput
namespace eval TdynOutput {
}

# -------------------------------------------------------------------
#  Procedure SpdData
#   
#      Devuelve una variable del archivo xml *.spd
#      nodepath es la seccin del archivo xml donde se ha de buscar
#
# -------------------------------------------------------------------

proc TdynOutput::SpdData { name {nodepath ""} } {

    set XmlRoot $processxmlfile::XmlRoot
    set XmlDoc $processxmlfile::XmlDoc

    if { $XmlDoc == "" } {
	WarnWin [= {SpdData error: XML file not initialised}]
	return
    }

    set ret -1
    if { $nodepath != "" } {
	set xpath "$nodepath/param\[@name='$name'\]"
	set paramNode [$XmlRoot selectNodes $xpath]
	if { $paramNode == "" } {
	    WarnWin "[= {SpdData error: Cannot find variable $name}]"
	} else {
	    set ret [[lindex $paramNode 0] text]
	}
    } else {
	set xpath "//*/param\[@name='$name'\]"
	set paramNode [$XmlRoot selectNodes $xpath]
	if { $paramNode == "" } {
	    WarnWin "[= {SpdData error: Cannot find variable $name}]"
	} else {
	    set ret [[lindex $paramNode 0] text]
	}
    }
    return $ret
}

# -------------------------------------------------------------------
#  Procedure SpdOperation
#   
#      Devuelve el resultado de una operacin entre variables del 
#      archivo xml *.spd
#      nodepath es la seccin del archivo xml donde se ha de buscar
#      No funciona con la operacin * desde GiD, pues este entiende 
#      el smbolo de manera especial (probar con **)
#
# -------------------------------------------------------------------

proc TdynOutput::SpdOperation { name1 name2 operation {nodepath ""} } {

    set XmlRoot $processxmlfile::XmlRoot
    set XmlDoc $processxmlfile::XmlDoc

    if { $XmlDoc == "" } {
	WarnWin [= {SpdData error: XML file not initialised}]
	return
    }

    set ret -1
    if { $nodepath != "" } {
	set xpath "$nodepath/param\[@name='$name1'\]"
	set paramNode [$XmlRoot selectNodes $xpath]
	if { $paramNode == "" } {
	    WarnWin "[= {SpdData error: Cannot find variable $name1}]"
	} else {
	    set ret1 [[lindex $paramNode 0] text]
	    set xpath "$nodepath/param\[@name='$name2'\]"
	    set paramNode [$XmlRoot selectNodes $xpath]
	    if { $paramNode == "" } {
		WarnWin "[= {SpdData error: Cannot find variable $name2}]"
	    } else {
		set ret2 [[lindex $paramNode 0] text]
		set ret [expr $ret1$operation$ret2]
	    }
	}
	
    } else {
	set xpath "//*/param\[@name='$name1'\]"
	set paramNode [$XmlRoot selectNodes $xpath]
	if { $paramNode == "" } {
	    WarnWin "[= {SpdData error: Cannot find variable $name1}]"
	} else {
	    set ret1 [[lindex $paramNode 0] text]
	    set xpath "//*/param\[@name='$name2'\]"
	    set paramNode [$XmlRoot selectNodes $xpath]
	    if { $paramNode == "" } {
		WarnWin "[= {SpdData error: Cannot find variable $name2}]"
	    } else {
		set ret2 [[lindex $paramNode 0] text]
		set ret [expr $ret1$operation$ret2]
	    }
	}
    }
    return $ret
}

# -------------------------------------------------------------------
#  Procedure strcmpSpdData
#   
#      Compara una variable del archivo xml *.spd
#      nodepath es la seccin del archivo xml donde se ha de buscar
#
# -------------------------------------------------------------------

proc TdynOutput::strcmpSpdData { name cmp {nodepath ""} } {

    return [string compare [TdynOutput::SpdData $name $nodepath] $cmp]
}

proc TdynOutput::!strcmpSpdData { name cmp {nodepath ""} } {

    return ![string compare [TdynOutput::SpdData $name $nodepath] $cmp]
}

# -------------------------------------------------------------------
#   Procedure FormatMatlist 
#
#   Devuelve una cadena que est compuesta por una serie de lneas
#   de formato:
#     Index 1, name1 = "field1"; ufield1,\n
#     ...
#     Index i, namei = "fieldi"; ufieldi,\n
#     ...
#     Index n, namen = "fieldn"; ufieldn;
#   Donde n es el nmero de campos en la lista names. 
#   Este comando puede utilizarse para escribir una lista de funciones y 
#   unidades en el *.bas de Tdyn.
#   La variable uoutput es utilizada para saber qu names tienen unidades.
#   Esta variable debe contener una lista de igual longitud que names con los
#   ndices de las unidades que le corresponden (0 si no tiene unidad). Por
#   ejemplo "0 1 2 2 0" asignar unidades al 2, 3er y 4 campos.
#   Si spaces > 1 se aaden esos espacios antes de cada nueva fila.
# -------------------------------------------------------------------

proc TdynOutput::FormatMatlist { material names field ufield {uoutput ""} {spaces 0}} {
    set nlen [llength $names] 
    set values [lrange [GiD_AccessValue get materials $material $field] 2 end]
    set uvalues [lrange [GiD_AccessValue get materials $material $ufield] 2 end]
    set vlen [llength $values]
    set plen [expr int($vlen/$nlen)]
    if { [expr $plen*$nlen] != $vlen } {
	WarnWin "[= {Error found calling FormatMatlist}].\n[= {%s values found for %s names} $vlen $nlen]."
    }
    set vulen [llength $uvalues]
    set nulen [expr $vulen/$plen]
    if { [expr $plen*$nulen] != $vulen } {
	WarnWin "[= {Error found calling FormatMatlist}].\n[= {%s values found for %s units} $vulen $nulen]."
	set uoutput ""
    }
    set sp [string repeat { } $spaces]
    set ret ""
    set ifield 0
    if { $uoutput != "" } {
	for {set iprop 0} {$iprop < $plen} {incr iprop} {
	    append ret "${sp}Index [expr $iprop+1], "
	    for {set iname 0} {$iname < $nlen} {incr iname} {
		append ret "[lindex $names $iname] = "
		append ret "\"[lindex $values $ifield];\" "
		incr ifield
		set iunit [lindex $uoutput $iname]
		if { $iunit <= 0 || $iunit > $vlen } {
		    append ret {[]}
		} else {
		    append ret [lindex $uvalues [expr $iprop*$nulen+$iunit-1]]
		}
		if { $iname != [expr $nlen-1] } {
		    append ret {, }
		} elseif { $iprop != [expr $plen-1] } {
		    append ret ";\n"
		} else {
		    append ret {;}
		}
	    }
	}
    } else {
	for {set iprop 0} {$iprop < $plen} {incr iprop} {
	    append ret "${sp}Index [expr $iprop+1], "
	    for {set iname 0} {$iname < $nlen} {incr iname} {
		append ret "[lindex $names $iname] = "
		append ret "\"[lindex $values $ifield];\" "
		append ret "[lindex $uvalues $ifield]"
		incr ifield
		if { $iname != [expr $nlen-1] } {
		    append ret {, }
		} elseif { $iprop != [expr $plen-1] } {
		    append ret ";\n"
		} else {
		    append ret {;}
		}
	    }
	}
    }
    return $ret
}

# Similar al anterior pero slo escribe una componente de la matriz, la dada por los
# ndices fidx para field y uidx para ufield. El ndice nidx marca el valor del vector
# names que se tiene que utilizar.
# Los ndices van de 1 a n para fidx y uidx
proc TdynOutput::FormatMatlval { material names nidx field fidx ufield uidx {spaces 0} } {
    set nlen [llength $names] 
    set values [lrange [GiD_AccessValue get materials $material $field] 2 end]
    set uvalues [lrange [GiD_AccessValue get materials $material $ufield] 2 end]
    set vlen [llength $values]
    set plen [expr int($vlen/$nlen)]
    if { [expr $plen*$nlen] != $vlen } {
	WarnWin "[= {Error found calling FormatMatlval}].\n[= {%s values found for %s names} $vlen $nlen]."
    }
    set vulen [llength $uvalues]
    set nulen [expr $vulen/$plen]

    set sp [string repeat { } $spaces]
    set ret ""  
    set iname 0
    for {set iprop 0} {$iprop < $plen} {incr iprop} {
	append ret $sp
	append ret "Index [expr $iname+1], [lindex $names [expr $nidx-1]] = "
	append ret "\"[lindex $values [expr $iprop*$nlen+$fidx-1]];\" "
	set iunit [expr $iprop*$nulen+$uidx-1]
	if { $iunit < 0 || $iunit >= $vlen } {
	    append ret {[]}
	} else {
	    append ret "[lindex $uvalues $iunit]"
	}
	if { $iprop == [expr $plen-1] } {
	    append ret ";"
	} else {
	    append ret ";\n"
	}
	incr iname
    }
    return $ret
}

# Similar a los anteriores pero slo escribe una componente de la matriz, la dada por los
# ndices fidx para field y uidx para ufield. El ndice nidx marca el valor del vector
# names que se tiene que utilizar. La diferencia de esta funcin es que la componente 
# referida slo se escribe si el campo siguiente toma el valor On, en caso contrario
# escribir en su lugar "Off". 
# Los ndices van de 1 a n para fidx y uidx
proc TdynOutput::FormatMatlvalIf { material names nidx field fidx ufield uidx {spaces 0} } {
    set nlen [llength $names] 
    set values [lrange [GiD_AccessValue get materials $material $field] 2 end]
    set uvalues [lrange [GiD_AccessValue get materials $material $ufield] 2 end]
    set vlen [llength $values]
    set plen [expr int($vlen/$nlen)]
    if { [expr $plen*$nlen] != $vlen } {
	WarnWin "[= {Error found calling FormatMatlval}].\n[= {%s values found for %s names} $vlen $nlen]."
    }
    set vulen [llength $uvalues]
    set nulen [expr $vulen/$plen]

    set sp [string repeat { } $spaces]
    set ret ""
    set iname 0
    for {set iprop 0} {$iprop < $plen} {incr iprop} {
	append ret $sp
	set onoff [lindex $values [expr $iprop*$nlen+$fidx]]
	if { $onoff == "On" } {
	    append ret "Index [expr $iname+1], [lindex $names [expr $nidx-1]] = "
	    append ret "\"[lindex $values [expr $iprop*$nlen+$fidx-1]];\" "
	    set iunit [expr $iprop*$nulen+$uidx-1]
	    if { $iunit < 0 || $iunit >= $vlen } {
		append ret {[]}
	    } else {
		append ret "[lindex $uvalues $iunit]"
	    }
	} else {
	    append ret "Off"
	}
	if { $iprop == [expr $plen-1] } {
	    append ret ";"
	} else {
	    append ret ";\n"
	}
	incr iname
    }
    return $ret
}

# Escribe un campo de ndice index de una lista de un campo del material.
# Por defecto los ndices van de 1 a n. Esto se define con base.
proc TdynOutput::Matlist { material field index {base 1} } {

    set values [lrange [GiD_AccessValue get materials $material $field] 2 end]
    set vlen [llength $values]

    if { $index > $vlen || $index < $base } {
	WarnWin "[= {Error found calling Matlist}].\n[= {Index %s found for %s values} $index $vlen]."
    }
    return [lindex $values [expr $index-$base]]
}


# Escribe un campo de ndice index de una lista de gendata.
# Por defecto los ndices van de 1 a n. Esto se define con base.
proc TdynOutput::Genlist { field index {base 1} } {
    set values [lrange [GiD_AccessValue get gendata $field] 2 end]
    if { [expr $index-$base] >= [llength $values] } {
	WarnWin "[= {Error found calling Genlist}].\n[= {Index %s found for %s values} [expt $index-$base+1] [llength $values]]."
    }
    return [lindex $values [expr $index-$base]]
}

# Devuelve unas unidades sin los corchetes que se guardan en el campo
proc TdynOutput::GiveUnits { field } {
    if { [catch {
	set ounits [GiD_AccessValue get gendata $field]
    } ] } {
	return ""
    }
    return [TUI_formatunits NumsToPlain $ounits]
}

# -------------------------------------------------------------------
#  Procedure Mat_DarcyReaction
#   
#      Devuelve una componente del vector de reaccin de la velocidad 
#      que contiene el trmino correspondiente de la ley de Darcy 
#      (-D*Mu*v)/vi (ojo Mu es viscosidad dividido por densidad).
#      Dado que los trminos de aceleracin que aparecen tienen la
#      forma k11*mu*vx+k12*mu*vy, lo que se escribe es ese trmino
#      dividido por vx (la reaccin k11*mu+k12*mu*vy/vx).
#      Argumentos: 
#        matname: indica el nombre del material 
#        type: X,Y or Z (component of the acceleration vector)
#      ejemplo:
#      
#      matfluid::write_acceleration fluid Y
#
# -------------------------------------------------------------------

proc TdynOutput::Mat_DarcyReaction { matname type } {

    global ProgramName

    # Read info from material
    if { $type=="X" || $type=="x" } {
	set DR1 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D11"]
	set DR2 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D12"]
	if { $ProgramName == "Tdyn3D" } {
	    set DR3 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D13"]
	} else {
	    set DR3 0.0 
	}
	set VEL "vx"
    } elseif { $type=="Y" || $type=="y" } {
	set DR1 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D12"]
	set DR2 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D22"]
	if { $ProgramName == "Tdyn3D" } {
	    set DR3 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D23"]
	} else {
	    set DR3 0.0 
	}
	set VEL "vy"
    } elseif { $type=="Z" || $type=="z" } {
	if { $ProgramName == "Tdyn2D" } { return 0.0 }
	set DR1 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D13"]
	set DR2 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D23"]
	set DR3 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D33"]
	set VEL "vz"
    } else { return 0.0 }
    # Calculate units factor
    # set ounits [GiD_AccessValue get gendata "Internal_Time_Units"]
    set gunits [GiD_AccessValue get materials $matname "Darcys_Coefficient_Units"]
    set iunits "\[1/[string range [GiD_AccessValue get gendata "Internal_Length_Units"] 1 end-1]2\]"
    set factor [TUI_exectdyntool [list -no_timetable \
		-tool convert_units 1 $gunits $iunits Permeability_Inverse]]
    if { $DR1==0.0 && $DR2==0.0 && $DR3==0.0 } { return 0.0 }
    set ret ""
    if { $DR1!=0.0 } { append retD +$DR1*vx }
    if { $DR2!=0.0 } { append retD +$DR2*vy }
    if { $DR3!=0.0 } { append retD +$DR3*vz }
    if { $factor != 1 } { 
	append ret "$factor*kvs*([string trimleft $retD {+}])/absmax($VEL,1.0e-20)"
    } else { 
	append ret "kvs*([string trimleft $retD {+}])/absmax($VEL,1.0e-20)" 
    }
    return $ret
}

# -------------------------------------------------------------------
#  Procedure Mat_Acceleration (Obsoleto, usar el anterior!!!)
#   
#      Devuelve una componente del vector de aceleracin a la que 
#      se aade el trmino correspondiente de la ley de Darcy 
#      (-D*Mu*v).
#      Argumentos: 
#        matname: indica el nombre del material 
#        type: X,Y or Z (component of the acceleration vector)
#      ejemplo:
#      
#      matfluid::write_acceleration fluid Y
#
# -------------------------------------------------------------------

proc TdynOutput::Mat_Acceleration { matname type } {

    global ProgramName

    # Read info from material
    if { $type=="X" || $type=="x" } {
	set Acc [GiD_AccessValue get materials $matname "AccOX_Field"]
	set DR1 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D11"]
	set DR2 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D12"]
	if { $ProgramName == "Tdyn3D" } {
	    set DR3 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D13"]
	} else {
	    set DR3 0.0 
	}
    } elseif { $type=="Y" || $type=="y" } {
	set Acc [GiD_AccessValue get materials $matname "AccOY_Field"]
	set DR1 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D12"]
	set DR2 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D22"]
	if { $ProgramName == "Tdyn3D" } {
	    set DR3 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D23"]
	} else {
	    set DR3 0.0 
	}
    } elseif { $type=="Z" || $type=="z" } {
	if { $ProgramName == "Tdyn2D" } { return 0.0 }
	set Acc [GiD_AccessValue get materials $matname "AccOY_Field"]
	set DR1 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D13"]
	set DR2 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D23"]
	set DR3 [GiD_AccessValue get materials $matname "Darcys_Coefficient_D33"]
    } else { return 0.0 }
    # Calculate units factor
    set gunits [GiD_AccessValue get materials $matname "Darcys_Coefficient_Units"]
    set iunits "\[1/[string range [GiD_AccessValue get gendata "Internal_Length_Units"] 1 end-1]2\]"
    set factor [TUI_exectdyntool [list -no_timetable \
		-tool convert_units 1 $gunits $iunits Permeability_Inverse]]
    if { $DR1==0.0 && $DR2==0.0 && $DR3==0.0 } { return $Acc }
    set ret ""
    if { $Acc!=0.0 } { append ret $Acc }
    set retD ""
    if { $DR1!=0.0 } { append retD +$DR1*vx }
    if { $DR2!=0.0 } { append retD +$DR2*vy }
    if { $DR3!=0.0 } { append retD +$DR3*vz }
    if { $factor != 1 } { 
	append ret "-$factor*kvs*([string trimleft $retD {+}])"
    } else { 
	append ret "-kvs*([string trimleft $retD {+}])" 
    }
    return $ret
}

# -------------------------------------------------------------------
#  Utilities for ramsolid output
# -------------------------------------------------------------------

namespace eval ramsolid {
    variable nsteps 1
}


proc ramsolid::OutputLoadcasesData {} {
    variable nsteps

    set Number_of_Steps [GidUtils::GetProblemDataValue Number_of_Steps]
    set Coupling_Step [GidUtils::GetProblemDataValue Coupling_Step]
    set Coupling_Start [GidUtils::GetProblemDataValue Coupling_Start]
    set Coupling_Type [GidUtils::GetProblemDataValue Coupling_Type]
    set Restart [GidUtils::GetProblemDataValue Restart] ;# On

    if { $Coupling_Type ne "Step_By_Step" } {
	set nsteps 1
    } elseif { $Restart ne "On" } {
	set nsteps [expr {int(($Number_of_Steps-$Coupling_Start)/$Coupling_Step+1)}]
    } else {
	set pn [GiveProjectName]
	set rstfile [file join $pn.gid [file tail $pn].flavia.rst]
	if { ![file readable $rstfile] } {
	    error "error: file '$rstfile' does not exist"
	}
	set fin [open $rstfile r]
	set data [read $fin]
	close $fin

	regexp {Current_Step\s*=\s*(\d+)} $data {} Current_Step
	set nsteps [expr {int(($Number_of_Steps-$Coupling_Start)/$Coupling_Step+1)}]
	if { $Current_Step >= $Coupling_Start } {
	    set nsteps [expr {($Number_of_Steps-$Current_Step)/$Coupling_Step}]
	}
    }
    set _ "combined_load_cases\nactive\n"
    for { set i 0 } { $i < $nsteps } { incr i } {
	append _ [format "combined_load name=RAMSOLID_tdynstep_%d type=strengths\n" $i]
	append _ [format "1.0,RAMSOLID_tdynstep_%d\nend combined_load\n" $i]
    }
    append _ "end combined_load_cases\n"
    return $_
}

proc ramsolid::OutputTdyn_Strain {} {
    variable nsteps

    set Coupling_Type [GidUtils::GetProblemDataValue Coupling_Type]
    if { $Coupling_Type eq "None" } { return "" }

    set data [GiD_Info conditions Tdyn_Strain mesh]
    if { ![llength $data] } { return "" }

    set _ ""
    append _ "temperature_loads\n"
    for { set i 0 } { $i < $nsteps } { incr i } {
	foreach j $data {
	    foreach "- num - -" $j break
	    append _ [format "%d 0.0 -1.0 name=RAMSOLID_tdynstep_%d\n" \
		    $num $i]
	}
    }
    append _ "end temperature_loads\n"
    return $_
}

