namespace eval unitdatwin {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_fluid [IsVarOnTdynDefaults show_fluid 1]
    variable show_solid [IsVarOnTdynDefaults show_solid 1]
    variable show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    variable show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    variable show_ransol [IsVarOnTdynDefaults show_ransol 1]
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    variable show_advect [IsVarOnTdynDefaults show_advect 1]
    variable show_naval [IsVarOnTdynDefaults show_naval 1]

    variable Aux_Length_Units [list m dm cm mm um pm in pica ft yd \u00c5 Nmi]
    variable Aux_Mass_Units [list Kg g t ton lb]
    variable Aux_Time_Units [list s ds cs ms min h day]
    variable Aux_Force_Units [list N KN Kp lbf dn]
    variable Aux_Velocity_Units [list m/s cm/s mm/s Km/h in/s ft/s knot mach]
    variable Aux_Pressure_Units [list m2/s2 dm2/s2 cm2/s2 psi]
    variable Aux_KEnergy_Units [list m\u00b2/s\u00b2 dm\u00b2/s\u00b2 mm\u00b2/s\u00b2 \
	    m\u00b2/cs\u00b2 m\u00b2/ms\u00b2 ft\u00b2/s\u00b2]
    variable Aux_Temperature_Units [list \u2070C K \u2070F]
    variable Aux_Angle_Units [list \u2070 rad \u0027 \u0022]
    variable Aux_Moment_Units [list N.m KN.m Kp.m lbf.in]

    variable Aux_Few_Length_Units [list m cm mm in]
    variable Aux_Few_Temperature_Units [list \u2070C \u2070F]

    # IN/OUT Units
   
    variable InOut_UnitsData [list \
	    [list Output_Length_Units "[= {Output Length Units}]" string 0 Aux_Length_Units Aux_Few_Length_Units] \
	    [list Output_Mass_Units "[= {Output Mass Units}]" string 0 Aux_Mass_Units] \
	    [list Output_Time_Units "[= {Output Time Units}]" string 0 Aux_Time_Units] \
	    [list Output_Temperature_Units "[= {Output Temperature Units}]" string 0 Aux_Temperature_Units Aux_Few_Temperature_Units] \
	    [list Output_Force_Units "[= {Output RamSolid Units}]" string 0 Aux_Force_Units] \
	    [list Internal_Length_Units "[= {Internal Length Units}]" string 0 Aux_Length_Units Aux_Few_Length_Units] \
	    [list Internal_Mass_Units "[= {Internal Mass Units}]" string 0 Aux_Mass_Units] \
	    [list Internal_Time_Units "[= {Internal Time Units}]" string 0 Aux_Time_Units] \
	    [list Internal_Temperature_Units "[= {Internal Temperature Units}]" string 0 Aux_Temperature_Units] \
	    ]
    foreach item $InOut_UnitsData {
	variable [lindex $item 0]
    }

    # Geometry Units
    variable Geometry_UnitsData [list \
	    [list Geometry_Units "[= {Geometry Units}]" string 0 Aux_Length_Units Aux_Few_Length_Units] \
	    ]
    foreach item $Geometry_UnitsData {
	variable [lindex $item 0]
    }

    # General Units
    variable General_UnitsData [list \
	    [list Boundary_Length_Units "[= {Boundary Length Units}]" string 0 Aux_Length_Units All] \
	    [list Boundary_Mass_Units "[= {Boundary Mass Units}]" string 0 Aux_Mass_Units All] \
	    [list Boundary_Time_Units "[= {Boundary Time Units}]" string 0 Aux_Time_Units All] \
	    [list Boundary_Velocity_Units "[= {Boundary Velocity Units}]" string 0 Aux_Velocity_Units Ransol] \
	    [list Boundary_Pressure_Units "[= {Boundary Pressure Units}]" string 0 Aux_Pressure_Units Ransol] \
	    [list Boundary_Kenr_Units "[= {Boundary Kenr Units}]" string 0 Aux_KEnergy_Units Ransol] \
	    [list Boundary_ELen_Units "[= {Boundary ELen Units}]" string 0 Aux_Length_Units Ransol] \
	    [list Boundary_Temperature_Units "[= {Boundary Temperature Units}]" string 0 Aux_Temperature_Units Heatrans] \
	    [list Boundary_Angle_Units "[= {Boundary Angle Units}]" string 0 Aux_Angle_Units All] \
	    [list Boundary_Force_Units "[= {Boundary Force Units}]" string 0 Aux_Force_Units Alemesh] \
	    [list Boundary_Moment_Units "[= {Boundary Moment Units}]" string 0 Aux_Moment_Units Alemesh] \
	    ]
    foreach item $General_UnitsData {
	variable [lindex $item 0]
    }

    # Function Editor variables
    variable fluid_variables "x y z nx ny nz t it rand dt exp(1) pi vx vy vz pr tm\
	    vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable example_fluid_variables "x y z nx ny nz t it rand dt exp(1)\
	    pi vx vy vz pr tm vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable fluid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
    variable predefined_ff [list $fluid_variables $example_fluid_variables $fluid_operators]
}

#  If show is set to all, all the units options will be available.
#  If show is set to few, only the basic options will be shown.
proc unitdatwin::init { parent {show "all"} {tags ""} } {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_naval
    variable show_alemesh

    set unw $parent.unw

    if { [TUI_closeotherwindows $unw 1] == 1 } {

	if { $show == "all" } {
	    InitWindow $unw "[= {Tdyn Units Data}]" PreTdynUnitsDataWindowGeom ""
	} else {
	    InitWindow $unw "[= {Units Data}]" PreTdynUnitsDataWindowGeom ""
	}
	set closecmd "unitdatwin::checkclose $unw 1 1 1"
	set acceptcmd "unitdatwin::checkclose $unw 1 0 0"
	wm protocol $unw WM_DELETE_WINDOW "$closecmd"

	# Reading values from Problem Data
	unitdatwin::getvalues

	# Top frame
	set topf [frame $unw.f -bg lightblue]
	# Picture Box
	set l_icon [label $topf.l_icon -image [imagefromtdyn units.gif] -bg lightblue]
	grid $topf -row 0 -column 0 -sticky nwe
	grid $l_icon -row 0 -column 0 -sticky nw
	grid columnconf $topf 1 -weight 1

	if { $show == "all" } {
	    # Main Notebook
	    set pages [NoteBook $unw.pages -homogeneous 1 -arcradius 0]
	    grid $pages -row 1 -column 0 -sticky nwes -pady 5
	    
	    grid columnconf $unw 0 -weight 1
	    grid rowconf $unw 1 -weight 1
	    
	    set pnum 0
	    set firstpage ""
	    
	    # In/Out Page
	    incr pnum
	    set pvar "pg_inout"
	    if { [string length $firstpage] < 1 } { 
		set firstpage $pvar
	    }
	    set fpvar "fpg_inout"
	    set $pvar [$pages insert $pnum $pvar -text "[= {IN/OUT}]"]
	    set $fpvar [$pages getframe $pvar]
	    unitdatwin::inout_data $pg_inout $show
	    #        $pages raise $pvar
	    
	    # Geometry Page
	    incr pnum
	    set pvar "pg_geometry"
	    if { [string length $firstpage] < 1 } { 
		set firstpage $pvar
	    }
	    set fpvar "fpg_geometry"
	    set $pvar [$pages insert $pnum $pvar -text "[= {GEOMETRY}]"]
	    set $fpvar [$pages getframe $pvar]
	    unitdatwin::geometry_data $pg_geometry $show
	    $pages raise $pvar
	    
	    # General Page
	    incr pnum
	    set pvar "pg_general"
	    if { [string length $firstpage] < 1 } { 
		set firstpage $pvar
	    }
	    set fpvar "fpg_general"
	    set $pvar [$pages insert $pnum $pvar -text "[= {GENERAL}]"]
	    set $fpvar [$pages getframe $pvar]
	    unitdatwin::general_data $pg_general $show
	    #        $pages raise $pvar
	} else {
	    # Main Frame
	    set frame [frame $unw.frame]
	    grid $frame -row 1 -column 0 -sticky nwes -pady 5
	    
	    grid columnconf $unw 0 -weight 1
	    grid rowconf $unw 1 -weight 1 

	    grid columnconf $frame 0 -weight 1
	    grid rowconf $frame 0 -weight 1
	    grid rowconf $frame 1 -weight 1 

	    # Geometry & In/Out Info
	    unitdatwin::fewunits_data $frame
	}

	# Buttons frame
	set bf [frame $unw.bf -relief raised -takefocus 1]
	# Accept button
	set btn_accept [button $bf.btn_accept -text [= {Accept Data}] -underline 0 \
		 -width 10 -relief raised -bd 1 -highlightthickness 1 -takefocus 1 \
		-command "$acceptcmd"]
	# Close button
	set btn_close [button $bf.btn_close -text [= {Close}] -underline 0 -width 10\
		-relief raised -bd 1 -highlightthickness 1 -takefocus 1 \
		-command "$closecmd"]

	# GRIDS
	grid $bf -row 2 -column 0 -sticky news
	grid $btn_accept -row 0 -column 0 -sticky ew -padx 5 -pady 5
	grid $btn_close -row 0 -column 1 -sticky ew -padx 5 -pady 5

	# BINDS
	bind $unw <Escape> "tkButtonInvoke $btn_close"
	bind $unw <Alt-c> "tkButtonInvoke $btn_close"
	bind $unw <Alt-a> "tkButtonInvoke $btn_accept"

	if { [info exist pages] } {
	    $pages compute_size
	}
	focus $unw

    }
}

################################################################################
#
#   FewUnits
#
################################################################################

#  If writes basic units info
proc unitdatwin::fewunits_data { frame } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable InOut_UnitsData
    foreach item $InOut_UnitsData {
	variable [lindex $item 0]
    }

    set help_Geometry_Units "[= {Units of the geometry}]."
    set help_Output_Temperature_Units "[= {Temperature units for input/output}]."
    
    variable Geometry_UnitsData
    foreach item $Geometry_UnitsData {
	variable [lindex $item 0]
    }

    # Scrolled Window
    set scrwindow [ScrolledWindow $frame.f -relief ridge -bd 2]
    set scrframe [ScrollableFrame $scrwindow.f]
    $scrwindow setwidget $scrframe
    set bframe [$scrframe getframe]
    grid $scrwindow -row 0 -column 0 -sticky ewns
#    grid rowconfigure $frame 0 -weight 1
#    grid columnconfigure $frame 0 -weight 1
    GidHelp $scrwindow "[= {Enter units data}]" 

    set irow 0
    foreach item $Geometry_UnitsData {
	set name [lindex $item 0]
	set l_$name [Label $bframe.l_$name  -text "[lindex $item 1] "]
	set e_$name [ComboBox $bframe.e_$name -values [eval set temp \$unitdatwin::[lindex $item 5]] \
		-textvariable unitdatwin::$name -command "unitdatwin::check_fewunits $item" -editable 0 \
		-width [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 5]]]]
	grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	incr irow
	if { [info exists help_$name] } {
	    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	}
    }

    set Few_InOut_UnitsData  [list \
	    [list Output_Temperature_Units "[= {Temperature Units}]" string 0 Aux_Few_Temperature_Units] \
	    ]
    foreach item $Few_InOut_UnitsData {
	set name [lindex $item 0]
	set l_$name [Label $bframe.l_$name  -text "[lindex $item 1] "]
	set e_$name [ComboBox $bframe.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		-textvariable unitdatwin::$name -command "unitdatwin::check_fewunits $item" -editable 0 \
		-width [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	incr irow
	if { [info exists help_$name] } {
	    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	}   
    }

}

proc unitdatwin::check_fewunits { var } {
    variable InOut_UnitsData
    foreach item $InOut_UnitsData {
	variable [lindex $item 0]
    }
    variable Geometry_UnitsData
    foreach item $Geometry_UnitsData {
	variable [lindex $item 0]
    }
    variable General_UnitsData
    foreach item $General_UnitsData {
	variable [lindex $item 0]
    }

    if { [regexp $var {Geometry}] } {
	if { [eval set temp $$var] == {[in]} } {
	    set Output_Temperature_Units [Fh]
	} else {
	    set Output_Temperature_Units [Gt]
	}
    } else {
	if { [eval set temp $$var] == {[Fh]} } {
	    set Geometry_Units [in]
	} elseif { $Geometry_Units == {[in]} } {
	    set Geometry_Units [cm]
	}
    }
    set Boundary_Temperature_Units $Output_Temperature_Units
}
################################################################################
#
#   In/Out
#
################################################################################

#  If show is set to all, all the units options will be available.
#  If show is set to few, only the basic options will be shown.
proc unitdatwin::inout_data { page {show "all"} } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable InOut_UnitsData
    foreach item $InOut_UnitsData {
	variable [lindex $item 0]
    }
    set help_Output_Length_Units "[= {Units used in the output data (written in the results file)}]."
    set help_Output_Mass_Units $help_Output_Length_Units
    set help_Output_Time_Units $help_Output_Length_Units
    set help_Output_Temperature_Units $help_Output_Length_Units
    set help_Output_Force_Units "[= {Results units for Ramsolid module (structural calculations)}]."
    set help_Internal_Length_Units "[= {Units used in the internal calculations}].\n\
	[= {Warning 1: These units are used to perform the evaluation of the functions\
	(physical data and other fields) during Tdyn execution}]. [= {If these are changed,\
	functions should be updated accordingly}].\n [= {Warning 2: Change of these units may\
	result in loss of accuracy in the internal calculations}]."
    set help_Internal_Mass_Units $help_Internal_Length_Units
    set help_Internal_Time_Units $help_Internal_Length_Units
    set help_Internal_Temperature_Units $help_Internal_Length_Units
    
    # Title Frame
    set tframe [TitleFrame $page.f -relief ridge -text "[= {Input/Output units}]"]
    set frame [$tframe getframe]
    set irow 0 
    GidHelp $tframe "[= {Select Input/Output units}]" 

    if { $show == "all" } {
	foreach item $InOut_UnitsData {
	    set name [lindex $item 0]
	    set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
	    set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		    -textvariable unitdatwin::$name -editable 1 -width \
		    [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	    grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	    grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	    incr irow
	    if { [info exists help_$name] } {
		GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	    }
	}
    } elseif { $show == "few" } {
	set Few_InOut_UnitsData  [list \
	    [list Output_Temperature_Units "[= {Output Temperature Units}]" string 0 Aux_Few_Temperature_Units] \
	    ]
	foreach item $Few_InOut_UnitsData {
	    set name [lindex $item 0]
	    set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
	    set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		    -textvariable unitdatwin::$name -editable 1 -width \
		    [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	    grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	    grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	    incr irow
	    if { [info exists help_$name] } {
		GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	    }   
	}
    } else {
	return
    }
    grid $tframe -row 0 -column 0 -sticky ewn
    grid columnconfigure $page 0 -weight 1
    grid rowconfigure $page 2 -weight 1
#    GidHelpRecursive $tframe "[= {Select Input/Output units}]"   
}

################################################################################
#
#   Geometry
#
################################################################################

#  If show is set to all, all the units options will be available.
#  If show is set to few, only the basic options will be shown.
proc unitdatwin::geometry_data { page {show "all"} } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable Geometry_UnitsData
    foreach item $Geometry_UnitsData {
	variable [lindex $item 0]
    }

    set help_Geometry_Units "[= {Units of the geometry}]."

    # Title Frame
    set tframe [TitleFrame $page.f -relief ridge -text "[= {Geometry units}]"]
    set frame [$tframe getframe]
    set irow 0 
    
    foreach item $Geometry_UnitsData {
	set name [lindex $item 0]
	set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
	if { $show=="all" } {
	    set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		    -textvariable unitdatwin::$name -editable 1 -width \
		    [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	} elseif { $show=="few" } { 
	    set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 5]] \
		    -textvariable unitdatwin::$name -editable 1 -width \
		    [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 5]]]]
	} else {
	    return
	}
	grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	incr irow
	if { [info exists help_$name] } {
	    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	}
    }
    grid $tframe -row 0 -column 0 -sticky ewn
    grid columnconfigure $page 0 -weight 1
    grid rowconfigure $page 2 -weight 1
    GidHelpRecursive $tframe "[= {Select the units of the geometry (CAD data)}]." 
}

################################################################################
#
#   General
#
################################################################################

#  If show is set to all, all the units options will be available.
#  If show is set to few, only the basic options will be shown.
proc unitdatwin::general_data { page {show "all"} } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable General_UnitsData
    foreach item $General_UnitsData {
	variable [lindex $item 0]
    }

    # Title Frame
    set tframe [TitleFrame $page.f -relief ridge -text "[= {General units}]"]
    set frame [$tframe getframe]
    set irow 0 
    
    if { $show == "all" } {
	foreach item $General_UnitsData {
	    if { ($show_ransol && [lindex $item 5]=="Ransol") || \
		($show_heatrans && [lindex $item 5]=="Heatrans") || [lindex $item 5]=="All" } {
		set name [lindex $item 0]
		set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
		set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		        -textvariable unitdatwin::$name -editable 1 -width \
		        [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
		grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
		grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
		incr irow
		if { [info exists help_$name] } {
		    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
		}
	    }
	}
    } elseif { $show == "few" } {   
	set Few_General_UnitsData [list \
		[list Boundary_Temperature_Units [= {Boundary Temperature Units}] string 0 Aux_Few_Temperature_Units Heatrans] \
		]
	foreach item $Few_General_UnitsData {
	    if { ($show_ransol && [lindex $item 5]=="Ransol") || \
		($show_heatrans && [lindex $item 5]=="Heatrans") || [lindex $item 5]=="All" } {
		set name [lindex $item 0]
		set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
		set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		        -textvariable unitdatwin::$name -editable 1 -width \
		        [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
		grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
		grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
		incr irow
		if { [info exists help_$name] } {
		    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
		}
	    }
	} 
    }

    grid $tframe -row 0 -column 0 -sticky ewn
    grid columnconfigure $page 0 -weight 1
    grid rowconfigure $page 2 -weight 1
    GidHelpRecursive $tframe "[= {Select the units used in the definition of initial and boundary\
	    conditions}].\n[= {These units will be used in the definition of the boundary conditions\
	    of the problem and other information}]." 
}

################################################################################
#
#   Get Values
#
################################################################################

proc unitdatwin::getvalues { } {

    set unilist [list InOut_UnitsData Geometry_UnitsData General_UnitsData]
    set gendata [.central.s info gendata]

    # Reading units
    for {set i 0} {$i < [llength $unilist]} {incr i} {
	set thisvar [lindex $unilist $i]
	variable $thisvar
	foreach item [eval set nothing $$thisvar] {
	    variable [lindex $item 0]
	}
	foreach item [eval set nothing $$thisvar] {
	    set itt [lindex $item 0]
	    set nitt [lindex $item 1]
	    set $itt [TUI_formatunits Nums2Unicode \
		    [DWUnder2Space [join [lindex $gendata [expr [lsearch -regexp $gendata \
		                        [eval set nothing ^${itt}(\$|\#CB)]] + 1]]]] 0]
	}
    }
}

proc unitdatwin::checkclose { win {dumpit 1} {askdump 1} {close 1} } {

    global tkPriv tcl_platform

    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver

    # Checking info 
    set valueschange 0
    set wtsave 0
    set usr_err ""
    set num_usr_err 0
    set unilist [list InOut_UnitsData Geometry_UnitsData General_UnitsData]
    set paglist [list {IN/OUT} GEOMETRY GENERAL]
    
    set gendata [.central.s info gendata]

    # Checking units
    for {set i 0} {$i < [llength $unilist]} {incr i} {
	set thisuni [lindex $unilist $i]
	set thispag [lindex $paglist $i]
	set uerrpg_$thispag 0
	if { $thisuni != "" } {
	    variable $thisuni
	    foreach item [eval set nothing $$thisuni] {
		variable [lindex $item 0]
	    }
	    foreach item [eval set nothing $$thisuni] {
		set itt [lindex $item 0]
		set jtt temp_$itt
		set $jtt [TUI_formatunits Unicode2Nums [eval set temp $$itt] 1]
		set temp [join [lindex $gendata [expr [lsearch -regexp $gendata \
		        [eval set nothing ^${itt}(\$|\#CB)]] + 1]]]
		if { [DWUnder2Space $temp] != [eval set nothing $$jtt] } {
		    set valueschange 1
		    if { [lindex $item 3] == 1} {
		        set errcheck [TUI_valuecheck [DWSpace2Under [eval set nothing $$jtt]] \
		                [lindex $item 2] 0 ""]
		        if { $errcheck != "ok"} {
		            incr num_usr_err
		            if { [eval set temp \$uerrpg_$thispag] != 1 } {
		                append usr_err "\n$thispag [= {PAGE}]\n"
		            set uerrpg_$thispag 1
		            }
		            append usr_err "$num_usr_err. " [DWUnder2Space $itt] ": $errcheck\n"
		        }
		    }
		}
	    }
	}
    }

    if { $valueschange == 1 } {
	if { $askdump == 0 } {
	    set answer 0
	} else {
	    set answer [tk_dialogRAM $win "[= {Tdyn Units Data}] " \
		    "[= {Units data has changed}].\n[= {Do you want to save changes?}]" question 0 Yes No]
	}

	if { $answer == 0 && $dumpit == 1} {
	    set wtsave 1
	    if { $num_usr_err > 0 } {
		if { $num_usr_err > 1 } {
		    set err_display "[= {The following %i errors was found in the data input} $num_usr_err]:\n"
		} else {
		    set err_display "[= {The following error was found in the data input}]:\n"
		}
		append err_display $usr_err
		WarnWin $err_display
		set close 0
	    } else {
		unitdatwin::dump
		FloatMessage $win.flm "[= {Units Data were saved}]."
	    }
	}
    } else {
	if { $dumpit == 1 } {
	    FloatMessage $win.flm "[= {No changes need to be saved}]."
	}
    }

    if { $close == 1 } {
	destroy $win
	return $valueschange
    } else {
	if { $num_usr_err == 0 } {
	    return $valueschange
	} else {
	    if { $wtsave == 1 } {
		return "dataerror"
	    } else {
		return "nosave.dataerror"
	    }
	}
    }
}

proc unitdatwin::dump { } {

    variable InOut_UnitsData
    variable Geometry_UnitsData
    variable General_UnitsData

    TUI_dumpsingleunits unitdatwin $InOut_UnitsData ProblemData
    TUI_dumpsingleunits unitdatwin $Geometry_UnitsData ProblemData
    TUI_dumpsingleunits unitdatwin $General_UnitsData ProblemData

    if { [info exists ::gdutils::Geom_Units] } {
	set ::gdutils::Geom_Units [TUI_formatunits Nums2Unicode \
		[GiD_AccessValue get gendata Geometry_Units]]
    }
    return 1
}
