# tdyn3d.tcl  -*- TCL -*-
#---------------------------------------------------------------------------
#This file is written in TCL lenguage 
#For more information about TCL look at: http://www.sunlabs.com/research/tcl/
#
#At least two procs must be in this file:
#
#    InitGIDProject dir - Will be called whenever a project is begun to be used.
#           where dir is the project's directory
#    EndGIDProject - Will be called whenever a project ends to be used.
#
#For more information about GID internals, check the program scripts.
#---------------------------------------------------------------------------

############################################################################
#   InitGIDProject
############################################################################
proc InitGIDProject { dir } {
    global VersionNumber ProgramName ProblemTypePriv GidPriv
#
#   Check GiD Version
#
    set VersionRequired "7.4.9"
    set comp -1
    catch {
	set comp [GiDVersionCmp $VersionRequired]
    }
    if { $comp < 0 } {
	tk_dialogRAM .gid.tempwinw Warning \
		[= {Tdyn requires GiD %s or later} $VersionRequired] error 0 OK
    }
#
#   Define name and version, initialise variables and open splash window
#
    set ProgramNameTmp Tdyn3D
    set VersionNumberTmp "4.11"
    set NewProblemType "No"
    if { ![info exists ProgramName] } { 
	set ProgramName ""
    }
    if { ![info exists VersionNumber] } { 
	set VersionNumber ""
    }

    catch { unset ProblemTypePriv }
    catch { unset TdynWizPriv }
    catch { unset TdynTutPriv }
    catch { unset SATWizPriv }
    catch { namespace delete TdynDefaults }

    set ProblemTypePriv(problemtypedir) $dir

    if { [string compare -nocase $ProgramName $ProgramNameTmp]!=0 || \
	    [string compare -nocase $VersionNumber $VersionNumberTmp]!=0 } {
	set ProgramName $ProgramNameTmp
	set VersionNumber $VersionNumberTmp
	Splash
	if { [winfo exists .splash] } {
	    tkwait window .splash
	}
	set NewProblemType "Yes"
    }

#
#   Load Tdyn utilities file
#
     if { [catch {
	 source [file join $dir .. scripts tdynutils.tcl]
     } errormsg] } {
	 WarnWin "[= {Error found}]: [= {Cannot load file %s} [file join $dir .. scripts tdynutils.tcl]].\n \
		 [= {Tdyn is not correctly installed}].\n$errormsg."
	 return
     }

#
#   Ventana de Configuracin (se abre la primera vez)
#
    if { [IsVarOnTdynDefaults first_start 1] } {
	WriteVarTdynDefaults first_start 0
	ConfigureTdynWindow 0
	tkwait window .gid.tdynconf
	#   Meshing surfaces with Rsurf
	.central.s process escape escape escape escape Utilities Variables \
		SurfaceMesher 1 escape escape escape
	#   Meshing surfaces with smoothing
	.central.s process escape escape escape escape Utilities Variables \
		Highqualitysmoothing 2 escape escape escape
    }

#
#   Inicializacin de algunas variables necesarias
#
    set ProblemTypePriv(check_functions) [IsVarOnTdynDefaults check_funct 1]
    set ProblemTypePriv(check_units) [IsVarOnTdynDefaults check_units 0]
    set ProblemTypePriv(edit_units) [IsVarOnTdynDefaults edit_units 1]
    set ProblemTypePriv(assembling_type) [IsVarOnTdynDefaults assembl_type 1]
#
#   Additional TCL Source Files
#
     if { [catch {
	 source [file join $dir .. scripts helpviewer.tcl]
	 source [file join $dir .. scripts tkwizard.tcl]
	 source [file join $dir .. scripts tdynwizard.tcl]
	 source [file join $dir .. scripts vofwizard.tcl]
	 source [file join $dir .. scripts tkwidget.tcl]
	 source [file join $dir .. scripts materials.tcl]
	 source [file join $dir .. scripts mat.fluid.tcl]
	 source [file join $dir .. scripts mat.solid.tcl]
	 source [file join $dir .. scripts bond.fluid.tcl]
	 source [file join $dir .. scripts bond.solid.tcl]
	 source [file join $dir .. scripts functioneditor.tcl]
	 source [file join $dir .. scripts interfaceutils.tcl]
	 source [file join $dir .. scripts init.datwin.tcl]
	 source [file join $dir .. scripts solv.datwin.tcl]
	 source [file join $dir .. scripts prb.datwin.tcl]
	 source [file join $dir .. scripts unt.datwin.tcl]
	 source [file join $dir .. scripts tdynoutput.tcl]
    } errormsg] } {
	WarnWin "[= {Error found}]: [= {Tdyn is not correctly installed}].\n$errormsg."
	return
    }

#
#   Define los problemas que se resolvern (por defecto)
#    
    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_naval [IsVarOnTdynDefaults show_naval 1]
    set show_ramsolid [IsVarOnTdynDefaults show_ramsolid 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    WriteVarTdynDefaults show_vof 0
    set show_vof [IsVarOnTdynDefaults show_vof 1]

    set AnyToBeSolved 0
    if { $ProgramName == "Tdyn2D" } {
	set ProblemList [list Solve_FluidFlow_in_Fluid Solve_Temperature_in_Fluid \
		Solve_SpeciesConcentration_in_Fluid Solve_PhiProblems_in_Fluid \
		Solve_FluidFlow_in_Solid Solve_Temperature_in_Solid \
		Solve_SpeciesConcentration_in_Solid Solve_PhiProblems_in_Solid]
    } else {
	set ProblemList [list Solve_FluidFlow_in_Fluid Solve_Temperature_in_Fluid \
		Solve_SpeciesConcentration_in_Fluid Solve_PhiProblems_in_Fluid \
		Solve_FluidFlow_in_Solid Solve_Temperature_in_Solid \
		Solve_SpeciesConcentration_in_Solid Solve_PhiProblems_in_Solid \
		Solve_StructuralProblem_in_Solid]
    }
    set gendata [.central.s info gendata]
    foreach item $ProblemList {
	variable $item
	set $item [DWUnder2Space [join [lindex $gendata [expr [lsearch -regexp $gendata \
		[eval set nothing ^${item}(\$|\#CB)]] + 1]]]]
	if { [eval set nothing $$item] == "Yes" } {
	    set AnyToBeSolved 1
	    break
	}
    }
    if { $AnyToBeSolved == 0 } {
	if { $show_fluid } {
	    GiD_AccessValue set gendata Solve_Fluid Yes
	    if { $show_ransol } { 
		GiD_AccessValue set gendata Solve_FluidFlow_in_Fluid Yes
	    } elseif { $show_heatrans } { 
		GiD_AccessValue set gendata Solve_Temperature_in_Fluid Yes
	    } elseif { $show_advect } { 
		GiD_AccessValue set gendata Solve_SpeciesConcentration_in_Fluid Yes 
	    } elseif { $show_ursolver } { 
		GiD_AccessValue set gendata Solve_PhiProblems_in_Fluid Yes 
	    }
	} else {
	    GiD_AccessValue set gendata Solve_Solid Yes
	    if { $show_ransol } { 
		GiD_AccessValue set gendata Solve_FluidFlow_in_Solid Yes
	    } elseif { $show_heatrans } { 
		GiD_AccessValue set gendata Solve_Temperature_in_Solid Yes
	    } elseif { $show_advect } { 
		GiD_AccessValue set gendata Solve_SpeciesConcentration_in_Solid Yes
	    } elseif { $show_ursolver } { 
		GiD_AccessValue set gendata Solve_PhiProblems_in_Solid Yes
	    } elseif { $show_ramsolid } { 
		GiD_AccessValue set gendata Solve_StructuralProblem_in_Solid Yes
	    }
	}
    }
#
#   Actualizamos los menus de Tdyn
#
    UpdateTdynMenus
    UpdateTdynContextualMenu
#
#   Creates Bitmaps Toolbars
#
    MyPreTdyn3DBitmaps $dir
    MyPostTdyn3DBitmaps $dir
#
#   Pass materials as sets to Postprocess
#
    set ProblemTypePriv(PropertyToPost) [.central.s info variables PropertyToPost]
    if { $ProblemTypePriv(PropertyToPost) != "Material" } {
	.central.s process escape escape escape escape Utilities Variables \
		PropertyToPost Material escape escape escape
    }
#
#   Define the title of the window
#
    global GidPriv
    set GidPriv(ProgName) $ProgramName
    ChangeWindowTitle
#
#   Open configure window
#
    if {[IsVarOnTdynDefaults isconfigured -1]==-1} {
#        WarnWin "[= {Tdyn is not configured}]"
#        ConfigureTdyn
#        WriteVarTdynDefaults isconfigured 1
    }
#
#   Open a tip
#
    set TipsWizPriv(show) [IsVarOnTdynDefaults showtips 1]
    if { $TipsWizPriv(show)==1 &&  $NewProblemType=="Yes" } {
	OpenTipsWizardWindow
    }
}

############################################################################
#   InitGIDPostProcess
############################################################################
proc InitGIDPostProcess {} {
}

############################################################################
#   Changed Language
############################################################################
proc ChangedLanguage { language } {
    global ProblemTypePriv
    # Update Menus
    UpdateTdynMenus
    # Update Tool Bar
    MyPreTdyn3DBitmaps $ProblemTypePriv(problemtypedir)
    MyPostTdyn3DBitmaps $ProblemTypePriv(problemtypedir)
}

############################################################################
#   EndGIDProject
############################################################################
proc EndGIDProject {} {
    # Close menu bar and define the original title of the window
    global ProblemTypePriv ProgramName GidPriv
    # Kill Bitmaps window
    EndMyBitmaps
    # Kill wizard windows (if exist)
    if { [winfo exists .gid.tdynwizard] } {
	destroy .gid.tdynwizard
	DestroyWindows
    }
    # Kill tips windows (if exist)
    if { [winfo exists .gid.tipswizard] } {
	destroy .gid.tipswizard
    }
    # Kill forces and motions windows
    ForcesGraph::kill
    MotionsGraph::kill
    # Kill rest of windows (if exist)
    set winlist [list .gid.tdynconf .gid.fluidmat .gid.solidmat \
	    .gid.fluidbond .gid.solidbond .gid.contacts .gid.tdw .gid.tiw]
    foreach winopen $winlist {
	if { [winfo exists $winopen] } {
	    destroy $winopen
	}      
    }
    # Delete memory
    catch {
	namespace delete initdatwin
	namespace delete solvdatwin
	namespace delete buttons
	namespace delete TUI_images
	namespace delete matfluid
	namespace delete matsolid
	namespace delete FEditor
	namespace delete HelpViewer
	namespace delete TdynDrawGraph
	namespace delete ForcesGraph
	namespace delete MotionsGraph
	namespace delete TdynDefaults
    } 
    # Borramos el contenido de SelectGIDBatFile
    proc SelectGIDBatFile {} {
    }
    # Define the original title of the window
    set GidPriv(ProgName) GiD
    ChangeWindowTitle ""
}

############################################################################
#   EndGIDPostprocess
############################################################################
proc EndGIDPostprocess {} {
}

############################################################################
#   Creamos las barras de herramientas de Tdyn
############################################################################
proc MyPreTdyn3DBitmaps { dir { type "DEFAULT INSIDELEFT"} } {
    global MyPreBitmapsNames MyPreBitmapsCommands MyPreBitmapsHelp 
    global ProgramName ProblemTypePriv

    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1] 
    set show_naval [IsVarOnTdynDefaults show_naval 1] 
    set show_ramsolid [IsVarOnTdynDefaults show_ramsolid 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_vof [IsVarOnTdynDefaults show_vof 1]

    set MyPreBitmapsNames(0) ""
    set MyPreBitmapsCommands(0) ""
    set MyPreBitmapsHelp(0) ""
#    if { $show_naval } { 
#        lappend MyPreBitmapsNames(0) "images/compbox.gif" 
#        lappend MyPreBitmapsCommands(0) [list -np- OpenTdynWizardWindow]
#        lappend MyPreBitmapsHelp(0) [= {Start Tdyn Wizard}]
#    }
    if { $show_ransol } { 
	lappend MyPreBitmapsNames(0) "images/fluidconstraints.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "RANSOL"]
	lappend MyPreBitmapsHelp(0) [= {Open fluid conditions window}]
    }
    if { $show_heatrans } { 
#        lappend MyPreBitmapsNames(0) "images/fire.gif" 
	lappend MyPreBitmapsNames(0) "images/thermo.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "HEATRANS"]
	lappend MyPreBitmapsHelp(0) [= {Open temperature conditions window}]
    }
    if { $show_advect } { 
	lappend MyPreBitmapsNames(0) "images/advect.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "ADVECT"]
	lappend MyPreBitmapsHelp(0) [= {Open species advection conditions window}]
    }
    if { $show_ursolver } { 
#        lappend MyPreBitmapsNames(0) "images/ursolver.gif" 
	lappend MyPreBitmapsNames(0) "images/phicap.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "URSOLVER"]
	lappend MyPreBitmapsHelp(0) [= {Open ursolver conditions window}]
    }
    if { $show_alemesh } { 
	lappend MyPreBitmapsNames(0) "images/alemesh.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "ALEMESH"]
	lappend MyPreBitmapsHelp(0) [= {Open mesh deformation conditions window}]
    }  
    if { $show_naval } { 
	lappend MyPreBitmapsNames(0) "images/wave.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "NAVAL"]
	lappend MyPreBitmapsHelp(0) [= {Open naval conditions window}]
    }
    if { $show_vof } { 
	lappend MyPreBitmapsNames(0) "images/vof.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "VOFLUID"]
	lappend MyPreBitmapsHelp(0) [= {Open naval conditions window}]
    }
    if { $show_ramsolid } { 
	lappend MyPreBitmapsNames(0) "images/constraints.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- GidOpenConditions "RAMSOLID"]
	lappend MyPreBitmapsHelp(0) [= {Open structural calculation (Ramsolid) conditions window}]
    } 

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    if { $show_fluid } { 
	lappend MyPreBitmapsNames(0) "images/drop.gif" 
#        lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "MATERIALS_(FLUID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "MATERIALS_(FLUID)"]
	lappend MyPreBitmapsHelp(0) [= {Open fluid materials window}]
    }
    if { $show_solid } { 
	lappend MyPreBitmapsNames(0) "images/column.gif" 
#        lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "MATERIALS_(SOLID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "MATERIALS_(SOLID)"]
	lappend MyPreBitmapsHelp(0) [= {Open solid materials window}]
    }
    if { $show_fluid } { 
	lappend MyPreBitmapsNames(0) "images/fboundaries.gif" 
#         lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "BOUNDARIES_(FLUID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "BOUNDARIES_(FLUID)"]
	lappend MyPreBitmapsHelp(0) [= {Open fluid boundaries window}]
    }
    if { $show_solid } { 
	lappend MyPreBitmapsNames(0) "images/sboundaries.gif" 
#         lappend MyPreBitmapsCommands(0) [list -np- GidOpenMaterials "BOUNDARIES_(SOLID)"]
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "BOUNDARIES_(SOLID)"]
	lappend MyPreBitmapsHelp(0) [= {Open solid boundaries window}]
    }
    if { $show_solid && $show_fluid } { 
	lappend MyPreBitmapsNames(0) "images/contacts.gif"
	lappend MyPreBitmapsCommands(0) [list -np- materials::init "CONTACTS"]
	lappend MyPreBitmapsHelp(0) [= {Open contacts window}]
    }
    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/initial.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- initdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open initial and conditional data window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/prdata.gif" 
#    lappend MyPreBitmapsCommands(0) [list -np- GidOpenProblemData "PROBLEM"]
    lappend MyPreBitmapsCommands(0) [list -np- prbdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open general data window}]

    lappend MyPreBitmapsNames(0) "images/solvers.gif" 
#    lappend MyPreBitmapsCommands(0) [list -np- GidOpenProblemData "SOLVER"]
    lappend MyPreBitmapsCommands(0) [list -np- solvdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open modules and solvers data window}]

    lappend MyPreBitmapsNames(0) "images/units.gif" 
#    lappend MyPreBitmapsCommands(0) [list -np- GidOpenProblemData "UNITS"]
    lappend MyPreBitmapsCommands(0) [list -np- unitdatwin::init .gid "all" ""]
    lappend MyPreBitmapsHelp(0) [= {Open units window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/mesh.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- MeshingWindow]
    lappend MyPreBitmapsHelp(0) [= {Start meshing window}]

    lappend MyPreBitmapsNames(0) "---" 
    lappend MyPreBitmapsCommands(0) ""
    lappend MyPreBitmapsHelp(0) ""

    lappend MyPreBitmapsNames(0) "images/calc.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- StartCalculate]
    lappend MyPreBitmapsHelp(0) [= {Launch calculation process}]

    lappend MyPreBitmapsNames(0) "images/info.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- PWViewOutput]
    lappend MyPreBitmapsHelp(0) [= {View process info}]

    if { $show_ransol } {
	lappend MyPreBitmapsNames(0) "images/loads.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- OpenForcesWindow]
	lappend MyPreBitmapsHelp(0) [= {View forces on bodies}]
    }
    if { $show_ransol } {
	lappend MyPreBitmapsNames(0) "images/graphfor.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- ForcesGraph::init]
	lappend MyPreBitmapsHelp(0) [= {Graphs of forces on bodies}]
    }
    if { $show_alemesh } {
	lappend MyPreBitmapsNames(0) "images/graphmov.gif" 
	lappend MyPreBitmapsCommands(0) [list -np- MotionsGraph::init]
	lappend MyPreBitmapsHelp(0) [= {Graphs of motions of bodies}]
    }
    lappend MyPreBitmapsNames(0) "images/stop.gif" 
    lappend MyPreBitmapsCommands(0) [list Utilities CancelProcess]
    lappend MyPreBitmapsHelp(0) [= {Cancel calculation process}]

    lappend MyPreBitmapsNames(0) "images/TdynIcon3D.gif" 
    lappend MyPreBitmapsCommands(0) [list -np- Splash 0]
    lappend MyPreBitmapsHelp(0) [= {Tdyn 3D Interface}]

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always
    set prefix Pre
    set - [= {Tdyn3D pre toolbar}] ;# only for translations, to pick the sentence
    set ProblemTypePriv(pretoolbarwin) [CreateOtherBitmaps MyPreTdyn3DBar [= {Tdyn3D pre toolbar}] \
	    MyPreBitmapsNames MyPreBitmapsCommands MyPreBitmapsHelp \
	    $dir "MyPreTdyn3DBitmaps [list $dir]" $type $prefix]
    AddNewToolbar "$ProgramName prebar" ${prefix}MyBarTdyn3DWindowGeom "MyPreTdyn3DBitmaps [list $dir]"
}

proc MyPostTdyn3DBitmaps { dir { type "DEFAULT INSIDELEFT"} } {
    global MyPostBitmapsNames MyPostBitmapsCommands MyPostBitmapsHelp ProgramName ProblemTypePriv

    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]

    set MyPostBitmapsNames(0) ""
    set MyPostBitmapsCommands(0) ""
    set MyPostBitmapsHelp(0) ""

    lappend MyPostBitmapsNames(0) "images/info.gif" 
    lappend MyPostBitmapsCommands(0) [list -np- PWViewOutput]
    lappend MyPostBitmapsHelp(0) [= {View process info}]

    if { $show_ransol } {
	lappend MyPostBitmapsNames(0) "images/loads.gif" 
	lappend MyPostBitmapsCommands(0) [list -np- OpenForcesWindow]
	lappend MyPostBitmapsHelp(0) [= {View forces on bodies}]
    }
    if { $show_ransol } {
	lappend MyPostBitmapsNames(0) "images/graphfor.gif" 
	lappend MyPostBitmapsCommands(0) [list -np- ForcesGraph::init]
	lappend MyPostBitmapsHelp(0) [= {Graphs of forces on bodies}]
    }
    if { $show_ransol } {
	lappend MyPostBitmapsNames(0) "images/graphmov.gif" 
	lappend MyPostBitmapsCommands(0) [list -np- MotionsGraph::init]
	lappend MyPostBitmapsHelp(0) [= {Graphs of motions of bodies}]
    }
    
    lappend MyPostBitmapsNames(0) "images/TdynIcon3D.gif" 
    lappend MyPostBitmapsCommands(0) [list -np- Splash 0]
    lappend MyPostBitmapsHelp(0) [= {Tdyn 3D Interface}]

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always
    set prefix Post
    set - [= {Tdyn3D post toolbar}] ;# only for translations, to pick the sentence
    set ProblemTypePriv(posttoolbarwin) [CreateOtherBitmaps MyPostTdyn3DBar [= {Tdyn3D post toolbar}] \
	    MyPostBitmapsNames MyPostBitmapsCommands MyPostBitmapsHelp \
	    $dir "MyPostTdyn3DBitmaps [list $dir]" $type $prefix]
    AddNewToolbar "$ProgramName postbar" ${prefix}MyBarTdyn3DWindowGeom "MyPostTdyn3DBitmaps [list $dir]"

}

proc EndMyBitmaps {} {
    global ProgramName ProblemTypePriv
    
    ReleaseToolbar "$ProgramName prebar"
    ReleaseToolbar "$ProgramName postbar"
    rename MyPreTdyn3DBitmaps ""
    rename MyPostTdyn3DBitmaps ""
    catch { destroy $ProblemTypePriv(pretoolbarwin) }
    catch { destroy $ProblemTypePriv(posttoolbarwin) } 
}

############################################################################
#   Actualizamos los menus de Tdyn
############################################################################
proc UpdateTdynMenus { } {
    global VersionNumber ProgramName ProblemTypePriv
    global MenuNames MenuEntries MenuCommands MenuAcceler
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP

    set dir $ProblemTypePriv(problemtypedir)
#
#   Oculta Books de conditiones no aplicables
#
    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_naval [IsVarOnTdynDefaults show_naval 1]
    set show_ramsolid [IsVarOnTdynDefaults show_ramsolid 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_vof [IsVarOnTdynDefaults show_vof 1]

    if { !$show_ransol } { 
	GidShowBook conditions RANSOL 0
    }
    if { !$show_heatrans } { 
	GidShowBook conditions HEATRANS 0
    }
    if { !$show_advect } { 
	GidShowBook conditions ADVECT 0
    }
    if { !$show_ursolver } { 
	GidShowBook conditions URSOLVER 0
    }
    if { !$show_alemesh } { 
	GidShowBook conditions ALEMESH 0
    }
    if { !$show_naval } { 
	GidShowBook conditions NAVAL 0
    }
    if { !$show_vof } { 
	GidShowBook conditions VOFLUID 0
    }
    if { !$show_ramsolid } { 
	GidShowBook conditions RAMSOLID 0
    }
    if { !$show_fluid } { 
	GidShowBook materials MATERIALS_(FLUID) 0
    }
    if { !$show_solid } { 
	GidShowBook materials MATERIALS_(SOLID) 0
    }
#
#   Add new entries in Utilities Pulldown
#    
    set ipos [lsearch $MenuNames [_ {Utilities}]]
    if { $ipos != -1 } {
	lappend MenuEntries($ipos) ---
	lappend MenuCommands($ipos) "" 
	lappend MenuAcceler($ipos) "" ""

	if { [IsVarOnTdynDefaults show_naval 1] } { 
	    lappend MenuEntries($ipos) [= {Create Floating Line}]
	    lappend MenuCommands($ipos) {-np- IntersectPlanarFreeSurface}
	    lappend MenuAcceler($ipos) "" ""

	    lappend MenuEntries($ipos) "[= {Read Free Surface}]..."
	    lappend MenuCommands($ipos) {-np- ReadFreeSurfaceFromPrevious}
	    lappend MenuAcceler($ipos) "" ""

	    lappend MenuEntries($ipos) ---
	    lappend MenuCommands($ipos) "" 
	    lappend MenuAcceler($ipos) "" ""
	}

	lappend MenuEntries($ipos) "[= {Tdyn Report}]..."
	lappend MenuCommands($ipos) {-np- UseCreateReportWindow}
	lappend MenuAcceler($ipos) "" ""
    }
    InsertMenuOption [= {Utilities}] "[= {Tdyn Preferences}]..." 3 ConfigureTdynWindow \
	    PRE insert
#
#   Add new entries in Help Preprocess Pulldown
#    
    set ipos [lsearch $MenuNames [_ {Help}]]
    if { $ipos != -1 } {
	set MenuEntries($ipos) [linsert $MenuEntries($ipos) 0 [= {Tdyn tutorials}] \
		[_ {Tdyn help}]]
	set fulldir $dir
	if { [file pathtype $dir]=="relative" } {
	    set fulldir [file join [pwd] $dir]
	}
	set MenuCommands($ipos) [linsert $MenuCommands($ipos) 0 \
				     [list -np- HelpWindow "CUSTOM_HELP_FILE" \
				     [file join $fulldir tdyn3dtutorials tdyn3d_tutorials.html]] \
				     [list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
				     [file join $fulldir ../info tdynreference_toc.html]]]

	set MenuAcceler($ipos) [linsert $MenuAcceler($ipos) 0 "" ""]

	lappend MenuEntries($ipos) "[= {About Tdyn3D}]..."
	lappend MenuCommands($ipos) [list -np- Splash 0]
	lappend MenuAcceler($ipos) ""

	lappend MenuEntries($ipos) "---"
	lappend MenuCommands($ipos) ""
	lappend MenuAcceler($ipos) ""

	lappend MenuEntries($ipos) "[= {Tip of the day}]..."
	lappend MenuCommands($ipos) [list -np- OpenTipsWizardWindow]
	lappend MenuAcceler($ipos) ""
    }
#
#   Add new entries in Help Postprocess Pulldown
#    
    set ipos [lsearch $MenuNamesP [_ {Help}]]
    if { $ipos != -1 } {
	set MenuEntriesP($ipos) [linsert $MenuEntriesP($ipos) 0 [= {Tdyn tutorials}] \
		[= {Tdyn help}]]
	set MenuCommandsP($ipos) [linsert $MenuCommandsP($ipos) 0 \
		[list -np- HelpWindow "CUSTOM_HELP_FILE" \
		[file join $dir tdyn3dtutorials tdyn3d_tutorials.html]] \
		[list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
		[file join $dir ../info tdynreference_toc.html]]]
	set MenuAccelerP($ipos) [linsert $MenuAccelerP($ipos) 0 "" ""]

	lappend MenuEntriesP($ipos) "[= {About Tdyn3D}]..."
	lappend MenuCommandsP($ipos) [list -np- Splash 0]
	lappend MenuAccelerP($ipos) ""

	lappend MenuEntriesP($ipos) "---"
	lappend MenuCommandsP($ipos) ""
	lappend MenuAccelerP($ipos) ""

	lappend MenuEntriesP($ipos) "[= {Tip of the day}]..."
	lappend MenuCommandsP($ipos) [list -np- OpenTipsWizardWindow]
	lappend MenuAccelerP($ipos) ""
    }
#
#   Remove entries in Help menu
#
    catch {
	RemoveMenuOptionIfExists [_ {Help}] [_ {Customization Help}] PREPOST
	RemoveMenuOptionIfExists [_ {Help}] [_ {What is new}] PREPOST
	RemoveMenuOptionIfExists [_ {Help}] [_ {FAQ}] PREPOST 
	RemoveMenuOptionIfExists [_ {Help}] [_ {Visit GiD web}] PREPOST
	RemoveMenuOptionIfExists [_ {Help}] [_ "Visit %s web" $::GidPriv(ProgName)] PREPOST
    }
#
#   Change entries in Meshing menu
#
    set ipos [lsearch $MenuNames [_ {Meshing}]]
    if { $ipos != -1 } {
	set jpos [lsearch $MenuEntries($ipos) "[_ {Generate}]..."]
	if { $jpos != -1 } {
	    set MenuCommands($ipos) [lreplace $MenuCommands($ipos) $jpos $jpos "-np- MeshingWindow"]
	} 
    }
    catch {
	RemoveMenuOption [_ {Meshing}] [_ {Element type}]>[_ {Only points}] PRE
	RemoveMenuOption [_ {Meshing}] [_ {Element type}]>[_ {Linear}] PRE
	RemoveMenuOption [_ {Meshing}] [_ {Element type}]>[_ {Triangle}] PRE
	RemoveMenuOption [_ {Meshing}] [_ {Element type}]>[_ {Quadrilateral}] PRE
    }
#
#   Change entries in Calculate menu
#
    set ipos [lsearch $MenuNames [_ {Calculate}]]
    if { $ipos != -1 } {
	set jpos [lsearch $MenuEntries($ipos) [_ {Calculate}]]
	if { $jpos != -1 } {
	    set MenuCommands($ipos) [lreplace $MenuCommands($ipos) $jpos $jpos "-np- StartCalculate"]
	} 
    }
#
#   Change entries in Data menu
#
    GidChangeDataLabel {Data units} ""
    GidChangeDataLabel {Interval} ""
    GidChangeDataLabel {Conditions} ""
    GidChangeDataLabel {Materials} ""
    GidChangeDataLabel {Interval Data} ""
    GidChangeDataLabel {Problem Data} ""

    GidAddUserDataOptionsMenu [= {Conditions}] "TdynConditionsSubmenu %W" 2
    GidAddUserDataOptions "---" "" 3
    GidAddUserDataOptionsMenu [= {Materials}] "TdynMaterialsSubmenu %W" 4
    GidAddUserDataOptionsMenu [= {Boundaries}] "TdynBoundariesSubmenu %W" 5
    GidAddUserDataOptions "---" "" 6
    GidAddUserDataOptions     [= {Initial Data}] {initdatwin::init .gid "all" ""} 7
    GidAddUserDataOptions "---" "" 8
    GidAddUserDataOptions     [= {Problem Data}] {prbdatwin::init .gid "all" ""} 9
    GidAddUserDataOptions     [= {Modules Data}] {solvdatwin::init .gid "all" ""} 10
    GidAddUserDataOptions     [= {Units Data}] {unitdatwin::init .gid "all" ""} 11
    GidAddUserDataOptions "---" "" 12
    if { [IsVarOnTdynDefaults show_fluid 1] } {
	GidAddUserDataOptionsMenu [= {Problem Wizards}] "TdynWizardsSubmenu %W" 13
	GidAddUserDataOptions "---" "" 14
    }
#
#   Add a new entries in View results, Utilities  and Calculate menus
#
    set idx [lsearch $MenuNamesP [_ {View results}]]
    if { $idx != -1 } {
	lappend MenuEntriesP($idx) "---"
	lappend MenuCommandsP($idx) ""
	lappend MenuEntriesP($idx) [= {Time Table}]
	lappend MenuCommandsP($idx) {-np- OpenTimeTableWindow} 
	if { $show_ransol || $show_alemesh } {
	    lappend MenuEntriesP($idx) "---"
	}
	if { $show_ransol } {
	    lappend MenuCommandsP($idx) ""
	    lappend MenuEntriesP($idx) [= {Forces on Boundaries}]
	    lappend MenuCommandsP($idx) "-np- OpenForcesWindow"
	}
	if { $show_ransol } {
	    lappend MenuEntriesP($idx) [= {Forces Graph}]
	    lappend MenuCommandsP($idx) "-np- ForcesGraph::init"
	    lappend MenuAccelerP($idx) ""
	}
	if { $show_alemesh } {
	    lappend MenuEntriesP($idx) [= {Motions Graph}]
	    lappend MenuCommandsP($idx) "-np- MotionsGraph::init"
	    lappend MenuAccelerP($idx) ""
	}
    } else {
	WarnWin "[= {Tdyn version and GiD version not compatible}]."
    }

    set idx [lsearch $MenuNamesP [_ {Utilities}]]
    if { $idx != -1 } {
	lappend MenuEntriesP($idx) "---"
	lappend MenuCommandsP($idx) ""
	lappend MenuAccelerP($idx) ""
	lappend MenuEntriesP($idx) "[= {Tdyn Report}]..."
	lappend MenuCommandsP($idx) {-np- UseCreateReportWindow} 
	lappend MenuAccelerP($idx) ""
    } else {
	WarnWin "[= {Tdyn version and GiD version not compatible}]."
    }

    set idx [lsearch $MenuNames [_ {Calculate}]]
    if { $idx != -1 } {
	if { $show_ransol || $show_alemesh } {
	    lappend MenuEntries($idx) "---"
	}
	if { $show_ransol } {
	    lappend MenuCommands($idx) ""
	    lappend MenuEntries($idx) [= {Forces on Boundaries}]
	    lappend MenuCommands($idx) "-np- OpenForcesWindow"
	}
	if { $show_ransol } {
	    lappend MenuEntries($idx) [= {Forces Graph}]
	    lappend MenuCommands($idx) "-np- ForcesGraph::init"
	    lappend MenuAcceler($idx) ""
	}
	if { $show_alemesh } {
	    lappend MenuEntries($idx) [= {Motions Graph}]
	    lappend MenuCommands($idx) "-np- MotionsGraph::init"
	    lappend MenuAcceler($idx) ""
	}
    } else {
	WarnWin "[= {Tdyn version and GiD version not compatible}]."
    }
    UpdateMenus
#    CreateTopMenus
}

proc UpdateTdynContextualMenu { } {

    catch {
	ContextualEntity::RegisterUserDataCommand StartTdynContextualMenu
    }
}

#
#  Submenus of Menu Data
#
proc TdynConditionsSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_ransol 1] } { 
	$w add command -label [= {Ransol}] -command [list GidOpenConditions RANSOL]
    }
    if { [IsVarOnTdynDefaults show_heatrans 1] } { 
	$w add command -label [= {Heatrans}] -command [list GidOpenConditions HEATRANS]
    }
    if { [IsVarOnTdynDefaults show_advect 1] } { 
	$w add command -label [= {Advect}] -command [list GidOpenConditions ADVECT]
    }
    if { [IsVarOnTdynDefaults show_ursolver 1] } { 
	$w add command -label [= {Ursolver}] -command [list GidOpenConditions URSOLVER]
    } 
    if { [IsVarOnTdynDefaults show_naval 1] } { 
	$w add command -label [= {Naval}] -command [list GidOpenConditions NAVAL]
    }
    if { [IsVarOnTdynDefaults show_alemesh 1] } { 
	$w add command -label [= {Alemesh}] -command [list GidOpenConditions ALEMESH]
    } 
    if { [IsVarOnTdynDefaults show_vof 1] } { 
	$w add command -label [= {VOFluid}] -command [list GidOpenConditions VOFLUID]
    } 
    if { [IsVarOnTdynDefaults show_ramsolid 1] } { 
	$w add command -label [= {Ramsolid}] -command [list GidOpenConditions RAMSOLID]
    }
    $w configure -postcommand ""
}
proc TdynMaterialsSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_fluid 1] } { 
	$w add command -label [= {Fluid}] -command "materials::init MATERIALS_(FLUID)"
    }
    if { [IsVarOnTdynDefaults show_solid 1] } { 
	$w add command -label [= {Solid}] -command "materials::init MATERIALS_(SOLID)"
    }
    $w configure -postcommand ""
}
proc TdynBoundariesSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_fluid 1] } {
	$w add command -label [= {Fluid}] -command "materials::init BOUNDARIES_(FLUID)"
    }
    if { [IsVarOnTdynDefaults show_solid 1] } { 
	$w add command -label [= {Solid}] -command "materials::init BOUNDARIES_(SOLID)"
    }
    if { [IsVarOnTdynDefaults show_solid 1] && [IsVarOnTdynDefaults show_fluid 1] && \
	([IsVarOnTdynDefaults show_heatrans 1] || [IsVarOnTdynDefaults show_advect 1] || \
	[IsVarOnTdynDefaults show_ursolver 1]) } { 
	$w add command -label [= {Contact}] -command "materials::init CONTACTS"
    }
    $w configure -postcommand ""
}
proc TdynWizardsSubmenu { w } {
    $w del 0 end
    if { [IsVarOnTdynDefaults show_fluid 1] } {
	$w add command -label "[= {Tdyn}]..." -command "OpenTdynWizardWindow"
	$w add cascade -label "[= {Tdyn Tools}]" -menu $w.tdyntools
	if { ![winfo exists $w.tdyntools] } {
	    menu $w.tdyntools -bd 1 -activeborderwidth 1
	} else {
	    $w.tdyntools delete 0 end
	}
	$w.tdyntools add command -label "[= {Control Volume}]..." -command "UseCreateCompBox"
	$w.tdyntools add command -label "[= {Define Data}]..." -command "UseCreateDefData"
	$w.tdyntools add command -label "[= {Generate Mesh}]..." -command "UseGenerateMesh"
	if { [IsVarOnTdynDefaults show_naval 1] } { 
	    $w add command -label "[= {Sink&Trim}]..." -command "OpenSinkTrimWizardWindow"
	}
    }
    if { [IsVarOnTdynDefaults show_vof 1] } {
#        $w add command -label "[= {VOFLUID}]..." -command "OpenVOFWizardWindow"
    }
    $w configure -postcommand ""
}

#
#   Assign Mesh Criteria to Surfaces and Generate the Mesh
#
proc MeshingWindow {} {
    global ProblemTypePriv TdynWizPriv

    if { [lsearch [image names] back-16] == -1 } {
	image create photo back-16 -data {
	    R0lGODlhEAAQAIUAAPwCBBRSdBRObCQ2TBxObISevAQCBNzu/BRGZPz6/FzC
	    3Pz+/HTS5ByyzJze7Mzq9ITC3AQWLAyWvBSavFyuxAwaLAwSHBRafBSOrDzW
	    5AyixCS61ETW3CzG1AQeLAweLAxefBSStEze7CSWtCyatBSCnBRWfAwmPBRW
	    dByixAQSHAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQABAAAAZi
	    QIBwSCwah4HjUTBQFgkFg3MoKBykU0QhoUAIAuAksbpgNByPxQMSGVsVDYlk
	    IqdUiJYLJqORbDgcHRseRR8gISIaEyMkGCVYRBEmeyAnlgaQkSgpmU4RAZ1O
	    KqFOpFNGfkEAIf5oQ3JlYXRlZCBieSBCTVBUb0dJRiBQcm8gdmVyc2lvbiAy
	    LjUNCqkgRGV2ZWxDb3IgMTk5NywxOTk4LiBBbGwgcmlnaHRzIHJlc2VydmVk
	    Lg0KaHR0cDovL3d3dy5kZXZlbGNvci5jb20AOw==
	}
	image create photo forward-16 -data {
	    R0lGODlhEAAQAIUAAPwCBAwyTBRObAw2VDR+nCRKZOzy/KTe7Pz+/KTK3Nzu
	    /Lze7FS+1AyexAyuzBSavAyOtBSmzOTy/BRqjNTm9IzO5ETS3ETa5By61Ayi
	    xByixBRmjAQGDBxCXGSivCySrCSWtBTC3AQOHAQWHAxWdEze7AQKFBRCXAwq
	    PAQCBAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
	    AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAAAALAAAAAAQABAAAAZj
	    QIBwSCwahYGjUjBQGgWEpHNYMBCaT4G2UDggos+EwmBYMBpf6VBgYDgeEMgj
	    IpmoAQVKxXLBPDIXGhscRB0eHyAgDSGBGyJFASMiIiMkJYImUwAnmJqbjp4A
	    KCmhAKSlTn5BACH+aENyZWF0ZWQgYnkgQk1QVG9HSUYgUHJvIHZlcnNpb24g
	    Mi41DQqpIERldmVsQ29yIDE5OTcsMTk5OC4gQWxsIHJpZ2h0cyByZXNlcnZl
	    ZC4NCmh0dHA6Ly93d3cuZGV2ZWxjb3IuY29tADs=
	}
    }

    set w .gid.meshing
    if { [winfo exists $w] } return
    if { [info procs InitWindow] != "" } {
	InitWindow $w "[= {Generate mesh}]" PreMeshingWindowGeom ""
    } else {
	toplevel $w
    }

    set ProjectStatus [.central.s info list_entities status]
    set ProblemTypePriv(sizetransit) 0.0 
    set ProblemTypePriv(genesize) 0.0
    regexp -line "Unstructured size transition\:[ ]?(.+)$" $ProjectStatus -> ProblemTypePriv(sizetransit)
    regexp -line "Last size used\:[ ]?(.+)$" $ProjectStatus -> ProblemTypePriv(genesize)
    if {![string is double -strict $ProblemTypePriv(genesize)] | $ProblemTypePriv(sizetransit) <= 0.0} {
	set ProblemTypePriv(sizetransit) [.central.s info variable SizeTransitionsFactor]
    }
    if {![string is double -strict $ProblemTypePriv(genesize)] | $ProblemTypePriv(genesize) <= 0.0} {
	if {![string is double -strict $ProblemTypePriv(genesize)] | $ProblemTypePriv(genesize) <= 0.0} {
	    set ProblemTypePriv(genesize) [format {%.3g} [expr 0.1*[LayerLength "" D]]]
	    if { $ProblemTypePriv(genesize) == 0 } {
		set ProblemTypePriv(genesize) 1.0
	    }
	}
    }
    set ProblemTypePriv(genesize) [string trim $ProblemTypePriv(genesize)]
    if { ![info exist ProblemTypePriv(meshingtype)] } {
	set ProblemTypePriv(meshingtype) meshall
    }

    labelframe $w.f0 -text [= "Unstructured size transitions"]

    set f01 [frame $w.f0.f1]
    label $f01.l1 -text [= slow] -anchor nw
    label $f01.l2 -text [= fast] -anchor ne
    grid $f01.l1 $f01.l2 -sticky new -padx 5
    grid columnconfigure $f01 "0 1" -weight 1

    set f02 [frame $w.f0.f2]
    button $f02.bt0 -image back-16 -bd 1 -relief flat -overrelief raised \
	    -command {set ProblemTypePriv(sizetransit) [expr $ProblemTypePriv(sizetransit)-0.1]}
    scale $f02.sc -orient h -from 0.1 -to 1.0 \
	    -resolution 0.1 -variable ProblemTypePriv(sizetransit)
    button $f02.bt1 -image forward-16  -bd 1 -relief flat -overrelief raised \
	    -command {set ProblemTypePriv(sizetransit) [expr $ProblemTypePriv(sizetransit)+0.1]}
    grid $f02.bt0 $f02.sc $f02.bt1 -sticky new
    grid columnconfigure $f02 1 -weight 1

    grid $f01     -     -sticky nwe -padx 2 -pady 2
    grid $f02     -     -sticky nsew -padx 2 -pady 2
    grid columnconfigure $w.f0 1 -weight 1
    grid rowconfigure $w.f0 2 -weight 1

    GidHelp "$w.f0" "[= {It controls whether the transitions between\
	    different element sizes are slow or fast}]."

    frame $w.f1 -relief ridge -bd 2
    radiobutton $w.f1.r1 -text "[= {Mesh all surfaces}]" \
	    -var ProblemTypePriv(meshingtype) -value meshall
    radiobutton $w.f1.r2 -text "[= {Mesh surfaces in active layers}]" \
	    -var ProblemTypePriv(meshingtype) -value meshlayers
    radiobutton $w.f1.r3 -text "[= {Mesh using default options}]" \
	    -var ProblemTypePriv(meshingtype) -value meshdefault
    
    GidHelp "$w.f1" "[= {Select what line entities will be meshed}]."

    grid $w.f1.r1 -sticky w
    grid $w.f1.r2 -sticky w
    grid $w.f1.r3 -sticky w
    grid columnconf $w.f1 0 -weight 1

    set ProblemTypePriv(deletefiles) yes
    frame $w.fa -relief ridge -bd 2
    checkbutton $w.fa.c1 -text [= {Delete result files}] \
	    -var ProblemTypePriv(deletefiles) -onvalue yes -offvalue no

    GidHelp "$w.fa" "[= {Select to delete existing result files}]."

    grid $w.fa.c1 -sticky w
    grid columnconf $w.fa 0 -weight 1

    frame $w.f2 -relief ridge -bd 2
    label $w.f2.lg -text [= {Maximum element size}]
    entry $w.f2.eg -relief sunken -textvariable ProblemTypePriv(genesize) -width 2    
    GidHelp "$w.f2" "[= {Insert a value for the global element size}]."
    tkTabToWindow $w.f2.eg

    grid $w.f2.lg $w.f2.eg -sticky w -padx 2 -pady 2
    grid conf $w.f2.eg -sticky we
    grid columnconf $w.f2 1 -weight 1

    grid $w.f2 -sticky we -padx 3 -pady 3
    grid $w.f1 -sticky we -padx 3 -pady 3
    grid $w.fa -sticky we -padx 3 -pady 3
    grid $w.f0 -sticky we -padx 3 -pady 3

    frame $w.buts
    button $w.buts.gn -text [= {Generate}] -width 10 -bd 1 -command  "MeshingCriteria $w"
    button $w.buts.cn -text [= {Cancel}] -width 10 -bd 1 -command "destroy $w"
    button $w.buts.hp -image [imagefromtdyn questionarrow.gif] -command "PickHelp $w.buts.hp" \
	    -highlightthickness 0 -bd 1 -relief flat -overrelief raised
    pack $w.buts.gn $w.buts.cn $w.buts.hp -side left -padx 5 -ipadx 5 -pady 5 

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.buts.cn"
    bind $w <Escape> "tkButtonInvoke $w.buts.cn"
    bind $w <Return> "tkButtonInvoke $w.buts.gn"
    grid $w.buts -sticky s

    grid rowconfigure $w {0 1 2} -weight 1
    grid columnconfigure $w 0 -weight 1   
    wm geom $w ""
}

proc MeshingCriteria {w} {
    global ProblemTypePriv
    #   Controles iniciales
    if {![IsWord real+ $ProblemTypePriv(genesize) $w "[= {In Elements general size field}]."] } {
	return 
    } else {
	if { $ProblemTypePriv(genesize) == 0.0 } {
	    tk_dialogRAM .gid.tempwin [= {Error window}] "[= {General size must be greater than 0.0}]." \
		    error 0 OK
	    return 
	}
	destroy $w
    }   
    #   Se desabilitan ventanas y grficos
    .central.s disable windows 1 
    .central.s disable graphinput 1 
    #   Se cancela la malla que pueda existir
    .central.s process escape escape escape escape Meshing CancelMesh Yes
    # Se borran los archivos de resultados
    if { $ProblemTypePriv(deletefiles) == "yes" } {
	set resfiles [list flavia flavia.res flavia.war flavia.inf flavia.tim \
		flavia.out flavia.ram.res flavia.ram.msh flavia.sat flavia.for \
		flavia.mov]
	DeleteFiles $resfiles
    }
    #   Se aplica la transicin de tamaos
    set ProblemTypePriv(oldsizetransit) [.central.s info variable SizeTransitionsFactor]
    .central.s process escape escape escape escape utilities variables \
	    SizeTransitionsFactor $ProblemTypePriv(sizetransit) escape escape
    #   Se definen las capas que se mallarn
    switch $ProblemTypePriv(meshingtype) {
	meshall {
	    .central.s process escape escape escape escape \
		    Meshing MeshCriteria Mesh Surfaces 1: escape
	}
	meshlayers  { 
	    set layers_on_list [.central.s info layers -on]
	    foreach curr_layer [lrange $layers_on_list 0 end] {
		.central.s process escape escape escape escape \
		        Meshing MeshCriteria Mesh Surfaces layer:$curr_layer escape
	    }
	
	}
    } 
    #   Se genera la malla
    .central.s disable windows 0
    .central.s process escape escape escape escape Meshing Generate $ProblemTypePriv(genesize)
    #   Se ha acabado todo
    .central.s process escape escape escape escape utilities variables \
	    SizeTransitionsFactor $ProblemTypePriv(oldsizetransit) escape escape
    .central.s disable graphinput 0
}    

#
#  Procedure to assign Mesh Criteria to Surfaces
#
proc AssignMeshCriteriaToSurfaces {} {

    set answer [tk_messageBox -message "[= {Assign <Mesh criteria> to all Surfaces?}]" \
		    -title "[= {Assign <Mesh criteria>}]"  -type yesnocancel -icon question]

    case $answer {
	yes {
		    .central.s process escape escape escape escape \
		    Meshing MeshCriteria Mesh Surfaces 1: escape
	}
	no  {
	}
	cancel return
    }
}

#
#   Start Calculation Proccess
#
proc StartCalculate {} {
    global GIDDEFAULT VersionNumber ProgramName ProblemTypePriv
   
    # Determina el nombre del archivo
    set aa [.central.s info Project]
    set ProjectName [lindex $aa 1]
    if { [file extension $ProjectName] == ".gid" } {
    set ProjectName [file root $ProjectName]
    }
    set basename [file tail $ProjectName]

    if { [GiD_AccessValue get gendata Solve_FluidFlow_in_Fluid] == "No" && 
	 [GiD_AccessValue get gendata Solve_Temperature_in_Fluid] == "No" && 
	 [GiD_AccessValue get gendata Solve_SpeciesConcentration_in_Fluid] == "No" && 
	 [GiD_AccessValue get gendata Solve_PhiProblems_in_Fluid] == "No" &&
	 [GiD_AccessValue get gendata Solve_FluidFlow_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_Temperature_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_SpeciesConcentration_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_PhiProblems_in_Solid] == "No" && 
	 [GiD_AccessValue get gendata Solve_StructuralProblem_in_Solid] == "No" } {
	tk_dialogRAM .gid.tempwin [= {Error window}] "[= {Select problem to be solved in window}] \"[= {Data > Problem Data > PROBLEM}]\"." \
	    error 0 OK
	return
    }
#
#   Results file format (ASCII, BINARY, ...)
#
    if { [GiD_AccessValue get gendata Solve_StructuralProblem_in_Solid] != "No" } {
	if { [GiD_AccessValue get gendata Results_File] != "Ascii" } {
	    set answer [tk_messageBox -message "[= {Ramsolid do not support selected results file format}].\n[= {Results file will be written in ASCII format}]" \
		    -title [= {ASCII results file}] -type yesnocancel -icon question]
	    
	    case $answer {
		yes {
		    GiD_AccessValue set gendata Results_File "Ascii"
		}
		no  {
		    return
		}
		cancel return
	    }
	}
    }
#
#   Assembling type definition
#
    if { [catch {
	set Assembling_Type [GiD_AccessValue get gendata Assembling_Type]
    } errormsg] } {
	WarnWin "[= {Old Tdyn file format found}].\n[= {It should be updated to Tdyn version %s or higher} $VersionNumber]."
    } else {
	if { ![string is integer -strict $Assembling_Type] } {
	    set Assembling_Type $ProblemTypePriv(assembling_type)
	}
	if { $Assembling_Type != $ProblemTypePriv(assembling_type) } {
	    if { $Assembling_Type == 1 } {
		set Assembly_Text [= {Nodal Assembling type}]
	    } else {
		set Assembly_Text [= {Elemental Assembling type}]
	    }
	    if { [GiveProjectName] != "UNNAMED" } {
		set answer [tk_messageBox -message "[= {This model was created with %s} $Assembly_Text].\n[= {Check\
		            default value in 'Utilities>Tdyn Preferences...>Algorithm'}].\n[= {Do you want to proceed?}]" \
		        -title [= {Assembling Preferences}] \
		        -type yesnocancel -icon question]
		case $answer {
		    yes {
		        # Let's continue                
		    }
		    no  {
		        return
		    }
		    cancel return
		}
	    }
	}
	GiD_AccessValue set gendata Assembling_Type $ProblemTypePriv(assembling_type)
    }
#
#   Check if results file exists
#   
    if { [ReadGiDVariable UseMoreWindows] != 0 } {
	set directory $ProjectName.gid
	if { [file pathtype $directory] == "relative" } {
	    set directory [file join [pwd] $directory]
	}
	set file [file join $directory $basename.flavia.res]
	
	if { [file exists $file] } {
	    set answer [tk_messageBox -message "[= {Results file exists}].\n[= {Do you want to overwrite it?}]" \
		    -title [= {Results file exists}] -type yesnocancel -icon question]
	    
	    case $answer {
		yes {
		    .central.s process escape escape escape escape  Utilities Calculate escape
		}
		no  {
		    return
		}
		cancel return
	    }
	} else {
	    .central.s process escape escape escape escape Utilities Calculate escape
	}
    } else {
	.central.s process escape escape escape escape Utilities Calculate escape
    }
}

proc Splash { { autodestroy 1 } } {
    global GIDDEFAULT VersionNumber ProgramName ProblemTypePriv

    if { [.gid.central.s disable windows] } { return }

    if { [ winfo exist .splash]} {
	destroy .splash
	update
    }

    if { [catch {
	toplevel .splash
	wm withdraw .splash
	
	set dir $ProblemTypePriv(problemtypedir)
	set im [image create photo -file [ file join $dir images/tdyn-open.gif]]
	set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
	set y [expr [winfo screenheight .splash]/2-[image height $im]/2]
	
	wm geom .splash +$x+$y
	wm transient .splash .gid
	wm overrideredirect .splash 1
	pack [label .splash.l -image $im -relief ridge -bd 2]
	
	label .splash.lv -text "$ProgramName: Ver. $VersionNumber" -relief sunken \
		-bg #9ab6ce -font "times 14"

	place .splash.lv -x 35 -y 270
	bind .splash <1> "destroy .splash"
	bind .splash <KeyPress> "destroy .splash"
	grab .splash
	focus .splash
	update
	wm deiconify .splash
	raise .splash .gid
	update
	if { $autodestroy } {
	    after 2000 "if { [ winfo exist .splash] } { 
		destroy .splash
	    }"
	}
    }] } {
	if { [ winfo exist .splash]} {
	    destroy .splash
	    update
	}
    }
}

#
#  Procedure to create Tdyn3D About window
#
proc AboutTdyn3D { dir { overw .gid } } {
    global GIDDEFAULT VersionNumber

    set ProgName "[= {Tdyn 3D Interface for GiD}]"
    set GenText "[= {Tdyn 3D Interface for GiD}]"
    set Developer "[= {Tdyn 3D (C) COMPASS IS}]"
    set Company [ join {"COMPASS Ingeniera y Sistemas\n" "Barcelona\n" "(Spain)"} ""]
    set Version [= {Ver. %s} $VersionNumber]
    set Adress "C/ Manuel Girona 61, bajos\ne-mail: info@compassis.com\nfax: +34-93-2041909"

    set im [image create photo -file [ file join $dir images/tdyn.gif]]
    
    toplevel .gid.pepe
    wm title .gid.pepe $ProgName

    wm geometry .gid.pepe +[expr [winfo x $overw]+[winfo width $overw]/2-400]+[ \
	    expr [winfo y $overw]+[winfo height $overw]/2-250]

    frame .gid.pepe.mainf -bd 2 -relief ridge

    frame .gid.pepe.mainf1 -bd 1 -relief ridge

    message .gid.pepe.l -justify center -text $ProgName -width 200 -font BigFont

    label .gid.pepe.l2 -text $GenText -justify center -wraplength 200
    label .gid.pepe.l23 -image $im -relief ridge -bd 2
    
    frame .gid.pepe.mainf2 -bd 1 -relief ridge

    label .gid.pepe.l25 -text $Developer -justify left
    label .gid.pepe.l27 -text $Company -wraplength 200
    label .gid.pepe.l30 -text $Adress -justify left
    label .gid.pepe.l3 -text $Version
    frame .gid.pepe.b
    button .gid.pepe.b.ok -text [= {OK}] -command "destroy .gid.pepe" -padx 10
#    button .gid.pepe.b.more -text [= {More}] -command "AddMoreWelcome"
    pack .gid.pepe.b.ok -padx 3 -side left -anchor center -pady 3
#    pack  .gid.pepe.l30 .gid.pepe.l27 .gid.pepe.l25 .gid.pepe.l23 .gid.pepe.l3 .gid.pepe.l2 \
#            .gid.pepe.l -in .gid.pepe.mainf -side bottom -pady 5 -fill x -expand yes

    pack .gid.pepe.l30 .gid.pepe.l27 .gid.pepe.l25 -in .gid.pepe.mainf2 -side bottom -pady 5 -fill both -expand yes
    pack .gid.pepe.l23 .gid.pepe.l3 .gid.pepe.l -in .gid.pepe.mainf1 -side bottom -pady 5 -fill both -expand yes

    pack .gid.pepe.mainf1 .gid.pepe.mainf2 -in .gid.pepe.mainf -side left -fill both -expand yes

    pack .gid.pepe.b -side bottom
    pack .gid.pepe.mainf -side top -expand yes -fill both

    tkwait visibility .gid.pepe
    grab  .gid.pepe 
    tkwait window .gid.pepe
}

################################################################################
# SelectGIDBatFile is called automagically by GiD and must return the
# .bat file name or nothing
#
# added by ramsan jul 2003
################################################################################

proc SelectGIDBatFile { directory basename } {
    set Solve_FluidFlow "No"
    set Solve_Temperature "No"
    set Solve_SpeciesConcentration "No"
    set Solve_PhiProblems "No"
    set Solve_StructuralProblem "No"
    if { [GidUtils::GetProblemDataValue "Solve_FluidFlow_in_Fluid#CB#(Yes,No)"] != "No" || \
	    [GidUtils::GetProblemDataValue "Solve_FluidFlow_in_Solid#CB#(Yes,No)"] != "No" } {
	set Solve_FluidFlow "Yes"
    }
    if { [GidUtils::GetProblemDataValue "Solve_Temperature_in_Fluid#CB#(Yes,No)"] != "No" || \
	    [GidUtils::GetProblemDataValue "Solve_Temperature_in_Solid#CB#(Yes,No)"] != "No" } {
	set Solve_Temperature "Yes"
    }
    if { [GidUtils::GetProblemDataValue "Solve_SpeciesConcentration_in_Fluid#CB#(Yes,No)"] != "No" || \
	    [GidUtils::GetProblemDataValue "Solve_SpeciesConcentration_in_Solid#CB#(Yes,No)"] != "No" } {
	set Solve_SpeciesConcentration "Yes"
    }
    if { [GidUtils::GetProblemDataValue "Solve_PhiProblems_in_Fluid#CB#(Yes,No)"] != "No" || \
	    [GidUtils::GetProblemDataValue "Solve_PhiProblems_in_Solid#CB#(Yes,No)"] != "No" } {
	set Solve_PhiProblems "Yes"
    }
    if { [GidUtils::GetProblemDataValue "Solve_StructuralProblem_in_Solid#CB#(Yes,No)"] != "No" } {
	set Solve_StructuralProblem "Yes"
    }

    # Is Tdyn going to be executed?
    if { $Solve_FluidFlow == "Yes" || $Solve_Temperature == "Yes" || $Solve_SpeciesConcentration == "Yes" || \
	    $Solve_PhiProblems == "Yes"} {
	# Is Ramsolid going to be executed?
	if { $Solve_StructuralProblem == "Yes" } {
	    if { [GidUtils::GetProblemDataValue "Coupling_Type"] != "None" } {
		switch $::tcl_platform(platform) {
		    "windows" { return tdyn3d-ramsolid.win.bat }
		    "default" { return tdyn3d-ramsolid.unix.bat }
		}
	# Error found. Ramsolid and Tdyn cannot run in parallel
	} else {
	    switch $::tcl_platform(platform) {
		    "windows" { return tdyn3dorramsolid.win.bat }
		    "default" { return tdyn3dorramsolid.unix.bat }
		}
	}
	# Tdyn is executed alone
	} else {
	    switch $::tcl_platform(platform) {
		"windows" { return tdyn3d.win.bat }
		"default" { return tdyn3d.unix.bat }
	    }
	}
	# Is Ramsolid going to be executed alone?
    } elseif { $Solve_StructuralProblem == "Yes" } {
	switch $::tcl_platform(platform) {
	    "windows" { return ramsolid.win.bat }
	    "default" { return ramsolid.unix.bat }
	}
    }
    return ""
}

proc RamResultsUnits { } {
    # Convierte las unidades de resultados al formato de RamSeries
    if { [GidUtils::GetProblemDataValue "Output_Length_Units"] == {[m]} } {
	set outlen m
    } elseif { [GidUtils::GetProblemDataValue "Output_Length_Units"] == {[dm]} } {
	set outlen dm
    } elseif { [GidUtils::GetProblemDataValue "Output_Length_Units"] == {[cm]} } {
	set outlen cm
    } elseif { [GidUtils::GetProblemDataValue "Output_Length_Units"] == {[mm]} } {
	set outlen mm
    } else {
	WarnWin [= {Ramsolid result units not accepted.\nN-m will be used.}]
	return N-m
    }
    if { [GidUtils::GetProblemDataValue "Output_Force_Units"] == {[N]} } {
	return "N-$outlen"
    } elseif { [GidUtils::GetProblemDataValue "Output_Force_Units"] == {[KN]} } {
	if { $outlen != "m" } { 
	    WarnWin [= {Ramsolid result units not accepted.\nKN-m-MPa will be used.}]
	    return "kN-m-MPa"
	} 
	return "kN-$outlen-MPa"
    } elseif { [GidUtils::GetProblemDataValue "Output_Force_Units"] == {[Kp]} } {
	if { $outlen != "cm" } { 
	    WarnWin [= {Ramsolid result units not accepted.\nKp-cm will be used.}]
	    return "kp-cm"
	} 
	return "Kp-$outlen"
    } else {
	WarnWin [= {Ramsolid result units not accepted.\nN-m will be used.}]
	return N-m
    }
}

############################################################################
#   Customized control-1 contextual menu
############################################################################

proc StartTdynContextualMenu { menu whatuse type entity } {
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    #   Problemas que se resolvern (preferences)
    set show_ransol [IsVarOnTdynDefaults show_ransol 1]
    set show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    set show_advect [IsVarOnTdynDefaults show_advect 1]
    set show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    set show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    set show_naval [IsVarOnTdynDefaults show_naval 1]
    set show_ramsolid [IsVarOnTdynDefaults show_ramsolid 1]
    set show_fluid [IsVarOnTdynDefaults show_fluid 1]
    set show_solid [IsVarOnTdynDefaults show_solid 1]
    set show_vof [IsVarOnTdynDefaults show_vof 1]

    switch $type Points { set ov ovpnt } Lines { set ov ovline } Surfaces { set ov ovsurf } \
	Volumes { set ov ovvol } Nodes { set ov ovpnt } Elements { set ov [list ovline ovsurf ovvol] }

    switch $type Points - Lines - Surfaces - Volumes { set where geometry } Nodes - \
	Elements { set where mesh }

    # Options for Boundary Conditions
    set books ""
    foreach ovi $ov {
	foreach cnd [.central.s info Conditions $ovi] {
	    set book [.central.s info Conditions $cnd BOOK]
	    lappend conds($book) $cnd
	    set varcnt "show_[string tolower $book]"
	    if { ![info exists $varcnt] } {
		set $varcnt 0
	    }
	    if { [lsearch $books $book] == -1 && [set $varcnt] } {
		lappend books $book
	    }
	}
    }
    if { $whatuse != "POSTUSE" } {
	if { [llength $books] > 0 } {
	    set j 1
	    while { [winfo exists $menu.m$j] } { incr j }
	    set submenu [menu $menu.m$j]
	    $menu add cascade -label [= {Conditions}] -menu $submenu
	    foreach book $books {
		set text $book
		if { $text == "Default" } { set text "" }
		ContextualEntity::AddToMenu $submenu [= {Open %s...} $text] [list GidOpenConditions $book]
	    }
	}
	$menu add separator
    }
    # Options for Materials and Boundaries
    if { $whatuse != "POSTUSE" } {
	set j 1
	while { [winfo exists $menu.m$j] } { incr j }
	set submenu1 [menu $menu.m$j]
	$menu add cascade -label [= {Materials}] -menu $submenu1
	incr j
	set submenu2 [menu $menu.m$j]
	$menu add cascade -label [= {Boundaries}] -menu $submenu2
	if { $show_fluid } {
	    ContextualEntity::AddToMenu $submenu1 [= {Fluid}] [list materials::init MATERIALS_(FLUID)]
	    ContextualEntity::AddToMenu $submenu2 [= {Fluid}] [list materials::init BOUNDARIES_(FLUID)]
	} 
	if { $show_solid } {
	    ContextualEntity::AddToMenu $submenu1 [= {Solid}] [list materials::init MATERIALS_(SOLID)]
	    ContextualEntity::AddToMenu $submenu2 [= {Solid}] [list materials::init BOUNDARIES_(SOLID)]
	} 
	if { $show_fluid && $show_solid && ($show_heatrans || $show_advect || $show_ursolver) } {
	    ContextualEntity::AddToMenu $submenu2 [= {Contact}] [list materials::init CONTACTS]
	}
	$menu add separator
    }

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    return
}